@@ -3,9 +3,9 @@
 # ./compile.sh [ i586-pc-msdosdjgpp | i686-pc-cygwin | i386-pc-mingw32 | powerpc-mac-darwin ]
 # Comment the version definition to not compile the library
 VER_E2FSPROGS=
-VER_PROGSREISERFS="0.3.1-rc8"
-#VER_LIBEWF=20100226
-VER_LIBEWF=20120504
+#VER_PROGSREISERFS="0.3.1-rc8"
+VER_PROGSREISERFS=
+VER_LIBEWF=20140608
 smp_mflags="-j 2"
 crosscompile_target=
 prefix=/usr/
@@ -57,10 +57,16 @@ case "$crosscompile_target" in
 	VER_NTFSPROGS=
 	VER_E2FSPROGS="1.42.8"
   ;;
+  arm-none-linux-gnueabi|powerpc-linux-gnuspe)
+	VER_LIBNTFS3G="2014.2.15"
+	VER_NTFSPROGS=
+	VER_E2FSPROGS="1.42.8"
+  ;;
   *)
 	VER_LIBNTFS3G="2014.2.15"
 	VER_NTFSPROGS=
 	VER_E2FSPROGS="1.42.8"
+	VER_LIBEWF=
   ;;
 esac
 prefix=/usr/$crosscompile_target
@@ -314,28 +320,31 @@ then
 # libewf should work under MacOSX but it hasn't been tested
 # use  --with-ncurses-lib=$prefix/usr/lib to get binaries that don't need libncurses
 # but users may be unable to navigate...
-		$confdir/configure --host=$crosscompile_target --prefix=$prefix $CONFIGUREOPT --without-ewf --enable-sudo --with-sudo-bin=/usr/bin/sudo  --disable-qt --disable-assert
+		$confdir/configure --host=$crosscompile_target --prefix=$prefix $CONFIGUREOPT --without-ewf --enable-sudo --with-sudo-bin=/usr/bin/sudo  --disable-qt --disable-assert --enable-record-compilation-date
                 ;;
 	  i686-apple-darwin9)
-		$confdir/configure --host=$crosscompile_target --prefix=$prefix $CONFIGUREOPT --enable-sudo --with-sudo-bin=/usr/bin/sudo --disable-qt --disable-stack-protector
+		$confdir/configure --host=$crosscompile_target --prefix=$prefix $CONFIGUREOPT --enable-sudo --with-sudo-bin=/usr/bin/sudo --disable-qt --disable-stack-protector --enable-record-compilation-date
                 ;;
           i586-pc-msdosdjgpp)
-		$confdir/configure --host=$crosscompile_target --prefix=$prefix $CONFIGUREOPT --without-ewf --without-iconv --disable-qt
+		$confdir/configure --host=$crosscompile_target --prefix=$prefix $CONFIGUREOPT --without-ewf --without-iconv --disable-qt --enable-record-compilation-date
                 ;;
           i386-mingw32)
-		$confdir/configure --host=$crosscompile_target --prefix=$prefix $CONFIGUREOPT --without-iconv --enable-missing-uuid-ok
+		$confdir/configure --host=$crosscompile_target --prefix=$prefix $CONFIGUREOPT --without-iconv --enable-missing-uuid-ok --enable-record-compilation-date
                 ;;
 	  i686-pc-mingw32|x86_64-pc-mingw32|i686-w64-mingw32|x86_64-w64-mingw32)
-		$confdir/configure --host=$crosscompile_target --prefix=$prefix $CONFIGUREOPT --enable-missing-uuid-ok --enable-qt
+		$confdir/configure --host=$crosscompile_target --prefix=$prefix $CONFIGUREOPT --enable-missing-uuid-ok --enable-qt --enable-record-compilation-date
                 ;;
 	  arm-marvell-linux-gnu)
-		$confdir/configure --host=$crosscompile_target --prefix=$prefix $CONFIGUREOPT --without-ewf --without-ntfs --disable-qt
+		$confdir/configure --host=$crosscompile_target --prefix=$prefix $CONFIGUREOPT --without-ewf --without-ntfs --disable-qt --enable-record-compilation-date
                 ;;
 	  arm-none-linux-gnueabi|powerpc-linux-gnuspe)
-		$confdir/configure --host=$crosscompile_target --prefix=$prefix $CONFIGUREOPT --without-ntfs --disable-qt
+		$confdir/configure --host=$crosscompile_target --prefix=$prefix $CONFIGUREOPT --without-ntfs --disable-qt --enable-record-compilation-date
+                ;;
+          *cygwin*)
+		$confdir/configure --host=$crosscompile_target --prefix=$prefix $CONFIGUREOPT --enable-missing-uuid-ok --enable-record-compilation-date
                 ;;
           *)
-		$confdir/configure --host=$crosscompile_target --prefix=$prefix $CONFIGUREOPT
+		$confdir/configure --host=$crosscompile_target --prefix=$prefix $CONFIGUREOPT --enable-record-compilation-date
                 ;;
 	esac
 	cd $pwd_saved
