@@ -79,7 +79,6 @@ char *get_default_location(void)
 }
 
 #ifdef HAVE_NCURSES
-extern const char *monstr[];
 
 #ifdef __MINGW32__
 #define SPATH_SEP "\\"
@@ -100,7 +99,7 @@ extern const char *monstr[];
 
 static void set_parent_directory(char *dst_directory);
 static void dir_aff_entry(WINDOW *window, file_info_t *file_info);
-static int aff_txt(int line, WINDOW *window, const char *_format, ...) __attribute__ ((format (printf, 3, 4)));
+static int aff_txt(const int line, WINDOW *window, const char *_format, ...) __attribute__ ((format (printf, 3, 4)));
 
 #if defined(DJGPP) || defined(__OS2__)
 void get_dos_drive_list(struct td_list_head *list);
@@ -178,10 +177,8 @@ char *ask_location(const char*msg, const char *src_dir, const char *dst_org)
   do
   {
     DIR* dir;
-    file_info_t dir_list = {
-      .list = TD_LIST_HEAD_INIT(dir_list.list),
-      .name = NULL
-    };
+    file_info_t dir_list;
+    TD_INIT_LIST_HEAD(&dir_list.list);
     wmove(window,5,0);
     wclrtoeol(window);	/* before addstr for BSD compatibility */
     if(has_colors())
@@ -232,7 +229,7 @@ char *ask_location(const char*msg, const char *src_dir, const char *dst_org)
 	  break;
 	/* hide filename beginning by '.' except '.' and '..' */
 	if(dir_entrie->d_name[0]=='.' &&
-	    !dir_entrie->d_name[1]=='\0' &&
+	    !(dir_entrie->d_name[1]=='\0') &&
 	    !(dir_entrie->d_name[1]=='.' && dir_entrie->d_name[2]=='\0'))
 	  continue;
         if(strlen(dst_directory) + 1 + strlen(dir_entrie->d_name) + 1 <= sizeof(current_file)
@@ -522,7 +519,6 @@ char *ask_location(const char*msg, const char *src_dir, const char *dst_org)
 		file_info=td_list_entry(current_file, file_info_t, list);
 		if(current_file!=&dir_list.list &&
 		  (LINUX_S_ISDIR(file_info->st_mode) || LINUX_S_ISLNK(file_info->st_mode)))
-		if(current_file!=&dir_list.list)
 		{
 		  if(strcmp(file_info->name, ".")==0)
 		  {
@@ -553,6 +549,8 @@ char *ask_location(const char*msg, const char *src_dir, const char *dst_org)
 	  if(offset+old_LINES <= pos_num+line_base+1)
 	    offset=pos_num + line_base + 2 - old_LINES;
         } while(quit==ASK_LOCATION_WAITKEY && old_LINES==LINES);
+	wmove(window, line_directory-1, 0);
+	wclrtoeol(window);	/* before addstr for BSD compatibility */
       } while(quit==ASK_LOCATION_UPDATE || old_LINES!=LINES);
       delete_list_file(&dir_list);
     }
@@ -562,6 +560,7 @@ char *ask_location(const char*msg, const char *src_dir, const char *dst_org)
       quit=ASK_LOCATION_NEWDIR;
     }
   } while(quit==ASK_LOCATION_NEWDIR);
+  wrefresh(window);
   delwin(window);
   (void) clearok(stdscr, TRUE);
 #ifdef HAVE_TOUCHWIN
@@ -574,18 +573,7 @@ static void dir_aff_entry(WINDOW *window, file_info_t *file_info)
 {
   char str[11];
   char datestr[80];
-  {
-    const struct tm *tm_p;
-    if(file_info->td_mtime!=0 && (tm_p= localtime(&file_info->td_mtime))!=NULL)
-    {
-      snprintf(datestr, sizeof(datestr),"%2d-%s-%4d %02d:%02d",
-	  tm_p->tm_mday, monstr[tm_p->tm_mon],
-	  1900 + tm_p->tm_year, tm_p->tm_hour,
-	  tm_p->tm_min);
-    } else {
-      strncpy(datestr, "                 ",sizeof(datestr));
-    }
-  }
+  set_datestr((char *)&datestr, sizeof(datestr), file_info->td_mtime);
   mode_string(file_info->st_mode, str);
   wprintw(window, "%s %5u %5u ", 
       str, (unsigned int)file_info->st_uid, (unsigned int)file_info->st_gid);
@@ -594,13 +582,14 @@ static void dir_aff_entry(WINDOW *window, file_info_t *file_info)
   wprintw(window, " %s %s", datestr, file_info->name);
 }
 
-static int aff_txt(int line, WINDOW *window, const char *_format, ...)
+static int aff_txt(const int line, WINDOW *window, const char *_format, ...)
 {
+  int next_line;
   va_list ap;
   va_start(ap,_format);
-  line=vaff_txt(line, window, _format, ap);
+  next_line=vaff_txt(line, window, _format, ap);
   va_end(ap);
-  return line;
+  return next_line;
 }
 #endif
 
