@@ -53,7 +53,9 @@ int change_arch_type_ncurses(disk_t *disk, const int verbose)
   /* arch_list must match the order from menuOptions */
   const arch_fnct_t *arch_list[]={&arch_i386, &arch_gpt, &arch_humax, &arch_mac, &arch_none, &arch_sun, &arch_xbox, NULL};
   unsigned int menu;
-  for(menu=0;arch_list[menu]!=NULL && disk->arch!=arch_list[menu];menu++);
+  for(menu=0;
+      arch_list[menu]!=NULL && disk->arch!=arch_list[menu];
+      menu++);
   if(arch_list[menu]==NULL)
   {
     menu=0;
@@ -68,7 +70,7 @@ int change_arch_type_ncurses(disk_t *disk, const int verbose)
       { 'I', arch_i386.part_name, "Intel/PC partition" },
       { 'G', arch_gpt.part_name, "EFI GPT partition map (Mac i386, some x86_64...)" },
       { 'H', arch_humax.part_name, "Humax partition table" },
-      { 'M', arch_mac.part_name, "Apple partition map" },
+      { 'M', arch_mac.part_name, "Apple partition map (legacy)" },
       { 'N', arch_none.part_name, "Non partitioned media" },
       { 'S', arch_sun.part_name, "Sun Solaris partition"},
       { 'X', arch_xbox.part_name, "XBox partition"},
@@ -91,10 +93,13 @@ int change_arch_type_ncurses(disk_t *disk, const int verbose)
 	wbkgdset(stdscr,' ' | COLOR_PAIR(0));
       wprintw(stdscr, " partition table type has been detected.");
     }
-    wmove(stdscr,20,0);
-    wprintw(stdscr,"Note: Do NOT select 'None' for media with only a single partition. It's very");
-    wmove(stdscr,21,0);
-    wprintw(stdscr,"rare for a disk to be 'Non-partitioned'.");
+    if(disk->arch_autodetected!=&arch_none)
+    {
+      wmove(stdscr,20,0);
+      wprintw(stdscr,"Note: Do NOT select 'None' for media with only a single partition. It's very");
+      wmove(stdscr,21,0);
+      wprintw(stdscr,"rare for a disk to be 'Non-partitioned'.");
+    }
     car=wmenuSelect_ext(stdscr, 23, INTER_PARTITION_Y, INTER_PARTITION_X, menuOptions, 7, "IGHMNSXQ", MENU_BUTTON | MENU_VERT | MENU_VERT_WARN, &menu,&real_key);
     switch(car)
     {
