@@ -23,6 +23,7 @@
 #include <config.h>
 #endif
 
+#ifdef HAVE_NCURSES
 #include <stdio.h>
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
@@ -42,24 +43,20 @@
 #include "guid_cmp.h"
 #include "guid_cpy.h"
 #include "partgpt.h"
-#include "hdaccess.h"
-#include "autoset.h"
 
 extern const arch_fnct_t arch_gpt;
 extern const arch_fnct_t arch_i386;
-extern const arch_fnct_t arch_mac;
 extern const arch_fnct_t arch_none;
 extern const arch_fnct_t arch_sun;
 extern const struct systypes_gtp gpt_sys_types[];
 
-#ifdef HAVE_NCURSES
 struct part_name_struct
 {
   unsigned int index;
   const char *name;
 };
 
-static void change_part_type_ncurses(const disk_t *disk_car,partition_t *partition)
+static void change_part_type_int_ncurses(const disk_t *disk_car, partition_t *partition)
 {
   partition_t *new_partition;
   char response[100];
@@ -67,7 +64,7 @@ static void change_part_type_ncurses(const disk_t *disk_car,partition_t *partiti
   int i;
   unsigned int last[3], done = 0, next = 0;
   struct part_name_struct part_name[0x100];
-  struct MenuItem menuType[]=
+  const struct MenuItem menuType[]=
   {
     { 'P', "Previous",""},
     { 'N', "Next","" },
@@ -112,9 +109,10 @@ static void change_part_type_ncurses(const disk_t *disk_car,partition_t *partiti
   aff_part(stdscr,AFF_PART_ORDER|AFF_PART_STATUS,disk_car,partition);
   screen_buffer_display(stdscr,"",menuType);
   wmove(stdscr,LINES-2,0);
+  wclrtoeol(stdscr);
   wprintw(stdscr,"New partition type [current %02x] ? ",partition->arch->get_part_type(partition));
-  if (get_string(response, sizeof(response), NULL) > 0) {
-    int tmp_val = strtol(response, NULL, 16);
+  if (get_string(stdscr, response, sizeof(response), NULL) > 0) {
+    const int tmp_val = strtol(response, NULL, 16);
     partition->arch->set_part_type(partition,tmp_val);
   }
 }
@@ -122,7 +120,7 @@ static void change_part_type_ncurses(const disk_t *disk_car,partition_t *partiti
 #define	INTER_CHGTYPE_X 0
 #define	INTER_CHGTYPE_Y 23
 
-static void change_part_type_ncurses2(const disk_t *disk_car, partition_t *partition)
+static void change_part_type_list_ncurses(const disk_t *disk_car, partition_t *partition)
 {
   partition_t *new_partition;
   unsigned int intr_nbr_line=0;
@@ -361,13 +359,8 @@ static void gpt_change_part_type(const disk_t *disk_car, partition_t *partition)
   }
 }
 
-void change_part_type(const disk_t *disk_car,partition_t *partition, char **current_cmd)
+void change_part_type_ncurses(const disk_t *disk_car, partition_t *partition)
 {
-  if(*current_cmd!=NULL)
-  {
-    change_part_type_cli(disk_car, partition, current_cmd);
-    return;
-  }
   if(partition->arch==NULL)
   {
     log_error("change_part_type arch==NULL\n");
@@ -379,7 +372,7 @@ void change_part_type(const disk_t *disk_car,partition_t *partition, char **curr
     log_info("Change partition type:\n");
     log_partition(disk_car,partition);
     partition->arch=&arch_none;
-    change_part_type_ncurses2(disk_car, partition);
+    change_part_type_list_ncurses(disk_car, partition);
     log_info("Change partition type:\n");
     log_partition(disk_car,partition);
     partition->arch=&arch_gpt;
@@ -391,52 +384,10 @@ void change_part_type(const disk_t *disk_car,partition_t *partition, char **curr
     return;
   }
   if(partition->arch==&arch_i386 || partition->arch==&arch_sun)
-    change_part_type_ncurses(disk_car, partition);
+    change_part_type_int_ncurses(disk_car, partition);
   else
-    change_part_type_ncurses2(disk_car, partition);
+    change_part_type_list_ncurses(disk_car, partition);
   log_info("Change partition type:\n");
   log_partition(disk_car,partition);
 }
 #endif
-
-int interface_partition_type(disk_t *disk_car, const int verbose, char**current_cmd)
-{
-  const arch_fnct_t *arch_list[]={&arch_i386, &arch_gpt, &arch_none, &arch_sun, &arch_mac, NULL};
-  int ask_user=1;
-  if(*current_cmd!=NULL)
-  {
-    int keep_asking;
-    do
-    {
-      int i;
-      ask_user=0;
-      keep_asking=0;
-      while(*current_cmd[0]==',')
-        (*current_cmd)++;
-      for(i=0;arch_list[i]!=NULL;i++)
-        if(strncmp(*current_cmd, arch_list[i]->part_name_option, strlen(arch_list[i]->part_name_option))==0)
-        {
-          (*current_cmd)+=strlen(arch_list[i]->part_name_option);
-          disk_car->arch=arch_list[i];
-	  autoset_unit(disk_car);
-          keep_asking=1;
-        }
-      if(strncmp(*current_cmd, "ask_type", 8)==0)
-      {
-	(*current_cmd)+=8;
-	ask_user=1;
-      }
-    } while(keep_asking>0);
-  }
-  if(ask_user>0)
-  {
-#ifdef HAVE_NCURSES
-    if(interface_partition_type_ncurses(disk_car))
-      return 1;
-#endif
-  }
-  log_info("%s\n",disk_car->description_short(disk_car));
-  log_info("Partition table type: %s\n",disk_car->arch->part_name);
-  hd_update_geometry(disk_car, 0,verbose);
-  return 0;
-}
