@@ -119,97 +119,108 @@ struct efi_guid_s
 #define PXBOX_FATX	1
 
 #define	GPT_ENT_TYPE_UNUSED		\
-	((efi_guid_t){le32(0x00000000),le16(0x0000),le16(0x0000),0x00,0x00,{0x00,0x00,0x00,0x00,0x00,0x00}})
+	((const efi_guid_t){le32(0x00000000),le16(0x0000),le16(0x0000),0x00,0x00,{0x00,0x00,0x00,0x00,0x00,0x00}})
 #define	GPT_ENT_TYPE_EFI		\
-	((efi_guid_t){le32(0xc12a7328),le16(0xf81f),le16(0x11d2),0xba,0x4b,{0x00,0xa0,0xc9,0x3e,0xc9,0x3b}})
+	((const efi_guid_t){le32(0xc12a7328),le16(0xf81f),le16(0x11d2),0xba,0x4b,{0x00,0xa0,0xc9,0x3e,0xc9,0x3b}})
+/* Extended Boot Partition */
+#define GPT_ENT_TYPE_EBP		\
+	((const efi_guid_t){le32(0xbc13c2ff),le16(0x59e6),le16(0x4262),0xa3,0x52,{0xb2,0x75,0xfd,0x6f,0x71,0x72}})
 #define	GPT_ENT_TYPE_MBR		\
-	((efi_guid_t){le32(0x024dee41),le16(0x33e7),le16(0x11d3),0x9d,0x69,{0x00,0x08,0xc7,0x81,0xf3,0x9f}})
+	((const efi_guid_t){le32(0x024dee41),le16(0x33e7),le16(0x11d3),0x9d,0x69,{0x00,0x08,0xc7,0x81,0xf3,0x9f}})
 #define	GPT_ENT_TYPE_FREEBSD		\
-	((efi_guid_t){le32(0x516e7cb4),le16(0x6ecf),le16(0x11d6),0x8f,0xf8,{0x00,0x02,0x2d,0x09,0x71,0x2b}})
+	((const efi_guid_t){le32(0x516e7cb4),le16(0x6ecf),le16(0x11d6),0x8f,0xf8,{0x00,0x02,0x2d,0x09,0x71,0x2b}})
 #define	GPT_ENT_TYPE_FREEBSD_SWAP	\
-	((efi_guid_t){le32(0x516e7cb5),le16(0x6ecf),le16(0x11d6),0x8f,0xf8,{0x00,0x02,0x2d,0x09,0x71,0x2b}})
+	((const efi_guid_t){le32(0x516e7cb5),le16(0x6ecf),le16(0x11d6),0x8f,0xf8,{0x00,0x02,0x2d,0x09,0x71,0x2b}})
 #define	GPT_ENT_TYPE_FREEBSD_UFS	\
-	((efi_guid_t){le32(0x516e7cb6),le16(0x6ecf),le16(0x11d6),0x8f,0xf8,{0x00,0x02,0x2d,0x09,0x71,0x2b}})
+	((const efi_guid_t){le32(0x516e7cb6),le16(0x6ecf),le16(0x11d6),0x8f,0xf8,{0x00,0x02,0x2d,0x09,0x71,0x2b}})
 #define GPT_ENT_TYPE_FREEBSD_ZFS	\
-	((efi_guid_t){le32(0x516e7cb),le16(0x6ecf),le16(0x11d6),0x8f,0xf8,{0x00,0x02,0x2d,0x09,0x71,0x2b}})
+	((const efi_guid_t){le32(0x516e7cb),le16(0x6ecf),le16(0x11d6),0x8f,0xf8,{0x00,0x02,0x2d,0x09,0x71,0x2b}})
 /*
  * The following is unused but documented here to avoid reuse.
  *
  * GPT_ENT_TYPE_FREEBSD_UFS2	\
- *	((efi_guid_t){le32(0x516e7cb7),le16(0x6ecf),le16(0x11d6),0x8f,0xf8,{0x00,0x02,0x2d,0x09,0x71,0x2b}})
+ *	((const efi_guid_t){le32(0x516e7cb7),le16(0x6ecf),le16(0x11d6),0x8f,0xf8,{0x00,0x02,0x2d,0x09,0x71,0x2b}})
  */
 
 #define	GPT_ENT_TYPE_FREEBSD_VINUM	\
-	((efi_guid_t){le32(0x516e7cb8),le16(0x6ecf),le16(0x11d6),0x8f,0xf8,{0x00,0x02,0x2d,0x09,0x71,0x2b}})
+	((const efi_guid_t){le32(0x516e7cb8),le16(0x6ecf),le16(0x11d6),0x8f,0xf8,{0x00,0x02,0x2d,0x09,0x71,0x2b}})
 
 
 #define	GPT_ENT_TYPE_MS_RESERVED	\
-	((efi_guid_t){le32(0xe3c9e316),le16(0x0b5c),le16(0x4db8),0x81,0x7d,{0xf9,0x2d,0xf0,0x02,0x15,0xae}})
+	((const efi_guid_t){le32(0xe3c9e316),le16(0x0b5c),le16(0x4db8),0x81,0x7d,{0xf9,0x2d,0xf0,0x02,0x15,0xae}})
 #define	GPT_ENT_TYPE_MS_BASIC_DATA	\
-	((efi_guid_t){le32(0xebd0a0a2),le16(0xb9e5),le16(0x4433),0x87,0xc0,{0x68,0xb6,0xb7,0x26,0x99,0xc7}})
+	((const efi_guid_t){le32(0xebd0a0a2),le16(0xb9e5),le16(0x4433),0x87,0xc0,{0x68,0xb6,0xb7,0x26,0x99,0xc7}})
 #define	GPT_ENT_TYPE_MS_LDM_METADATA	\
-	((efi_guid_t){le32(0x5808c8aa),le16(0x7e8f),le16(0x42e0),0x85,0xd2,{0xe1,0xe9,0x04,0x34,0xcf,0xb3}})
+	((const efi_guid_t){le32(0x5808c8aa),le16(0x7e8f),le16(0x42e0),0x85,0xd2,{0xe1,0xe9,0x04,0x34,0xcf,0xb3}})
 #define	GPT_ENT_TYPE_MS_LDM_DATA	\
-	((efi_guid_t){le32(0xaf9b60a0),le16(0x1431),le16(0x4f62),0xbc,0x68,{0x33,0x11,0x71,0x4a,0x69,0xad}})
+	((const efi_guid_t){le32(0xaf9b60a0),le16(0x1431),le16(0x4f62),0xbc,0x68,{0x33,0x11,0x71,0x4a,0x69,0xad}})
+#define	GPT_ENT_TYPE_MS_RECOVERY	\
+	((const efi_guid_t){le32(0xde94bba4),le16(0x06d1),le16(0x4d40),0xa1,0x6a,{0xbf,0xd5,0x01,0x79,0xd6,0xac}})
 
-#define	GPT_ENT_TYPE_LINUX_DATA		GPT_ENT_TYPE_MS_BASIC_DATA
 #define	GPT_ENT_TYPE_LINUX_RAID		\
-	((efi_guid_t){le32(0xa19d880f),le16(0x05fc),le16(0x4d3b),0xa0,0x06,{0x74,0x3f,0x0f,0x84,0x91,0x1e}})
+	((const efi_guid_t){le32(0xa19d880f),le16(0x05fc),le16(0x4d3b),0xa0,0x06,{0x74,0x3f,0x0f,0x84,0x91,0x1e}})
 #define	GPT_ENT_TYPE_LINUX_SWAP		\
-	((efi_guid_t){le32(0x0657fd6d),le16(0xa4ab),le16(0x43c4),0x84,0xe5,{0x09,0x33,0xc8,0x4b,0x4f,0x4f}})
+	((const efi_guid_t){le32(0x0657fd6d),le16(0xa4ab),le16(0x43c4),0x84,0xe5,{0x09,0x33,0xc8,0x4b,0x4f,0x4f}})
 #define	GPT_ENT_TYPE_LINUX_LVM		\
-	((efi_guid_t){le32(0xe6d6d379),le16(0xf507),le16(0x44c2),0xa2,0x3c,{0x23,0x8f,0x2a,0x3d,0xf9,0x28}})
+	((const efi_guid_t){le32(0xe6d6d379),le16(0xf507),le16(0x44c2),0xa2,0x3c,{0x23,0x8f,0x2a,0x3d,0xf9,0x28}})
 #define GPT_ENT_TYPE_LINUX_RESERVED	\
-	((efi_guid_t){le32(0x8da63339),le16(0x0007),le16(0x60c0),0xc4,0x36,{0x08,0x3a,0xc8,0x23,0x09,0x08}})
-
+	((const efi_guid_t){le32(0x8da63339),le16(0x0007),le16(0x60c0),0xc4,0x36,{0x08,0x3a,0xc8,0x23,0x09,0x08}})
+#define GPT_ENT_TYPE_LINUX_DATA	\
+	((const efi_guid_t){le32(0x0fc63daf),le16(0x8483),le16(0x4772),0x8e,0x79,{0x3d,0x69,0xd8,0x47,0x7d,0xe4}})
+#define GPT_ENT_TYPE_LINUX_HOME \
+	((const efi_guid_t){le32(0x933ac7e1),le16(0x2eb4),le16(0x4f13),0xb8,0x44,{0x0e,0x14,0xe2,0xae,0xf9,0x15}})
+#define GPT_ENT_TYPE_LINUX_SRV	\
+	((const efi_guid_t){le32(0x3b8f8425),le16(0x20e0),le16(0x4f3b),0x90,0x7f,{0x1a,0x25,0xa7,0x6f,0x98,0xe8}})
 
 #define GPT_ENT_TYPE_HPUX_DATA	\
-	((efi_guid_t){le32(0x75894c1e),le16(0x3aeb),le16(0x11d3),0xb7,0xc1,{0x7b,0x03,0xa0,0x00,0x00,0x00}})
+	((const efi_guid_t){le32(0x75894c1e),le16(0x3aeb),le16(0x11d3),0xb7,0xc1,{0x7b,0x03,0xa0,0x00,0x00,0x00}})
 #define GPT_ENT_TYPE_HPUX_SERVICE \
-	((efi_guid_t){le32(0xe2a1e728),le16(0x32e3),le16(0x11d6),0xa6,0x82,{0x7b,0x03,0xa0,0x00,0x00,0x00}})
+	((const efi_guid_t){le32(0xe2a1e728),le16(0x32e3),le16(0x11d6),0xa6,0x82,{0x7b,0x03,0xa0,0x00,0x00,0x00}})
 
+#define GPT_ENT_TYPE_MAC_AFS		\
+	((const efi_guid_t){le32(0x7c3457ef),le16(0x0000),le16(0x11aa),0xaa,0x11,{0x00,0x30,0x65,0x43,0xec,0xac}})
 #define GPT_ENT_TYPE_MAC_HFS		\
-	((efi_guid_t){le32(0x48465300),le16(0x0000),le16(0x11aa),0xaa,0x11,{0x00,0x30,0x65,0x43,0xec,0xac}})
+	((const efi_guid_t){le32(0x48465300),le16(0x0000),le16(0x11aa),0xaa,0x11,{0x00,0x30,0x65,0x43,0xec,0xac}})
 #define GPT_ENT_TYPE_MAC_UFS		\
-	((efi_guid_t){le32(0x55465300),le16(0x0000),le16(0x11aa),0xaa,0x11,{0x00,0x30,0x65,0x43,0xec,0xac}})
+	((const efi_guid_t){le32(0x55465300),le16(0x0000),le16(0x11aa),0xaa,0x11,{0x00,0x30,0x65,0x43,0xec,0xac}})
 #define GPT_ENT_TYPE_MAC_RAID	\
-	((efi_guid_t){le32(0x52414944),le16(0x0000),le16(0x11aa),0xaa,0x11,{0x00,0x30,0x65,0x43,0xec,0xac}})
+	((const efi_guid_t){le32(0x52414944),le16(0x0000),le16(0x11aa),0xaa,0x11,{0x00,0x30,0x65,0x43,0xec,0xac}})
 #define GPT_ENT_TYPE_MAC_RAID_OFFLINE 	\
-	((efi_guid_t){le32(0x52414944),le16(0x5f4f),le16(0x11aa),0xaa,0x11,{0x00,0x30,0x65,0x43,0xec,0xac}})
+	((const efi_guid_t){le32(0x52414944),le16(0x5f4f),le16(0x11aa),0xaa,0x11,{0x00,0x30,0x65,0x43,0xec,0xac}})
 #define GPT_ENT_TYPE_MAC_BOOT	\
-	((efi_guid_t){le32(0x426f6f74),le16(0x0000),le16(0x11aa),0xaa,0x11,{0x00,0x30,0x65,0x43,0xec,0xac}})
+	((const efi_guid_t){le32(0x426f6f74),le16(0x0000),le16(0x11aa),0xaa,0x11,{0x00,0x30,0x65,0x43,0xec,0xac}})
 #define GPT_ENT_TYPE_MAC_LABEL 	\
-	((efi_guid_t){le32(0x4c616265),le16(0x6c00),le16(0x11aa),0xaa,0x11,{0x00,0x30,0x65,0x43,0xec,0xac}})
+	((const efi_guid_t){le32(0x4c616265),le16(0x6c00),le16(0x11aa),0xaa,0x11,{0x00,0x30,0x65,0x43,0xec,0xac}})
 #define GPT_ENT_TYPE_MAC_TV_RECOVERY  	\
-	((efi_guid_t){le32(0x5265636f),le16(0x7665),le16(0x11aa),0xaa,0x11,{0x00,0x30,0x65,0x43,0xec,0xac}})
+	((const efi_guid_t){le32(0x5265636f),le16(0x7665),le16(0x11aa),0xaa,0x11,{0x00,0x30,0x65,0x43,0xec,0xac}})
 
 #define GPT_ENT_TYPE_SOLARIS_BOOT  	\
-	((efi_guid_t){le32(0x6a82cb45),le16(0x1dd2),le16(0x11b2),0x99,0xa6,{0x08,0x00,0x20,0x73,0x66,0x31}})
+	((const efi_guid_t){le32(0x6a82cb45),le16(0x1dd2),le16(0x11b2),0x99,0xa6,{0x08,0x00,0x20,0x73,0x66,0x31}})
 #define GPT_ENT_TYPE_SOLARIS_ROOT  	\
-	((efi_guid_t){le32(0x6a85cf4d),le16(0x1dd2),le16(0x11b2),0x99,0xa6,{0x08,0x00,0x20,0x73,0x66,0x31}})
+	((const efi_guid_t){le32(0x6a85cf4d),le16(0x1dd2),le16(0x11b2),0x99,0xa6,{0x08,0x00,0x20,0x73,0x66,0x31}})
 #define GPT_ENT_TYPE_SOLARIS_SWAP  	\
-	((efi_guid_t){le32(0x6a87c46f),le16(0x1dd2),le16(0x11b2),0x99,0xa6,{0x08,0x00,0x20,0x73,0x66,0x31}})
+	((const efi_guid_t){le32(0x6a87c46f),le16(0x1dd2),le16(0x11b2),0x99,0xa6,{0x08,0x00,0x20,0x73,0x66,0x31}})
 #define GPT_ENT_TYPE_SOLARIS_BACKUP  	\
-	((efi_guid_t){le32(0x6a8b642b),le16(0x1dd2),le16(0x11b2),0x99,0xa6,{0x08,0x00,0x20,0x73,0x66,0x31}})
+	((const efi_guid_t){le32(0x6a8b642b),le16(0x1dd2),le16(0x11b2),0x99,0xa6,{0x08,0x00,0x20,0x73,0x66,0x31}})
 #define GPT_ENT_TYPE_SOLARIS_USR  	\
-	((efi_guid_t){le32(0x6a898cc3),le16(0x1dd2),le16(0x11b2),0x99,0xa6,{0x08,0x00,0x20,0x73,0x66,0x31}})
+	((const efi_guid_t){le32(0x6a898cc3),le16(0x1dd2),le16(0x11b2),0x99,0xa6,{0x08,0x00,0x20,0x73,0x66,0x31}})
 #define GPT_ENT_TYPE_MAC_ZFS		GPT_ENT_TYPE_SOLARIS_USR  
 #define GPT_ENT_TYPE_SOLARIS_VAR  	\
-	((efi_guid_t){le32(0x6a8ef2e9),le16(0x1dd2),le16(0x11b2),0x99,0xa6,{0x08,0x00,0x20,0x73,0x66,0x31}})
+	((const efi_guid_t){le32(0x6a8ef2e9),le16(0x1dd2),le16(0x11b2),0x99,0xa6,{0x08,0x00,0x20,0x73,0x66,0x31}})
 #define GPT_ENT_TYPE_SOLARIS_HOME  	\
-	((efi_guid_t){le32(0x6a90ba39),le16(0x1dd2),le16(0x11b2),0x96,0xa6,{0x08,0x00,0x20,0x73,0x66,0x31}})
+	((const efi_guid_t){le32(0x6a90ba39),le16(0x1dd2),le16(0x11b2),0x96,0xa6,{0x08,0x00,0x20,0x73,0x66,0x31}})
 #define GPT_ENT_TYPE_SOLARIS_EFI_ALTSCTR \
-	((efi_guid_t){le32(0x6a9283a5),le16(0x1dd2),le16(0x11b2),0x96,0xa6,{0x08,0x00,0x20,0x73,0x66,0x31}})
+	((const efi_guid_t){le32(0x6a9283a5),le16(0x1dd2),le16(0x11b2),0x96,0xa6,{0x08,0x00,0x20,0x73,0x66,0x31}})
 #define GPT_ENT_TYPE_SOLARIS_RESERVED1	\
-	((efi_guid_t){le32(0x6a945a3b),le16(0x1dd2),le16(0x11b2),0x96,0xa6,{0x08,0x00,0x20,0x73,0x66,0x31}})
+	((const efi_guid_t){le32(0x6a945a3b),le16(0x1dd2),le16(0x11b2),0x96,0xa6,{0x08,0x00,0x20,0x73,0x66,0x31}})
 #define GPT_ENT_TYPE_SOLARIS_RESERVED2	\
-	((efi_guid_t){le32(0x6a9630d1),le16(0x1dd2),le16(0x11b2),0x96,0xa6,{0x08,0x00,0x20,0x73,0x66,0x31}})
+	((const efi_guid_t){le32(0x6a9630d1),le16(0x1dd2),le16(0x11b2),0x96,0xa6,{0x08,0x00,0x20,0x73,0x66,0x31}})
 #define GPT_ENT_TYPE_SOLARIS_RESERVED3	\
-	((efi_guid_t){le32(0x6a980767),le16(0x1dd2),le16(0x11b2),0x96,0xa6,{0x08,0x00,0x20,0x73,0x66,0x31}})
+	((const efi_guid_t){le32(0x6a980767),le16(0x1dd2),le16(0x11b2),0x96,0xa6,{0x08,0x00,0x20,0x73,0x66,0x31}})
 #define GPT_ENT_TYPE_SOLARIS_RESERVED4	\
-	((efi_guid_t){le32(0x6a96237f),le16(0x1dd2),le16(0x11b2),0x96,0xa6,{0x08,0x00,0x20,0x73,0x66,0x31}})
+	((const efi_guid_t){le32(0x6a96237f),le16(0x1dd2),le16(0x11b2),0x96,0xa6,{0x08,0x00,0x20,0x73,0x66,0x31}})
 #define GPT_ENT_TYPE_SOLARIS_RESERVED5	\
-	((efi_guid_t){le32(0x6a8d2ac7),le16(0x1dd2),le16(0x11b2),0x96,0xa6,{0x08,0x00,0x20,0x73,0x66,0x31}})
+	((const efi_guid_t){le32(0x6a8d2ac7),le16(0x1dd2),le16(0x11b2),0x96,0xa6,{0x08,0x00,0x20,0x73,0x66,0x31}})
 
 #define TESTDISK_O_RDONLY     00
 #define TESTDISK_O_RDWR	      02
@@ -220,19 +231,20 @@ struct efi_guid_s
 
 enum upart_type {
   UP_UNK=0,
-  UP_BEOS=1,
-  UP_BTRFS=2,
-  UP_CRAMFS=3,
-  UP_EXFAT=4,
-  UP_EXT2=5,
-  UP_EXT3=6,
-  UP_EXT4=7,
-  UP_EXTENDED=8,
-  UP_FAT12=9,
-  UP_FAT16=0x0a,
-  UP_FAT32=0x0b,
+  UP_BEOS,
+  UP_BTRFS,
+  UP_CRAMFS,
+  UP_EXFAT,
+  UP_EXT2,
+  UP_EXT3,
+  UP_EXT4,
+  UP_EXTENDED,
+  UP_FAT12,
+  UP_FAT16,
+  UP_FAT32,
   UP_FATX,
   UP_FREEBSD,
+  UP_F2FS,
   UP_GFS2,
   UP_HFS,
   UP_HFSP,
@@ -254,6 +266,7 @@ enum upart_type {
   UP_NTFS,
   UP_OPENBSD,
   UP_OS2MB,
+  UP_ReFS,
   UP_RFS,
   UP_RFS2,
   UP_RFS3,
@@ -270,6 +283,7 @@ enum upart_type {
   UP_XFS2,
   UP_XFS3,
   UP_XFS4,
+  UP_XFS5,
   UP_ZFS};
 typedef enum upart_type upart_type_t;
 enum status_type { STATUS_DELETED, STATUS_PRIM, STATUS_PRIM_BOOT, STATUS_LOG, STATUS_EXT, STATUS_EXT_IN_EXT};
@@ -445,19 +459,19 @@ time_t td_ntfs2utc (int64_t ntfstime);
     (((x)&0x00ff)<<8))
 #define le24(x)  (x)
 #define le32(x)  (x)
-#define be32(x)  ((((x)&0xff000000L)>>24)                | \
-    (((x)&0x00ff0000L)>>8)                  | \
-    (((x)&0x0000ff00L)<<8)                  | \
-    (((x)&0x000000ffL)<<24))
+#define be32(x)  ((((x)&0xff000000UL)>>24)                | \
+    (((x)&0x00ff0000UL)>>8)                  | \
+    (((x)&0x0000ff00UL)<<8)                  | \
+    (((x)&0x000000ffUL)<<24))
 #define le64(x)  (x)
-#define be64(x)  ((((x)&0xff00000000000000LL)>>56)       | \
-    (((x)&0x00ff000000000000LL)>>40)        | \
-    (((x)&0x0000ff0000000000LL)>>24)        | \
-    (((x)&0x000000ff00000000LL)>>8)         | \
-    (((x)&0x00000000ff000000LL)<<8)         | \
-    (((x)&0x0000000000ff0000LL)<<24)        | \
-    (((x)&0x000000000000ff00LL)<<40)        | \
-    (((x)&0x00000000000000ffLL)<<56))
+#define be64(x)  ((((x)&0xff00000000000000ULL)>>56)       | \
+    (((x)&0x00ff000000000000ULL)>>40)        | \
+    (((x)&0x0000ff0000000000ULL)>>24)        | \
+    (((x)&0x000000ff00000000ULL)>>8)         | \
+    (((x)&0x00000000ff000000ULL)<<8)         | \
+    (((x)&0x0000000000ff0000ULL)<<24)        | \
+    (((x)&0x000000000000ff00ULL)<<40)        | \
+    (((x)&0x00000000000000ffULL)<<56))
 #else /* bigendian */
 #define le16(x)  ((((x)&0xff00)>>8)                      | \
     (((x)&0x00ff)<<8))
@@ -466,19 +480,19 @@ time_t td_ntfs2utc (int64_t ntfstime);
 #define le24(x) ((((x) & 0x000000ffUL) << 16) | \
     ((x) & 0x0000ff00UL)        | \
     (((x) & 0x00ff0000UL) >> 16))
-#define le32(x)  ((((x)&0xff000000L)>>24)                | \
-    (((x)&0x00ff0000L)>>8)                  | \
-    (((x)&0x0000ff00L)<<8)                  | \
-    (((x)&0x000000ffL)<<24))
+#define le32(x)  ((((x)&0xff000000UL)>>24)                | \
+    (((x)&0x00ff0000UL)>>8)                  | \
+    (((x)&0x0000ff00UL)<<8)                  | \
+    (((x)&0x000000ffUL)<<24))
 #define be32(x)  (x)
-#define le64(x)  ((((x)&0xff00000000000000LL)>>56)       | \
-    (((x)&0x00ff000000000000LL)>>40)        | \
-    (((x)&0x0000ff0000000000LL)>>24)        | \
-    (((x)&0x000000ff00000000LL)>>8)         | \
-    (((x)&0x00000000ff000000LL)<<8)         | \
-    (((x)&0x0000000000ff0000LL)<<24)        | \
-    (((x)&0x000000000000ff00LL)<<40)        | \
-    (((x)&0x00000000000000ffLL)<<56))
+#define le64(x)  ((((x)&0xff00000000000000ULL)>>56)       | \
+    (((x)&0x00ff000000000000ULL)>>40)        | \
+    (((x)&0x0000ff0000000000ULL)>>24)        | \
+    (((x)&0x000000ff00000000ULL)>>8)         | \
+    (((x)&0x00000000ff000000ULL)<<8)         | \
+    (((x)&0x0000000000ff0000ULL)<<24)        | \
+    (((x)&0x000000000000ff00ULL)<<40)        | \
+    (((x)&0x00000000000000ffULL)<<56))
 #define be64(x)  (x)
 #endif
 #ifndef HAVE_SNPRINTF
@@ -494,6 +508,9 @@ int strncasecmp(const char * s1, const char * s2, size_t len);
 #ifndef HAVE_STRCASESTR
 char * strcasestr (const char *haystack, const char *needle);
 #endif
+#if ! defined(HAVE_LOCALTIME_R) && ! defined(__MINGW32__)
+struct tm *localtime_r(const time_t *timep, struct tm *result);
+#endif
 /*
  * td_min()/td_max() macros that also do
  * strict type-checking.. See the
@@ -512,6 +529,9 @@ char * strcasestr (const char *haystack, const char *needle);
         (void) (&_x == &_y);            \
         _x > _y ? _x : _y; })
 
+int check_command(char **current_cmd, const char *cmd, size_t n);
+void skip_comma_in_command(char **current_cmd);
+uint64_t get_int_from_command(char **current_cmd);
 #ifdef __cplusplus
 } /* closing brace for extern "C" */
 #endif
