@@ -41,10 +41,14 @@
 #ifdef HAVE_IO_H
 #include <io.h>
 #endif
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
 #include "common.h"
 #include "dir.h"
 #include "log.h"
 #include "log_part.h"
+#define MAX_DIR_NBR 256
 
 const char *monstr[] = { "Jan", "Feb", "Mar", "Apr", "May", "Jun",
 				"Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};
@@ -107,15 +111,15 @@ static char ftypelet (unsigned int bits)
 void mode_string (const unsigned int mode, char *str)
 {
   str[0] = ftypelet(mode);
-  str[1] = mode & LINUX_S_IRUSR ? 'r' : '-';
-  str[2] = mode & LINUX_S_IWUSR ? 'w' : '-';
-  str[3] = mode & LINUX_S_IXUSR ? 'x' : '-';
-  str[4] = mode & LINUX_S_IRGRP ? 'r' : '-';
-  str[5] = mode & LINUX_S_IWGRP ? 'w' : '-';
-  str[6] = mode & LINUX_S_IXGRP ? 'x' : '-';
-  str[7] = mode & LINUX_S_IROTH ? 'r' : '-';
-  str[8] = mode & LINUX_S_IWOTH ? 'w' : '-';
-  str[9] = mode & LINUX_S_IXOTH ? 'x' : '-';
+  str[1] = (mode & LINUX_S_IRUSR) ? 'r' : '-';
+  str[2] = (mode & LINUX_S_IWUSR) ? 'w' : '-';
+  str[3] = (mode & LINUX_S_IXUSR) ? 'x' : '-';
+  str[4] = (mode & LINUX_S_IRGRP) ? 'r' : '-';
+  str[5] = (mode & LINUX_S_IWGRP) ? 'w' : '-';
+  str[6] = (mode & LINUX_S_IXGRP) ? 'x' : '-';
+  str[7] = (mode & LINUX_S_IROTH) ? 'r' : '-';
+  str[8] = (mode & LINUX_S_IWOTH) ? 'w' : '-';
+  str[9] = (mode & LINUX_S_IXOTH) ? 'x' : '-';
   str[10]='\0';
 #ifdef LINUX_S_ISUID
   if (mode & LINUX_S_ISUID)
@@ -149,6 +153,36 @@ void mode_string (const unsigned int mode, char *str)
 #endif
 }
 
+int set_datestr(char *datestr, size_t n, const time_t timev)
+{
+  const struct tm *tm_p;
+#if ! defined(__MINGW32__)
+  struct tm tmp;
+#endif
+  if(timev==0)
+  {
+    strncpy(datestr, "                 ", n);
+    return 0;
+  }
+#if defined(__MINGW32__)
+  tm_p=localtime(&timev);
+#else
+  tm_p=localtime_r(&timev, &tmp);
+#endif
+  if(tm_p==NULL)
+  {
+    strncpy(datestr, "                 ", n);
+    return 0;
+  }
+  snprintf(datestr, n,"%2d-%s-%4d %02d:%02d",
+      tm_p->tm_mday, monstr[tm_p->tm_mon],
+      1900 + tm_p->tm_year, tm_p->tm_hour,
+      tm_p->tm_min);
+  if(1900+tm_p->tm_year>=2000)
+    return 1;
+  return 0;
+}
+
 int dir_aff_log(const dir_data_t *dir_data, const file_info_t *dir_list)
 {
   int test_date=0;
@@ -159,26 +193,10 @@ int dir_aff_log(const dir_data_t *dir_data, const file_info_t *dir_list)
   }
   td_list_for_each(file_walker, &dir_list->list)
   {
-    const file_info_t *current_file=td_list_entry(file_walker, file_info_t, list);
+    const file_info_t *current_file=td_list_entry_const(file_walker, const file_info_t, list);
     char		datestr[80];
     char str[11];
-    {
-      const struct tm *tm_p;
-      if(current_file->td_mtime && (tm_p = localtime(&current_file->td_mtime))!=NULL)
-      {
-	snprintf(datestr, sizeof(datestr),"%2d-%s-%4d %02d:%02d",
-	    tm_p->tm_mday, monstr[tm_p->tm_mon],
-	    1900 + tm_p->tm_year, tm_p->tm_hour,
-	    tm_p->tm_min);
-	/* FIXME: a check using current_file->name will be better */
-	if(1900+tm_p->tm_year>=2000 && 1900+tm_p->tm_year<=2014)
-	{
-	  test_date=1;
-	}
-      } else {
-	strncpy(datestr, "                 ",sizeof(datestr));
-      }
-    }
+    test_date=set_datestr((char *)&datestr, sizeof(datestr), current_file->td_mtime);
     mode_string(current_file->st_mode, str);
     if((current_file->status&FILE_STATUS_DELETED)!=0)
       log_info("X");
@@ -203,9 +221,8 @@ int dir_aff_log(const dir_data_t *dir_data, const file_info_t *dir_list)
   return test_date;
 }
 
-int log_list_file(const disk_t *disk, const partition_t *partition, const dir_data_t *dir_data, const file_info_t*list)
+void log_list_file(const disk_t *disk, const partition_t *partition, const dir_data_t *dir_data, const file_info_t*list)
 {
-  int test_date=0;
   struct td_list_head *tmp;
   log_partition(disk, partition);
   if(dir_data!=NULL)
@@ -216,28 +233,12 @@ int log_list_file(const disk_t *disk, const partition_t *partition, const dir_da
   {
     char		datestr[80];
     char str[11];
-    const file_info_t *current_file=td_list_entry(tmp, file_info_t, list);
+    const file_info_t *current_file=td_list_entry_const(tmp, const file_info_t, list);
     if((current_file->status&FILE_STATUS_DELETED)!=0)
       log_info("X");
     else
       log_info(" ");
-    {
-      const struct tm *tm_p;
-      if(current_file->td_mtime && (tm_p = localtime(&current_file->td_mtime))!=NULL)
-      {
-	snprintf(datestr, sizeof(datestr),"%2d-%s-%4d %02d:%02d",
-	    tm_p->tm_mday, monstr[tm_p->tm_mon],
-	    1900 + tm_p->tm_year, tm_p->tm_hour,
-	    tm_p->tm_min);
-	/* FIXME: a check using current_file->name will be better */
-	if(1900+tm_p->tm_year>=2000 && 1900+tm_p->tm_year<=2014)
-	{
-	  test_date=1;
-	}
-      } else {
-	strncpy(datestr, "                 ",sizeof(datestr));
-      }
-    }
+    set_datestr((char *)&datestr, sizeof(datestr), current_file->td_mtime);
     mode_string(current_file->st_mode, str);
     log_info("%7lu ",(unsigned long int)current_file->st_ino);
     log_info("%s %5u %5u ", 
@@ -245,7 +246,6 @@ int log_list_file(const disk_t *disk, const partition_t *partition, const dir_da
     log_info("%9llu", (long long unsigned int)current_file->st_size);
     log_info(" %s %s\n", datestr, current_file->name);
   }
-  return test_date;
 }
 
 unsigned int delete_list_file(file_info_t *file_info)
@@ -265,17 +265,28 @@ unsigned int delete_list_file(file_info_t *file_info)
   return nbr;
 }
 
+static int is_inode_valid(const file_info_t *current_file, const unsigned int dir_nbr, const unsigned long int *inode_known)
+{
+  const unsigned long int new_inode=current_file->st_ino;
+  unsigned int i;
+  if(new_inode<2)
+    return 0;
+  if(strcmp(current_file->name, "..")==0)
+    return 0;
+  for(i=0; i<dir_nbr; i++)
+    if(new_inode==inode_known[i]) /* Avoid loop */
+      return 0;
+  return 1;
+}
+
 static int dir_whole_partition_log_aux(disk_t *disk, const partition_t *partition, dir_data_t *dir_data, const unsigned long int inode)
 {
   struct td_list_head *file_walker = NULL;
-#define MAX_DIR_NBR 256
   static unsigned int dir_nbr=0;
   static unsigned long int inode_known[MAX_DIR_NBR];
   const unsigned int current_directory_namelength=strlen(dir_data->current_directory);
-  file_info_t dir_list = {
-    .list = TD_LIST_HEAD_INIT(dir_list.list),
-    .name = NULL
-  };
+  file_info_t dir_list;
+  TD_INIT_LIST_HEAD(&dir_list.list);
   if(dir_nbr==MAX_DIR_NBR)
     return 1;	/* subdirectories depth is too high => Back */
   if(dir_data->verbose>0)
@@ -286,31 +297,17 @@ static int dir_whole_partition_log_aux(disk_t *disk, const partition_t *partitio
   inode_known[dir_nbr++]=inode;
   td_list_for_each(file_walker, &dir_list.list)
   {
-    const file_info_t *current_file=td_list_entry(file_walker, file_info_t, list);
-    if(LINUX_S_ISDIR(current_file->st_mode)!=0)
+    const file_info_t *current_file=td_list_entry_const(file_walker, const file_info_t, list);
+    if(LINUX_S_ISDIR(current_file->st_mode)!=0 &&
+	is_inode_valid(current_file, dir_nbr, inode_known)>0 &&
+	strlen(dir_data->current_directory)+1+strlen(current_file->name)<sizeof(dir_data->current_directory)-1)
     {
-      const unsigned long int new_inode=current_file->st_ino;
-      unsigned int new_inode_ok=1;
-      unsigned int i;
-      if(new_inode<2)
-	new_inode_ok=0;
-      for(i=0;i<dir_nbr && new_inode_ok!=0;i++)
-	if(new_inode==inode_known[i]) /* Avoid loop */
-	  new_inode_ok=0;
-      if(strcmp(current_file->name, "..")==0)
-	  new_inode_ok=0;
-      if(new_inode_ok>0)
-      {
-	if(strlen(dir_data->current_directory)+1+strlen(current_file->name)<sizeof(dir_data->current_directory)-1)
-	{
-	  if(strcmp(dir_data->current_directory,"/"))
-	    strcat(dir_data->current_directory,"/");
-	  strcat(dir_data->current_directory,current_file->name);
-	  dir_whole_partition_log_aux(disk, partition, dir_data, new_inode);
-	  /* restore current_directory name */
-	  dir_data->current_directory[current_directory_namelength]='\0';
-	}
-      }
+      if(strcmp(dir_data->current_directory,"/"))
+	strcat(dir_data->current_directory,"/");
+      strcat(dir_data->current_directory,current_file->name);
+      dir_whole_partition_log_aux(disk, partition, dir_data, current_file->st_ino);
+      /* restore current_directory name */
+      dir_data->current_directory[current_directory_namelength]='\0';
     }
   }
   delete_list_file(&dir_list);
@@ -324,6 +321,70 @@ int dir_whole_partition_log(disk_t *disk, const partition_t *partition, dir_data
   return dir_whole_partition_log_aux(disk, partition, dir_data, inode);
 }
 
+static int dir_whole_partition_copy_aux(disk_t *disk, const partition_t *partition, dir_data_t *dir_data, const unsigned long int inode, unsigned int *copy_ok, unsigned int *copy_bad)
+{
+  struct td_list_head *file_walker = NULL;
+  static unsigned int dir_nbr=0;
+  static unsigned long int inode_known[MAX_DIR_NBR];
+  const unsigned int current_directory_namelength=strlen(dir_data->current_directory);
+  file_info_t dir_list;
+  TD_INIT_LIST_HEAD(&dir_list.list);
+  if(dir_nbr==MAX_DIR_NBR)
+    return 1;	/* subdirectories depth is too high => Back */
+  dir_data->get_dir(disk, partition, dir_data, inode, &dir_list);
+  /* Not perfect for FAT32 root cluster */
+  inode_known[dir_nbr++]=inode;
+  td_list_for_each(file_walker, &dir_list.list)
+  {
+    const file_info_t *current_file=td_list_entry_const(file_walker, const file_info_t, list);
+    if(strlen(dir_data->current_directory) + 1 + strlen(current_file->name) <
+	sizeof(dir_data->current_directory)-1)
+    {
+      if(strcmp(dir_data->current_directory,"/"))
+	strcat(dir_data->current_directory,"/");
+      strcat(dir_data->current_directory,current_file->name);
+      if(LINUX_S_ISDIR(current_file->st_mode)!=0)
+      {
+	if(is_inode_valid(current_file, dir_nbr, inode_known)>0)
+	{
+	  dir_whole_partition_copy_aux(disk, partition, dir_data, current_file->st_ino, copy_ok, copy_bad);
+	}
+      }
+      else if(LINUX_S_ISREG(current_file->st_mode)!=0)
+      {
+	if(dir_data->copy_file(disk, partition, dir_data, current_file) == 0)
+	  (*copy_ok)++;
+	else
+	  (*copy_bad)++;
+      }
+    }
+    /* restore current_directory name */
+    dir_data->current_directory[current_directory_namelength]='\0';
+  }
+  delete_list_file(&dir_list);
+  dir_nbr--;
+  return 0;
+}
+
+void dir_whole_partition_copy(disk_t *disk, const partition_t *partition, dir_data_t *dir_data, const unsigned long int inode)
+{
+  unsigned int copy_ok=0;
+  unsigned int copy_bad=0;
+  char *dst_directory=(char *)MALLOC(4096);
+  dst_directory[0]='.';
+  dst_directory[1]='\0';
+#ifdef HAVE_GETCWD
+  if(getcwd(dst_directory, 4096)==NULL)
+  {
+    free(dst_directory);
+    return ;
+  }
+#endif
+  dir_data->local_dir=dst_directory;
+  dir_whole_partition_copy_aux(disk, partition, dir_data, inode, &copy_ok, &copy_bad);
+  log_info("Copy done! %u ok, %u failed", copy_ok, copy_bad);
+}
+
 int filesort(const struct td_list_head *a, const struct td_list_head *b)
 {
   const file_info_t *file_a=td_list_entry_const(a, const file_info_t, list);
@@ -416,6 +477,17 @@ int set_mode(const char *pathname, unsigned int mode)
 #endif
 }
 
+static void strip_fn(char *fn)
+{
+  unsigned int i;
+  for(i=0;fn[i]!='\0';i++);
+  while(i>0 && (fn[i-1]==' '||fn[i-1]=='.'))
+    i--;
+  if(i==0 && (fn[i]==' '||fn[i]=='.'))
+    fn[i++]='_';
+  fn[i]='\0';
+}
+
 #ifdef DJGPP
 static inline unsigned char convert_char_dos(unsigned char car)
 {
@@ -650,7 +722,7 @@ char *mkdir_local(const char *localroot, const char *pathname)
 #endif
   }
 #else
-#warning You need a mkdir function!
+#warning "You need a mkdir function!"
 #endif
   return localdir;
 }
@@ -681,6 +753,7 @@ FILE *fopen_local(char **localfilename, const char *localroot, const char *filen
   memcpy(dst, localroot, l1);
   memcpy(dst+l1, filename, l2+1);
   *localfilename=dst;
+  strip_fn(dst);
   f_out=fopen(dst,"wb");
   if(f_out)
     return f_out;
