@@ -34,6 +34,7 @@ extern "C" {
 #define FLAG_LIST_MASK16	4
 #define FLAG_LIST_PATHNAME	8
 #define FLAG_LIST_ADS		16
+#define FLAG_LIST_SYSTEM	32
 /* capabilities */
 #define CAPA_LIST_DELETED	1
 #define CAPA_LIST_ADS		2
@@ -74,10 +75,12 @@ struct dir_data
 #define	FILE_STATUS_MARKED	2
 #define	FILE_STATUS_ADS		4
 
+int set_datestr(char *datestr, size_t n, const time_t timev);
 int dir_aff_log(const dir_data_t *dir_data, const file_info_t*dir_list);
-int log_list_file(const disk_t *disk_car, const partition_t *partition, const dir_data_t *dir_data, const file_info_t*list);
+void log_list_file(const disk_t *disk_car, const partition_t *partition, const dir_data_t *dir_data, const file_info_t*list);
 unsigned int delete_list_file(file_info_t *list);
 int dir_whole_partition_log(disk_t *disk_car, const partition_t *partition, dir_data_t *dir_data, const unsigned long int inode);
+void dir_whole_partition_copy(disk_t *disk_car, const partition_t *partition, dir_data_t *dir_data, const unsigned long int inode);
 void mode_string (const unsigned int mode, char *str);
 int set_mode(const char *pathname, unsigned int mode);
 FILE *fopen_local(char **localfilename, const char *localroot, const char *filename);
