@@ -40,7 +40,7 @@ struct exfat_super_block {
 	unsigned char   xxxx01;                 /* ??? (0x00 or any value (?)) */
 	unsigned char   xxxx02;                 /* ??? (0x01 or 0x00 (?)) */
 	uint16_t  	state;                  /* state of this volume */
-	unsigned char   blocksize_bits;         /* bits of block size */
+	unsigned char   blocksize_bits;         /* 0x6c bits of block size */
 	unsigned char   block_per_clus_bits;    /* bits of blocks per cluster */
 	unsigned char   number_of_fats;
 	unsigned char   drive_select;           /* Used by INT 13 */
@@ -94,9 +94,9 @@ struct exfat_alloc_bitmap_entry
 
 uint64_t exfat_cluster_to_offset(const struct exfat_super_block *exfat_header, const unsigned int cluster);
 int exfat_read_cluster(disk_t *disk, const partition_t *partition, const struct exfat_super_block*exfat_header, void *buffer, const unsigned int cluster);
-int check_EXFAT(disk_t *disk, partition_t *partition);
-int recover_EXFAT(const disk_t *disk, const struct exfat_super_block *exfat_header, partition_t *partition);
-int test_EXFAT(const struct exfat_super_block *exfat_header, partition_t *partition);
+int check_exFAT(disk_t *disk, partition_t *partition);
+int recover_exFAT(const disk_t *disk, const struct exfat_super_block *exfat_header, partition_t *partition);
+int test_exFAT(const struct exfat_super_block *exfat_header);
 
 #ifdef __cplusplus
 } /* closing brace for extern "C" */
