@@ -52,7 +52,7 @@
 #include "setdate.h"
 #include "fat.h"
 
-#define EXFAT_MKMODE(a,m) ((m & (a & ATTR_RO ? LINUX_S_IRUGO|LINUX_S_IXUGO : LINUX_S_IRWXUGO)) | (a & ATTR_DIR ? LINUX_S_IFDIR : LINUX_S_IFREG))
+#define EXFAT_MKMODE(a,m) ((m & ((a & ATTR_RO) ? LINUX_S_IRUGO|LINUX_S_IXUGO : LINUX_S_IRWXUGO)) | ((a & ATTR_DIR) ? LINUX_S_IFDIR : LINUX_S_IFREG))
 struct exfat_dir_struct
 {
   struct exfat_super_block*boot_sector;
@@ -106,17 +106,16 @@ static int exfat_ucstoutf8(iconv_t cd, const unsigned char *ins, const unsigned
 #define ATTR_SYS     4  /* system */
 #define ATTR_DIR     16 /* directory */
 #define ATTR_ARCH    32 /* archived */
-#define EXFAT_MKMODE(a,m) ((m & (a & ATTR_RO ? LINUX_S_IRUGO|LINUX_S_IXUGO : LINUX_S_IRWXUGO)) | (a & ATTR_DIR ? LINUX_S_IFDIR : LINUX_S_IFREG))
 
-static unsigned int exfat_get_next_cluster(disk_t *disk_car,const partition_t *partition, const int offset, const unsigned int cluster)
+static unsigned int exfat_get_next_cluster(disk_t *disk_car,const partition_t *partition, const uint64_t offset, const unsigned int cluster)
 {
   unsigned char *buffer=(unsigned char*)MALLOC(disk_car->sector_size);
   unsigned int next_cluster;
   const uint32_t *p32=(const uint32_t*)buffer;
-  const unsigned long int offset_s=cluster/(disk_car->sector_size/4);
-  const unsigned long int offset_o=cluster%(disk_car->sector_size/4);
+  const uint64_t offset_s=cluster / (disk_car->sector_size/4);
+  const uint64_t offset_o=cluster % (disk_car->sector_size/4);
   if((unsigned)disk_car->pread(disk_car, buffer, disk_car->sector_size,
-	partition->part_offset + offset + (uint64_t)offset_s * disk_car->sector_size) != disk_car->sector_size)
+	partition->part_offset + offset + offset_s * disk_car->sector_size) != disk_car->sector_size)
   {
     log_error("exfat_get_next_cluster read error\n");
     free(buffer);
@@ -191,10 +190,15 @@ static int dir_exfat_aux(const unsigned char*buffer, const unsigned int size, co
 #ifdef HAVE_ICONV
 	char *outs;
 #endif
-	unsigned int i,j;
-	for(j=0; j<255 && current_file->name[j]!='\0'; j++);
+	unsigned int i;
+	unsigned int j;
+	for(j=0;
+	    j<255 && current_file->name[j]!='\0';
+	    j++);
 #ifdef HAVE_ICONV
-	for(i=2; i<32 && (buffer[offset+i]!=0 || buffer[offset+i+1]!=0); i+=2);
+	for(i=2;
+	    i<32 && (buffer[offset+i]!=0 || buffer[offset+i+1]!=0);
+	    i+=2);
 	i-=2;
 	outs=&current_file->name[j];
 	if(exfat_ucstoutf8(ls->cd, &buffer[offset+2], i, &outs, 512-j) < 0)
@@ -230,7 +234,7 @@ static int exfat_dir(disk_t *disk, const partition_t *partition, dir_data_t *dir
   const unsigned int total_clusters=le32(exfat_header->total_clusters);
   exfat_method_t exfat_meth=exFAT_FOLLOW_CLUSTER;
   int stop=0;
-  const uint64_t start_exfat1=le32(exfat_header->fat_blocknr) << exfat_header->blocksize_bits;
+  const uint64_t start_exfat1=(uint64_t)le32(exfat_header->fat_blocknr) << exfat_header->blocksize_bits;
   if(first_cluster<2)
     cluster=le32(exfat_header->rootdir_clusnr);
   else
@@ -334,11 +338,11 @@ dir_partition_t dir_partition_exfat_init(disk_t *disk, const partition_t *partit
   dir_data->param=FLAG_LIST_DELETED;
   dir_data->verbose=verbose;
   dir_data->capabilities=CAPA_LIST_DELETED;
-  dir_data->copy_file=exfat_copy;
-  dir_data->close=dir_partition_exfat_close;
+  dir_data->copy_file=&exfat_copy;
+  dir_data->close=&dir_partition_exfat_close;
   dir_data->local_dir=NULL;
   dir_data->private_dir_data=ls;
-  dir_data->get_dir=exfat_dir;
+  dir_data->get_dir=&exfat_dir;
   return DIR_PART_OK;
 }
 
@@ -364,7 +368,8 @@ static int exfat_copy(disk_t *disk, const partition_t *partition, dir_data_t *di
   unsigned int cluster;
   uint64_t file_size=file->st_size;
   exfat_method_t exfat_meth=exFAT_FOLLOW_CLUSTER;
-  uint64_t start_exfat1,clus_blocknr;
+  uint64_t start_exfat1;
+  unsigned long int clus_blocknr;
   unsigned long int total_clusters;
   f_out=fopen_local(&new_file, dir_data->local_dir, dir_data->current_directory);
   if(!f_out)
@@ -375,7 +380,7 @@ static int exfat_copy(disk_t *disk, const partition_t *partition, dir_data_t *di
     return -1;
   }
   cluster = file->st_ino;
-  start_exfat1=le32(exfat_header->fat_blocknr) << exfat_header->blocksize_bits;
+  start_exfat1=(uint64_t)le32(exfat_header->fat_blocknr) << exfat_header->blocksize_bits;
   clus_blocknr=le32(exfat_header->clus_blocknr);
   total_clusters=le32(exfat_header->total_clusters);
   log_trace("exfat_copy dst=%s first_cluster=%u (%llu) size=%lu\n", new_file,
