@@ -44,38 +44,20 @@
 #include "log.h"
 #include "log_part.h"
 #include "fat_common.h"
+#include "dir.h"
+#include "fat_dir.h"
 
 extern const arch_fnct_t arch_i386;
 extern const arch_fnct_t arch_mac;
 
-static int set_FAT_info(disk_t *disk_car, const struct fat_boot_sector *fat_header, partition_t *partition);
+static void set_FAT_info(disk_t *disk_car, const struct fat_boot_sector *fat_header, partition_t *partition);
 static int fat32_set_part_name(disk_t *disk_car, partition_t *partition, const struct fat_boot_sector*fat_header);
 static int log_fat_info(const struct fat_boot_sector*fh1, const upart_type_t upart_type, const unsigned int sector_size);
-static int test_OS2MB(const disk_t *disk, const struct fat_boot_sector *fat_header, partition_t *partition,const int verbose, const int dump_ind);
+static int test_OS2MB(const disk_t *disk, const struct fat_boot_sector *fat_header, const partition_t *partition, const int verbose, const int dump_ind);
 static int is_fat12(const partition_t *partition);
 static int is_fat16(const partition_t *partition);
 static int is_fat32(const partition_t *partition);
 
-#define DELETED_FLAG 0xe5 /* marks file as deleted when in name[0] */
-#define IS_FREE(n) (!*(n) || *(const unsigned char *) (n) == DELETED_FLAG)
-#define ATTR_RO      1  /* read-only */
-#define ATTR_HIDDEN  2  /* hidden */
-#define ATTR_SYS     4  /* system */
-#define ATTR_VOLUME  8  /* volume label */
-#define ATTR_DIR     16 /* directory */
-#define ATTR_ARCH    32 /* archived */
-
-#define ATTR_NONE    0 /* no attribute bits */
-#define ATTR_UNUSED  (ATTR_VOLUME | ATTR_ARCH | ATTR_SYS | ATTR_HIDDEN)
-	/* attribute bits that are copied "as is" */
-#define ATTR_EXT     (ATTR_RO | ATTR_HIDDEN | ATTR_SYS | ATTR_VOLUME)
-#define ATTR_EXT_MASK     (ATTR_RO | ATTR_HIDDEN | ATTR_SYS | ATTR_VOLUME | ATTR_DIR | ATTR_ARCH)
-	/* bits that are used by the Windows 95/Windows NT extended FAT */
-#define FAT12_BAD	0x0FF7
-#define FAT12_EOC	0x0FF8
-#define FAT16_BAD	0xFFF7
-#define FAT16_EOC	0xFFF8
-
 static int log_fat_info(const struct fat_boot_sector*fh1, const upart_type_t upart_type, const unsigned int sector_size)
 {
   log_info("sector_size  %u\n", fat_sector_size(fh1));
@@ -196,115 +178,135 @@ int check_FAT(disk_t *disk_car,partition_t *partition,const int verbose)
   return 0;
 }
 
-static int set_FAT_info(disk_t *disk_car, const struct fat_boot_sector *fat_header, partition_t *partition)
+static void set_FAT_info(disk_t *disk_car, const struct fat_boot_sector *fat_header, partition_t *partition)
 {
-  const char*buffer=(const char*)fat_header;
+  uint64_t start_fat1;
+  uint64_t start_data;
+  uint64_t part_size;
+  unsigned long int no_of_cluster;
+  unsigned long int fat_length;
+  const char *buffer=(const char*)fat_header;
   partition->fsname[0]='\0';
   partition->blocksize=fat_sector_size(fat_header)* fat_header->sectors_per_cluster;
-  switch(partition->upart_type)
+  fat_length=le16(fat_header->fat_length)>0?le16(fat_header->fat_length):le32(fat_header->fat32_length);
+  part_size=(fat_sectors(fat_header)>0?fat_sectors(fat_header):le32(fat_header->total_sect));
+  start_fat1=le16(fat_header->reserved);
+  start_data=start_fat1+fat_header->fats*fat_length+(get_dir_entries(fat_header)*32+fat_sector_size(fat_header)-1)/fat_sector_size(fat_header);
+  no_of_cluster=(part_size-start_data)/fat_header->sectors_per_cluster;
+  if(no_of_cluster<4085)
   {
-    case UP_FAT12:
-      snprintf(partition->info, sizeof(partition->info), "FAT12, blocksize=%u", partition->blocksize);
-      if(buffer[38]==0x29)	/* BS_BootSig */
-      {
-        set_part_name_chomp(partition,((const unsigned char*)fat_header)+FAT1X_PART_NAME,11);
-        if(check_VFAT_volume_name(partition->fsname, 11))
-          partition->fsname[0]='\0';
-      }
-      break;
-    case UP_FAT16:
-      snprintf(partition->info, sizeof(partition->info), "FAT16, blocksize=%u", partition->blocksize);
-      if(buffer[38]==0x29)	/* BS_BootSig */
-      {
-        set_part_name_chomp(partition,((const unsigned char*)fat_header)+FAT1X_PART_NAME,11);
-        if(check_VFAT_volume_name(partition->fsname, 11))
-          partition->fsname[0]='\0';
-      }
-      break;
-    case UP_FAT32:
-      if(partition->sb_offset==0)
-	snprintf(partition->info, sizeof(partition->info), "FAT32, blocksize=%u", partition->blocksize);
-      else
-	snprintf(partition->info, sizeof(partition->info), "FAT32 found using backup sector, blocksize=%u", partition->blocksize);
-      fat32_set_part_name(disk_car,partition,fat_header);
-      break;
-    default:
-      log_critical("set_FAT_info unknown upart_type\n");
-      return 1;
+    partition->upart_type=UP_FAT12;
+    snprintf(partition->info, sizeof(partition->info), "FAT12, blocksize=%u", partition->blocksize);
+    if(buffer[38]==0x29)	/* BS_BootSig */
+    {
+      set_part_name_chomp(partition,((const unsigned char*)fat_header)+FAT1X_PART_NAME,11);
+      if(check_VFAT_volume_name(partition->fsname, 11))
+	partition->fsname[0]='\0';
+    }
+  }
+  else if(no_of_cluster<65525)
+  {
+    partition->upart_type=UP_FAT16;
+    snprintf(partition->info, sizeof(partition->info), "FAT16, blocksize=%u", partition->blocksize);
+    if(buffer[38]==0x29)	/* BS_BootSig */
+    {
+      set_part_name_chomp(partition,((const unsigned char*)fat_header)+FAT1X_PART_NAME,11);
+      if(check_VFAT_volume_name(partition->fsname, 11))
+	partition->fsname[0]='\0';
+    }
+  }
+  else
+  {
+    partition->upart_type=UP_FAT32;
+    if(partition->sb_offset==0)
+      snprintf(partition->info, sizeof(partition->info), "FAT32, blocksize=%u", partition->blocksize);
+    else
+      snprintf(partition->info, sizeof(partition->info), "FAT32 found using backup sector, blocksize=%u", partition->blocksize);
+    fat32_set_part_name(disk_car,partition,fat_header);
   }
-  return 0;
 }
 
-unsigned int get_next_cluster(disk_t *disk_car,const partition_t *partition, const upart_type_t upart_type,const int offset, const unsigned int cluster)
+static unsigned int get_next_cluster_fat12(disk_t *disk, const partition_t *partition, const int offset, const unsigned int cluster)
+{
+  unsigned int next_cluster;
+  unsigned long int offset_s;
+  unsigned long int offset_o;
+  unsigned char *buffer=(unsigned char*)MALLOC(2*disk->sector_size);
+  offset_s=(cluster+cluster/2)/disk->sector_size;
+  offset_o=(cluster+cluster/2)%disk->sector_size;
+  if((unsigned)disk->pread(disk, buffer, 2 * disk->sector_size,
+	partition->part_offset + (uint64_t)(offset + offset_s) * disk->sector_size) != 2 * disk->sector_size)
+  {
+    log_error("get_next_cluster_fat12 read error\n");
+    free(buffer);
+    return 0;
+  }
+  if((cluster&1)!=0)
+    next_cluster=le16((*((uint16_t*)&buffer[offset_o])))>>4;
+  else
+    next_cluster=le16(*((uint16_t*)&buffer[offset_o]))&0x0FFF;
+  free(buffer);
+  return next_cluster;
+}
+
+static unsigned int get_next_cluster_fat16(disk_t *disk, const partition_t *partition, const int offset, const unsigned int cluster)
+{
+  unsigned int next_cluster;
+  unsigned long int offset_s;
+  unsigned long int offset_o;
+  unsigned char *buffer=(unsigned char*)MALLOC(disk->sector_size);
+  const uint16_t *p16=(const uint16_t*)buffer;
+  offset_s=cluster/(disk->sector_size/2);
+  offset_o=cluster%(disk->sector_size/2);
+  if((unsigned)disk->pread(disk, buffer, disk->sector_size,
+	partition->part_offset + (uint64_t)(offset + offset_s) * disk->sector_size) != disk->sector_size)
+  {
+    log_error("get_next_cluster_fat16 read error\n");
+    free(buffer);
+    return 0;
+  }
+  next_cluster=le16(p16[offset_o]);
+  free(buffer);
+  return next_cluster;
+}
+
+static unsigned int get_next_cluster_fat32(disk_t *disk, const partition_t *partition, const int offset, const unsigned int cluster)
+{
+  unsigned int next_cluster;
+  unsigned long int offset_s;
+  unsigned long int offset_o;
+  unsigned char *buffer=(unsigned char*)MALLOC(disk->sector_size);
+  const uint32_t *p32=(const uint32_t*)buffer;
+  offset_s=cluster/(disk->sector_size/4);
+  offset_o=cluster%(disk->sector_size/4);
+  if((unsigned)disk->pread(disk, buffer, disk->sector_size,
+	partition->part_offset + (uint64_t)(offset + offset_s) * disk->sector_size) != disk->sector_size)
+  {
+    log_error("get_next_cluster_fat32 read error\n");
+    free(buffer);
+    return 0;
+  }
+  /* FAT32 used 28 bits, the 4 high bits are reserved
+   * 0x00000000: free cluster
+   * 0x0FFFFFF7: bad cluster
+   * 0x0FFFFFF8+: EOC End of cluster
+   * */
+  next_cluster=le32(p32[offset_o])&0xFFFFFFF;
+  free(buffer);
+  return next_cluster;
+}
+
+unsigned int get_next_cluster(disk_t *disk,const partition_t *partition, const upart_type_t upart_type,const int offset, const unsigned int cluster)
 {
   /* Offset can be offset to FAT1 or to FAT2 */
-  /* log_trace("get_next_cluster(upart_type=%u,offset=%u,cluster=%u\n",upart_type,offset,cluster); */
   switch(upart_type)
   {
     case UP_FAT12:
-      {
-	unsigned int next_cluster;
-	unsigned long int offset_s,offset_o;
-	unsigned char *buffer=(unsigned char*)MALLOC(2*disk_car->sector_size);
-        offset_s=(cluster+cluster/2)/disk_car->sector_size;
-        offset_o=(cluster+cluster/2)%disk_car->sector_size;
-        if((unsigned)disk_car->pread(disk_car, buffer, 2 * disk_car->sector_size,
-	      partition->part_offset + (uint64_t)(offset + offset_s) * disk_car->sector_size) != 2 * disk_car->sector_size)
-        {
-          log_error("get_next_cluster read error\n");
-	  free(buffer);
-	  return 0;
-        }
-        if((cluster&1)!=0)
-          next_cluster=le16((*((uint16_t*)&buffer[offset_o])))>>4;
-        else
-          next_cluster=le16(*((uint16_t*)&buffer[offset_o]))&0x0FFF;
-        free(buffer);
-        return next_cluster;
-      }
+      return get_next_cluster_fat12(disk, partition, offset, cluster);
     case UP_FAT16:
-      {
-	unsigned int next_cluster;
-	unsigned long int offset_s,offset_o;
-	unsigned char *buffer=(unsigned char*)MALLOC(disk_car->sector_size);
-        const uint16_t *p16=(const uint16_t*)buffer;
-        offset_s=cluster/(disk_car->sector_size/2);
-        offset_o=cluster%(disk_car->sector_size/2);
-        if((unsigned)disk_car->pread(disk_car, buffer, disk_car->sector_size,
-	      partition->part_offset + (uint64_t)(offset + offset_s) * disk_car->sector_size) != disk_car->sector_size)
-        {
-          log_error("get_next_cluster read error\n");
-	  free(buffer);
-	  return 0;
-        }
-        next_cluster=le16(p16[offset_o]);
-        free(buffer);
-        return next_cluster;
-      }
+      return get_next_cluster_fat16(disk, partition, offset, cluster);
     case UP_FAT32:
-      {
-	unsigned int next_cluster;
-	unsigned long int offset_s,offset_o;
-	unsigned char *buffer=(unsigned char*)MALLOC(disk_car->sector_size);
-        const uint32_t *p32=(const uint32_t*)buffer;
-        offset_s=cluster/(disk_car->sector_size/4);
-        offset_o=cluster%(disk_car->sector_size/4);
-        if((unsigned)disk_car->pread(disk_car, buffer, disk_car->sector_size,
-	      partition->part_offset + (uint64_t)(offset + offset_s) * disk_car->sector_size) != disk_car->sector_size)
-        {
-          log_error("get_next_cluster read error\n");
-	  free(buffer);
-	  return 0;
-        }
-        /* FAT32 used 28 bits, the 4 high bits are reserved
-         * 0x00000000: free cluster
-         * 0x0FFFFFF7: bad cluster
-         * 0x0FFFFFF8+: EOC End of cluster
-         * */
-        next_cluster=le32(p32[offset_o])&0xFFFFFFF;
-        free(buffer);
-        return next_cluster;
-      }
+      return get_next_cluster_fat32(disk, partition, offset, cluster);
     default:
       log_critical("fat.c get_next_cluster unknown fat type\n");
       return 0;
@@ -425,10 +427,17 @@ static unsigned int get_prev_cluster(disk_t *disk_car,const partition_t *partiti
 }
 */
 
-int test_FAT(disk_t *disk_car, const struct fat_boot_sector *fat_header, partition_t *partition,const int verbose, const int dump_ind)
+int test_FAT(disk_t *disk_car, const struct fat_boot_sector *fat_header, const partition_t *partition, const int verbose, const int dump_ind)
 {
-  uint64_t start_fat1,start_fat2,start_rootdir,start_data,part_size,end_data;
-  unsigned long int no_of_cluster,fat_length,fat_length_calc;
+  uint64_t start_fat1;
+  uint64_t start_fat2;
+  uint64_t start_rootdir;
+  uint64_t start_data;
+  uint64_t part_size;
+  uint64_t end_data;
+  unsigned long int no_of_cluster;
+  unsigned long int fat_length;
+  unsigned long int fat_length_calc;
   const char *buffer=(const char*)fat_header;
   if(!(le16(fat_header->marker)==0xAA55
         && (fat_header->ignored[0]==0xeb || fat_header->ignored[0]==0xe9)
@@ -534,7 +543,6 @@ int test_FAT(disk_t *disk_car, const struct fat_boot_sector *fat_header, partiti
     }
     start_rootdir=start_fat2+fat_length;
     fat_length_calc=((no_of_cluster+2+fat_sector_size(fat_header)*2/3-1)*3/2/fat_sector_size(fat_header));
-    partition->upart_type=UP_FAT12;
     if(memcmp(buffer+FAT_NAME1,"FAT12   ",8)!=0) /* 2 Mo max */
     {
       screen_buffer_add("Should be marked as FAT12\n");
@@ -574,7 +582,6 @@ int test_FAT(disk_t *disk_car, const struct fat_boot_sector *fat_header, partiti
     }
     start_rootdir=start_fat2+fat_length;
     fat_length_calc=((no_of_cluster+2+fat_sector_size(fat_header)/2-1)*2/fat_sector_size(fat_header));
-    partition->upart_type=UP_FAT16;
     if(memcmp(buffer+FAT_NAME1,"FAT16   ",8)!=0)
     {
       screen_buffer_add("Should be marked as FAT16\n");
@@ -620,7 +627,6 @@ int test_FAT(disk_t *disk_car, const struct fat_boot_sector *fat_header, partiti
     }
     start_rootdir=start_data+(uint64_t)(le32(fat_header->root_cluster)-2)*fat_header->sectors_per_cluster;
     fat_length_calc=((no_of_cluster+2+fat_sector_size(fat_header)/4-1)*4/fat_sector_size(fat_header));
-    partition->upart_type=UP_FAT32;
     if(memcmp(buffer+FAT_NAME2,"FAT32   ",8)!=0)
     {
       screen_buffer_add("Should be marked as FAT32\n");
@@ -662,7 +668,7 @@ int test_FAT(disk_t *disk_car, const struct fat_boot_sector *fat_header, partiti
     log_info("FAT1 : %lu-%lu\n", (long unsigned)start_fat1, (long unsigned)(start_fat1+fat_length-1));
     log_info("FAT2 : %lu-%lu\n", (long unsigned)start_fat2, (long unsigned)(start_fat2+fat_length-1));
     log_info("start_rootdir : %lu", (long unsigned)start_rootdir);
-    if(partition->upart_type==UP_FAT32)
+    if(no_of_cluster >= 65525)	/* FAT32 */
       log_info(" root cluster : %u",(unsigned int)le32(fat_header->root_cluster));
     log_info("\nData : %lu-%lu\n", (long unsigned)start_data, (long unsigned)end_data);
     log_info("sectors : %lu\n", (long unsigned)part_size);
@@ -700,8 +706,10 @@ int comp_FAT(disk_t *disk, const partition_t *partition, const unsigned long int
   return 0 if FATs match
   */
   unsigned int reste;
-  uint64_t hd_offset, hd_offset2;
-  unsigned char *buffer, *buffer2;
+  uint64_t hd_offset;
+  uint64_t hd_offset2;
+  unsigned char *buffer;
+  unsigned char *buffer2;
   buffer=(unsigned char *)MALLOC(16*disk->sector_size);
   buffer2=(unsigned char *)MALLOC(16*disk->sector_size);
   hd_offset=partition->part_offset+(uint64_t)sect_res*disk->sector_size;
@@ -756,8 +764,34 @@ unsigned long int fat32_get_next_free(const unsigned char *boot_fat32, const uns
   return le32(fsinfo->nextfree);
 }
 
+static int fat_has_EFI_entry(disk_t *disk, partition_t *partition, const int verbose)
+{
+  dir_data_t dir_data;
+  struct td_list_head *file_walker = NULL;
+  file_info_t dir_list;
+  const dir_partition_t res=dir_partition_fat_init(disk, partition, &dir_data, verbose);
+  if(res!=DIR_PART_OK)
+    return 0;
+  TD_INIT_LIST_HEAD(&dir_list.list);
+  dir_data.get_dir(disk, partition, &dir_data, 0, &dir_list);
+  td_list_for_each(file_walker, &dir_list.list)
+  {
+    const file_info_t *current_file=td_list_entry_const(file_walker, const file_info_t, list);
+    if(strcmp(current_file->name, "EFI")==0)
+    {
+      delete_list_file(&dir_list);
+      dir_data.close(&dir_data);
+      return 1;
+    }
+  }
+  delete_list_file(&dir_list);
+  dir_data.close(&dir_data);
+  return 0;
+}
+
 int recover_FAT(disk_t *disk_car, const struct fat_boot_sector*fat_header, partition_t *partition, const int verbose, const int dump_ind, const int backup)
 {
+  int efi=0;
   if(test_FAT(disk_car, fat_header, partition, verbose, dump_ind))
     return 1;
   partition->part_size=(uint64_t)(fat_sectors(fat_header)>0?fat_sectors(fat_header):le32(fat_header->total_sect)) *
@@ -809,10 +843,7 @@ int recover_FAT(disk_t *disk_car, const struct fat_boot_sector*fat_header, parti
       else
         partition->part_type_i386=P_32FAT_LBA;
       partition->part_type_mac=PMAC_FAT32;
-      if(memcmp(partition->fsname,"EFI",4)==0)
-	partition->part_type_gpt=GPT_ENT_TYPE_EFI;
-      else
-	partition->part_type_gpt=GPT_ENT_TYPE_MS_BASIC_DATA;
+      partition->part_type_gpt=GPT_ENT_TYPE_MS_BASIC_DATA;
       if(backup)
       {
         partition->sb_offset=6*512;
@@ -823,6 +854,15 @@ int recover_FAT(disk_t *disk_car, const struct fat_boot_sector*fat_header, parti
       log_critical("recover_FAT unknown FAT type\n");
       return 1;
   }
+  if(memcmp(partition->fsname,"EFI",4)==0)
+    efi=1;
+  if(efi==0)
+    efi=fat_has_EFI_entry(disk_car, partition, verbose);
+  if(efi)
+  {
+    partition->part_type_gpt=GPT_ENT_TYPE_EFI;
+    strcpy(partition->partname, "EFI System Partition");
+  }
   return 0;
 }
 
@@ -870,11 +910,12 @@ static int fat32_set_part_name(disk_t *disk_car, partition_t *partition, const s
 
 int check_OS2MB(disk_t *disk, partition_t *partition, const int verbose)
 {
-  unsigned char buffer[disk->sector_size];
-  if((unsigned)disk->pread(disk, &buffer, disk->sector_size, partition->part_offset) != disk->sector_size)
+  unsigned char *buffer=(unsigned char *)MALLOC(disk->sector_size);
+  if((unsigned)disk->pread(disk, buffer, disk->sector_size, partition->part_offset) != disk->sector_size)
   {
     screen_buffer_add("check_OS2MB: Read error\n");
     log_error("check_OS2MB: Read error\n");
+    free(buffer);
     return 1;
   }
   if(test_OS2MB(disk,(const struct fat_boot_sector *)buffer,partition,verbose,0)!=0)
@@ -884,8 +925,11 @@ int check_OS2MB(disk_t *disk, partition_t *partition, const int verbose)
       log_info("\n\ntest_OS2MB()\n");
       log_partition(disk, partition);
     }
+    free(buffer);
     return 1;
   }
+  partition->upart_type=UP_OS2MB;
+  free(buffer);
   return 0;
 }
 
@@ -894,6 +938,7 @@ int recover_OS2MB(const disk_t *disk, const struct fat_boot_sector*fat_header, p
   if(test_OS2MB(disk, fat_header, partition, verbose, dump_ind))
     return 1;
   /* 1 cylinder */
+  partition->upart_type=UP_OS2MB;
   partition->part_size=(uint64_t)disk->geom.heads_per_cylinder * disk->geom.sectors_per_head * disk->sector_size;
   partition->part_type_i386=P_OS2MB;
   partition->fsname[0]='\0';
@@ -901,7 +946,7 @@ int recover_OS2MB(const disk_t *disk, const struct fat_boot_sector*fat_header, p
   return 0;
 }
 
-static int test_OS2MB(const disk_t *disk, const struct fat_boot_sector *fat_header, partition_t *partition,const int verbose, const int dump_ind)
+static int test_OS2MB(const disk_t *disk, const struct fat_boot_sector *fat_header, const partition_t *partition, const int verbose, const int dump_ind)
 {
   const char*buffer=(const char*)fat_header;
   if(le16(fat_header->marker)==0xAA55 && memcmp(buffer+FAT_NAME1,"FAT     ",8)==0)
@@ -915,7 +960,6 @@ static int test_OS2MB(const disk_t *disk, const struct fat_boot_sector *fat_head
     }
     if(dump_ind)
       dump_log(buffer, DEFAULT_SECTOR_SIZE);
-    partition->upart_type=UP_OS2MB;
     return 0;
   }
   return 1;
