@@ -66,7 +66,7 @@ static void dump_fat1x(disk_t *disk_car, partition_t *partition, const unsigned
 {
   log_info("Boot sector\n");
   dump_log(buffer_bs, FAT1x_BOOT_SECTOR_SIZE);
-  if(*current_cmd==NULL)
+  if(current_cmd==NULL || *current_cmd==NULL)
   {
 #ifdef HAVE_NCURSES
     dump_fat1x_ncurses(disk_car, partition, buffer_bs);
@@ -77,7 +77,6 @@ static void dump_fat1x(disk_t *disk_car, partition_t *partition, const unsigned
 int fat1x_boot_sector(disk_t *disk_car, partition_t *partition, const int verbose, const int dump_ind, const unsigned int expert, char **current_cmd)
 {
   unsigned char *buffer_bs;
-  const char *options="DR";
 #ifdef HAVE_NCURSES
   const struct MenuItem menu_fat1x[]=
   {
@@ -95,6 +94,7 @@ int fat1x_boot_sector(disk_t *disk_car, partition_t *partition, const int verbos
   buffer_bs=(unsigned char*)MALLOC(FAT1x_BOOT_SECTOR_SIZE);
   while(1)
   {
+    const char *options;
 #ifdef HAVE_NCURSES
     unsigned int menu=3;
 #endif
@@ -138,33 +138,27 @@ int fat1x_boot_sector(disk_t *disk_car, partition_t *partition, const int verbos
     if(*current_cmd!=NULL)
     {
       command=0;
-      while(*current_cmd[0]==',')
-	(*current_cmd)++;
-      if(strncmp(*current_cmd,"rebuildbs",9)==0)
+      skip_comma_in_command(current_cmd);
+      if(check_command(current_cmd,"rebuildbs",9)==0)
       {
-	(*current_cmd)+=9;
 	command='R';
       }
-      else if(strncmp(*current_cmd,"dump",4)==0)
+      else if(check_command(current_cmd,"dump",4)==0)
       {
-	(*current_cmd)+=4;
 	command='D';
       }
-      else if(strncmp(*current_cmd,"list",4)==0)
+      else if(check_command(current_cmd,"list",4)==0)
       {
-	(*current_cmd)+=4;
 	if(strchr(options,'L')!=NULL)
 	  command='L';
       }
-      else if(strncmp(*current_cmd,"repairfat",9)==0)
+      else if(check_command(current_cmd,"repairfat",9)==0)
       {
-	(*current_cmd)+=9;
 	if(strchr(options,'C')!=NULL)
 	  command='C';
       }
-      else if(strncmp(*current_cmd,"initroot",8)==0)
+      else if(check_command(current_cmd,"initroot",8)==0)
       {
-	(*current_cmd)+=8;
 	if(strchr(options,'I')!=NULL)
 	    command='I';
       }
@@ -184,7 +178,7 @@ int fat1x_boot_sector(disk_t *disk_car, partition_t *partition, const int verbos
 	free(buffer_bs);
 	return 0;
       case 'R': /* R : rebuild boot sector */
-	rebuild_FAT_BS(disk_car,partition,verbose,dump_ind,1,expert,current_cmd);
+	rebuild_FAT_BS(disk_car, partition, verbose, dump_ind, expert, current_cmd);
 	break;
       case 'D':
 	dump_fat1x(disk_car, partition, buffer_bs, current_cmd);
@@ -196,7 +190,7 @@ int fat1x_boot_sector(disk_t *disk_car, partition_t *partition, const int verbos
 	FAT_init_rootdir(disk_car, partition, verbose, current_cmd);
 	break;
       case 'L':
-	dir_partition(disk_car, partition, 0,current_cmd);
+	dir_partition(disk_car, partition, 0, 0, current_cmd);
 	break;
     }
   }
