@@ -62,7 +62,6 @@
 #define INTER_FAT_ASK_Y	23
 #define INTER_FATBS_X		0
 #define INTER_FATBS_Y		22
-extern const char *monstr[];
 
 typedef struct info_offset_struct info_offset_t;
 
@@ -73,13 +72,13 @@ struct info_offset_struct
   unsigned int  fat_type;
 };
 
-static upart_type_t fat_find_info(disk_t *disk_car,unsigned int*reserved, unsigned int*fat_length, const partition_t *partition,const uint64_t max_offset,const int p_fat12,const int p_fat16,const int p_fat32,const int verbose,const int dump_ind,const int interface, const unsigned int expert, unsigned int *fats);
-static int fat_find_type(disk_t *disk_car,const partition_t *partition,const uint64_t max_offset,const int p_fat12,const int p_fat16,const int p_fat32,const int verbose,const int dump_ind,const int interface,unsigned int *nbr_offset,info_offset_t *info_offset, const unsigned int max_nbr_offset);
+static upart_type_t fat_find_info(disk_t *disk_car,unsigned int*reserved, unsigned int*fat_length, const partition_t *partition, const uint64_t max_offset, const int p_fat12, const int p_fat16, const int p_fat32, const int verbose, const int dump_ind, const unsigned int expert, unsigned int *fats);
+static int fat_find_type(disk_t *disk_car, const partition_t *partition, const uint64_t max_offset, const int p_fat12, const int p_fat16, const int p_fat32, const int verbose, const int dump_ind, unsigned int *nbr_offset, info_offset_t *info_offset, const unsigned int max_nbr_offset);
 
 static unsigned int fat_find_fat_start(const unsigned char *buffer,const int p_fat12, const int p_fat16, const int p_fat32,unsigned long int*fat_offset, const unsigned int sector_size);
 
-static void create_fat_boot_sector(disk_t *disk_car, partition_t *partition, const unsigned int reserved, const int verbose, const unsigned int dir_entries, const unsigned long int root_cluster, const unsigned int sectors_per_cluster, const unsigned int fat_length,const int interface, const upart_type_t upart_type, const unsigned int fats, char **current_cmd);
-static unsigned int fat32_find_root_cluster(disk_t *disk_car,const partition_t *partition,const unsigned int sectors_per_cluster, const unsigned long int no_of_cluster, const unsigned int reserved, const unsigned int fat_length, const int interface, const int verbose, const unsigned int expert, const unsigned int first_free_cluster, const unsigned int fats);
+static void create_fat_boot_sector(disk_t *disk_car, partition_t *partition, const unsigned int reserved, const int verbose, const unsigned int dir_entries, const unsigned long int root_cluster, const unsigned int sectors_per_cluster, const unsigned int fat_length, const upart_type_t upart_type, const unsigned int fats, char **current_cmd);
+static unsigned int fat32_find_root_cluster(disk_t *disk_car,const partition_t *partition,const unsigned int sectors_per_cluster, const unsigned long int no_of_cluster, const unsigned int reserved, const unsigned int fat_length, const int verbose, const unsigned int expert, const unsigned int first_free_cluster, const unsigned int fats);
 static int write_FAT_boot_code_aux(unsigned char *buffer);
 static int find_dir_entries(disk_t *disk_car,const partition_t *partition, const unsigned int offset,const int verbose);
 static int analyse_dir_entries(disk_t *disk_car,const partition_t *partition, const unsigned int offset, const int verbose);
@@ -181,7 +180,7 @@ static int ask_root_directory(disk_t *disk_car, const partition_t *partition, co
    *  1: back
    *  other: new cluster
    * */
-  int car='A';
+  int car;
   int quit=0;
   int offset=0;
   int pos_num=0;
@@ -205,7 +204,7 @@ static int ask_root_directory(disk_t *disk_car, const partition_t *partition, co
 	file_walker!=&dir_list->list && (i-offset)<INTER_DIR;
 	file_walker=file_walker->next,i++)
     {
-      const file_info_t *current_file=td_list_entry(file_walker, file_info_t, list);
+      const file_info_t *current_file=td_list_entry_const(file_walker, const file_info_t, list);
       char str[11];
       char		datestr[80];
       wmove(window,8+i-offset,0);
@@ -217,18 +216,7 @@ static int ask_root_directory(disk_t *disk_car, const partition_t *partition, co
       }
       else
 	waddstr(window, " ");
-      {
-	const struct tm *tm_p;
-	if(current_file->td_mtime!=0 && (tm_p=localtime(&current_file->td_mtime))!=NULL)
-	{
-	  snprintf(datestr, sizeof(datestr),"%2d-%s-%4d %02d:%02d",
-	      tm_p->tm_mday, monstr[tm_p->tm_mon],
-	      1900 + tm_p->tm_year, tm_p->tm_hour,
-	      tm_p->tm_min);
-	} else {
-	  strncpy(datestr, "                 ",sizeof(datestr));
-	}
-      }
+      set_datestr((char *)&datestr, sizeof(datestr), current_file->td_mtime);
       mode_string(current_file->st_mode, str);
       wprintw(window, "%s %3u %3u ", 
 	  str, (unsigned int)current_file->st_uid, (unsigned int)current_file->st_gid);
@@ -310,7 +298,14 @@ static int ask_root_directory(disk_t *disk_car, const partition_t *partition, co
 }
 #endif
 
-static unsigned int fat32_find_root_cluster(disk_t *disk_car,const partition_t *partition,const unsigned int sectors_per_cluster, const unsigned long int no_of_cluster,const unsigned int reserved, const unsigned int fat_length, const int interface, const int verbose, const unsigned int expert, const unsigned int first_free_cluster, const unsigned int fats)
+static int is_root_cluster_candidat(const file_info_t *dir_list)
+{
+  const file_info_t *file1=td_list_first_entry(&dir_list->list, const file_info_t, list);
+  const file_info_t *file2=td_list_next_entry(file1, list);
+  return (!td_list_empty(&dir_list->list) && (&file2->list==&dir_list->list || file1->st_ino!=file2->st_ino));
+}
+
+static unsigned int fat32_find_root_cluster(disk_t *disk_car,const partition_t *partition,const unsigned int sectors_per_cluster, const unsigned long int no_of_cluster,const unsigned int reserved, const unsigned int fat_length, const int verbose, const unsigned int expert, const unsigned int first_free_cluster, const unsigned int fats)
 {
   unsigned long int root_cluster=0;
   const uint64_t start_data=reserved+fats*fat_length;
@@ -326,25 +321,20 @@ static unsigned int fat32_find_root_cluster(disk_t *disk_car,const partition_t *
 #endif
     unsigned char *buffer;
     int ind_stop=0;
-    file_info_t rootdir_list= {
-      .list = TD_LIST_HEAD_INIT(rootdir_list.list),
-      .name = NULL
-    };
+    file_info_t rootdir_list;
+    TD_INIT_LIST_HEAD(&rootdir_list.list);
     buffer=(unsigned char *)MALLOC(cluster_size);
 #ifdef HAVE_NCURSES
-    if(interface)
-    {
-      wmove(stdscr,22,0);
-      wattrset(stdscr, A_REVERSE);
-      waddstr(stdscr,"  Stop  ");
-      wattroff(stdscr, A_REVERSE);
-    }
+    wmove(stdscr,22,0);
+    wattrset(stdscr, A_REVERSE);
+    waddstr(stdscr,"  Stop  ");
+    wattroff(stdscr, A_REVERSE);
 #endif
     for(root_cluster=2;(root_cluster<2+no_of_cluster)&&(ind_stop==0);root_cluster++)
     {
 #ifdef HAVE_NCURSES
       const unsigned long int percent=root_cluster*100/(2+no_of_cluster);
-      if(interface>0 && (root_cluster&0xfff)==0)
+      if((root_cluster&0xfff)==0)
       {
         wmove(stdscr,9,0);
         wclrtoeol(stdscr);
@@ -371,10 +361,8 @@ static unsigned int fat32_find_root_cluster(disk_t *disk_car,const partition_t *
           if(fat_get_cluster_from_entry(entry2)==0 &&
 	      buffer[0x40]!=0) /* First-level directory with files */
           {
-	    file_info_t dir_list = {
-	      .list = TD_LIST_HEAD_INIT(dir_list.list),
-	      .name = NULL
-	    };
+	    file_info_t dir_list;
+	    TD_INIT_LIST_HEAD(&dir_list.list);
             log_info("First-level directory found at cluster %lu\n",root_cluster);
             dir_fat_aux(buffer, cluster_size, 0, &dir_list);
             if(verbose>0)
@@ -382,7 +370,7 @@ static unsigned int fat32_find_root_cluster(disk_t *disk_car,const partition_t *
               dir_aff_log(NULL, &dir_list);
             }
             {
-	      const file_info_t *first_entry=td_list_entry(&dir_list.list, file_info_t, list);
+	      const file_info_t *first_entry=td_list_first_entry(&dir_list.list, const file_info_t, list);
               file_info_t *new_file=(file_info_t *)MALLOC(sizeof(*new_file));
               memcpy(new_file, first_entry, sizeof(*new_file));
 	      new_file->name=(char*)MALLOC(32);
@@ -448,47 +436,49 @@ static unsigned int fat32_find_root_cluster(disk_t *disk_car,const partition_t *
             {
               unsigned long int new_root_cluster;
               unsigned long int tmp=root_cluster;
-              int back=0;	/* To avoid an endless loop... */
+              int back;	/* To avoid an endless loop... */
               /* Il faut ajouter un parcours arriere de la FAT 
                * car on localise le dernier cluster du root_cluster */
               if(verbose>0)
                 log_verbose("cluster %lu, etat=%d, found=%u,nb_subdir=%d,nb_subdir_ok=%d\n",
 		    root_cluster, etat, found, nb_subdir, nb_subdir_ok);
-              do
-              {
-                new_root_cluster=tmp;
-                tmp=fat32_get_prev_cluster(disk_car,partition,reserved,new_root_cluster,no_of_cluster);
-                if(verbose>0)
-                  log_verbose("prev cluster(%lu)=>%lu\n",new_root_cluster,tmp);
-                if(tmp)
-                {
-                  /* Check cluster number */
-                  if((tmp<2) || (tmp>=2+no_of_cluster))
-                  {
-                    log_error("bad cluster number\n");
-                    free(buffer);
-                    return new_root_cluster;
-                  }
-                  /* Read the cluster */
-                  if((unsigned)disk_car->pread(disk_car, buffer, cluster_size,
-			partition->part_offset + (start_data + (uint64_t)(tmp - 2) * sectors_per_cluster) * disk_car->sector_size) != cluster_size)
-                  {
-                    log_critical("cluster can't be read\n");
-                    free(buffer);
-                    return new_root_cluster;
-                  }
-                  /* Check if this cluster is a directory structure. FAT can be damaged */
-                  for(i=0;i<cluster_size/32;i++)
-                  {
-                    if(check_FAT_dir_entry(&buffer[i*0x20],i)!=1)
-                    {
-                      log_error("cluster data is not a directory structure\n");
-                      free(buffer);
-                      return new_root_cluster;
-                    }
-                  }
-                }
-              } while(tmp && (++back<10));
+	      for(back=0; back<10; back++)
+	      {
+		new_root_cluster=tmp;
+		tmp=fat32_get_prev_cluster(disk_car,partition,reserved,new_root_cluster,no_of_cluster);
+		if(verbose>0)
+		  log_verbose("prev cluster(%lu)=>%lu\n",new_root_cluster,tmp);
+		if(tmp==0)
+		{
+		  free(buffer);
+		  return new_root_cluster;
+		}
+		/* Check cluster number */
+		if((tmp<2) || (tmp>=2+no_of_cluster))
+		{
+		  log_error("bad cluster number\n");
+		  free(buffer);
+		  return new_root_cluster;
+		}
+		/* Read the cluster */
+		if((unsigned)disk_car->pread(disk_car, buffer, cluster_size,
+		      partition->part_offset + (start_data + (uint64_t)(tmp - 2) * sectors_per_cluster) * disk_car->sector_size) != cluster_size)
+		{
+		  log_critical("cluster can't be read\n");
+		  free(buffer);
+		  return new_root_cluster;
+		}
+		/* Check if this cluster is a directory structure. FAT can be damaged */
+		for(i=0;i<cluster_size/32;i++)
+		{
+		  if(check_FAT_dir_entry(&buffer[i*0x20],i)!=1)
+		  {
+		    log_error("cluster data is not a directory structure\n");
+		    free(buffer);
+		    return new_root_cluster;
+		  }
+		}
+	      }
               free(buffer);
               return new_root_cluster;
             }
@@ -501,14 +491,10 @@ static unsigned int fat32_find_root_cluster(disk_t *disk_car,const partition_t *
               }
             }
             {
-	      file_info_t dir_list = {
-		.list = TD_LIST_HEAD_INIT(dir_list.list),
-		.name = NULL
-	      };
-	      const file_info_t *file1=td_list_entry(dir_list.list.next, file_info_t, list);
-	      const file_info_t *file2=td_list_entry(file1->list.next, file_info_t, list);
+	      file_info_t dir_list;
+	      TD_INIT_LIST_HEAD(&dir_list.list);
               dir_fat_aux(buffer, cluster_size, 0, &dir_list);
-              if(!td_list_empty(&dir_list.list) && (&file2->list==&dir_list.list || file1->st_ino!=file2->st_ino))
+              if(is_root_cluster_candidat(&dir_list))
               {
                 int test_date=1;
                 if(verbose>0)
@@ -517,7 +503,7 @@ static unsigned int fat32_find_root_cluster(disk_t *disk_car,const partition_t *
                   test_date=dir_aff_log(NULL, &dir_list);
                 }
 #ifdef HAVE_NCURSES
-                if(interface>0 && interactive>0 && test_date>0)
+                if(interactive>0 && test_date>0)
                 {
                   switch(ask_root_directory(disk_car, partition, &dir_list, root_cluster))
                   {
@@ -558,7 +544,7 @@ static unsigned int fat32_find_root_cluster(disk_t *disk_car,const partition_t *
     {
       dir_aff_log(NULL, &rootdir_list);
 #ifdef HAVE_NCURSES
-      if(interface && expert>0)
+      if(expert>0)
       {
         if(ask_confirmation("Create a new root cluster with %u first-level directories (Expert only) (Y/N)",dir_nbr)!=0 && ask_confirmation("Write root cluster, confirm ? (Y/N)")!=0)
         {
@@ -592,7 +578,8 @@ static void fat_date_unix2dos(int unix_date,unsigned short *mstime, unsigned sho
       (((unix_date/3600) % 24) << 11));
   day = unix_date/86400-3652;
   year = day/365;
-  if ((year+3)/4+365*year > day) year--;
+  if ((year+3)/4+365*year > day)
+    year--;
   day -= (year+3)/4+365*year;
   if (day == 59 && !(year & 3)) {
     nl_day = day;
@@ -600,8 +587,9 @@ static void fat_date_unix2dos(int unix_date,unsigned short *mstime, unsigned sho
   }
   else {
     nl_day = (year & 3) || day <= 59 ? day : day-1;
-    for (month = 0; month < 12; month++)
-      if (day_n[month] > nl_day) break;
+    for (month = 1; month < 12; month++)
+      if (day_n[month] > nl_day)
+	break;
   }
   *msdate = le16(nl_day-day_n[month-1]+1+(month << 5)+(year << 9));
 }
@@ -610,7 +598,9 @@ static int file2entry(struct msdos_dir_entry *de, const file_info_t *current_fil
 {
   unsigned int i,j;
   /* Name */
-  for(i=0;(i<8)&&(current_file->name[i]!='.')&&(current_file->name[i]!='\0');i++)
+  for(i=0;
+      i<8 && current_file->name[i]!='.' && current_file->name[i]!='\0';
+      i++)
   {
     de->name[i]=current_file->name[i];
   }
@@ -619,8 +609,12 @@ static int file2entry(struct msdos_dir_entry *de, const file_info_t *current_fil
     de->name[j]=' ';
   }
   /* Extension */
-  for(;(current_file->name[i]!='.')&&(current_file->name[i]!='\0');i++);
-  for(j=0;(j<3)&&(current_file->name[i]!='\0');j++)
+  for(;
+      current_file->name[i]!='.' && current_file->name[i]!='\0';
+      i++);
+  for(j=0;
+      j<3 && current_file->name[i]!='\0';
+      j++)
   {
     de->ext[j]=current_file->name[i];
   }
@@ -653,7 +647,7 @@ static int fat32_create_rootdir(disk_t *disk_car,const partition_t *partition, c
   memset(buffer,0,cluster_size);
   td_list_for_each(file_walker, &rootdir_list->list)
   {
-    const file_info_t *current_file=td_list_entry(file_walker, file_info_t, list);
+    const file_info_t *current_file=td_list_entry_const(file_walker, const file_info_t, list);
     file2entry((struct msdos_dir_entry*)buffer+current_entry,current_file);
     if(++current_entry==(cluster_size/sizeof(struct msdos_dir_entry)))
     {
@@ -692,10 +686,8 @@ static int fat32_create_rootdir(disk_t *disk_car,const partition_t *partition, c
   }
 #ifdef DEBUG
   {
-    file_info_t dir_list = {
-      .list = TD_LIST_HEAD_INIT(dir_list.list),
-      .name = NULL
-    };
+    file_info_t dir_list;
+    TD_INIT_LIST_HEAD(&dir_list.list);
     dir_fat_aux(buffer, cluster_size, 0, &dir_list);
     dir_aff_log(NULL, dir_list);
     delete_list_file(&dir_list);
@@ -821,11 +813,9 @@ static int analyse_dir_entries2(disk_t *disk_car,const partition_t *partition, c
 {
   unsigned char *buffer_dir;
   unsigned int root_dir_size;
-  file_info_t dir_list = {
-    .list = TD_LIST_HEAD_INIT(dir_list.list),
-    .name = NULL
-  };
+  file_info_t dir_list;
   struct td_list_head *file_walker = NULL;
+  TD_INIT_LIST_HEAD(&dir_list.list);
   if(root_size_max==0)
   {
     root_size_max=4096;
@@ -847,7 +837,7 @@ static int analyse_dir_entries2(disk_t *disk_car,const partition_t *partition, c
   }
   td_list_for_each(file_walker, &dir_list.list)
   {
-    const file_info_t *current_file=td_list_entry(file_walker, file_info_t, list);
+    const file_info_t *current_file=td_list_entry_const(file_walker, const file_info_t, list);
     if(LINUX_S_ISDIR(current_file->st_mode) && (current_file->status&FILE_STATUS_DELETED)==0)
     {
       const unsigned long int new_inode=current_file->st_ino;
@@ -979,28 +969,23 @@ static void menu_write_fat_boot_sector(disk_t *disk_car, partition_t *partition,
     if(*current_cmd!=NULL)
     {
       command='Q';
-      while(*current_cmd[0]==',')
-	(*current_cmd)++;
-      if(strncmp(*current_cmd,"list",4)==0)
+      skip_comma_in_command(current_cmd);
+      if(check_command(current_cmd,"list",4)==0)
       {
-	(*current_cmd)+=4;
 	command='L';
       }
-      else if(strncmp(*current_cmd,"dump",4)==0)
+      else if(check_command(current_cmd,"dump",4)==0)
       {
-	(*current_cmd)+=4;
 	command='D';
       }
-      else if(strncmp(*current_cmd,"noconfirm",9)==0)
+      else if(check_command(current_cmd,"noconfirm",9)==0)
       {
 	command=0;	/* do nothing */
 	no_confirm=1;
-	(*current_cmd)+=9;
       }
-      else if(strncmp(*current_cmd,"write",5)==0)
+      else if(check_command(current_cmd,"write",5)==0)
       {
 	command='W';
-	(*current_cmd)+=5;
       }
     }
     else
@@ -1029,7 +1014,7 @@ static void menu_write_fat_boot_sector(disk_t *disk_car, partition_t *partition,
 	  const upart_type_t old_upart_type=upart_type;
 	  partition->upart_type=upart_type;
 	  io_redir_add_redir(disk_car,partition->part_offset,3*disk_car->sector_size,0,newboot);
-	  dir_partition(disk_car, partition, verbose, current_cmd);
+	  dir_partition(disk_car, partition, verbose, 0, current_cmd);
 	  io_redir_del_redir(disk_car,partition->part_offset);
 	  partition->upart_type=old_upart_type;
 	}
@@ -1075,7 +1060,7 @@ static void menu_write_fat_boot_sector(disk_t *disk_car, partition_t *partition,
     log_info("Don't write new boot!\n");
 }
 
-static void create_fat_boot_sector(disk_t *disk_car, partition_t *partition, const unsigned int reserved, const int verbose, const unsigned int dir_entries, const unsigned long int root_cluster, const unsigned int sectors_per_cluster, const unsigned int fat_length,const int interface, const upart_type_t upart_type, const unsigned int fats, char **current_cmd)
+static void create_fat_boot_sector(disk_t *disk_car, partition_t *partition, const unsigned int reserved, const int verbose, const unsigned int dir_entries, const unsigned long int root_cluster, const unsigned int sectors_per_cluster, const unsigned int fat_length, const upart_type_t upart_type, const unsigned int fats, char **current_cmd)
 {
   unsigned char *orgboot;
   unsigned char *newboot;
@@ -1093,8 +1078,9 @@ static void create_fat_boot_sector(disk_t *disk_car, partition_t *partition, con
     memset(orgboot,0,3*disk_car->sector_size);
   }
   memcpy(newboot,orgboot,3*disk_car->sector_size);
-  if((le16(fat_header->marker)!=0xAA55)||
-      !((fat_header->ignored[0]==0xeb && fat_header->ignored[2]==0x90)||fat_header->ignored[0]==0xe9))
+  if(3*disk_car->sector_size >= DEFAULT_SECTOR_SIZE &&
+      (le16(fat_header->marker)!=0xAA55 ||
+       !((fat_header->ignored[0]==0xeb && fat_header->ignored[2]==0x90)||fat_header->ignored[0]==0xe9)))
   {
     write_FAT_boot_code_aux(newboot);
   }
@@ -1270,8 +1256,7 @@ static void create_fat_boot_sector(disk_t *disk_car, partition_t *partition, con
   {
     log_info("Extrapolated boot sector and current boot sector are identical.\n");
   }
-  if(interface)
-    menu_write_fat_boot_sector(disk_car, partition, verbose, upart_type, orgboot, newboot, error, current_cmd);
+  menu_write_fat_boot_sector(disk_car, partition, verbose, upart_type, orgboot, newboot, error, current_cmd);
   free(orgboot);
   free(newboot);
 }
@@ -1572,7 +1557,7 @@ static unsigned int fat_find_fat_start(const unsigned char *buffer,const int p_f
   return 0;
 }
 
-static int fat_find_type(disk_t *disk_car,const partition_t *partition,const uint64_t max_offset,const int p_fat12,const int p_fat16,const int p_fat32,const int verbose,const int dump_ind,const int interface,unsigned int *nbr_offset,info_offset_t *info_offset, const unsigned int max_nbr_offset)
+static int fat_find_type(disk_t *disk_car, const partition_t *partition, const uint64_t max_offset, const int p_fat12, const int p_fat16, const int p_fat32, const int verbose, const int dump_ind, unsigned int *nbr_offset, info_offset_t *info_offset, const unsigned int max_nbr_offset)
 {
   uint64_t offset;
 #ifdef HAVE_NCURSES
@@ -1586,15 +1571,12 @@ static int fat_find_type(disk_t *disk_car,const partition_t *partition,const uin
         (long unsigned)(max_offset/disk_car->sector_size), p_fat12, p_fat16, p_fat32, verbose, dump_ind);
   }
 #ifdef HAVE_NCURSES
-  if(interface)
-  {
-      wmove(stdscr,8,0);
-      wprintw(stdscr,"FAT : %s%s%s?\n",p_fat12?"12 ":"", p_fat16?"16 ":"", p_fat32?"32 ":"");
-      wmove(stdscr,22,0);
-      wattrset(stdscr, A_REVERSE);
-      waddstr(stdscr,"  Stop  ");
-      wattroff(stdscr, A_REVERSE);
-  }
+  wmove(stdscr,8,0);
+  wprintw(stdscr,"FAT : %s%s%s?\n",p_fat12?"12 ":"", p_fat16?"16 ":"", p_fat32?"32 ":"");
+  wmove(stdscr,22,0);
+  wattrset(stdscr, A_REVERSE);
+  waddstr(stdscr,"  Stop  ");
+  wattroff(stdscr, A_REVERSE);
 #endif
   for(offset=disk_car->sector_size;
       offset<max_offset && !ind_stop;
@@ -1602,7 +1584,7 @@ static int fat_find_type(disk_t *disk_car,const partition_t *partition,const uin
   {
 #ifdef HAVE_NCURSES
     const unsigned long int percent=offset*100/max_offset;
-    if(interface && (percent!=old_percent))
+    if(percent!=old_percent)
     {
       wmove(stdscr,8,30);
       wclrtoeol(stdscr);	/* before addstr for BSD compatibility */
@@ -1653,24 +1635,21 @@ static int fat_find_type(disk_t *disk_car,const partition_t *partition,const uin
     }
   }
 #ifdef HAVE_NCURSES
-  if(interface)
-  {
-    wmove(stdscr,22,0);
-    wclrtoeol(stdscr);
-    wrefresh(stdscr);
-  }
+  wmove(stdscr,22,0);
+  wclrtoeol(stdscr);
+  wrefresh(stdscr);
 #endif
   free(buffer);
   return 0;
 }
 
-static upart_type_t fat_find_info(disk_t *disk_car,unsigned int*reserved, unsigned int*fat_length, const partition_t *partition,const uint64_t max_offset,const int p_fat12,const int p_fat16,const int p_fat32,const int verbose,const int dump_ind,const int interface, const unsigned int expert, unsigned int *fats)
+static upart_type_t fat_find_info(disk_t *disk_car,unsigned int*reserved, unsigned int*fat_length, const partition_t *partition, const uint64_t max_offset, const int p_fat12, const int p_fat16, const int p_fat32, const int verbose, const int dump_ind, const unsigned int expert, unsigned int *fats)
 {
   unsigned int nbr_offset=0;
   unsigned int i;
   info_offset_t info_offset[0x400];
   upart_type_t upart_type=UP_UNK;
-  fat_find_type(disk_car, partition,max_offset,p_fat12,p_fat16,p_fat32,verbose,dump_ind,interface,&nbr_offset,&info_offset[0], 0x400);
+  fat_find_type(disk_car, partition, max_offset, p_fat12, p_fat16, p_fat32, verbose, dump_ind, &nbr_offset, &info_offset[0], 0x400);
   for(i=0;i<nbr_offset;i++)
   {
     const uint64_t end=partition->part_offset+(uint64_t)info_offset[i].offset*disk_car->sector_size;
@@ -1680,7 +1659,7 @@ static upart_type_t fat_find_info(disk_t *disk_car,unsigned int*reserved, unsign
 	offset2sector(disk_car,end),
 	info_offset[i].nbr);
 #ifdef HAVE_NCURSES
-    if(dump_ind>0 && interface>0)
+    if(dump_ind>0)
     {
       unsigned char *buffer=(unsigned char *)MALLOC(disk_car->sector_size);
       if((unsigned)disk_car->pread(disk_car, &buffer, disk_car->sector_size, end) == disk_car->sector_size)
@@ -1712,11 +1691,17 @@ static upart_type_t fat_find_info(disk_t *disk_car,unsigned int*reserved, unsign
     }
     switch(info_offset[offset_for_max_nbr].fat_type)
     {
-      case 12: upart_type=UP_FAT12; break;
-      case 16: upart_type=UP_FAT16; break;
-      case 32: upart_type=UP_FAT32; break;
+      case 12:
+	upart_type=UP_FAT12;
+	break;
+      case 16:
+	upart_type=UP_FAT16;
+	break;
+      case 32:
+	upart_type=UP_FAT32;
+	break;
     }
-    for(i=0;i<nbr_offset;i++)
+    for(i=0; i<nbr_offset; i++)
     {
       if(info_offset[i].fat_type==info_offset[offset_for_max_nbr].fat_type)
       {
@@ -1797,7 +1782,7 @@ static upart_type_t fat_find_info(disk_t *disk_car,unsigned int*reserved, unsign
       log_info("first_fat %lu, second_fat %lu\n",info_offset[first_fat].offset, info_offset[second_fat].offset);
     }
   }
-  if(expert>0 && interface>0)
+  if(expert>0)
   {
 #ifdef HAVE_NCURSES
     return select_fat_info(info_offset,nbr_offset,reserved,fat_length,max_offset/disk_car->sector_size,fats);
@@ -1840,6 +1825,7 @@ static upart_type_t select_fat_info(const info_offset_t *info_offset, const unsi
     if(fat2_location == *reserved)
     {
       *fats=1;
+      *fat_length=0;
     }
     else if(fat2_location > *reserved)
     {
@@ -1886,7 +1872,7 @@ static upart_type_t select_fat_info(const info_offset_t *info_offset, const unsi
 }
 #endif
 
-int rebuild_FAT_BS(disk_t *disk_car, partition_t *partition, const int verbose, const int dump_ind,const int interface,const unsigned int expert, char**current_cmd)
+int rebuild_FAT_BS(disk_t *disk_car, partition_t *partition, const int verbose, const int dump_ind, const unsigned int expert, char**current_cmd)
 {
   unsigned long int max_offset;
   unsigned int fat_length=0;
@@ -1963,44 +1949,38 @@ int rebuild_FAT_BS(disk_t *disk_car, partition_t *partition, const int verbose,
        log_debug("search_fat16(partition,max_offset=%d,p_fat12=%d,p_fat16=%d,p_fat32=%d,debug=%d,dump_ind=%d)\n",max_offset,p_fat12,p_fat16,p_fat32,verbose,dump_ind);
    */
 #ifdef HAVE_NCURSES
-  if(interface)
-  {
-    aff_copy(stdscr);
-    wmove(stdscr,4,0);
-    wprintw(stdscr,"%s",disk_car->description(disk_car));
-    mvwaddstr(stdscr,5,0,msg_PART_HEADER_LONG);
-    wmove(stdscr,6,0);
-    aff_part(stdscr,AFF_PART_ORDER|AFF_PART_STATUS,disk_car,partition);
-    wrefresh(stdscr);
-  }
+  aff_copy(stdscr);
+  wmove(stdscr,4,0);
+  wprintw(stdscr,"%s",disk_car->description(disk_car));
+  mvwaddstr(stdscr,5,0,msg_PART_HEADER_LONG);
+  wmove(stdscr,6,0);
+  aff_part(stdscr,AFF_PART_ORDER|AFF_PART_STATUS,disk_car,partition);
+  wrefresh(stdscr);
 #endif
-  upart_type=fat_find_info(disk_car,&reserved, &fat_length, partition,max_offset,p_fat12,p_fat16,p_fat32,verbose,dump_ind,interface,expert,&fats);
+  upart_type=fat_find_info(disk_car, &reserved, &fat_length, partition, max_offset, p_fat12, p_fat16, p_fat32, verbose, dump_ind, expert, &fats);
 #ifdef HAVE_NCURSES
-  if(interface)
+  aff_copy(stdscr);
+  wmove(stdscr,4,0);
+  wprintw(stdscr,"%s",disk_car->description(disk_car));
+  mvwaddstr(stdscr,5,0,msg_PART_HEADER_LONG);
+  wmove(stdscr,6,0);
+  aff_part(stdscr,AFF_PART_ORDER|AFF_PART_STATUS,disk_car,partition);
+  wmove(stdscr,8,0);
+  wclrtoeol(stdscr);
+  switch(upart_type)
   {
-    aff_copy(stdscr);
-    wmove(stdscr,4,0);
-    wprintw(stdscr,"%s",disk_car->description(disk_car));
-    mvwaddstr(stdscr,5,0,msg_PART_HEADER_LONG);
-    wmove(stdscr,6,0);
-    aff_part(stdscr,AFF_PART_ORDER|AFF_PART_STATUS,disk_car,partition);
-    wmove(stdscr,8,0);
-    wclrtoeol(stdscr);
-    switch(upart_type)
-    {
-      case UP_FAT12:
-	waddstr(stdscr,"FAT : 12");
-	break;
-      case UP_FAT16:
-	waddstr(stdscr,"FAT : 16");
-	break;
-      case UP_FAT32:
-	waddstr(stdscr,"FAT : 32");
-	break;
-      default:
-	waddstr(stdscr,"No FAT found");
-	break;
-    }
+    case UP_FAT12:
+      waddstr(stdscr,"FAT : 12");
+      break;
+    case UP_FAT16:
+      waddstr(stdscr,"FAT : 16");
+      break;
+    case UP_FAT32:
+      waddstr(stdscr,"FAT : 32");
+      break;
+    default:
+      waddstr(stdscr,"No FAT found");
+      break;
   }
 #endif
   if(verbose>0)
@@ -2026,7 +2006,7 @@ int rebuild_FAT_BS(disk_t *disk_car, partition_t *partition, const int verbose,
       (fat_length==0)||(reserved==0))
   {
     uint64_t start_data=0;
-    if(find_sectors_per_cluster(disk_car, partition, verbose, dump_ind, interface, &sectors_per_cluster, &start_data, upart_type)==0)
+    if(find_sectors_per_cluster(disk_car, partition, verbose, dump_ind, &sectors_per_cluster, &start_data, upart_type)==0)
     {
       display_message("Can't find cluster size\n");
       return 0;
@@ -2089,12 +2069,9 @@ int rebuild_FAT_BS(disk_t *disk_car, partition_t *partition, const int verbose,
       log_info("fat_length %u\n",fat_length);
   }
 #ifdef HAVE_NCURSES
-  if(interface)
-  {
-    if(fat_length==0)
-      waddstr(stdscr," Can't find FAT length\n");
-    wrefresh(stdscr);
-  }
+  if(fat_length==0)
+    waddstr(stdscr," Can't find FAT length\n");
+  wrefresh(stdscr);
 #endif
   if(upart_type && (fat_length>1))
   {
@@ -2185,7 +2162,7 @@ int rebuild_FAT_BS(disk_t *disk_car, partition_t *partition, const int verbose,
       /* Use first fat */
       fat32_free_info(disk_car,partition,reserved,data_size/sectors_per_cluster,&first_free_cluster,&free_cluster_count);
       /* FAT32 : Find root cluster */
-      root_cluster=fat32_find_root_cluster(disk_car,partition,sectors_per_cluster,data_size/sectors_per_cluster,reserved,fat_length,interface,verbose,expert,first_free_cluster,fats);
+      root_cluster=fat32_find_root_cluster(disk_car, partition, sectors_per_cluster, data_size/sectors_per_cluster, reserved, fat_length, verbose, expert, first_free_cluster, fats);
       if(expert>0)
       {
 #ifdef HAVE_NCURSES
@@ -2205,14 +2182,11 @@ int rebuild_FAT_BS(disk_t *disk_car, partition_t *partition, const int verbose,
       }
     }
 #ifdef HAVE_NCURSES
-    if(interface)
-    {
-      wmove(stdscr,9,0);
-      wclrtoeol(stdscr);
-      wrefresh(stdscr);
-    }
+    wmove(stdscr,9,0);
+    wclrtoeol(stdscr);
+    wrefresh(stdscr);
 #endif
-    create_fat_boot_sector(disk_car,partition, reserved, verbose,dir_entries,root_cluster,sectors_per_cluster,fat_length,interface,upart_type,fats,current_cmd);
+    create_fat_boot_sector(disk_car, partition, reserved, verbose, dir_entries, root_cluster, sectors_per_cluster, fat_length, upart_type, fats, current_cmd);
     if(verbose)
     {
       log_info("\n");
