@@ -25,7 +25,7 @@
 extern "C" {
 #endif
 
-int rebuild_FAT_BS(disk_t *disk_car,partition_t *partition, const int verbose, const int dump_ind,const int interface, const unsigned int expert, char**current_cmd);
+int rebuild_FAT_BS(disk_t *disk_car,partition_t *partition, const int verbose, const int dump_ind, const unsigned int expert, char**current_cmd);
 int FAT_init_rootdir(disk_t *disk_car, partition_t *partition, const int verbose, char **current_cmd);
 int repair_FAT_table(disk_t *disk_car, partition_t *partition, const int verbose, char **current_cmd);
 
