@@ -32,25 +32,16 @@
 #include "filegen.h"
 
 static void register_header_check_au(file_stat_t *file_stat);
-static int header_check_au(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only,  const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new);
 
 const file_hint_t file_hint_au= {
   .extension="au",
   .description="Sun/NeXT audio data",
-  .min_header_distance=0,
   .max_filesize=PHOTOREC_MAX_FILE_SIZE,
   .recover=1,
   .enable_by_default=1,
   .register_header_check=&register_header_check_au
 };
 
-static const unsigned char au_header[4]= {'.','s','n','d'};
-
-static void register_header_check_au(file_stat_t *file_stat)
-{
-  register_header_check(0, au_header,sizeof(au_header), &header_check_au, file_stat);
-}
-
 /* http://en.wikipedia.org/wiki/Au_file_format */
 struct header_au_s
 {
@@ -65,8 +56,7 @@ struct header_au_s
 static int header_check_au(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
 {
   const struct header_au_s *au=(const struct header_au_s *)buffer;
-  if(memcmp(buffer,au_header,sizeof(au_header))==0 &&
-    (const uint32_t)be32(au->offset) >= sizeof(struct header_au_s) &&
+  if((const uint32_t)be32(au->offset) >= sizeof(struct header_au_s) &&
     be32(au->encoding)>0 && be32(au->encoding)<=27 &&
     be32(au->channels)>0 && be32(au->channels)<=256)
   {
@@ -77,7 +67,7 @@ static int header_check_au(const unsigned char *buffer, const unsigned int buffe
       reset_file_recovery(file_recovery_new);
       file_recovery_new->min_filesize=111;
       file_recovery_new->extension=file_hint_au.extension;
-      file_recovery_new->calculated_file_size=be32(au->offset)+be32(au->size);
+      file_recovery_new->calculated_file_size=(uint64_t)be32(au->offset)+be32(au->size);
       file_recovery_new->data_check=&data_check_size;
       file_recovery_new->file_check=&file_check_size;
       return 1;
@@ -89,3 +79,9 @@ static int header_check_au(const unsigned char *buffer, const unsigned int buffe
   }
   return 0;
 }
+
+static void register_header_check_au(file_stat_t *file_stat)
+{
+  static const unsigned char au_header[4]= {'.','s','n','d'};
+  register_header_check(0, au_header,sizeof(au_header), &header_check_au, file_stat);
+}
