@@ -37,7 +37,6 @@ static void register_header_check_axx(file_stat_t *file_stat);
 const file_hint_t file_hint_axx= {
   .extension="axx",
   .description="AxCrypt",
-  .min_header_distance=0,
   .max_filesize=PHOTOREC_MAX_FILE_SIZE,
   .recover=1,
   .enable_by_default=1,
@@ -49,11 +48,6 @@ struct SHeader
   uint32_t aoLength;
   uint8_t   oType;
 } __attribute__ ((gcc_struct, __packed__));
-#if defined(HAVE_FSEEKO) && !defined(__MINGW32__)
-#define my_fseek fseeko
-#else
-#define my_fseek fseek
-#endif
 
 static void file_check_axx(file_recovery_t *fr)
 {
