@@ -40,7 +40,6 @@ static void register_header_check_caf(file_stat_t *file_stat);
 const file_hint_t file_hint_caf= {
   .extension="caf",
   .description="Core Audio Format",
-  .min_header_distance=0,
   .max_filesize=PHOTOREC_MAX_FILE_SIZE,
   .recover=1,
   .enable_by_default=1,
@@ -66,8 +65,8 @@ static data_check_t data_check_caf(const unsigned char *buffer, const unsigned i
 #ifdef DEBUG_CAF
     log_trace("file_caf.c: %s chunk %c%c%c%c (0x%02x%02x%02x%02x) size %llu, calculated_file_size %llu (0x%llx)\n",
 	file_recovery->filename,
-        buffer[i],buffer[i+1],buffer[i+2],buffer[i+3], 
-        buffer[i],buffer[i+1],buffer[i+2],buffer[i+3], 
+        buffer[i],buffer[i+1],buffer[i+2],buffer[i+3],
+        buffer[i],buffer[i+1],buffer[i+2],buffer[i+3],
         (long long unsigned)chunk_size,
         (long long unsigned)file_recovery->calculated_file_size,
         (long long unsigned)file_recovery->calculated_file_size);
@@ -79,7 +78,7 @@ static data_check_t data_check_caf(const unsigned char *buffer, const unsigned i
     }
     if(chunk_size >= 0)
     {
-      file_recovery->calculated_file_size+=12+chunk_size;
+      file_recovery->calculated_file_size+=(uint64_t)12+chunk_size;
     }
     else
     {
@@ -117,7 +116,7 @@ static void register_header_check_caf(file_stat_t *file_stat)
 {
   static const unsigned char caf_header[12]=  {
     'c' , 'a' , 'f' , 'f' , 0x00, 0x01, 0x00, 0x00,
-    'd' , 'e' , 's' , 'c' 
+    'd' , 'e' , 's' , 'c'
   };
   register_header_check(0, caf_header, sizeof(caf_header), &header_check_caf, file_stat);
 }
