@@ -30,6 +30,7 @@
 #include <string.h>
 #endif
 #include <stdio.h>
+#include <assert.h>
 #include "types.h"
 #include "common.h"
 #include "filegen.h"
@@ -37,30 +38,24 @@
 #include "log.h"
 #include "memmem.h"
 #include "setdate.h"
+#include "file_doc.h"
 
 static void register_header_check_doc(file_stat_t *file_stat);
 static void file_check_doc(file_recovery_t *file_recovery);
 static int header_check_doc(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new);
-static void file_rename_doc(const char *old_filename);
-static uint32_t *OLE_load_FAT(FILE *IN, const struct OLE_HDR *header);
-static uint32_t *OLE_load_MiniFAT(FILE *IN, const struct OLE_HDR *header, const uint32_t *fat, const unsigned int fat_entries);
+static void file_rename_doc(file_recovery_t *file_recovery);
+static uint32_t *OLE_load_FAT(FILE *IN, const struct OLE_HDR *header, const uint64_t offset);
+static uint32_t *OLE_load_MiniFAT(FILE *IN, const struct OLE_HDR *header, const uint32_t *fat, const unsigned int fat_entries, const uint64_t offset);
 
 const file_hint_t file_hint_doc= {
   .extension="doc",
   .description="Microsoft Office Document (doc/xls/ppt/vsd/...), 3ds Max, MetaStock, Wilcom ES",
-  .min_header_distance=0,
   .max_filesize=PHOTOREC_MAX_FILE_SIZE,
   .recover=1,
   .enable_by_default=1,
   .register_header_check=&register_header_check_doc
 };
 
-static const unsigned char doc_header[]= { 0xd0, 0xcf, 0x11, 0xe0, 0xa1, 0xb1, 0x1a, 0xe1};
-
-static void register_header_check_doc(file_stat_t *file_stat)
-{
-  register_header_check(0, doc_header,sizeof(doc_header), &header_check_doc, file_stat);
-}
 
 const char WilcomDesignInformationDDD[56]=
 {
@@ -73,22 +68,23 @@ const char WilcomDesignInformationDDD[56]=
   'D', '\0', 'D', '\0', 'D', '\0', '\0', '\0'
 };
 
-static void file_check_doc(file_recovery_t *file_recovery)
+void file_check_doc_aux(file_recovery_t *file_recovery, const uint64_t offset)
 {
   unsigned char buffer_header[512];
   uint64_t doc_file_size;
   uint32_t *fat;
   unsigned long int i;
-  unsigned int freesect_count=0;  
+  unsigned int freesect_count=0;
   const struct OLE_HDR *header=(const struct OLE_HDR*)&buffer_header;
   const uint64_t doc_file_size_org=file_recovery->file_size;
-  file_recovery->file_size=0;
+  file_recovery->file_size=offset;
   /*reads first sector including OLE header */
-  if(fseek(file_recovery->handle, 0, SEEK_SET) < 0 ||
+  if(my_fseek(file_recovery->handle, offset, SEEK_SET) < 0 ||
       fread(&buffer_header, sizeof(buffer_header), 1, file_recovery->handle) != 1)
     return ;
 #ifdef DEBUG_OLE
   log_info("file_check_doc %s\n", file_recovery->filename);
+  log_trace("sector size          %u\n",1<<le16(header->uSectorShift));
   log_trace("num_FAT_blocks       %u\n",le32(header->num_FAT_blocks));
   log_trace("num_extra_FAT_blocks %u\n",le32(header->num_extra_FAT_blocks));
 #endif
@@ -97,7 +93,7 @@ static void file_check_doc(file_recovery_t *file_recovery)
       le32(header->num_extra_FAT_blocks)>50 ||
       le32(header->num_FAT_blocks)>109+le32(header->num_extra_FAT_blocks)*((1<<le16(header->uSectorShift))-1))
     return ;
-  if((fat=OLE_load_FAT(file_recovery->handle, header))==NULL)
+  if((fat=OLE_load_FAT(file_recovery->handle, header, offset))==NULL)
   {
 #ifdef DEBUG_OLE
     log_info("OLE_load_FAT failed\n");
@@ -109,11 +105,12 @@ static void file_check_doc(file_recovery_t *file_recovery)
       i>0 && le32(fat[i])==0xFFFFFFFF;
       i--)
     freesect_count++;
-  doc_file_size=512+(((le32(header->num_FAT_blocks)<<le16(header->uSectorShift))/4-freesect_count)<<le16(header->uSectorShift));
+  doc_file_size=offset + ((1+(le32(header->num_FAT_blocks)<<le16(header->uSectorShift))/4-freesect_count)<<le16(header->uSectorShift));
   if(doc_file_size > doc_file_size_org)
   {
 #ifdef DEBUG_OLE
-    log_info("doc_file_size=512+((%u<<%u)/4-%u)<<%u\n",
+    log_info("doc_file_size=%llu + (1+(%u<<%u)/4-%u)<<%u\n",
+	(unsigned long long)offset,
 	le32(header->num_FAT_blocks), le16(header->uSectorShift),
 	freesect_count, le16(header->uSectorShift));
     log_info("doc_file_size %llu > doc_file_size_org %llu\n",
@@ -148,7 +145,7 @@ static void file_check_doc(file_recovery_t *file_recovery)
 	free(fat);
 	return ;
       }
-      if(fseek(file_recovery->handle, 512+(block<<le16(header->uSectorShift)), SEEK_SET)<0)
+      if(my_fseek(file_recovery->handle, offset + ((1+block)<<le16(header->uSectorShift)), SEEK_SET)<0)
       {
 #ifdef DEBUG_OLE
 	log_info("fseek failed\n");
@@ -173,7 +170,8 @@ static void file_check_doc(file_recovery_t *file_recovery)
 	    sid<(1<<le16(header->uSectorShift))/sizeof(struct OLE_DIR) && dir_entry->type!=NO_ENTRY;
 	    sid++,dir_entry++)
 	{
-	    if(le32(dir_entry->start_block) > 0 && le32(dir_entry->size) > 0 &&
+	    if(offset +
+		le32(dir_entry->start_block) > 0 && le32(dir_entry->size) > 0 &&
 		((le32(dir_entry->size) >= le32(header->miniSectorCutoff)
 		  && le32(dir_entry->start_block) > fat_entries) ||
 		 le32(dir_entry->size) > doc_file_size))
@@ -194,6 +192,11 @@ static void file_check_doc(file_recovery_t *file_recovery)
   file_recovery->file_size=doc_file_size;
 }
 
+static void file_check_doc(file_recovery_t *file_recovery)
+{
+  file_check_doc_aux(file_recovery, 0);
+}
+
 static const char *ole_get_file_extension(const unsigned char *buffer, const unsigned int buffer_size)
 {
   const struct OLE_HDR *header=(const struct OLE_HDR *)buffer;
@@ -211,7 +214,7 @@ static const char *ole_get_file_extension(const unsigned char *buffer, const uns
   else
   {
     const uint32_t *fati=(const uint32_t *)(header+1);
-    const unsigned int fat_offset=(le32(fati[0]) << le16(header->uSectorShift))+512;
+    const unsigned int fat_offset=(1+le32(fati[0])) << le16(header->uSectorShift);
     fat=(const uint32_t *)&buffer[fat_offset];
     fat_entries=(le32(header->num_FAT_blocks) << le16(header->uSectorShift))/4;
     if(fat_offset>buffer_size)
@@ -228,7 +231,7 @@ static const char *ole_get_file_extension(const unsigned char *buffer, const uns
       block<fat_entries && block!=0xFFFFFFFE && i<fat_entries;
       block=le32(fat[block]), i++)
   {
-    const unsigned int offset_root_dir=512+(block<<le16(header->uSectorShift));
+    const unsigned int offset_root_dir=(1+block)<<le16(header->uSectorShift);
 #ifdef DEBUG_OLE
     log_info("Root Directory block=%u (0x%x)\n", block, block);
 #endif
@@ -251,7 +254,7 @@ static const char *ole_get_file_extension(const unsigned char *buffer, const uns
 	}
 	for(;j<64;j+=2)
 	  log_info(" ");
-	log_info(" type %u", dir_entry->type);
+	log_info(" namsiz=%u type %u", le16(dir_entry->namsiz), dir_entry->type);
 	log_info(" Flags=%s", (dir_entry->bflags==0?"Red  ":"Black"));
 	log_info(" sector %u (%u bytes)\n",
 	    (unsigned int)le32(dir_entry->start_block),
@@ -264,6 +267,10 @@ static const char *ole_get_file_extension(const unsigned char *buffer, const uns
 	  is_db++;
 	switch(le16(dir_entry->namsiz))
 	{
+	  case 10:
+	    if(memcmp(dir_entry->name, ".\0Q\0D\0F\0\0\0",10)==0)
+	      return "qdf-backup";
+	    break;
 	  case 12:
 	    /* 3ds max */
 	    if(memcmp(dir_entry->name, "S\0c\0e\0n\0e\0\0\0",12)==0)
@@ -288,13 +295,19 @@ static const char *ole_get_file_extension(const unsigned char *buffer, const uns
 	    break;
 	  case 22:
 	    /* SigmaPlot .jnb */
-	    if(memcmp(dir_entry->name, "J\0N\0B\0V\0e\0r\0s\0i\0o\0n\0\0", 22)==0)
+	    if(memcmp(dir_entry->name, "J\0N\0B\0V\0e\0r\0s\0i\0o\0n\0\0\0", 22)==0)
 	      return "jnb";
+	    /* Autodesk Inventor part ipt or iam file */
+	    if(memcmp(dir_entry->name, "R\0S\0e\0S\0t\0o\0r\0a\0g\0e\0\0\0", 22)==0)
+	      return "ipt";
 	    break;
 	  case 24:
 	    /* HP Photosmart Photo Printing Album */
 	    if(memcmp(dir_entry->name,"I\0m\0a\0g\0e\0s\0S\0t\0o\0r\0e\0\0\0",24)==0)
 	      return "albm";
+	    /* Lotus Approch */
+	    if(memcmp(dir_entry->name,"A\0p\0p\0r\0o\0a\0c\0h\0D\0o\0c\0\0\0",24)==0)
+	      return "apr";
 	    break;
 	  case 28:
 	    /* Microsoft Works Spreadsheet or Chart */
@@ -313,6 +326,11 @@ static const char *ole_get_file_extension(const unsigned char *buffer, const uns
 #endif
 	    }
 	    break;
+	  case 32:
+	    /* Revit */
+	    if(memcmp(dir_entry->name, "R\0e\0v\0i\0t\0P\0r\0e\0v\0i\0e\0w\0004\0.\0000\0\0", 32)==0)
+	      return "rvt";
+	    break;
 	  case 34:
 	    if(memcmp(dir_entry->name, "S\0t\0a\0r\0C\0a\0l\0c\0D\0o\0c\0u\0m\0e\0n\0t\0\0\0",34)==0)
 	      return "sdc";
@@ -370,113 +388,116 @@ static const char *ole_get_file_extension(const unsigned char *buffer, const uns
 
 static int header_check_doc(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
 {
-  if(memcmp(buffer,doc_header,sizeof(doc_header))==0)
-  {
-    const struct OLE_HDR *header=(const struct OLE_HDR *)buffer;
-    if(le16(header->reserved)!=0 || le32(header->reserved1)!=0)
-      return 0;
-    if(le16(header->uMiniSectorShift)!=6)
-      return 0;
-    /* max and qbb file have uSectorShift=12 */
-    if(le16(header->uSectorShift)!=9 && le16(header->uSectorShift)!=12)
-      return 0;
-    if(le16(header->uSectorShift)==9 && le32(header->csectDir)!=0)
-      return 0;
-    /* max file have csectDir=1
-     * qbb file have csectDir=4 */
-    if(le16(header->uSectorShift)==12 && le32(header->csectDir)==0)
-      return 0;
-    /*
-       num_FAT_blocks=109+num_extra_FAT_blocks*(512-1);
-       maximum file size is 512+(num_FAT_blocks*128)*512, about 1.6GB
+  const struct OLE_HDR *header=(const struct OLE_HDR *)buffer;
+  /* Check for Little Endian */
+  if(le16(header->uByteOrder)!=0xFFFE)
+    return 0;
+  if(le16(header->uDllVersion)!=3 && le16(header->uDllVersion)!=4)
+    return 0;
+  if(le16(header->reserved)!=0 || le32(header->reserved1)!=0)
+    return 0;
+  if(le16(header->uMiniSectorShift)!=6)
+    return 0;
+  if(le16(header->uDllVersion)==3 && le16(header->uSectorShift)!=9)
+    return 0;
+  /* max and qbb file have uSectorShift=12 */
+  if(le16(header->uDllVersion)==4 && le16(header->uSectorShift)!=12)
+    return 0;
+  if(le16(header->uDllVersion)==3 && le32(header->csectDir)!=0)
+    return 0;
+  /* max file have csectDir=1
+   * qbb file have csectDir=4 */
+  if(le16(header->uDllVersion)==4 && le32(header->csectDir)==0)
+    return 0;
+  /*
+     num_FAT_blocks=109+num_extra_FAT_blocks*(512-1);
+     maximum file size is 512+(num_FAT_blocks*128)*512, about 1.6GB
      */
-    if(le32(header->num_FAT_blocks)==0 ||
-	le32(header->num_extra_FAT_blocks)>50 ||
-	le32(header->num_FAT_blocks)>109+le32(header->num_extra_FAT_blocks)*((1<<le16(header->uSectorShift))-1))
-      return 0;
-    reset_file_recovery(file_recovery_new);
-    file_recovery_new->file_check=&file_check_doc;
-    file_recovery_new->file_rename=&file_rename_doc;
-    file_recovery_new->extension=ole_get_file_extension(buffer, buffer_size);
-    if(file_recovery_new->extension!=NULL)
+  if(le32(header->num_FAT_blocks)==0 ||
+      le32(header->num_extra_FAT_blocks)>50 ||
+      le32(header->num_FAT_blocks)>109+le32(header->num_extra_FAT_blocks)*((1<<le16(header->uSectorShift))-1))
+    return 0;
+  reset_file_recovery(file_recovery_new);
+  file_recovery_new->file_check=&file_check_doc;
+  file_recovery_new->file_rename=&file_rename_doc;
+  file_recovery_new->extension=ole_get_file_extension(buffer, buffer_size);
+  if(file_recovery_new->extension!=NULL)
+  {
+    if(strcmp(file_recovery_new->extension,"sda")==0)
     {
-      if(strcmp(file_recovery_new->extension,"sda")==0)
-      {
-	if(td_memmem(buffer,buffer_size,"StarImpress",11)!=NULL)
-	  file_recovery_new->extension="sdd";
-      }
-      else if(strcmp(file_recovery_new->extension,"wps")==0)
-      {
-	/* Distinguish between MS Works .wps and MS Publisher .pub */
-	if(td_memmem(buffer,buffer_size,"Microsoft Publisher",19)!=NULL)
-	  file_recovery_new->extension="pub";
-      }
-      return 1;
+      if(td_memmem(buffer,buffer_size,"StarImpress",11)!=NULL)
+	file_recovery_new->extension="sdd";
     }
-    if(td_memmem(buffer,buffer_size,"WordDocument",12)!=NULL)
+    else if(strcmp(file_recovery_new->extension,"wps")==0)
     {
-      file_recovery_new->extension="doc";
+      /* Distinguish between MS Works .wps and MS Publisher .pub */
+      if(td_memmem(buffer,buffer_size,"Microsoft Publisher",19)!=NULL)
+	file_recovery_new->extension="pub";
     }
-    else if(td_memmem(buffer,buffer_size,"StarDraw",8)!=NULL)
-    {
-      file_recovery_new->extension="sda";
-    }
-    else if(td_memmem(buffer,buffer_size,"StarCalc",8)!=NULL)
-    {
-      file_recovery_new->extension="sdc";
-    }
-    else if(td_memmem(buffer,buffer_size,"StarImpress",11)!=NULL)
-    {
-      file_recovery_new->extension="sdd";
-    }
-    else if(td_memmem(buffer,buffer_size,"Worksheet",9)!=NULL ||
-	td_memmem(buffer,buffer_size,"Book",4)!=NULL || 
-	td_memmem(buffer,buffer_size,"Workbook",8)!=NULL || 
-	td_memmem(buffer,buffer_size,"Calc",4)!=NULL)
-    {
-      file_recovery_new->extension="xls";
-    }
-    else if(td_memmem(buffer,buffer_size,"Power",5)!=NULL)
-    {
-      file_recovery_new->extension="ppt";
-    }
-    else if(td_memmem(buffer,buffer_size,"AccessObjSiteData",17)!=NULL)
-    {
-      file_recovery_new->extension="mdb";
-    }
-    else if(td_memmem(buffer,buffer_size,"Visio",5)!=NULL)
-    {
-      file_recovery_new->extension="vsd";
-    }
-    else if(td_memmem(buffer,buffer_size,"SfxDocument",11)!=NULL)
-    {
-      file_recovery_new->extension="sdw";
-    }
-    else if(td_memmem(buffer,buffer_size,"CPicPage",8)!=NULL)
-    {	/* Flash Project File */
-      file_recovery_new->extension="fla";
-    }
-    else if(td_memmem(buffer,buffer_size,"Microsoft Publisher",19)!=NULL)
-    { /* Publisher */
-      file_recovery_new->extension="pub";
-    }
-    else if(td_memmem(buffer, buffer_size, "Microsoft Works Database", 24)!=NULL
-	|| td_memmem( buffer, buffer_size, "MSWorksDBDoc", 12)!=NULL)
-    { /* Microsoft Works .wdb */
-      file_recovery_new->extension="wdb";
-    }
-    else if(td_memmem(buffer,buffer_size,"MetaStock",9)!=NULL)
-    { /* MetaStock */
-      file_recovery_new->extension="mws";
-    }
-    else
-      file_recovery_new->extension=file_hint_doc.extension;
     return 1;
   }
-  return 0;
+  if(td_memmem(buffer,buffer_size,"WordDocument",12)!=NULL)
+  {
+    file_recovery_new->extension="doc";
+  }
+  else if(td_memmem(buffer,buffer_size,"StarDraw",8)!=NULL)
+  {
+    file_recovery_new->extension="sda";
+  }
+  else if(td_memmem(buffer,buffer_size,"StarCalc",8)!=NULL)
+  {
+    file_recovery_new->extension="sdc";
+  }
+  else if(td_memmem(buffer,buffer_size,"StarImpress",11)!=NULL)
+  {
+    file_recovery_new->extension="sdd";
+  }
+  else if(td_memmem(buffer,buffer_size,"Worksheet",9)!=NULL ||
+      td_memmem(buffer,buffer_size,"Book",4)!=NULL ||
+      td_memmem(buffer,buffer_size,"Workbook",8)!=NULL ||
+      td_memmem(buffer,buffer_size,"Calc",4)!=NULL)
+  {
+    file_recovery_new->extension="xls";
+  }
+  else if(td_memmem(buffer,buffer_size,"Power",5)!=NULL)
+  {
+    file_recovery_new->extension="ppt";
+  }
+  else if(td_memmem(buffer,buffer_size,"AccessObjSiteData",17)!=NULL)
+  {
+    file_recovery_new->extension="mdb";
+  }
+  else if(td_memmem(buffer,buffer_size,"Visio",5)!=NULL)
+  {
+    file_recovery_new->extension="vsd";
+  }
+  else if(td_memmem(buffer,buffer_size,"SfxDocument",11)!=NULL)
+  {
+    file_recovery_new->extension="sdw";
+  }
+  else if(td_memmem(buffer,buffer_size,"CPicPage",8)!=NULL)
+  {	/* Flash Project File */
+    file_recovery_new->extension="fla";
+  }
+  else if(td_memmem(buffer,buffer_size,"Microsoft Publisher",19)!=NULL)
+  { /* Publisher */
+    file_recovery_new->extension="pub";
+  }
+  else if(td_memmem(buffer, buffer_size, "Microsoft Works Database", 24)!=NULL
+      || td_memmem( buffer, buffer_size, "MSWorksDBDoc", 12)!=NULL)
+  { /* Microsoft Works .wdb */
+    file_recovery_new->extension="wdb";
+  }
+  else if(td_memmem(buffer,buffer_size,"MetaStock",9)!=NULL)
+  { /* MetaStock */
+    file_recovery_new->extension="mws";
+  }
+  else
+    file_recovery_new->extension=file_hint_doc.extension;
+  return 1;
 }
 
-static uint32_t *OLE_load_FAT(FILE *IN, const struct OLE_HDR *header)
+static uint32_t *OLE_load_FAT(FILE *IN, const struct OLE_HDR *header, const uint64_t offset)
 {
   uint32_t *fat;
   uint32_t *dif;
@@ -491,7 +512,7 @@ static uint32_t *OLE_load_FAT(FILE *IN, const struct OLE_HDR *header)
 	i<le32(header->num_extra_FAT_blocks) && block!=0xFFFFFFFF && block!=0xFFFFFFFE;
 	i++, block=le32(dif[109+i*(((1<<le16(header->uSectorShift))/4)-1)]))
     {
-      if(fseek(IN, 512+(block<<le16(header->uSectorShift)), SEEK_SET) < 0)
+      if(my_fseek(IN, offset + ((1+block)<<le16(header->uSectorShift)), SEEK_SET) < 0)
       {
 	free(dif);
 	return NULL;
@@ -512,7 +533,7 @@ static uint32_t *OLE_load_FAT(FILE *IN, const struct OLE_HDR *header)
 	j<le32(header->num_FAT_blocks);
 	j++, data+=(1<<le16(header->uSectorShift)))
     {
-      if(fseek(IN,512+(le32(dif[j])<<le16(header->uSectorShift)),SEEK_SET)<0)
+      if(my_fseek(IN, offset + ((1+le32(dif[j]))<<le16(header->uSectorShift)), SEEK_SET)<0)
       {
 	free(dif);
 	free(fat);
@@ -532,7 +553,7 @@ static uint32_t *OLE_load_FAT(FILE *IN, const struct OLE_HDR *header)
 
 static void *OLE_read_stream(FILE *IN,
     const uint32_t *fat, const unsigned int fat_entries, const unsigned int uSectorShift,
-    const unsigned int block_start, const unsigned int len)
+    const unsigned int block_start, const unsigned int len, const uint64_t offset)
 {
   unsigned char *dataPt;
   unsigned int block;
@@ -547,7 +568,7 @@ static void *OLE_read_stream(FILE *IN,
       free(dataPt);
       return NULL;
     }
-    if(fseek(IN, 512+(block<<uSectorShift), SEEK_SET)<0)
+    if(my_fseek(IN, offset + ((1+block)<<uSectorShift), SEEK_SET)<0)
     {
       free(dataPt);
       return NULL;
@@ -561,7 +582,7 @@ static void *OLE_read_stream(FILE *IN,
   return dataPt;
 }
 
-static uint32_t *OLE_load_MiniFAT(FILE *IN, const struct OLE_HDR *header, const uint32_t *fat, const unsigned int fat_entries)
+static uint32_t *OLE_load_MiniFAT(FILE *IN, const struct OLE_HDR *header, const uint32_t *fat, const unsigned int fat_entries, const uint64_t offset)
 {
   unsigned char*minifat_pos;
   uint32_t *minifat;
@@ -574,7 +595,7 @@ static uint32_t *OLE_load_MiniFAT(FILE *IN, const struct OLE_HDR *header, const
   block=le32(header->MiniFat_block);
   for(i=0; i < le32(header->csectMiniFat) && block < fat_entries; i++)
   {
-    if(fseek(IN, ((uint64_t)block << le16(header->uSectorShift)) + 512, SEEK_SET) < 0)
+    if(my_fseek(IN, offset + (((uint64_t)1+block) << le16(header->uSectorShift)), SEEK_SET) < 0)
     {
       free(minifat);
       return NULL;
@@ -645,6 +666,8 @@ static void software2ext(const char **ext, const unsigned int count, const unsig
 
 static const char *software_uni2ext(const unsigned int count, const unsigned char *software)
 {
+  if(count>=15 && memcmp(software, "M\0i\0c\0r\0o\0s\0o\0f\0t\0 \0E\0x\0c\0e\0l\0", 30)==0)
+    return "et";
   if(count>=17 && memcmp(software, "D\0e\0l\0c\0a\0m\0 \0P\0o\0w\0e\0r\0S\0H\0A\0P\0E\0", 34)==0)
     return "psmodel";
   return NULL;
@@ -653,18 +676,19 @@ static const char *software_uni2ext(const unsigned int count, const unsigned cha
 static void OLE_parse_summary_aux(const unsigned char *dataPt, const unsigned int dirLen, const char **ext, char **title, time_t *file_time)
 {
   unsigned int pos;
+  assert(dirLen >= 48 && dirLen<=1024*1024);
 #ifdef DEBUG_OLE
   dump_log(dataPt, dirLen);
 #endif
   if(dataPt[0]!=0xfe || dataPt[1]!=0xff)
     return ;
   pos=get32u(dataPt, 44);
+  if(pos > dirLen - 8)
+    return ;
   {
 //    unsigned int size;
     unsigned int numEntries;
     unsigned int i;
-    if(pos+8 > dirLen)
-      return ;
     numEntries=get32u(dataPt, pos+4);
 #ifdef DEBUG_OLE
     {
@@ -778,7 +802,8 @@ static void *OLE_read_ministream(unsigned char *ministream,
 
 static void OLE_parse_summary(FILE *file, const uint32_t *fat, const unsigned int fat_entries,
     const struct OLE_HDR *header, const unsigned int ministream_block, const unsigned int ministream_size,
-    const unsigned int block, const unsigned int len, const char **ext, char **title, time_t *file_time)
+    const unsigned int block, const unsigned int len, const char **ext, char **title, time_t *file_time,
+    const uint64_t offset)
 {
   unsigned char *summary=NULL;
   if(len < 48 || len>1024*1024)
@@ -790,11 +815,11 @@ static void OLE_parse_summary(FILE *file, const uint32_t *fat, const unsigned in
       const unsigned int mini_fat_entries=(le32(header->csectMiniFat) << le16(header->uSectorShift)) / 4;
       uint32_t *minifat;
       unsigned char *ministream;
-      if((minifat=OLE_load_MiniFAT(file, header, fat, fat_entries))==NULL)
+      if((minifat=OLE_load_MiniFAT(file, header, fat, fat_entries, offset))==NULL)
 	return ;
       ministream=(unsigned char *)OLE_read_stream(file,
 	  fat, fat_entries, le16(header->uSectorShift),
-	  ministream_block, ministream_size);
+	  ministream_block, ministream_size, offset);
       if(ministream != NULL)
       {
 	summary=(unsigned char*)OLE_read_ministream(ministream,
@@ -808,7 +833,7 @@ static void OLE_parse_summary(FILE *file, const uint32_t *fat, const unsigned in
   else
     summary=(unsigned char *)OLE_read_stream(file,
 	fat, fat_entries, le16(header->uSectorShift),
-	block, len);
+	block, len, offset);
   if(summary!=NULL)
   {
     OLE_parse_summary_aux(summary, len, ext, title, file_time);
@@ -816,7 +841,7 @@ static void OLE_parse_summary(FILE *file, const uint32_t *fat, const unsigned in
   }
 }
 
-static void file_rename_doc(const char *old_filename)
+static void file_rename_doc(file_recovery_t *file_recovery)
 {
   const char *ext=NULL;
   char *title=NULL;
@@ -826,15 +851,15 @@ static void file_rename_doc(const char *old_filename)
   const struct OLE_HDR *header=(const struct OLE_HDR*)&buffer_header;
   time_t file_time=0;
   unsigned int fat_entries;
-  if(strstr(old_filename, ".sdd")!=NULL)
+  if(strstr(file_recovery->filename, ".sdd")!=NULL)
     ext="sdd";
-  if((file=fopen(old_filename, "rb"))==NULL)
+  if((file=fopen(file_recovery->filename, "rb"))==NULL)
     return;
 #ifdef DEBUG_OLE
-  log_info("file_rename_doc(%s)\n", old_filename);
+  log_info("file_rename_doc(%s)\n", file_recovery->filename);
 #endif
   /*reads first sector including OLE header */
-  if(fseek(file, 0, SEEK_SET) < 0 ||
+  if(my_fseek(file, 0, SEEK_SET) < 0 ||
       fread(&buffer_header, sizeof(buffer_header), 1, file) != 1)
   {
     fclose(file);
@@ -848,16 +873,7 @@ static void file_rename_doc(const char *old_filename)
     fclose(file);
     return ;
   }
-  if(le16(header->uSectorShift)==12)
-  {
-    fclose(file);
-    if(le32(header->csectDir)==1)
-      file_rename(old_filename, NULL, 0, 0, "max", 1);
-    else
-      file_rename(old_filename, NULL, 0, 0, "qbb", 1);
-    return ;
-  }
-  if((fat=OLE_load_FAT(file, header))==NULL)
+  if((fat=OLE_load_FAT(file, header, 0))==NULL)
   {
     fclose(file);
     return ;
@@ -880,18 +896,20 @@ static void file_rename_doc(const char *old_filename)
 	block=le32(fat[block]), i++)
     {
       struct OLE_DIR *dir_entries;
-      if(fseek(file, 512+(block<<le16(header->uSectorShift)), SEEK_SET)<0)
+      if(my_fseek(file, (1+block)<<le16(header->uSectorShift), SEEK_SET)<0)
       {
 	free(fat);
 	fclose(file);
+	free(title);
 	return ;
       }
       dir_entries=(struct OLE_DIR *)MALLOC(1<<le16(header->uSectorShift));
-      if(fread(dir_entries, (1<<le16(header->uSectorShift)), 1, file)!=1)
+      if(fread(dir_entries, 1<<le16(header->uSectorShift), 1, file)!=1)
       {
 	free(fat);
 	free(dir_entries);
 	fclose(file);
+	free(title);
 	return ;
       }
 
@@ -934,6 +952,10 @@ static void file_rename_doc(const char *old_filename)
 #endif
 	    switch(le16(dir_entry->namsiz))
 	    {
+	      case 10:
+		if(memcmp(dir_entry->name, ".\0Q\0D\0F\0\0\0",10)==0)
+		  ext="qdf-backup";
+		break;
 	      case 12:
 		/* 3ds max */
 		if(memcmp(dir_entry->name, "S\0c\0e\0n\0e\0\0\0",12)==0)
@@ -966,13 +988,19 @@ static void file_rename_doc(const char *old_filename)
 		break;
 	      case 22:
 		/* SigmaPlot .jnb */
-		if(memcmp(dir_entry->name, "J\0N\0B\0V\0e\0r\0s\0i\0o\0n\0\0", 22)==0)
+		if(memcmp(dir_entry->name, "J\0N\0B\0V\0e\0r\0s\0i\0o\0n\0\0\0", 22)==0)
 		  ext="jnb";
+		/* Autodesk Inventor part ipt or iam file */
+		if(memcmp(dir_entry->name, "R\0S\0e\0S\0t\0o\0r\0a\0g\0e\0\0\0", 22)==0)
+		  ext="ipt";
 		break;
 	      case 24:
 		/* HP Photosmart Photo Printing Album */
 		if(memcmp(dir_entry->name,"I\0m\0a\0g\0e\0s\0S\0t\0o\0r\0e\0\0\0",24)==0)
 		  ext="albm";
+		/* Lotus Approach */
+		else if(memcmp(dir_entry->name,"A\0p\0p\0r\0o\0a\0c\0h\0D\0o\0c\0\0\0",24)==0)
+		  ext="apr";
 		break;
 	      case 28:
 		/* Microsoft Works Spreadsheet or Chart */
@@ -991,6 +1019,10 @@ static void file_rename_doc(const char *old_filename)
 #endif
 		}
 		break;
+	      case 32:
+		if(memcmp(dir_entry->name, "m\0a\0n\0i\0f\0e\0s\0t\0.\0c\0a\0m\0x\0m\0l\0\0\0",32)==0)
+		  ext="camrec";
+		break;
 	      case 34:
 		if(memcmp(dir_entry->name, "S\0t\0a\0r\0C\0a\0l\0c\0D\0o\0c\0u\0m\0e\0n\0t\0\0\0",34)==0)
 		  ext="sdc";
@@ -1000,6 +1032,8 @@ static void file_rename_doc(const char *old_filename)
 		if((ext==NULL || strcmp(ext,"sdd")!=0) &&
 		    memcmp(dir_entry->name, "S\0t\0a\0r\0D\0r\0a\0w\0D\0o\0c\0u\0m\0e\0n\0t\0003\0\0\0", 36)==0)
 		  ext="sda";
+		else if(memcmp(dir_entry->name, "f\0i\0l\0e\0_\0C\0O\0M\0P\0A\0N\0Y\0_\0F\0I\0L\0E\0\0\0", 36)==0)
+		    ext="qbb";
 		break;
 	      case 38:
 		/* Quattro Pro spreadsheet */
@@ -1014,7 +1048,7 @@ static void file_rename_doc(const char *old_filename)
 		  OLE_parse_summary(file, fat, fat_entries, header,
 		      ministream_block, ministream_size,
 		      le32(dir_entry->start_block), le32(dir_entry->size),
-		      &ext, &title, &file_time);
+		      &ext, &title, &file_time, 0);
 		}
 		else if(memcmp(dir_entry->name,"P\0o\0w\0e\0r\0P\0o\0i\0n\0t\0 \0D\0o\0c\0u\0m\0e\0n\0t\0\0\0", 40)==0)
 		  ext="ppt";
@@ -1055,12 +1089,18 @@ static void file_rename_doc(const char *old_filename)
   free(fat);
   fclose(file);
   if(file_time!=0 && file_time!=(time_t)-1)
-    set_date(old_filename, file_time, file_time);
+    set_date(file_recovery->filename, file_time, file_time);
   if(title!=NULL)
   {
-    file_rename(old_filename, (const unsigned char*)title, strlen(title), 0, ext, 1);
+    file_rename(file_recovery, (const unsigned char*)title, strlen(title), 0, ext, 1);
     free(title);
   }
   else
-    file_rename(old_filename, NULL, 0, 0, ext, 1);
+    file_rename(file_recovery, NULL, 0, 0, ext, 1);
+}
+
+static void register_header_check_doc(file_stat_t *file_stat)
+{
+  static const unsigned char doc_header[]= { 0xd0, 0xcf, 0x11, 0xe0, 0xa1, 0xb1, 0x1a, 0xe1};
+  register_header_check(0, doc_header,sizeof(doc_header), &header_check_doc, file_stat);
 }
