@@ -36,7 +36,6 @@ static void register_header_check_drw(file_stat_t *file_stat);
 const file_hint_t file_hint_drw= {
   .extension="drw",
   .description="Pro/ENGINEER Drawing",
-  .min_header_distance=0,
   .max_filesize=PHOTOREC_MAX_FILE_SIZE,
   .recover=1,
   .enable_by_default=1,
@@ -56,7 +55,7 @@ static int header_check_drw(const unsigned char *buffer, const unsigned int buff
   if(!isprint(buffer[14]) || !isprint(buffer[15]) || !isprint(buffer[16]) || !isprint(buffer[17]))
     return 0;
   reset_file_recovery(file_recovery_new);
-  file_recovery_new->file_check=file_check_drw;
+  file_recovery_new->file_check=&file_check_drw;
   file_recovery_new->extension=file_hint_drw.extension;
   return 1;
 }
