@@ -36,7 +36,6 @@ static void register_header_check_edb(file_stat_t *file_stat);
 const file_hint_t file_hint_edb= {
   .extension="edb",
   .description="Exchange Database",
-  .min_header_distance=0,
   .max_filesize=PHOTOREC_MAX_FILE_SIZE,
   .recover=1,
   .enable_by_default=1,
@@ -50,7 +49,10 @@ static int header_check_edb(const unsigned char *buffer, const unsigned int buff
   if(file_recovery->file_stat!=NULL &&
       file_recovery->file_stat->file_hint==&file_hint_edb &&
       file_recovery->file_size==4096)
+  {
+    /* header_ignored(file_recovery_new); is useless as there is no file check */
     return 0;
+  }
   reset_file_recovery(file_recovery_new);
   file_recovery_new->extension=file_hint_edb.extension;
   return 1;
