@@ -38,12 +38,11 @@
 
 static void register_header_check_exe(file_stat_t *file_stat);
 static int header_check_exe(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new);
-static void file_rename_pe_exe(const char *old_filename);
+static void file_rename_pe_exe(file_recovery_t *file_recovery);
 
 const file_hint_t file_hint_exe= {
   .extension="exe",
   .description="MS Windows executable",
-  .min_header_distance=0,
   .max_filesize=PHOTOREC_MAX_FILE_SIZE,
   .recover=1,
   .enable_by_default=1,
@@ -120,7 +119,10 @@ static int header_check_exe(const unsigned char *buffer, const unsigned int buff
       uint64_t sum=0;
       const struct pe_image_section_hdr *pe_image_section=(const struct pe_image_section_hdr*)
 	((const unsigned char*)pe_hdr + sizeof(struct pe_image_file_hdr) + le16(pe_hdr->SizeOfOptionalHeader));
-      for(i=0;i<le16(pe_hdr->NumberOfSections) && (const unsigned char*)pe_image_section < buffer+buffer_size;i++,pe_image_section++)
+      for(i=0;
+	  i<le16(pe_hdr->NumberOfSections) &&
+	  (const unsigned char*)(pe_image_section+1) <= buffer+buffer_size;
+	  i++,pe_image_section++)
       {
 	if(le32(pe_image_section->SizeOfRawData)>0)
 	{
@@ -138,7 +140,7 @@ static int header_check_exe(const unsigned char *buffer, const unsigned int buff
 	if(le16(pe_image_section->NumberOfRelocations)>0)
 	{
 #ifdef DEBUG_EXE
-	  log_debug("relocations 0x%lx-0x%lx\n", 
+	  log_debug("relocations 0x%lx-0x%lx\n",
 	      (unsigned long)le32(pe_image_section->PointerToRelocations),
 	      (unsigned long)le32(pe_image_section->PointerToRelocations)+1*le16(pe_image_section->NumberOfRelocations)-1);
 #endif
@@ -176,7 +178,7 @@ static int header_check_exe(const unsigned char *buffer, const unsigned int buff
     if(le16(dos_hdr->bytes_in_last_block))
       coff_offset-=512-le16(dos_hdr->bytes_in_last_block);
 
-    if(coff_offset+1 < buffer_size &&
+    if(coff_offset < buffer_size-1 &&
 	buffer[coff_offset]==0x4c && buffer[coff_offset+1]==0x01)
     { /*  COFF_I386MAGIC */
       reset_file_recovery(file_recovery_new);
@@ -253,7 +255,7 @@ static unsigned int ReadUnicodeStr(const char *buffer, unsigned int pos, const u
   return pos;
 }
 
-static int PEVersion_aux(const char*buffer, const unsigned int end, const char *old_filename, const char *needle, const unsigned int needle_len, const int force_ext)
+static int PEVersion_aux(file_recovery_t *file_recovery, const char*buffer, const unsigned int end, const char *needle, const unsigned int needle_len, const int force_ext)
 {
   unsigned int pos=0;
   while(1)
@@ -306,7 +308,7 @@ static int PEVersion_aux(const char*buffer, const unsigned int end, const char *
 	    {
 	      if(do_rename)
 	      {
-		file_rename_unicode(old_filename, buffer, end, pt, NULL, force_ext);
+		file_rename_unicode(file_recovery, buffer, end, pt, NULL, force_ext);
 		return 0;
 	      }
 #ifdef DEBUG_EXE
@@ -328,7 +330,7 @@ static int PEVersion_aux(const char*buffer, const unsigned int end, const char *
   }
 }
 
-static void PEVersion(FILE *file, const unsigned int offset, const unsigned int length, const char *old_filename)
+static void PEVersion(FILE *file, const unsigned int offset, const unsigned int length, file_recovery_t *file_recovery)
 {
   char *buffer;
   if(length==0 || length > 1024*1024)
@@ -341,16 +343,16 @@ static void PEVersion(FILE *file, const unsigned int offset, const unsigned int
     free(buffer);
     return ;
   }
-  if(PEVersion_aux(buffer, length, old_filename, OriginalFilename, sizeof(OriginalFilename), 0)==0)
+  if(PEVersion_aux(file_recovery, buffer, length, OriginalFilename, sizeof(OriginalFilename), 0)==0)
   {
     free(buffer);
     return;
   }
-  PEVersion_aux(buffer, length, old_filename, InternalName, sizeof(InternalName), 1);
+  PEVersion_aux(file_recovery, buffer, length, InternalName, sizeof(InternalName), 1);
   free(buffer);
 }
 
-static void file_exe_ressource(FILE *file, const unsigned int base, const unsigned int dir_start, const unsigned int size, const unsigned int rsrcType, const unsigned int level, const struct pe_image_section_hdr *pe_sections, unsigned int nbr_sections, const char *old_filename)
+static void file_exe_ressource(FILE *file, const unsigned int base, const unsigned int dir_start, const unsigned int size, const unsigned int rsrcType, const unsigned int level, const struct pe_image_section_hdr *pe_sections, unsigned int nbr_sections, file_recovery_t *file_recovery)
 {
   struct rsrc_entries_s *rsrc_entries;
   struct rsrc_entries_s *rsrc_entry;
@@ -401,17 +403,23 @@ static void file_exe_ressource(FILE *file, const unsigned int base, const unsign
 	    size,
 	    (level==0?le32(rsrc_entry->Type):rsrcType),
 	    level + 1,
-	    pe_sections, nbr_sections, old_filename);
+	    pe_sections, nbr_sections, file_recovery);
       }
       if(level==2)
       {
 	unsigned int off;
 	unsigned int len;
 	if(fseek(file, base + (le32(rsrc_entry->Pos) & 0x7fffffff), SEEK_SET)<0)
+	{
+	  free(rsrc_entries);
 	  return ;
+	}
 	buffer_size=fread(buffer, 1, sizeof(buffer), file);
 	if(buffer_size<16)
+	{
+	  free(rsrc_entries);
 	  return ;
+	}
 	off=buffer[0]+ (buffer[1]<<8) + (buffer[2]<<16) + (buffer[3]<<24);
 	len=buffer[4]+ (buffer[5]<<8) + (buffer[6]<<16) + (buffer[7]<<24);
 	{
@@ -421,7 +429,7 @@ static void file_exe_ressource(FILE *file, const unsigned int base, const unsign
 	    if(le32(pe_section->VirtualAddress) <= off
 	      && off < le32(pe_section->VirtualAddress) + le32(pe_section->SizeOfRawData))
 	    {
-	      PEVersion(file, off - le32(pe_section->VirtualAddress) + base, len, old_filename);
+	      PEVersion(file, off - le32(pe_section->VirtualAddress) + base, len, file_recovery);
 	      free(rsrc_entries);
 	      return ;
 	    }
@@ -435,14 +443,14 @@ static void file_exe_ressource(FILE *file, const unsigned int base, const unsign
   free(rsrc_entries);
 }
 
-static void file_rename_pe_exe(const char *old_filename)
+static void file_rename_pe_exe(file_recovery_t *file_recovery)
 {
   unsigned char buffer[4096];
   FILE *file;
   int buffer_size;
   const struct dos_image_file_hdr *dos_hdr=(const struct dos_image_file_hdr*)buffer;
   const struct pe_image_file_hdr *pe_hdr;
-  if((file=fopen(old_filename, "rb"))==NULL)
+  if((file=fopen(file_recovery->filename, "rb"))==NULL)
     return;
   buffer_size=fread(buffer, 1, sizeof(buffer), file);
   if(buffer_size < (int)sizeof(struct dos_image_file_hdr))
@@ -495,7 +503,7 @@ static void file_rename_pe_exe(const char *old_filename)
     {
       if(le32(pe_section->SizeOfRawData)>0)
       {
-	if(strcmp((const char*)pe_section->Name, ".rsrc")==0)
+	if(memcmp((const char*)pe_section->Name, ".rsrc", 6)==0)
 	{
 	  file_exe_ressource(file,
 	      le32(pe_section->PointerToRawData),
@@ -503,7 +511,7 @@ static void file_rename_pe_exe(const char *old_filename)
 	      le32(pe_section->SizeOfRawData),
 	      0,
 	      0,
-	      pe_sections, nbr_sections, old_filename);
+	      pe_sections, nbr_sections, file_recovery);
 	  fclose(file);
 	  return;
 	}
