@@ -39,7 +39,6 @@ static int header_check_ext2_fs(const unsigned char *buffer, const unsigned int
 const file_hint_t file_hint_ext2_fs= {
   .extension="ext",
   .description="ext2/ext3/ext4 Filesystem",
-  .min_header_distance=0,
   .max_filesize=0,
   .recover=1,
   .enable_by_default=0,
@@ -63,7 +62,10 @@ static int header_check_ext2_fs(const unsigned char *buffer, const unsigned int
   if(file_recovery->file_stat!=NULL &&
        file_recovery->file_stat->file_hint==&file_hint_ext2_fs &&
        file_recovery->calculated_file_size==(uint64_t)le32(sb->s_blocks_count)*(EXT2_MIN_BLOCK_SIZE<<le32(sb->s_log_block_size)))
-    return 0;
+  {
+    if(header_ignored_adv(file_recovery, file_recovery_new)==0)
+      return 0;
+  }
   reset_file_recovery(file_recovery_new);
   file_recovery_new->extension=file_hint_ext2_fs.extension;
   file_recovery_new->calculated_file_size=(uint64_t)le32(sb->s_blocks_count)*(EXT2_MIN_BLOCK_SIZE<<le32(sb->s_log_block_size));
