@@ -44,7 +44,6 @@ static void register_header_check_fh5(file_stat_t *file_stat);
 const file_hint_t file_hint_fh5= {
   .extension="fh5",
   .description="Macromedia Freehand 5",
-  .min_header_distance=0,
   .max_filesize=PHOTOREC_MAX_FILE_SIZE,
   .recover=1,
   .enable_by_default=1,
@@ -68,7 +67,7 @@ static int header_check_fh5(const unsigned char *buffer, const unsigned int buff
   file_recovery_new->min_filesize=4096;
   file_recovery_new->calculated_file_size=be32(fh5_buffer->datalen);
   file_recovery_new->extension=file_hint_fh5.extension;
-  file_recovery_new->file_check=file_check_fh5;
+  file_recovery_new->file_check=&file_check_fh5;
   //    log_debug("header_check_fh5: Guessed length : %u\n", fh5_file_size);
   return 1;
 }
