@@ -37,7 +37,6 @@ static void register_header_check_flac(file_stat_t *file_stat);
 const file_hint_t file_hint_flac= {
   .extension="flac",
   .description="FLAC audio",
-  .min_header_distance=0,
   .max_filesize=(uint64_t)1500*1024*1024,
   .recover=1,
   .enable_by_default=1,
@@ -66,7 +65,7 @@ static data_check_t data_check_flac_metadata(const unsigned char *buffer, const
 #endif
     if((buffer[i]&0x7f)==0x7f)
 	return DC_ERROR;
-    file_recovery->calculated_file_size+=4+size;
+    file_recovery->calculated_file_size+=(uint64_t)4+size;
     if((buffer[i]&0x80)==0x80)
     {
       file_recovery->data_check=&data_check_flac_frame;
