@@ -37,8 +37,7 @@ static void register_header_check_flv(file_stat_t *file_stat);
 const file_hint_t file_hint_flv= {
   .extension="flv",
   .description="Macromedia",
-  .min_header_distance=0,
-  .max_filesize=200*1024*1024,
+  .max_filesize=PHOTOREC_MAX_FILE_SIZE,
   .recover=1,
   .enable_by_default=1,
   .register_header_check=&register_header_check_flv
@@ -83,7 +82,8 @@ static data_check_t data_check_flv(const unsigned char *buffer, const unsigned i
 	file_recovery->calculated_file_size+=4;
 	return DC_STOP;
       }
-      file_recovery->calculated_file_size+=4+11+datasize;
+      /* 4+11=15*/
+      file_recovery->calculated_file_size+=(uint64_t)15+datasize;
     }
     else
       return DC_ERROR;
