@@ -34,19 +34,12 @@
 #include "log.h"
 #endif
 
-#if defined(HAVE_FSEEKO) && !defined(__MINGW32__)
-#define my_fseek fseeko
-#else
-#define my_fseek fseek
-#endif
-
 static void register_header_check_gpg(file_stat_t *file_stat);
 static int header_check_gpg(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new);
 
 const file_hint_t file_hint_gpg= {
   .extension="gpg",
   .description="OpenPGP/GPG (Partial support)",
-  .min_header_distance=0,
   .max_filesize=PHOTOREC_MAX_FILE_SIZE,
   .recover=1,
   .enable_by_default=1,
@@ -264,7 +257,7 @@ static void file_check_gpg(file_recovery_t *file_recovery)
 
     if(partial_body_length==0)
     {
-      if((buffer[i]&0x80)==0)	
+      if((buffer[i]&0x80)==0)
 	break;	/* Invalid */
       tag=openpgp_packet_tag(&buffer[i]);
       if((buffer[i]&0x40)==0)
@@ -417,9 +410,9 @@ static void file_check_gpg(file_recovery_t *file_recovery)
 
 static int header_check_gpg(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
 {
+  uint64_t i=0;
   unsigned int packet_tag[16];
   unsigned int nbr=0;
-  unsigned int i=0;
   int partial_body_length=0;
   int stop=0;
   memset(packet_tag, 0, sizeof(packet_tag));
@@ -431,7 +424,7 @@ static int header_check_gpg(const unsigned char *buffer, const unsigned int buff
     const int old_partial_body_length=partial_body_length;
     if(partial_body_length==0)
     {
-      if((buffer[i]&0x80)==0)	
+      if((buffer[i]&0x80)==0)
 	break;	/* Invalid */
       packet_tag[nbr]=openpgp_packet_tag(&buffer[i]);
       if((buffer[i]&0x40)==0)
