@@ -50,16 +50,11 @@
 #include "file_tiff.h"
 #include "setdate.h"
 
-#if defined(HAVE_FSEEKO) && !defined(__MINGW32__)
-#define my_fseek fseeko
-#else
-#define my_fseek fseek
-#endif
-
 extern const file_hint_t file_hint_doc;
 extern const file_hint_t file_hint_indd;
 extern const file_hint_t file_hint_mov;
 extern const file_hint_t file_hint_riff;
+extern const file_hint_t file_hint_rw2;
 extern data_check_t data_check_avi_stream(const unsigned char *buffer, const unsigned int buffer_size, file_recovery_t *file_recovery);
 
 static void register_header_check_jpg(file_stat_t *file_stat);
@@ -71,7 +66,6 @@ static int jpg_check_dht(const unsigned char *buffer, const unsigned int buffer_
 const file_hint_t file_hint_jpg= {
   .extension="jpg",
   .description="JPG picture",
-  .min_header_distance=0,
   .max_filesize=50*1024*1024,
   .recover=1,
   .enable_by_default=1,
@@ -276,13 +270,25 @@ static uint64_t check_mpo_le(const unsigned char *mpo, const uint64_t mpo_offset
   return max_offset;
 }
 
+static uint64_t check_mpo(const unsigned char *mpo, const uint64_t offset, const unsigned int size)
+{
+  if(mpo[0]=='I' && mpo[1]=='I' && mpo[2]=='*' && mpo[3]==0)
+  {
+    return check_mpo_le(mpo, offset, size);
+  }
+  else if(mpo[0]=='M' && mpo[1]=='M' && mpo[2]==0 && mpo[3]=='*')
+  {
+    return check_mpo_be(mpo, offset, size);
+  }
+  return 0;
+}
+
 static void file_check_mpo(file_recovery_t *fr)
 {
   unsigned char buffer[512];
   uint64_t offset=0;
   unsigned int size=0;
   size_t nbytes;
-  const unsigned char *mpo;
   {
     /* Check the first jpg */
     const uint64_t fs=fr->file_size;
@@ -293,7 +299,7 @@ static void file_check_mpo(file_recovery_t *fr)
   }
   do
   {
-    offset+=2+size;
+    offset+=(uint64_t)2+size;
     if(my_fseek(fr->handle, offset, SEEK_SET) < 0)
     {
       fr->file_size=0;
@@ -321,21 +327,10 @@ static void file_check_mpo(file_recovery_t *fr)
     fr->file_size=0;
     return ;
   }
-  offset+=8;
-  size-=8;
-  mpo=buffer+8;
-  if(mpo[0]=='I' && mpo[1]=='I' && mpo[2]=='*' && mpo[3]==0)
   {
-    const uint64_t max_offset=check_mpo_le(mpo, offset, size);
+    const uint64_t max_offset=check_mpo(buffer+8, offset+8, size-8);
     fr->file_size=(max_offset > fr->file_size ? 0 : max_offset);
   }
-  else if(mpo[0]=='M' && mpo[1]=='M' && mpo[2]==0 && mpo[3]=='*')
-  {
-    const uint64_t max_offset=check_mpo_be(mpo, offset, size);
-    fr->file_size=(max_offset > fr->file_size ? 0 : max_offset);
-  }
-  else
-    fr->file_size=0;
 }
 
 static int is_marker_valid(const unsigned int marker)
@@ -344,10 +339,21 @@ static int is_marker_valid(const unsigned int marker)
   {
     case 0x02 ... 0xbf:	/* Reserved */
     case 0xc0:		/* SOF0 Start of Frame */
+    case 0xc1:		/* SOF1 Extended sequential */
+    case 0xc2:		/* SOF2 Progressive */
+    case 0xc3:		/* SOF3 Lossless */
     case 0xc4:		/* Define Huffman table */
+    case 0xc5:		/* SOF5 Differential sequential */
+    case 0xc6:		/* SOF6 Differential progressive */
+    case 0xc7:		/* SOF7 Differential lossless */
     case 0xc8:		/* Start of Frame (JPG) (Reserved for JPEG extensions) */
+    case 0xc9:		/* SOF9 Extended sequential, arithmetic coding */
+    case 0xca:		/* SOF10 Progressive, arithmetic coding */
+    case 0xcb:		/* SOF11 Lossless, arithmetic coding */
     case 0xcc:		/* DAC arithmetic-coding conditioning*/
-    case 0xcf:		/* SOF15 */
+    case 0xcd:		/* SOF13 Differential sequential, arithmetic coding */
+    case 0xce:		/* SOF14 Differential progressive, arithmetic coding */
+    case 0xcf:		/* SOF15 Differential lossless, arithmetic coding */
     case 0xd0 ... 0xd7:	/* JPEG_RST0 .. JPEG_RST7 markers */
 //    case 0xd8:	/* SOI Start of Image */
 //    case 0xd9:	/* EOI End of Image */
@@ -383,10 +389,16 @@ static int header_check_jpg(const unsigned char *buffer, const unsigned int buff
     unsigned int height=0;
     jpg_get_size(buffer, buffer_size, &height, &width);
     if(file_recovery->file_stat->file_hint==&file_hint_indd)
-      return 0;
+    {
+      if(header_ignored_adv(file_recovery, file_recovery_new)==0)
+	return 0;
+    }
     if(file_recovery->file_stat->file_hint==&file_hint_doc &&
       strstr(file_recovery->filename, ".albm")!=NULL)
-      return 0;
+    {
+      if(header_ignored_adv(file_recovery, file_recovery_new)==0)
+	return 0;
+    }
     if( file_recovery->file_stat->file_hint==&file_hint_jpg)
     {
       /* Don't recover the thumb instead of the jpg itself */
@@ -396,43 +408,77 @@ static int header_check_jpg(const unsigned char *buffer, const unsigned int buff
 	log_info("jpg %llu %llu\n",
 	    (long long unsigned)file_recovery->calculated_file_size,
 	    (long long unsigned)file_recovery->file_size);
-	return 0;
+	if(header_ignored_adv(file_recovery, file_recovery_new)==0)
+	  return 0;
       }
       /* Don't recover the thumb instead of the jpg itself */
       if(file_recovery->file_size <= 4096 &&
 	  buffer[3]==0xe0 &&
 	  width>0 && width<200 && height>0 && height<200)
       {
-	return 0;
+	if(header_ignored_adv(file_recovery, file_recovery_new)==0)
+	  return 0;
       }
       /* Some JPG have two APP1 markers, avoid to dicard the first one */
       if( buffer[3]==0xe1	&&
 	  memcmp(&buffer[6], "http://ns.adobe.com/xap/", 24)==0)
       {
-	return 0;
+	if(header_ignored_adv(file_recovery, file_recovery_new)==0)
+	  return 0;
       }
       if(file_recovery->file_check==&file_check_mpo)
-	return 0;
+      {
+	if(header_ignored_adv(file_recovery, file_recovery_new)==0)
+	  return 0;
+      }
     }
     /* Don't extract jpg inside AVI */
     if( file_recovery->file_stat->file_hint==&file_hint_riff &&
 	(memcmp(buffer,  jpg_header_app0_avi, sizeof(jpg_header_app0_avi))==0 ||
 	 file_recovery->data_check == &data_check_avi_stream))
+    {
+      header_ignored(file_recovery_new);
       return 0;
+    }
     /* Don't extract jpg inside MOV */
     if( file_recovery->file_stat->file_hint==&file_hint_mov &&
 	memcmp(buffer,  jpg_header_app0_jfif11_null, sizeof(jpg_header_app0_jfif11_null))==0)
+    {
+      header_ignored(file_recovery_new);
       return 0;
+    }
+    /* Don't extract jpg inside rw2 */
+    if( file_recovery->file_stat->file_hint==&file_hint_rw2 &&
+      file_recovery->file_size <= 8192)
+    {
+      if(header_ignored_adv(file_recovery, file_recovery_new)==0)
+	return 0;
+    }
     if(buffer[3]==0xdb)	/* DQT */
+    {
+      header_ignored(file_recovery_new);
       return 0;
+    }
     if(buffer[3]==0xc4)	/* DHT - needed to recover .cr2 */
+    {
+      header_ignored(file_recovery_new);
       return 0;
+    }
     if(buffer[3]==0xe0 && (buffer[6]!='J' || buffer[7]!='F'))	/* Should be JFIF/JFXX */
+    {
+      header_ignored(file_recovery_new);
       return 0;
+    }
     if(buffer[3]==0xe1 && (buffer[6]!='E' || buffer[7]!='x' || buffer[8]!='i'|| buffer[9]!='f'))	/* Should be EXIF */
+    {
+      header_ignored(file_recovery_new);
       return 0;
+    }
     if(buffer[3]==0xfe && (!isprint(buffer[6]) || !isprint(buffer[7])))
+    {
+      header_ignored(file_recovery_new);
       return 0;
+    }
   }
   while(i+4<buffer_size && buffer[i]==0xff && is_marker_valid(buffer[i+1]))
   {
@@ -723,11 +769,11 @@ static void jpeg_testdisk_src (j_decompress_ptr cinfo, FILE * infile, uint64_t o
   }
 
   src = (my_source_mgr *) cinfo->src;
-  src->pub.init_source = jpg_init_source;
-  src->pub.fill_input_buffer = jpg_fill_input_buffer;
-  src->pub.skip_input_data = jpg_skip_input_data;
-  src->pub.resync_to_restart = jpeg_resync_to_restart; /* use default method */
-  src->pub.term_source = jpg_term_source;
+  src->pub.init_source = &jpg_init_source;
+  src->pub.fill_input_buffer = &jpg_fill_input_buffer;
+  src->pub.skip_input_data = &jpg_skip_input_data;
+  src->pub.resync_to_restart = &jpeg_resync_to_restart; /* use default method */
+  src->pub.term_source = &jpg_term_source;
   src->pub.bytes_in_buffer = 0; /* forces fill_input_buffer on first read */
   src->pub.next_input_byte = NULL; /* until buffer loaded */
   src->infile = infile;
@@ -794,7 +840,7 @@ static void jpeg_session_start(struct jpeg_session_struct *jpeg_session)
   {
     log_critical("jpeg_session_start: fseek failed.\n");
   }
-  jpeg_create_decompress(&jpeg_session->cinfo); 
+  jpeg_create_decompress(&jpeg_session->cinfo);
   jpeg_testdisk_src(&jpeg_session->cinfo, jpeg_session->handle, jpeg_session->offset, jpeg_session->blocksize);
   (void) jpeg_read_header(&jpeg_session->cinfo, TRUE);
   jpeg_session->cinfo.two_pass_quantize = FALSE;
@@ -828,10 +874,10 @@ static uint64_t jpg_xy_to_offset(FILE *infile, const unsigned int x, const unsig
       x, y, (long unsigned)offset_rel1, (long unsigned)offset_rel2);
 #endif
   jpeg_session.cinfo.err = jpeg_std_error(&jerr.pub);
-  jerr.pub.output_message = my_output_message;
-  jerr.pub.error_exit = my_error_exit;
+  jerr.pub.output_message = &my_output_message;
+  jerr.pub.error_exit = &my_error_exit;
   /* Establish the setjmp return context for my_error_exit to use. */
-  if (setjmp(jerr.setjmp_buffer)) 
+  if (setjmp(jerr.setjmp_buffer))
   {
     if(jpeg_session.frame!=NULL && jpeg_session.cinfo.output_scanline >= y)
     {
@@ -1009,8 +1055,15 @@ static unsigned int jpg_find_border(const unsigned int output_scanline, const un
 #define JPG_MAX_OFFSETS	10240
 
 /* FIXME: it doesn handle correctly when there is a few extra sectors */
-static uint64_t jpg_find_error(FILE *handle, const unsigned int output_scanline, const unsigned int output_width, const unsigned int output_components, const unsigned char *frame, const unsigned int *offsets, const uint64_t offset, const unsigned int blocksize, const uint64_t checkpoint_offset)
+static uint64_t jpg_find_error(struct jpeg_session_struct *jpeg_session, const unsigned int *offsets, const uint64_t checkpoint_offset)
 {
+  FILE *handle                         = jpeg_session->handle;
+  const unsigned int output_scanline   = jpeg_session->cinfo.output_scanline;
+  const unsigned int output_width      = jpeg_session->output_width;
+  const unsigned int output_components = jpeg_session->output_components;
+  //const unsigned int blocksize         = jpeg_session->blocksize;
+  const unsigned char *frame           = jpeg_session->frame;
+  const uint64_t offset                = jpeg_session->offset;
   const unsigned int row_stride = output_width * output_components;
   unsigned int result=0;
   unsigned int result_max=0;
@@ -1022,6 +1075,8 @@ static uint64_t jpg_find_error(FILE *handle, const unsigned int output_scanline,
   unsigned int output_scanline_max;
   if(output_scanline/8 >= JPG_MAX_OFFSETS)
     return 0;
+  if(jpeg_session->output_height < 10)
+    return 0;
   output_scanline_max=jpg_find_border(output_scanline, output_width, output_components, frame);
   for(i = 0, pos_new= 8 * row_stride;
       i < row_stride;
@@ -1102,14 +1157,14 @@ static uint64_t jpg_check_thumb(FILE *infile, const uint64_t offset, const unsig
   jpeg_session.offset=offset;
   jpeg_session.blocksize=blocksize;
   jpeg_session.cinfo.err = jpeg_std_error(&jerr.pub);
-  jerr.pub.output_message = my_output_message;
-  jerr.pub.error_exit = my_error_exit;
-  jerr.pub.emit_message= my_emit_message;
+  jerr.pub.output_message = &my_output_message;
+  jerr.pub.error_exit = &my_error_exit;
+  jerr.pub.emit_message= &my_emit_message;
 #ifdef DEBUG_JPEG
   jerr.pub.trace_level= 3;
 #endif
   /* Establish the setjmp return context for my_error_exit to use. */
-  if (setjmp(jerr.setjmp_buffer)) 
+  if (setjmp(jerr.setjmp_buffer))
   {
     /* If we get here, the JPEG code has signaled an error.
      * We need to clean up the JPEG object and return.
@@ -1120,7 +1175,7 @@ static uint64_t jpg_check_thumb(FILE *infile, const uint64_t offset, const unsig
     offset_error=jpeg_session.offset + src->file_size - src->pub.bytes_in_buffer;
     if(jpeg_session.frame!=NULL && jpeg_session.flags!=0)
     {
-      const uint64_t tmp=jpg_find_error(jpeg_session.handle, jpeg_session.cinfo.output_scanline, jpeg_session.output_width, jpeg_session.output_components, jpeg_session.frame, &offsets[0], jpeg_session.offset, blocksize, checkpoint_offset);
+      const uint64_t tmp=jpg_find_error(&jpeg_session, &offsets[0], checkpoint_offset);
 //      log_info("jpg_check_thumb jpeg corrupted near   %llu\n", offset_error);
       if(tmp !=0 && offset_error > tmp)
 	offset_error=tmp;
@@ -1131,9 +1186,9 @@ static uint64_t jpg_check_thumb(FILE *infile, const uint64_t offset, const unsig
   }
   memset(offsets, 0, sizeof(offsets));
   jpeg_session_start(&jpeg_session);
-  jpeg_session.frame = (unsigned char*)MALLOC(jpeg_session.output_height * jpeg_session.row_stride);
+  jpeg_session.frame = (unsigned char*)MALLOC((jpeg_session.output_height+1) * jpeg_session.row_stride);
   /* 0x100/2=0x80, medium value */
-  memset(jpeg_session.frame, 0x80, jpeg_session.row_stride * jpeg_session.cinfo.output_height);
+  memset(jpeg_session.frame, 0x80, jpeg_session.row_stride * (jpeg_session.cinfo.output_height+1));
   while (jpeg_session.cinfo.output_scanline < jpeg_session.cinfo.output_height)
   {
     JSAMPROW row_pointer[1];
@@ -1169,14 +1224,14 @@ static void jpg_check_picture(file_recovery_t *file_recovery)
   }
   jpeg_session.handle=file_recovery->handle;
   jpeg_session.cinfo.err = jpeg_std_error(&jerr.pub);
-  jerr.pub.output_message = my_output_message;
-  jerr.pub.error_exit = my_error_exit;
-  jerr.pub.emit_message= my_emit_message;
+  jerr.pub.output_message = &my_output_message;
+  jerr.pub.error_exit = &my_error_exit;
+  jerr.pub.emit_message= &my_emit_message;
 #ifdef DEBUG_JPEG
   jerr.pub.trace_level= 3;
 #endif
   /* Establish the setjmp return context for my_error_exit to use. */
-  if (setjmp(jerr.setjmp_buffer)) 
+  if (setjmp(jerr.setjmp_buffer))
   {
     /* If we get here, the JPEG code has signaled an error.
      * We need to clean up the JPEG object and return.
@@ -1198,7 +1253,7 @@ static void jpg_check_picture(file_recovery_t *file_recovery)
 #if 1
     if(jpeg_session.frame!=NULL && jpeg_session.flags!=0)
     {
-      const uint64_t offset_error=jpg_find_error(jpeg_session.handle, jpeg_session.cinfo.output_scanline, jpeg_session.output_width, jpeg_session.output_components, jpeg_session.frame, &offsets[0], jpeg_session.offset, jpeg_session.blocksize, file_recovery->checkpoint_offset);
+      const uint64_t offset_error=jpg_find_error(&jpeg_session, &offsets[0], file_recovery->checkpoint_offset);
       if(offset_error !=0 && file_recovery->offset_error > offset_error)
 	file_recovery->offset_error=offset_error;
 #ifdef DEBUG_JPEG
@@ -1294,7 +1349,7 @@ static int jpg_check_dht(const unsigned char *buffer, const unsigned int buffer_
       return 2;
     j++;
     for(l=0; l < 16; l++)
-      if(j < buffer_size)
+      if(j+l < buffer_size)
 	sum+=buffer[j+l];
     if(sum>255)
       return 2;
@@ -1317,7 +1372,7 @@ struct sof_header
   unsigned char data[0];
 } __attribute__ ((gcc_struct, __packed__));
 
-static int jpg_check_sof0(const unsigned char *buffer, const unsigned int buffer_size, const unsigned i)
+static int jpg_check_sof0(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int i)
 {
   const struct sof_header *h=(const struct sof_header *)&buffer[i];
   if(i+4 > buffer_size)
@@ -1426,7 +1481,7 @@ static uint64_t jpg_check_structure(file_recovery_t *file_recovery, const unsign
 #ifdef DEBUG_JPEG
       log_info("%s marker 0x%02x at 0x%x\n", file_recovery->filename, buffer[i+1], i);
 #endif
-      offset+=2+size;
+      offset+=(uint64_t)2+size;
       if(buffer[i+1]==0xda)	/* SOS: Start Of Scan */
       {
 	file_recovery->offset_ok=i+1;
@@ -1611,6 +1666,8 @@ static void file_check_jpg(file_recovery_t *file_recovery)
 {
   uint64_t thumb_offset;
   static uint64_t thumb_error=0;
+  if(file_recovery->calculated_file_size<=2)
+    file_recovery->calculated_file_size=0;
   /* FIXME REMOVE ME */
   file_recovery->flags=1;
   file_recovery->file_size=0;
@@ -1714,6 +1771,13 @@ static data_check_t data_check_jpg2(const unsigned char *buffer, const unsigned
 	old_marker=buffer[i];
 #endif
       }
+      else if(buffer[i] == 0xda || buffer[i] == 0xc4)
+      {
+	/* SOS and DHT may be embedded by progressive jpg */
+	file_recovery->data_check=NULL;
+	file_recovery->calculated_file_size=0;
+	return DC_CONTINUE;
+      }
       else if(buffer[i]!=0x00)
       {
 #ifdef DEBUG_JPEG
@@ -1744,12 +1808,12 @@ data_check_t data_check_jpg(const unsigned char *buffer, const unsigned int buff
     else if(buffer[i]==0xFF)
     {
       const unsigned int size=(buffer[i+2]<<8)+buffer[i+3];
-#if 0
+#ifdef DEBUG_JPEG
       log_info("data_check_jpg %02x%02x at %llu, next expected at %llu\n", buffer[i], buffer[i+1],
 	  (long long unsigned)file_recovery->calculated_file_size,
 	  (long long unsigned)file_recovery->calculated_file_size+2+size);
 #endif
-      file_recovery->calculated_file_size+=2+size;
+      file_recovery->calculated_file_size+=(uint64_t)2+size;
       if(buffer[i+1]==0xc0)	/* SOF0 */
       {
 	if(jpg_check_sof0(buffer, buffer_size, i)!=0)
@@ -1771,7 +1835,7 @@ data_check_t data_check_jpg(const unsigned char *buffer, const unsigned int buff
 	    buffer[i+4]=='M' && buffer[i+5]=='P' && buffer[i+6]=='F' && buffer[i+7]==0)
 	{
 	  unsigned int size_test=size;
-	  if(i + 2 + size_test >= buffer_size)
+	  if(i + 2 + size >= buffer_size)
 	  {
 	    size_test=buffer_size-i-2;
 	  }
@@ -1783,17 +1847,7 @@ data_check_t data_check_jpg(const unsigned char *buffer, const unsigned int buff
 	  if(size>12)
 	  {
 	    const uint64_t offset=file_recovery->calculated_file_size-(2+size)+8;
-	    const unsigned char *mpo=buffer+i+8;
-	    uint64_t calculated_file_size=0;
-	    size_test-=8;
-	    if(mpo[0]=='I' && mpo[1]=='I' && mpo[2]=='*' && mpo[3]==0)
-	    {
-	      calculated_file_size=check_mpo_le(mpo, offset, size-8);
-	    }
-	    else if(mpo[0]=='M' && mpo[1]=='M' && mpo[2]==0 && mpo[3]=='*')
-	    {
-	      calculated_file_size=check_mpo_be(mpo, offset, size-8);
-	    }
+	    const uint64_t calculated_file_size=check_mpo(buffer+i+8, offset, size_test-8);
 	    if(calculated_file_size > 0)
 	    {
 	      /* Multi-picture format */
