@@ -30,19 +30,12 @@
 #include "types.h"
 #include "filegen.h"
 
-#if defined(HAVE_FSEEKO) && !defined(__MINGW32__)
-#define my_fseek fseeko
-#else
-#define my_fseek fseek
-#endif
-
 static void register_header_check_m2ts(file_stat_t *file_stat);
 static void register_header_check_ts(file_stat_t *file_stat);
 
 const file_hint_t file_hint_m2ts= {
   .extension="m2ts",
   .description="Blu-ray MPEG-2",
-  .min_header_distance=0,
   .max_filesize=PHOTOREC_MAX_FILE_SIZE,
   .recover=1,
   .enable_by_default=1,
@@ -52,7 +45,6 @@ const file_hint_t file_hint_m2ts= {
 const file_hint_t file_hint_ts= {
   .extension="ts",
   .description="MPEG transport stream (TS)",
-  .min_header_distance=0,
   .max_filesize=PHOTOREC_MAX_FILE_SIZE,
   .recover=1,
   .enable_by_default=0,
@@ -77,13 +69,13 @@ static data_check_t data_check_ts_192(const unsigned char *buffer, const unsigne
   return DC_CONTINUE;
 }
 
-static void file_rename_ts_188(const char *old_filename)
+static void file_rename_ts_188(file_recovery_t *file_recovery)
 {
   FILE *file;
   unsigned char buffer[188];
   char buffer_pid[32];
   unsigned int pid;
-  if((file=fopen(old_filename, "rb"))==NULL)
+  if((file=fopen(file_recovery->filename, "rb"))==NULL)
     return;
   if(my_fseek(file, 0, SEEK_SET) < 0 ||
       fread(&buffer, sizeof(buffer), 1, file) != 1)
@@ -94,16 +86,16 @@ static void file_rename_ts_188(const char *old_filename)
   fclose(file);
   pid=((buffer[1]<<8)|buffer[2])&0x1fff;
   sprintf(buffer_pid, "pid_%u", pid);
-  file_rename(old_filename, (const unsigned char*)buffer_pid, strlen(buffer_pid), 0, NULL, 1);
+  file_rename(file_recovery, (const unsigned char*)buffer_pid, strlen(buffer_pid), 0, NULL, 1);
 }
 
-static void file_rename_ts_192(const char *old_filename)
+static void file_rename_ts_192(file_recovery_t *file_recovery)
 {
   FILE *file;
   unsigned char buffer[192];
   char buffer_pid[32];
   unsigned int pid;
-  if((file=fopen(old_filename, "rb"))==NULL)
+  if((file=fopen(file_recovery->filename, "rb"))==NULL)
     return;
   if(my_fseek(file, 0, SEEK_SET) < 0 ||
       fread(&buffer, sizeof(buffer), 1, file) != 1)
@@ -114,22 +106,25 @@ static void file_rename_ts_192(const char *old_filename)
   fclose(file);
   pid=((buffer[5]<<8)|buffer[6])&0x1fff;
   sprintf(buffer_pid, "pid_%u", pid);
-  file_rename(old_filename, (const unsigned char*)buffer_pid, strlen(buffer_pid), 0, NULL, 1);
+  file_rename(file_recovery, (const unsigned char*)buffer_pid, strlen(buffer_pid), 0, NULL, 1);
 }
 
 static int header_check_m2ts(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
 {
   unsigned int i;
-  if(file_recovery->file_stat!=NULL &&
-      file_recovery->file_stat->file_hint==&file_hint_m2ts &&
-      (file_recovery->data_check==&data_check_ts_192 ||
-       file_recovery->blocksize < 5))
-    return 0;
   /* BDAV MPEG-2 transport stream */
   /* Each frame is 192 byte long and begins by a TS_SYNC_BYTE */
   for(i=4; i<buffer_size && buffer[i]==0x47; i+=192);
   if(i<buffer_size)
     return 0;
+  if(file_recovery->file_stat!=NULL &&
+      file_recovery->file_stat->file_hint==&file_hint_m2ts &&
+      (file_recovery->data_check==&data_check_ts_192 ||
+       file_recovery->blocksize < 5))
+  {
+    header_ignored(file_recovery_new);
+    return 0;
+  }
   reset_file_recovery(file_recovery_new);
   if( memcmp(&buffer[0xd7], &buffer[0xe8], 4)==0)
   {
@@ -155,7 +150,7 @@ static int header_check_m2ts(const unsigned char *buffer, const unsigned int buf
     return 1;
   file_recovery_new->calculated_file_size=0;
   file_recovery_new->data_check=&data_check_ts_192;
-  file_recovery_new->file_check=&file_check_size_lax;
+  file_recovery_new->file_check=&file_check_size_max;
   return 1;
 }
 
@@ -190,7 +185,7 @@ static int header_check_m2t(const unsigned char *buffer, const unsigned int buff
   file_recovery_new->min_filesize=188;
   file_recovery_new->calculated_file_size=0;
   file_recovery_new->data_check=&data_check_ts_188;
-  file_recovery_new->file_check=&file_check_size_lax;
+  file_recovery_new->file_check=&file_check_size_max;
   file_recovery_new->file_rename=&file_rename_ts_188;
   return 1;
 }
