@@ -36,14 +36,13 @@ static void register_header_check_mpg(file_stat_t *file_stat);
 const file_hint_t file_hint_mpg= {
   .extension="mpg",
   .description="Moving Picture Experts Group video",
-  .min_header_distance=0,
   .max_filesize=PHOTOREC_MAX_FILE_SIZE,
   .recover=1,
   .enable_by_default=1,
   .register_header_check=&register_header_check_mpg
 };
 
-static int calculate_packet_size(const unsigned char *buffer)
+static unsigned int calculate_packet_size(const unsigned char *buffer)
 {
   /* http://dvd.sourceforge.net/dvdinfo/mpeghdrs.html */
   if(buffer[0]!=0 || buffer[1]!=0 || buffer[2]!=1)
@@ -71,13 +70,13 @@ static int calculate_packet_size(const unsigned char *buffer)
       /* Extension */
     case 0xB5:
       /* Sequence_Extension */
-      if((buffer[4]&0xF0)==0x10 && (buffer[7]&1)==1)	
+      if((buffer[4]&0xF0)==0x10 && (buffer[7]&1)==1)
 	return 10;
       /* Sequence_Display_Extension without color description */
-      if((buffer[4]&0xF0)==0x20 && (buffer[4]&1)==0 && (buffer[6]&2)==2)	
+      if((buffer[4]&0xF0)==0x20 && (buffer[4]&1)==0 && (buffer[6]&2)==2)
 	return 9;
       /* Sequence_Display_Extension with color description */
-      if((buffer[4]&0xF0)==0x20 && (buffer[4]&1)==1 && (buffer[9]&2)==2)	
+      if((buffer[4]&0xF0)==0x20 && (buffer[4]&1)==1 && (buffer[9]&2)==2)
 	return 12;
       /* Picture_Coding_Extension */
       if((buffer[4]&0xF0)==0x40)
@@ -134,7 +133,6 @@ static int calculate_packet_size(const unsigned char *buffer)
   }
 }
 
-
 static data_check_t data_check_mpg(const unsigned char *buffer, const unsigned int buffer_size, file_recovery_t *file_recovery)
 {
   while(file_recovery->calculated_file_size + buffer_size/2  >= file_recovery->file_size &&
@@ -152,6 +150,17 @@ static data_check_t data_check_mpg(const unsigned char *buffer, const unsigned i
   return DC_CONTINUE;
 }
 
+static int header_mpg_found(file_recovery_t *file_recovery_new)
+{
+  reset_file_recovery(file_recovery_new);
+  file_recovery_new->extension=file_hint_mpg.extension;
+  if(file_recovery_new->blocksize < 14)
+    return 1;
+  file_recovery_new->data_check=&data_check_mpg;
+  file_recovery_new->file_check=&file_check_size;
+  return 1;
+}
+
 static int header_check_mpg_Pack(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
 {
   unsigned int i=0;
@@ -169,23 +178,14 @@ static int header_check_mpg_Pack(const unsigned char *buffer, const unsigned int
   {
     if(buffer[5]==0 && buffer[6]==1 && buffer[7]==0 && buffer[8]==1)
     {
-      reset_file_recovery(file_recovery_new);
-      file_recovery_new->extension=file_hint_mpg.extension;
-      if(file_recovery_new->blocksize < 14)
-	return 1;
-      file_recovery_new->data_check=&data_check_mpg;
-      file_recovery_new->file_check=&file_check_size;
-      return 1;
+      return header_mpg_found(file_recovery_new);
     }
     if(file_recovery->file_stat!=NULL && file_recovery->file_stat->file_hint==&file_hint_mpg)
+    {
+      header_ignored(file_recovery_new);
       return 0;
-    reset_file_recovery(file_recovery_new);
-    file_recovery_new->extension=file_hint_mpg.extension;
-      if(file_recovery_new->blocksize < 14)
-	return 1;
-    file_recovery_new->data_check=&data_check_mpg;
-    file_recovery_new->file_check=&file_check_size;
-    return 1;
+    }
+    return header_mpg_found(file_recovery_new);
   }
   /* MPEG-2 Program stream http://neuron2.net/library/mpeg2/iso13818-1.pdf */
   /* MPEG2 system header start code, several per file */
@@ -212,31 +212,20 @@ static int header_check_mpg_Pack(const unsigned char *buffer, const unsigned int
 
     if(buffer[4]==0x44 && buffer[5]==0 && buffer[6]==4 && buffer[7]==0 && (buffer[8]&0xfc)==4)
     { /* SCR=0 */
-      reset_file_recovery(file_recovery_new);
-      file_recovery_new->extension=file_hint_mpg.extension;
-      if(file_recovery_new->blocksize < 14)
-	return 1;
-      file_recovery_new->data_check=&data_check_mpg;
-      file_recovery_new->file_check=&file_check_size;
-      return 1;
+      return header_mpg_found(file_recovery_new);
     }
     if(file_recovery->file_stat!=NULL && file_recovery->file_stat->file_hint==&file_hint_mpg)
+    {
+      header_ignored(file_recovery_new);
       return 0;
-    reset_file_recovery(file_recovery_new);
-    file_recovery_new->extension=file_hint_mpg.extension;
-    if(file_recovery_new->blocksize < 14)
-      return 1;
-    file_recovery_new->data_check=&data_check_mpg;
-    file_recovery_new->file_check=&file_check_size;
-    return 1;
+    }
+    return header_mpg_found(file_recovery_new);
   }
   return 0;
 }
 
 static int header_check_mpg_System(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
 {
-  if(file_recovery->file_stat!=NULL && file_recovery->file_stat->file_hint==&file_hint_mpg)
-    return 0;
   /* MPEG-1 http://andrewduncan.ws/MPEG/MPEG-1.ps */
   /*   ISO/IEC INTERNATIONAL 13818-1 STANDARD
   	system_header_start_code           32
@@ -266,19 +255,18 @@ static int header_check_mpg_System(const unsigned char *buffer, const unsigned i
 	return 0;
       i+=ret;
     }
-    reset_file_recovery(file_recovery_new);
-    file_recovery_new->extension=file_hint_mpg.extension;
-    file_recovery_new->data_check=&data_check_mpg;
-    file_recovery_new->file_check=&file_check_size;
-    return 1;
+    if(file_recovery->file_stat!=NULL && file_recovery->file_stat->file_hint==&file_hint_mpg)
+    {
+      header_ignored(file_recovery_new);
+      return 0;
+    }
+    return header_mpg_found(file_recovery_new);
   }
   return 0;
 }
 
 static int header_check_mpg_Sequence(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
 {
-  if(file_recovery->file_stat!=NULL && file_recovery->file_stat->file_hint==&file_hint_mpg)
-    return 0;
   /* MPEG-1 sequence header code 0x1B3 */
   /* horizontal size>0 */
   if((buffer[4]<<4)+(buffer[5]>>4)>0 &&
@@ -301,19 +289,18 @@ static int header_check_mpg_Sequence(const unsigned char *buffer, const unsigned
 	return 0;
       i+=ret;
     }
-    reset_file_recovery(file_recovery_new);
-    file_recovery_new->extension=file_hint_mpg.extension;
-    file_recovery_new->data_check=&data_check_mpg;
-    file_recovery_new->file_check=&file_check_size;
-    return 1;
+    if(file_recovery->file_stat!=NULL && file_recovery->file_stat->file_hint==&file_hint_mpg)
+    {
+      header_ignored(file_recovery_new);
+      return 0;
+    }
+    return header_mpg_found(file_recovery_new);
   }
   return 0;
 }
 
 static int header_check_mpg4_ElemVideo(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
 {
-  if(file_recovery->file_stat!=NULL && file_recovery->file_stat->file_hint==&file_hint_mpg)
-    return 0;
   /* ISO/IEC 14496-2 (MPEG-4 video) ELEMENTARY VIDEO HEADER - visual object start code */
   /* is_visual_object_identifier */
   if((buffer[4]&0xf0)==0x80 &&
@@ -333,11 +320,12 @@ static int header_check_mpg4_ElemVideo(const unsigned char *buffer, const unsign
 	return 0;
       i+=ret;
     }
-    reset_file_recovery(file_recovery_new);
-    file_recovery_new->extension=file_hint_mpg.extension;
-    file_recovery_new->data_check=&data_check_mpg;
-    file_recovery_new->file_check=&file_check_size;
-    return 1;
+    if(file_recovery->file_stat!=NULL && file_recovery->file_stat->file_hint==&file_hint_mpg)
+    {
+      header_ignored(file_recovery_new);
+      return 0;
+    }
+    return header_mpg_found(file_recovery_new);
   }
   return 0;
 }
