@@ -36,7 +36,6 @@ static void register_header_check_oci(file_stat_t *file_stat);
 const file_hint_t file_hint_oci= {
   .extension="oci",
   .description="OpenCanvas Image",
-  .min_header_distance=0,
   .max_filesize=PHOTOREC_MAX_FILE_SIZE,
   .recover=1,
   .enable_by_default=1,
@@ -60,8 +59,8 @@ static data_check_t data_check_oci(const unsigned char *buffer, const unsigned i
 #ifdef DEBUG_MOV
     log_trace("file_oci.c: %s atom %c%c%c%c (0x%02x%02x%02x%02x) size %llu, calculated_file_size %llu\n",
 	file_recovery->filename,
-        buffer[i],buffer[i+1],buffer[i+2],buffer[i+3], 
-        buffer[i],buffer[i+1],buffer[i+2],buffer[i+3], 
+        buffer[i],buffer[i+1],buffer[i+2],buffer[i+3],
+        buffer[i],buffer[i+1],buffer[i+2],buffer[i+3],
         (long long unsigned)atom_size,
         (long long unsigned)file_recovery->calculated_file_size);
 #endif
@@ -70,7 +69,7 @@ static data_check_t data_check_oci(const unsigned char *buffer, const unsigned i
       (buffer[i+2]>='A' && buffer[i+2]<='Z') &&
       (buffer[i+3]>='A' && buffer[i+3]<='Z'))
     {
-      file_recovery->calculated_file_size+=atom_size+8;
+      file_recovery->calculated_file_size+=(uint64_t)8+atom_size;
     }
     else
     {
