@@ -31,6 +31,7 @@
 #include "common.h"
 #include "log.h"
 
+extern const file_hint_t file_hint_indd;
 static void register_header_check_pct(file_stat_t *file_stat);
 static int header_check_pct(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new);
 static void file_check_pct(file_recovery_t *file_recovery);
@@ -38,14 +39,12 @@ static void file_check_pct(file_recovery_t *file_recovery);
 const file_hint_t file_hint_pct= {
   .extension="pct",
   .description="Macintosh Picture",
-  .min_header_distance=0,
   .max_filesize=PHOTOREC_MAX_FILE_SIZE,
   .recover=1,
   .enable_by_default=1,
   .register_header_check=&register_header_check_pct
 };
 
-static const unsigned char pct_header[6]= { 0x00, 0x11, 0x02, 0xff, 0x0c, 0x00};
 /* We are searching for PICTv2 files
    http://www.fileformat.info/format/macpict/
    SHORT    Version operator (0x0011)
@@ -80,16 +79,10 @@ struct pct_file_entry {
   uint32_t Reserved2;		/* 0x24 */
 } __attribute__ ((gcc_struct, __packed__));
 
-static void register_header_check_pct(file_stat_t *file_stat)
-{
-  register_header_check(0x20a, pct_header,sizeof(pct_header), &header_check_pct, file_stat);
-}
-
 static int header_check_pct(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
 {
   const struct pct_file_entry *pct=(const struct pct_file_entry *)(&buffer[0x200]);
-  if(memcmp(&buffer[0x20a],pct_header,sizeof(pct_header))==0 &&
-      be16(pct->XMin) <= be16(pct->XMax) &&
+  if(be16(pct->XMin) <= be16(pct->XMax) &&
       be16(pct->YMin) <= be16(pct->YMax) &&
       ((be16(pct->OXMin) <= be16(pct->OXMax) &&
 	be16(pct->OYMin) <= be16(pct->OYMax)) ||
@@ -101,6 +94,12 @@ static int header_check_pct(const unsigned char *buffer, const unsigned int buff
 	be16(pct->VersionOperator)==0x0011 &&
 	be16(pct->VersionNumber)==0x02ff)
   {
+    if(file_recovery->file_stat != NULL &&
+	file_recovery->file_stat->file_hint==&file_hint_indd)
+    {
+      if(header_ignored_adv(file_recovery, file_recovery_new)==0)
+	return 0;
+    }
     reset_file_recovery(file_recovery_new);
     file_recovery_new->extension=file_hint_pct.extension;
     /* We only have the low 16bits of the filesystem */
@@ -130,3 +129,8 @@ static void file_check_pct(file_recovery_t *file_recovery)
   file_recovery->file_size-=((file_recovery->file_size-file_recovery->min_filesize)&0xFFFF);
 }
 
+static void register_header_check_pct(file_stat_t *file_stat)
+{
+  static const unsigned char pct_header[6]= { 0x00, 0x11, 0x02, 0xff, 0x0c, 0x00};
+  register_header_check(0x20a, pct_header,sizeof(pct_header), &header_check_pct, file_stat);
+}
