@@ -39,19 +39,12 @@
 #include "memmem.h"
 #include "common.h"
 
-#if defined(HAVE_FSEEKO) && !defined(__MINGW32__)
-#define my_fseek fseeko
-#else
-#define my_fseek fseek
-#endif
-
 static void register_header_check_pdf(file_stat_t *file_stat);
 static void file_date_pdf(file_recovery_t *file_recovery);
 
 const file_hint_t file_hint_pdf= {
   .extension="pdf",
   .description="Portable Document Format, Adobe Illustrator",
-  .min_header_distance=0,
   .max_filesize=PHOTOREC_MAX_FILE_SIZE,
   .recover=1,
   .enable_by_default=1,
@@ -69,7 +62,7 @@ static int hex(int c)
   return -1;
 }
 
-static void file_rename_pdf(const char *old_filename)
+static void file_rename_pdf(file_recovery_t *file_recovery)
 {
   char title[512];
   const unsigned char pattern[6]={ '/', 'T', 'i', 't', 'l', 'e' };
@@ -80,7 +73,7 @@ static void file_rename_pdf(const char *old_filename)
   unsigned int j;
   size_t bsize;
   const unsigned char utf16[3]= { 0xfe, 0xff, 0x00};
-  if((handle=fopen(old_filename, "rb"))==NULL)
+  if((handle=fopen(file_recovery->filename, "rb"))==NULL)
     return;
   if(my_fseek(handle, 0, SEEK_END)<0)
   {
@@ -187,7 +180,7 @@ static void file_rename_pdf(const char *old_filename)
       (memcmp(&title[j-5], ".docx", 5)==0 ||
        memcmp(&title[j-5], ".xlsx", 5)==0))
     j-=5;
-  file_rename(old_filename, title, j, 0, NULL, 1);
+  file_rename(file_recovery, title, j, 0, NULL, 1);
   free(buffer);
 }
 
@@ -252,38 +245,18 @@ static void file_date_pdf(file_recovery_t *file_recovery)
       {
 	if(++j==sizeof(pattern))
 	{
-	  const unsigned char *date_asc;
-	  struct tm tm_time;
-	  if(my_fseek(file_recovery->handle, offset+i+1, SEEK_SET)<0)
+	  if(my_fseek(file_recovery->handle, offset+i+1, SEEK_SET)>=0 &&
+	      fread(buffer, 1, 22, file_recovery->handle) == 22)
 	  {
-	    free(buffer);
-	    return ;
+	    if(buffer[0]=='=' && (buffer[1]=='\'' || buffer[1]=='"'))
+	    {
+	      file_recovery->time=get_time_from_YYYY_MM_DD_HH_MM_SS((const char *)&buffer[2]);
+	    }
+	    else if(buffer[0]=='>')
+	    {
+	      file_recovery->time=get_time_from_YYYY_MM_DD_HH_MM_SS((const char *)&buffer[1]);
+	    }
 	  }
-	  if(fread(buffer, 1, 22, file_recovery->handle) < 22)
-	  {
-	    free(buffer);
-	    return ;
-	  }
-	  if(buffer[0]=='=' && (buffer[1]=='\'' || buffer[1]=='"'))
-	    date_asc=&buffer[2];
-	  else if(buffer[i]=='>')
-	    date_asc=&buffer[1];
-	  else
-	  {
-	    free(buffer);
-	    return ;
-	  }
-	  /* */
-	  memset(&tm_time, 0, sizeof(tm_time));
-	  tm_time.tm_sec=(date_asc[17]-'0')*10+(date_asc[18]-'0');      /* seconds 0-59 */
-	  tm_time.tm_min=(date_asc[14]-'0')*10+(date_asc[15]-'0');      /* minutes 0-59 */
-	  tm_time.tm_hour=(date_asc[11]-'0')*10+(date_asc[12]-'0');     /* hours   0-23*/
-	  tm_time.tm_mday=(date_asc[8]-'0')*10+(date_asc[9]-'0');	/* day of the month 1-31 */
-	  tm_time.tm_mon=(date_asc[5]-'0')*10+(date_asc[6]-'0')-1;	/* month 0-11 */
-	  tm_time.tm_year=(date_asc[0]-'0')*1000+(date_asc[1]-'0')*100+
-	    (date_asc[2]-'0')*10+(date_asc[3]-'0')-1900;        	/* year */
-	  tm_time.tm_isdst = -1;		/* unknown daylight saving time */
-	  file_recovery->time=mktime(&tm_time);
 	  free(buffer);
 	  return ;
 	}
