@@ -39,7 +39,6 @@ static data_check_t data_check_ps(const unsigned char *buffer, const unsigned in
 const file_hint_t file_hint_ps= {
   .extension="ps",
   .description="PostScript or Encapsulated PostScript document",
-  .min_header_distance=0,
   .max_filesize=PHOTOREC_MAX_FILE_SIZE,
   .recover=1,
   .enable_by_default=1,
@@ -55,36 +54,33 @@ static void register_header_check_ps(file_stat_t *file_stat)
 
 static int header_check_ps(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
 {
-  if(memcmp(buffer,ps_header,sizeof(ps_header))==0)
-  { /* PS or EPSF */
-    int i;
-    for(i=sizeof(ps_header);i<20;i++)
+  /* PS or EPSF */
+  int i;
+  for(i=sizeof(ps_header);i<20;i++)
+  {
+    switch(buffer[i])
     {
-      switch(buffer[i])
-      {
-	case '\n':
+      case '\n':
+	reset_file_recovery(file_recovery_new);
+	file_recovery_new->extension=file_hint_ps.extension;
+	file_recovery_new->data_check=&data_check_ps;
+	file_recovery_new->file_check=&file_check_size;
+	return 1;
+      case 'E':
+	if(memcmp(&buffer[i],"EPSF-",5)==0)
+	{
 	  reset_file_recovery(file_recovery_new);
-	  file_recovery_new->extension=file_hint_ps.extension;
-	  file_recovery_new->data_check=&data_check_ps;
-	  file_recovery_new->file_check=&file_check_size;
+	  file_recovery_new->extension="eps";
 	  return 1;
-	case 'E':
-	  if(memcmp(&buffer[i],"EPSF-",5)==0)
-	  {
-	    reset_file_recovery(file_recovery_new);
-	    file_recovery_new->extension="eps";
-	    return 1;
-	  }
-	  break;
-      }
+	}
+	break;
     }
-    reset_file_recovery(file_recovery_new);
-    file_recovery_new->extension=file_hint_ps.extension;
-    file_recovery_new->data_check=&data_check_ps;
-    file_recovery_new->file_check=&file_check_size;
-    return 1;
   }
-  return 0;
+  reset_file_recovery(file_recovery_new);
+  file_recovery_new->extension=file_hint_ps.extension;
+  file_recovery_new->data_check=&data_check_ps;
+  file_recovery_new->file_check=&file_check_size;
+  return 1;
 }
 
 static data_check_t data_check_ps(const unsigned char *buffer, const unsigned int buffer_size, file_recovery_t *file_recovery)
