@@ -36,7 +36,6 @@ static void register_header_check_ra(file_stat_t *file_stat);
 const file_hint_t file_hint_ra= {
   .extension="ra",
   .description="Real Audio",
-  .min_header_distance=0,
   .max_filesize=PHOTOREC_MAX_FILE_SIZE,
   .recover=1,
   .enable_by_default=1,
@@ -88,7 +87,7 @@ static int header_check_ra(const unsigned char *buffer, const unsigned int buffe
     const struct ra3_header *ra3=(const struct ra3_header *)buffer;
     reset_file_recovery(file_recovery_new);
     file_recovery_new->extension=file_hint_ra.extension;
-    file_recovery_new->calculated_file_size=8 + be16(ra3->header_size) + be32(ra3->data_size);
+    file_recovery_new->calculated_file_size=(uint64_t)8 + be16(ra3->header_size) + be32(ra3->data_size);
     file_recovery_new->data_check=&data_check_size;
     file_recovery_new->file_check=&file_check_size;
     return 1;
@@ -99,7 +98,7 @@ static int header_check_ra(const unsigned char *buffer, const unsigned int buffe
     const struct ra4_header *ra4=(const struct ra4_header *)buffer;
     reset_file_recovery(file_recovery_new);
     file_recovery_new->extension=file_hint_ra.extension;
-    file_recovery_new->calculated_file_size=40 + be16(ra4->header_size) + be32(ra4->data_size);
+    file_recovery_new->calculated_file_size=(uint64_t)40 + be16(ra4->header_size) + be32(ra4->data_size);
     file_recovery_new->data_check=&data_check_size;
     file_recovery_new->file_check=&file_check_size;
     return 1;
