@@ -32,47 +32,17 @@
 #include "filegen.h"
 #include "common.h"
 
-#if defined(HAVE_FSEEKO) && !defined(__MINGW32__)
-#define my_fseek fseeko
-#else
-#define my_fseek fseek
-#endif
-
 static void register_header_check_spf(file_stat_t *file_stat);
-static int header_check_spf(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new);
-static void file_check_spf(file_recovery_t *file_recovery);
 
 const file_hint_t file_hint_spf= {
   .extension="spf",
   .description="ShadowProtect",
-  .min_header_distance=0,
   .max_filesize=PHOTOREC_MAX_FILE_SIZE,
   .recover=1,
   .enable_by_default=1,
   .register_header_check=&register_header_check_spf
 };
 
-static const unsigned char spf_header[12]= {
-  'S', 'P', 'F', 'I', 0x00, 0x02, 0x00, 0x00, 0x41, 0x00, 0x00, 0x00
-};
-
-static void register_header_check_spf(file_stat_t *file_stat)
-{
-  register_header_check(0, spf_header,sizeof(spf_header), &header_check_spf, file_stat);
-}
-
-static int header_check_spf(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
-{
-  if(memcmp(buffer,spf_header,sizeof(spf_header))==0)
-  {
-    reset_file_recovery(file_recovery_new);
-    file_recovery_new->extension=file_hint_spf.extension;
-    file_recovery_new->file_check=file_check_spf;
-    return 1;
-  }
-  return 0;
-}
-
 enum { READ_SIZE=32*512 };
 
 static void file_check_spf(file_recovery_t *file_recovery)
@@ -112,3 +82,19 @@ static void file_check_spf(file_recovery_t *file_recovery)
     }
   }
 }
+
+static int header_check_spf(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
+{
+  reset_file_recovery(file_recovery_new);
+  file_recovery_new->extension=file_hint_spf.extension;
+  file_recovery_new->file_check=&file_check_spf;
+  return 1;
+}
+
+static void register_header_check_spf(file_stat_t *file_stat)
+{
+  static const unsigned char spf_header[12]= {
+    'S', 'P', 'F', 'I', 0x00, 0x02, 0x00, 0x00, 0x41, 0x00, 0x00, 0x00
+  };
+  register_header_check(0, spf_header,sizeof(spf_header), &header_check_spf, file_stat);
+}
