@@ -44,6 +44,7 @@
 extern const file_hint_t file_hint_doc;
 extern const file_hint_t file_hint_jpg;
 extern const file_hint_t file_hint_pdf;
+extern const file_hint_t file_hint_sld;
 extern const file_hint_t file_hint_tiff;
 extern const file_hint_t file_hint_zip;
 
@@ -52,15 +53,24 @@ static inline int filtre(unsigned int car);
 static void register_header_check_txt(file_stat_t *file_stat);
 static int header_check_txt(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new);
 static void register_header_check_fasttxt(file_stat_t *file_stat);
+static void register_header_check_snz(file_stat_t *file_stat);
 static int header_check_fasttxt(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new);
 #ifdef UTF16
 static int header_check_le16_txt(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new);
 #endif
 
+const file_hint_t file_hint_snz= {
+  .extension="snz",
+  .description="Olfaction SeeNez odorama",
+  .max_filesize=PHOTOREC_MAX_FILE_SIZE,
+  .recover=1,
+  .enable_by_default=1,
+  .register_header_check=&register_header_check_snz
+};
+
 const file_hint_t file_hint_fasttxt= {
   .extension="tx?",
   .description="Text files with header: rtf,xml,xhtml,mbox/imm,pm,ram,reg,sh,slk,stp,jad,url",
-  .min_header_distance=0,
   .max_filesize=PHOTOREC_MAX_FILE_SIZE,
   .recover=1,
   .enable_by_default=1,
@@ -70,7 +80,6 @@ const file_hint_t file_hint_fasttxt= {
 const file_hint_t file_hint_txt= {
   .extension="txt",
   .description="Other text files: txt,html,asp,bat,C,jsp,perl,php,py/emlx... scripts",
-  .min_header_distance=0,
   .max_filesize=PHOTOREC_MAX_FILE_SIZE,
   .recover=1,
   .enable_by_default=1,
@@ -106,6 +115,9 @@ static const txt_header_t fasttxt_headers[] = {
   { "#!/bin/bash", 					11, "sh"},
   { "#!/bin/ksh",					10, "sh"},
   { "#!/bin/sh",					 9, "sh"},
+  { "#! /bin/bash", 					12, "sh"},
+  { "#! /bin/ksh",					11, "sh"},
+  { "#! /bin/sh",					10, "sh"},
   /* Opera Hotlist bookmark/contact list/notes */
   { "Opera Hotlist version 2.0",			25, "adr"},
   /* Microsoft VB Class module */
@@ -119,6 +131,7 @@ static const txt_header_t fasttxt_headers[] = {
   { "TABLE\r\n0,1\r\n",					12, "dif"},
   /* Designer, a Photobook Designer Software */
   { "vSg4q7j8GLrtf",					13, "dp"},
+  { "-----BEGIN DSA PRIVATE KEY-----",			31, "dsa"},
   /* EMKA IOX file */
   { "1\t\t\t\t\tthis file\t", 				16,
 #ifdef DJGPP
@@ -127,14 +140,18 @@ static const txt_header_t fasttxt_headers[] = {
     "emka"
 #endif
   },
+  /* Source code in go language */
+  { "package main",					12, "go"},
   /* ENVI */
   { "ENVI\r\ndescription",				17, "hdr"},
   /* Java Application Descriptor
    * http://en.wikipedia.org/wiki/JAD_%28file_format%29 */
   { "MIDlet-1:",					 9, "jad"},
   { "{\"title\":\"\",\"id\":1,\"dateAdded\":",		31, "json"},
+  { "-----BEGIN RSA PRIVATE KEY-----",			31, "key"},
   /* Lyx http://www.lyx.org */
   { "#LyX 1.", 						 7, "lyx"},
+  { "#LyX 2.", 						 7, "lyx"},
   /* LilyPond http://lilypond.org*/
   { "\n\\version \"", 					11, "ly"},
   /* Moving Picture Experts Group Audio Layer 3 Uniform Resource Locator */
@@ -149,8 +166,14 @@ static const txt_header_t fasttxt_headers[] = {
   { "-- phpMyAdmin SQL Dump",				22, "sql"},
   { "--\n-- PostgreSQL database cluster dump",		38, "sql"},
   { "--\r\n-- PostgreSQL database cluster dump",	39, "sql"},
+  { "---- BEGIN SSH2 PUBLIC KEY ----",			31, "ppk"},
+  { "PuTTY-User-Key-File-2:",				22, "ppk"},
+  { "-----BEGIN PGP PRIVATE KEY BLOCK-----",		37, "priv"},
+  { "-----BEGIN PGP PUBLIC KEY BLOCK-----",		36, "pub"},
   /* PTGui,  panoramic stitching software */
   { "# ptGui project file",				20, "pts"},
+  { "ssh-dss AAAAB3",					14, "pub"},
+  { "ssh-rsa AAAAB3",					14, "pub"},
   /* Quantum GIS  */
   { "<!DOCTYPE qgis ",					15, "qgs"},
   /* Real Media  */
@@ -163,9 +186,6 @@ static const txt_header_t fasttxt_headers[] = {
   { "#SeeNez ",						 8, "SeeNezSST"},
   /* Sylk, Multiplan Symbolic Link Interchange  */
   { "ID;PSCALC3",					10, "slk"},
-  /* Olfaction SeeNez odorama */
-  { "DEFAULT\n",					 8, "snz"},
-  { "DEFAULT\r\n",					 9, "snz"},
   /* ISO 10303 is an ISO standard for the computer-interpretable
    * representation and exchange of industrial product data.
    * - Industrial automation systems and integration - Product data representation and exchange
@@ -190,6 +210,10 @@ static const txt_header_t fasttxt_headers[] = {
   },
   /* Mathlab Model .mdl */
   { "Model {", 7, "mdl"},
+  /* Windows folder settings for file explorer */
+  { "[.ShellClassInfo]",				17, "Desktop.ini" },
+  /* Fotobook */
+  { "<fotobook ",					10, "mcf"}, 
   {NULL, 0, NULL}
 };
 
@@ -226,27 +250,27 @@ static int filtre(unsigned int car)
     case 0xd7:	/* '×' */
     case 0xd9:  /* 'Ù' */
     case 0xdf:	/* 'ß' */
-    case 0xe0: 	/* 'à' */
-    case 0xe1: 	/* 'á' */
-    case 0xe2: 	/* 'â' */
+    case 0xe0:	/* 'à' */
+    case 0xe1:	/* 'á' */
+    case 0xe2:	/* 'â' */
     case 0xe3:  /* 'ã' */
-    case 0xe4: 	/* 'ä' */
+    case 0xe4:	/* 'ä' */
     case 0xe6:  /* 'æ' */
-    case 0xe7: 	/* 'ç' */
-    case 0xe8: 	/* 'è' */
-    case 0xe9: 	/* 'é' */
-    case 0xea: 	/* 'ê' */
-    case 0xeb: 	/* 'ë' */
+    case 0xe7:	/* 'ç' */
+    case 0xe8:	/* 'è' */
+    case 0xe9:	/* 'é' */
+    case 0xea:	/* 'ê' */
+    case 0xeb:	/* 'ë' */
     case 0xed:  /* 'í' */
-    case 0xee: 	/* 'î' */
-    case 0xef: 	/* 'ï' */
-    case 0xf4: 	/* 'ô' */
-    case 0xf6: 	/* 'ö' */
+    case 0xee:	/* 'î' */
+    case 0xef:	/* 'ï' */
+    case 0xf4:	/* 'ô' */
+    case 0xf6:	/* 'ö' */
     case 0xf8:  /* 'ø' */
-    case 0xf9: 	/* 'ù' */
+    case 0xf9:	/* 'ù' */
     case 0xfa:  /* 'ú' */
-    case 0xfb: 	/* 'û' */
-    case 0xfc: 	/* 'ü' */
+    case 0xfb:	/* 'û' */
+    case 0xfc:	/* 'ü' */
       return 1;
   }
   if(car=='\b' || car=='\t' || car=='\r' || car=='\n' ||
@@ -261,10 +285,10 @@ static int filtre(unsigned int car)
    return read size */
 int UTF2Lat(unsigned char *buffer_lower, const unsigned char *buffer, const int buf_len)
 {
-  const unsigned char *p; 	/* pointers to actual position in source buffer */
+  const unsigned char *p;	/* pointers to actual position in source buffer */
   unsigned char *q;	/* pointers to actual position in destination buffer */
   int i; /* counter of remaining bytes available in destination buffer */
-  for (i = buf_len, p = buffer, q = buffer_lower; p-buffer<buf_len && i > 0 && *p!='\0';) 
+  for (i = buf_len, p = buffer, q = buffer_lower; p-buffer<buf_len && i > 0 && *p!='\0';)
   {
     const unsigned char *p_org=p;
     if((*p & 0xf0)==0xe0 && (*(p+1) & 0xc0)==0x80 && (*(p+2) & 0xc0)==0x80)
@@ -275,12 +299,12 @@ int UTF2Lat(unsigned char *buffer_lower, const unsigned char *buffer, const int
       *q = '\0';
       switch (*p)
       {
-        case 0xE2 : 
+        case 0xE2 :
           switch (*(p+1))
-          { 
-            case 0x80 : 
+          {
+            case 0x80 :
               switch (*(p+2))
-              { 
+              {
                 case 0x93 : (*q) = 150; break;
                 case 0x94 : (*q) = 151; break;
                 case 0x98 : (*q) = 145; break;
@@ -299,15 +323,15 @@ int UTF2Lat(unsigned char *buffer_lower, const unsigned char *buffer, const int
                 case 0xBA : (*q) = 155; break;
               }
               break;
-            case 0x82 : 
+            case 0x82 :
               switch (*(p+2))
-              { 
+              {
                 case 0xAC : (*q) = 128; break;
               }
               break;
-            case 0x84 : 
+            case 0x84 :
               switch (*(p+2))
-              { 
+              {
                 case 0xA2 : (*q) = 153; break;
               }
               break;
@@ -321,22 +345,22 @@ int UTF2Lat(unsigned char *buffer_lower, const unsigned char *buffer, const int
       *q = '\0';
       switch (*p)
       {
-        case 0xC2 : 
+        case 0xC2 :
           (*q) = ((*(p+1)) | 0x80) & 0xBF; /* A0-BF and a few 80-9F */
           if((*q)==0xA0)
             (*q)=' ';
           break;
-        case 0xC3 : 
+        case 0xC3 :
           switch (*(p+1))
-	  { 
+	  {
 	    case 0xB3 : (*q) = 162; break;
 	    default:
 			(*q) = (*(p+1)) | 0xC0; /* C0-FF */
 			break;
 	  }
           break;
-        case 0xC5 : 
-          switch (*(p+1)) { 
+        case 0xC5 :
+          switch (*(p+1)) {
             case 0x92 : (*q) = 140; break;
             case 0x93 : (*q) = 156; break;
             case 0xA0 : (*q) = 138; break;
@@ -346,13 +370,13 @@ int UTF2Lat(unsigned char *buffer_lower, const unsigned char *buffer, const int
             case 0xBE : (*q) = 158; break;
           }
           break;
-        case 0xC6: 
-          switch (*(p+1)) { 
+        case 0xC6:
+          switch (*(p+1)) {
             case 0x92 : (*q) = 131; break;
           }
           break;
-        case 0xCB : 
-          switch (*(p+1)) { 
+        case 0xCB :
+          switch (*(p+1)) {
             case 0x86 : (*q) = 136; break;
             case 0x9C : (*q) = 152; break;
           }
@@ -365,7 +389,8 @@ int UTF2Lat(unsigned char *buffer_lower, const unsigned char *buffer, const int
 #ifdef DEBUG_TXT
       log_info("UTF8 Ascii UCS 0x%02x\n", *p);
 #endif
-      *q = tolower(*p++);
+      *q = tolower(*p);
+      p++;
     }
     if (*q=='\0' || filtre(*q)==0)
     {
@@ -384,38 +409,70 @@ int UTF2Lat(unsigned char *buffer_lower, const unsigned char *buffer, const int
 
 static int UTFsize(const unsigned char *buffer, const unsigned int buf_len)
 {
-  const unsigned char *p=buffer; 	/* pointers to actual position in source buffer */
+  const unsigned char *p=buffer;	/* pointers to actual position in source buffer */
   unsigned int i=0;
   while(i<buf_len && *p!='\0')
   {
     /* Reject some invalid UTF-8 sequences */
     if(*p==0xc0 || *p==0xc1 || *p==0xf7 || *p>=0xfd)
       return i;
-    if((*p & 0xf0)==0xe0 && i+3 <= buf_len && (*(p+1) & 0xc0)==0x80 && (*(p+2) & 0xc0)==0x80)
+    if((*p & 0xf0)==0xe0 && (i+2 >= buf_len || ((*(p+1) & 0xc0)==0x80 && (*(p+2) & 0xc0)==0x80)))
     { /* UTF8 l=3 */
-      const unsigned int car=(((*p)&0x1f)<<12) | (((*(p+1))&0x3f)<<6) | ((*(p+2))&0x3f);
-      if(filtre(car)==0)
-	return i;
+#ifdef DEBUG_TXT
+      log_info("UTFsize i=%u l=3\n", i);
+#endif
       p+=3;
       i+=3;
     }
-    else if((*p & 0xe0)==0xc0 && i+2 <= buf_len && (*(p+1) & 0xc0)==0x80)
+    else if((*p & 0xe0)==0xc0 && (i+1 >= buf_len || (*(p+1) & 0xc0)==0x80))
     { /* UTF8 l=2 */
-      const unsigned int car=(((*p)&0x1f)<<6) | ((*(p+1))&0x3f);
-      if(filtre(car)==0)
-	return i;
+#ifdef DEBUG_TXT
+      log_info("UTFsize i=%u l=2\n", i);
+#endif
       p+=2;
       i+=2;
     }
     else
     { /* Ascii UCS */
-      if(filtre(*p)==0)
-	return i;
+#ifdef DEBUG_TXT
+      log_info("UTFsize i=%u l=1 ? *p=%c\n", i, *p);
+#endif
+      switch(*p)
+      {
+	case 0x00:
+	case 0x01:
+	case 0x02:
+	case 0x03:
+	case 0x04:
+	case 0x05:
+	case 0x06:
+	case 0x07:
+	case 0x0b:
+	case 0x0c:
+	case 0x10:
+	case 0x11:
+	case 0x12:
+	case 0x13:
+	case 0x14:
+	case 0x15:
+	case 0x16:
+	case 0x17:
+	case 0x18:
+	case 0x19:
+	case 0x1a:
+	case 0x1b:
+	case 0x1c:
+	case 0x1d:
+	case 0x1e:
+	case 0x1f:
+	case 0x7f:
+	  return i;
+      }
       p++;
       i++;
     }
   }
-  return i;
+  return (i<buf_len?i:buf_len);
 }
 
 static data_check_t data_check_html(const unsigned char *buffer, const unsigned int buffer_size, file_recovery_t *file_recovery)
@@ -519,20 +576,9 @@ static int header_check_ics(const unsigned char *buffer, const unsigned int buff
   date_asc=strstr(buffer2, "DTSTART");
   if(date_asc!=NULL)
     date_asc=strchr(date_asc, ':');
-  if(date_asc!=NULL && date_asc-buffer2<=buffer_size-14)
+  if(date_asc!=NULL && date_asc+1+14 < buffer2+buffer_size)
   {
-    struct tm tm_time;
-    memset(&tm_time, 0, sizeof(tm_time));
-    date_asc++;
-    tm_time.tm_sec=(date_asc[13]-'0')*10+(date_asc[14]-'0');      /* seconds 0-59 */
-    tm_time.tm_min=(date_asc[11]-'0')*10+(date_asc[12]-'0');      /* minutes 0-59 */
-    tm_time.tm_hour=(date_asc[9]-'0')*10+(date_asc[10]-'0');      /* hours   0-23*/
-    tm_time.tm_mday=(date_asc[6]-'0')*10+(date_asc[7]-'0');	/* day of the month 1-31 */
-    tm_time.tm_mon=(date_asc[4]-'0')*10+(date_asc[5]-'0')-1;	/* month 0-11 */
-    tm_time.tm_year=(date_asc[0]-'0')*1000+(date_asc[1]-'0')*100+
-      (date_asc[2]-'0')*10+(date_asc[3]-'0')-1900;        	/* year */
-    tm_time.tm_isdst = -1;		/* unknown daylight saving time */
-    file_recovery_new->time=mktime(&tm_time);
+    file_recovery_new->time=get_time_from_YYYYMMDD_HHMMSS(date_asc+1);
   }
   free(buffer2);
   return 1;
@@ -582,6 +628,77 @@ static int header_check_dc(const unsigned char *buffer, const unsigned int buffe
   return 0;
 }
 
+static void file_rename_fods(file_recovery_t *file_recovery)
+{
+  FILE *file;
+  char buffer[4096];
+  char *tmp;
+  size_t lu;
+  if((file=fopen(file_recovery->filename, "rb"))==NULL)
+    return;
+  if((lu=fread(&buffer, 1, sizeof(buffer)-1, file)) <= 0)
+  {
+    fclose(file);
+    return ;
+  }
+  buffer[lu]='\0';
+  tmp=strchr(buffer,'<');
+  while(tmp!=NULL)
+  {
+    if(strncasecmp(tmp, "<office:meta><dc:title>", 23)==0)
+    {
+      const char *title=tmp+23;
+      tmp=strchr(title,'<');
+      if(tmp!=NULL)
+	*tmp='\0';
+      file_rename(file_recovery, (const unsigned char*)title, strlen(title), 0, NULL, 1);
+      fclose(file);
+      return ;
+    }
+    tmp++;
+    tmp=strchr(tmp,'<');
+  }
+  fclose(file);
+}
+
+static void file_rename_html(file_recovery_t *file_recovery)
+{
+  FILE *file;
+  char buffer[4096];
+  char *tmp;
+  size_t lu;
+  if((file=fopen(file_recovery->filename, "rb"))==NULL)
+    return;
+  if((lu=fread(&buffer, 1, sizeof(buffer)-1, file)) <= 0)
+  {
+    fclose(file);
+    return ;
+  }
+  buffer[lu]='\0';
+  tmp=strchr(buffer,'<');
+  while(tmp!=NULL)
+  {
+    if(strncasecmp(tmp, "</head", 5)==0)
+    {
+      fclose(file);
+      return ;
+    }
+    if(strncasecmp(tmp, "<title>", 7)==0)
+    {
+      const char *title=tmp+7;
+      tmp=strchr(title,'<');
+      if(tmp!=NULL)
+	*tmp='\0';
+      file_rename(file_recovery, (const unsigned char*)title, strlen(title), 0, NULL, 1);
+      fclose(file);
+      return ;
+    }
+    tmp++;
+    tmp=strchr(tmp,'<');
+  }
+  fclose(file);
+}
+
 static int header_check_html(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
 {
   if(file_recovery->file_stat!=NULL &&
@@ -599,9 +716,31 @@ static int header_check_html(const unsigned char *buffer, const unsigned int buf
 #else
   file_recovery_new->extension="html";
 #endif
+  file_recovery_new->file_rename=&file_rename_html;
   return 1;
 }
 
+static void file_check_vbm(file_recovery_t *file_recovery)
+{
+  file_search_footer(file_recovery, "</BackupMeta>", 13, 0);
+  file_allow_nl(file_recovery, NL_BARENL|NL_CRLF|NL_BARECR);
+}
+
+static int header_check_vbm(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
+{
+  reset_file_recovery(file_recovery_new);
+  file_recovery_new->data_check=&data_check_txt;
+  file_recovery_new->extension="vbm";
+  file_recovery_new->file_check=&file_check_vbm;
+  return 1;
+}
+
+static void file_check_gpx(file_recovery_t *file_recovery)
+{
+  file_search_footer(file_recovery, "</gpx>", 6, 0);
+  file_allow_nl(file_recovery, NL_BARENL|NL_CRLF|NL_BARECR);
+}
+
 static void file_check_xml(file_recovery_t *file_recovery)
 {
   file_search_footer(file_recovery, ">", 1, 0);
@@ -614,6 +753,63 @@ static void file_check_svg(file_recovery_t *file_recovery)
   file_allow_nl(file_recovery, NL_BARENL|NL_CRLF|NL_BARECR);
 }
 
+static data_check_t data_check_xml_utf8(const unsigned char *buffer, const unsigned int buffer_size, file_recovery_t *file_recovery)
+{
+  unsigned int i;
+  if(buffer_size<=8)
+    return DC_CONTINUE;
+  i=UTFsize(&buffer[buffer_size/2+4], buffer_size/2-4)+4;
+  if(i<buffer_size/2)
+  {
+    file_recovery->calculated_file_size=file_recovery->file_size+i;
+    return DC_STOP;
+  }
+  file_recovery->calculated_file_size=file_recovery->file_size+(buffer_size/2);
+  file_recovery->data_check=&data_check_txt;
+  return DC_CONTINUE;
+}
+
+static int header_check_xml_utf8(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
+{
+  const char *tmp;
+  /* buffer may not be null-terminated */
+  char *buf=(char *)MALLOC(buffer_size+1);
+  memcpy(buf, buffer, buffer_size);
+  buf[buffer_size]='\0';
+  reset_file_recovery(file_recovery_new);
+  file_recovery_new->data_check=&data_check_xml_utf8;
+  file_recovery_new->extension=NULL;
+  tmp=strchr(buf,'<');
+  while(tmp!=NULL && file_recovery_new->extension==NULL)
+  {
+    if(strncasecmp(tmp, "<Archive name=\"Root\">", 8)==0)
+    {
+      /* Grasshopper archive */
+      file_recovery_new->extension="ghx";
+    }
+    tmp++;
+    tmp=strchr(tmp,'<');
+  }
+  if(file_recovery_new->extension==NULL)
+  {
+    file_recovery_new->extension="xml";
+  }
+  file_recovery_new->file_check=&file_check_xml;
+  free(buf);
+  return 1;
+}
+
+static int header_check_xml_utf16(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
+{
+  /* Avoid false positive with .sldprt */
+  if(file_recovery->file_stat!=NULL &&
+      file_recovery->file_stat->file_hint==&file_hint_doc)
+    return 0;
+  reset_file_recovery(file_recovery_new);
+  file_recovery_new->extension="xml";
+  return 1;
+}
+
 static int header_check_xml(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
 {
   const char *tmp;
@@ -645,6 +841,7 @@ static int header_check_xml(const unsigned char *buffer, const unsigned int buff
 #else
       file_recovery_new->extension="html";
 #endif
+      file_recovery_new->file_rename=&file_rename_html;
     }
     else if(strncasecmp(tmp, "<Version>QBFSD", 14)==0)
     {
@@ -668,6 +865,14 @@ static int header_check_xml(const unsigned char *buffer, const unsigned int buff
       file_recovery_new->extension="plist";
 #endif
     }
+    else if(strncasecmp(tmp, "<gpx ", 5)==0)
+    {
+      /* GPS eXchange Format */
+      file_recovery_new->extension="gpx";
+      file_recovery_new->file_check=&file_check_gpx;
+      free(buf);
+      return 1;
+    }
     else if(strncasecmp(tmp, "<PremiereData Version=", 22)==0)
     {
       /* Adobe Premiere project  */
@@ -684,6 +889,13 @@ static int header_check_xml(const unsigned char *buffer, const unsigned int buff
       /* FictionBook, see http://www.fictionbook.org */
       file_recovery_new->extension="fb2";
     }
+    else if(strncasecmp(tmp, "<office:document", 16)==0)
+    {
+      /* OpenDocument Flat XML Spreadsheet */
+      file_recovery_new->extension="fods";
+      file_recovery_new->data_check=NULL;
+      file_recovery_new->file_rename=&file_rename_fods;
+    }
     tmp++;
     tmp=strchr(tmp,'<');
   }
@@ -720,7 +932,8 @@ static int header_check_xmp(const unsigned char *buffer, const unsigned int buff
   if(buffer[35]=='\0')
     return 0;
   if(file_recovery->file_stat!=NULL &&
-      (file_recovery->file_stat->file_hint==&file_hint_pdf ||
+      (file_recovery->file_stat->file_hint==&file_hint_jpg ||
+       file_recovery->file_stat->file_hint==&file_hint_pdf ||
        file_recovery->file_stat->file_hint==&file_hint_tiff))
     return 0;
   /* Adobe's Extensible Metadata Platform */
@@ -731,6 +944,33 @@ static int header_check_xmp(const unsigned char *buffer, const unsigned int buff
   return 1;
 }
 
+static void file_check_thunderbird(file_recovery_t *file_recovery)
+{
+  if(file_recovery->file_size<file_recovery->calculated_file_size)
+  {
+    file_recovery->file_size=0;
+    return;
+  }
+  file_recovery->file_size=file_recovery->calculated_file_size;
+}
+
+static int header_check_thunderbird(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
+{
+  unsigned int i;
+  if(file_recovery->file_stat!=NULL &&
+      file_recovery->file_stat->file_hint==&file_hint_fasttxt &&
+      strcmp(file_recovery->extension,"mbox")==0)
+    return 0;
+  for(i=0; i<64; i++)
+    if(buffer[i]==0)
+      return 0;
+  reset_file_recovery(file_recovery_new);
+  file_recovery_new->data_check=&data_check_txt;
+  file_recovery_new->file_check=&file_check_thunderbird;
+  file_recovery_new->extension="mbox";
+  return 1;
+}
+
 static int header_check_mbox(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
 {
   unsigned int i;
@@ -871,7 +1111,7 @@ static int header_check_txt(const unsigned char *buffer, const unsigned int buff
     reset_file_recovery(file_recovery_new);
     file_recovery_new->data_check=&data_check_txt;
     file_recovery_new->file_check=&file_check_size;
-    /* Dos/Windows bath */
+    /* Dos/Windows batch */
     file_recovery_new->extension="bat";
     return 1;
   }
@@ -950,33 +1190,12 @@ static int header_check_txt(const unsigned char *buffer, const unsigned int buff
   }
   if(file_recovery->file_stat!=NULL)
   {
-    if(file_recovery->file_stat->file_hint == &file_hint_doc)
-    {
-      if(strstr(file_recovery->filename,".doc")==NULL)
-	return 0;
-    }
-    else if(file_recovery->file_stat->file_hint == &file_hint_fasttxt ||
+    if(file_recovery->file_stat->file_hint == &file_hint_fasttxt ||
 	file_recovery->file_stat->file_hint == &file_hint_txt)
     {
       if(strstr(file_recovery->filename,".html")==NULL)
 	return 0;
     }
-    else if(file_recovery->file_stat->file_hint == &file_hint_jpg)
-    {
-      /* Don't search text at the beginning of JPG */
-      if(file_recovery->file_size < file_recovery->min_filesize)
-	return 0;
-      /* Text should not be found in JPEG */
-      if(td_memmem(buffer, buffer_size_test, "8BIM", 4)!=NULL ||
-	  td_memmem(buffer, buffer_size_test, "adobe", 5)!=NULL ||
-	  td_memmem(buffer, buffer_size_test, "exif:", 5)!=NULL ||
-	  td_memmem(buffer, buffer_size_test, "<rdf:", 5)!=NULL ||
-	  td_memmem(buffer, buffer_size_test, "<?xpacket", 9)!=NULL ||
-	  td_memmem(buffer, buffer_size_test, "<dict>", 6)!=NULL ||
-	  td_memmem(buffer, buffer_size_test, "xmp:CreatorTool>", 16)!=NULL ||
-	  td_memmem(buffer, buffer_size_test, "[camera info]", 13)!=NULL)
-	return 0;
-    }
     else
       return 0;
   }
@@ -1011,20 +1230,29 @@ static int header_check_txt(const unsigned char *buffer, const unsigned int buff
     reset_file_recovery(file_recovery_new);
     file_recovery_new->data_check=&data_check_txt;
     file_recovery_new->file_check=&file_check_size;
-    /* Dos/Windows bath */
+    /* Dos/Windows batch */
     file_recovery_new->extension="bat";
     return 1;
   }
+  if(strncasecmp((const char *)buffer, "dn: ", 4)==0)
+  {
+    reset_file_recovery(file_recovery_new);
+    file_recovery_new->data_check=&data_check_txt;
+    file_recovery_new->file_check=&file_check_size;
+    file_recovery_new->extension="ldif";
+    return 1;
+  }
   {
     const char *ext=NULL;
     /* ind=~0: random
      * ind=~1: constant	*/
-    double ind=1;
+    double ind;
     unsigned int nbrf=0;
     unsigned int is_csv=1;
+    char *str;
     /* Detect Fortran */
     {
-      char *str=buffer_lower;
+      str=buffer_lower;
       while((str=strstr(str, "\n      "))!=NULL)
       {
 	nbrf++;
@@ -1103,7 +1331,21 @@ static int header_check_txt(const unsigned char *buffer, const unsigned int buff
       ext="java";
 #endif
     }
-    else if(strstr(buffer_lower, "class")!=NULL &&
+    else if(strstr(buffer_lower, "\nimport (")!=NULL)
+    {
+      ext="go";
+    }
+    else if((str=strstr(buffer_lower, "\nimport "))!=NULL)
+    {
+      str+=8;
+      while(*str!='\0' && *str!='\n' && *str!=';')
+	str++;
+      if(*str==';')
+	ext="java";
+      else
+	ext="py";
+    }
+    else if(strstr(buffer_lower, "class ")!=NULL &&
 	(l>=100 || file_recovery->file_stat==NULL))
     {
 #ifdef DJGPP
@@ -1170,7 +1412,10 @@ static int header_check_txt(const unsigned char *buffer, const unsigned int buff
     }
     reset_file_recovery(file_recovery_new);
     if(strcmp(ext, "html")==0)
+    {
+      file_recovery_new->file_rename=&file_rename_html;
       file_recovery_new->data_check=&data_check_html;
+    }
     else
       file_recovery_new->data_check=&data_check_txt;
     file_recovery_new->file_check=&file_check_size;
@@ -1210,22 +1455,53 @@ static int header_check_stl(const unsigned char *buffer, const unsigned int buff
   return 1;
 }
 
+static int header_check_svg(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
+{
+  /* Scalable Vector Graphics */
+  reset_file_recovery(file_recovery_new);
+  file_recovery_new->extension="svg";
+  file_recovery_new->file_check=&file_check_svg;
+  return 1;
+}
+
+static int header_check_snz(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
+{
+  const unsigned int buffer_size_test=(buffer_size < 512? buffer_size : 512);
+  const unsigned char *pos=(const unsigned char *)td_memmem(buffer, buffer_size_test, ".snz", 4);
+  if(pos==NULL)
+    return 0;
+  reset_file_recovery(file_recovery_new);
+  file_recovery_new->data_check=&data_check_txt;
+  file_recovery_new->file_check=&file_check_size;
+  file_recovery_new->extension="snz";
+  file_recovery_new->min_filesize=pos-buffer;
+  return 1;
+}
+
+static void register_header_check_snz(file_stat_t *file_stat)
+{
+  register_header_check(0, "DEFAULT\n",   8, &header_check_snz, file_stat);
+  register_header_check(0, "DEFAULT\r\n", 9, &header_check_snz, file_stat);
+}
+
 static void register_header_check_fasttxt(file_stat_t *file_stat)
 {
   static const unsigned char header_xml_utf8[17]	= {0xef, 0xbb, 0xbf, '<', '?', 'x', 'm', 'l', ' ', 'v', 'e', 'r', 's', 'i', 'o', 'n', '='};
+  static const unsigned char header_xml_utf16[30]	= {0xff, 0xfe, '<', 0, '?', 0, 'x', 0, 'm', 0, 'l', 0, ' ', 0, 'v', 0, 'e', 0, 'r', 0, 's', 0, 'i', 0, 'o', 0, 'n', 0, '=', 0};
   const txt_header_t *header=&fasttxt_headers[0];
   while(header->len > 0)
   {
     register_header_check(0, header->string, header->len, &header_check_fasttxt, file_stat);
     header++;
   }
-  register_header_check(4, "SC V10", 		6,  &header_check_dc, file_stat);
+  register_header_check(4, "SC V10",		6,  &header_check_dc, file_stat);
   register_header_check(0, "DatasetHeader Begin", 19, &header_check_ers, file_stat);
 //  register_header_check(0, "\n<!DOCTYPE html",	15, &header_check_html, file_stat);
   register_header_check(0, "<!DOCTYPE html",	14, &header_check_html, file_stat);
   register_header_check(0, "<!DOCTYPE HTML",	14, &header_check_html, file_stat);
 //  register_header_check(0, "<html",		 5, &header_check_html, file_stat);
   register_header_check(0, "BEGIN:VCALENDAR",	15, &header_check_ics, file_stat);
+  register_header_check(0, "From - ",		 7, &header_check_thunderbird, file_stat);
   register_header_check(0, "From ",		 5, &header_check_mbox, file_stat);
   register_header_check(0, "Message-ID: ",	12, &header_check_mbox, file_stat);
   register_header_check(0, "MIME-Version:",	13, &header_check_mbox, file_stat);
@@ -1234,13 +1510,17 @@ static void register_header_check_fasttxt(file_stat_t *file_stat)
   register_header_check(0, "Return-path: ",	13, &header_check_mbox, file_stat);
   register_header_check(0, "Return-Path: ",	13, &header_check_mbox, file_stat);
   register_header_check(0, "package ",		 8, &header_check_perlm, file_stat);
-  register_header_check(0, "package\t", 	 8, &header_check_perlm, file_stat);
+  register_header_check(0, "package\t",		 8, &header_check_perlm, file_stat);
   register_header_check(0, "{\\rtf",		 5, &header_check_rtf, file_stat);
   register_header_check(0, "<smil>",		 6, &header_check_smil, file_stat);
   register_header_check(0, "solid ",		 6, &header_check_stl, file_stat);
   register_header_check(0, "<?xml version=",	14, &header_check_xml, file_stat);
-  register_header_check(0, header_xml_utf8, sizeof(header_xml_utf8), &header_check_xml, file_stat);
+  register_header_check(0, header_xml_utf8, sizeof(header_xml_utf8), &header_check_xml_utf8, file_stat);
+  register_header_check(0, header_xml_utf16, sizeof(header_xml_utf16), &header_check_xml_utf16, file_stat);
+  /* Veeam Backup */
+  register_header_check(0, "<BackupMeta Version=",	20, &header_check_vbm, file_stat);
   /* TinyTag */
   register_header_check(0, "FF 09 FF FF FF FF FF FF FF FF FF FF FF FF FF FF FFFF 00", 55, &header_check_ttd, file_stat);
   register_header_check(0, "<x:xmpmeta xmlns:x=\"adobe:ns:meta/\"", 35, &header_check_xmp, file_stat);
+  register_header_check(0, "<svg xmlns=\"http://www.w3.org/2000/svg\"", 39, &header_check_svg, file_stat);
 }
