@@ -37,7 +37,6 @@ static data_check_t data_check_wv(const unsigned char *buffer, const unsigned in
 const file_hint_t file_hint_wv= {
   .extension="wv",
   .description="WavPack, Hybrid Lossless Wavefile Compressor",
-  .min_header_distance=0,
   .max_filesize=100*1024*1024,
   .recover=1,
   .enable_by_default=1,
@@ -90,14 +89,14 @@ static data_check_t data_check_wv(const unsigned char *buffer, const unsigned in
     const WavpackHeader *wv=(const WavpackHeader*)&buffer[i];
     if(memcmp(wv, wv_header, sizeof(wv_header))==0)
     {
-      file_recovery->calculated_file_size+=le32(wv->ckSize)+8;
+      file_recovery->calculated_file_size+=(uint64_t)8+le32(wv->ckSize);
     }
     else if(buffer[i]=='A' && buffer[i+1]=='P' && buffer[i+2]=='E' && buffer[i+3]=='T' && buffer[i+4]=='A' && buffer[i+5]=='G' && buffer[i+6]=='E' && buffer[i+7]=='X')
     { /* APE Tagv2 (APE Tagv1 has no header) http://wiki.hydrogenaudio.org/index.php?title=APE_Tags_Header */
       const unsigned int ape_tag_size = (buffer[i+12] + (buffer[i+13]<<8) + (buffer[i+14]<<16) + (buffer[i+15]<<24))+32;
       file_recovery->calculated_file_size+=ape_tag_size;
     }
-    else if(buffer[i]=='T' && buffer[i+1]=='A' && buffer[i+2]=='G')	
+    else if(buffer[i]=='T' && buffer[i+1]=='A' && buffer[i+2]=='G')
     { /* http://www.id3.org/ID3v1 TAGv1 size = 128 bytes with header "TAG" */
       file_recovery->calculated_file_size+=128;
     }
@@ -105,7 +104,7 @@ static data_check_t data_check_wv(const unsigned char *buffer, const unsigned in
     {
       return DC_CONTINUE;
     }
-    else 
+    else
     {
       return DC_STOP;
     }
