@@ -42,24 +42,17 @@
 
 /* #define DEBUG_ZIP */
 
-#if defined(HAVE_FSEEKO) && !defined(__MINGW32__)
-#define my_fseek fseeko
-#else
-#define my_fseek fseek
-#endif
-
 extern const file_hint_t file_hint_doc;
 static void register_header_check_zip(file_stat_t *file_stat);
 static int header_check_zip(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new);
 static void file_check_zip(file_recovery_t *file_recovery);
 static unsigned int pos_in_mem(const unsigned char *haystack, const unsigned int haystack_size, const unsigned char *needle, const unsigned int needle_size);
-static void file_rename_zip(const char *old_filename);
+static void file_rename_zip(file_recovery_t *file_recovery);
 static char first_filename[256];
 
 const file_hint_t file_hint_zip= {
   .extension="zip",
   .description="zip archive including OpenOffice and MSOffice 2007",
-  .min_header_distance=0,
   .max_filesize=PHOTOREC_MAX_FILE_SIZE,
   .recover=1,
   .enable_by_default=1,
@@ -67,7 +60,6 @@ const file_hint_t file_hint_zip= {
 };
 
 static const unsigned char zip_header[4]  = { 'P', 'K', 0x03, 0x04};
-static const unsigned char zip_header2[8]  = { 'P', 'K', '0', '0', 'P', 'K', 0x03, 0x04}; /* WinZIPv8-compressed files. */
 #define ZIP_CENTRAL_DIR         0x02014B50
 #define ZIP_FILE_ENTRY          0x04034B50
 #define ZIP_SIGNATURE           0x05054B50
@@ -136,16 +128,20 @@ static int64_t file_get_pos(FILE *f, const void* needle, const unsigned int size
       if (buffer[count]==*(const char *)needle && memcmp(buffer+count, needle, size)==0)
       {
 	free(buffer);
-        if(my_fseek(f, (long)count-read_size, SEEK_CUR)<0)
+        if(my_fseek(f, (off_t)count-read_size, SEEK_CUR)<0)
+	{
+	  log_trace("zip: file_get_pos count-read failed\n");
           return -1;
+	}
         return total;
       }
       count++;
       total++;
       left--;
     }
-    if(feof(f) || my_fseek(f, (long)1-size, SEEK_CUR)<0)
+    if(feof(f) || my_fseek(f, (off_t)1-size, SEEK_CUR)<0)
     {
+      log_trace("zip: file_get_pos 1-size failed\n");
       free(buffer);
       return -1;
     }
@@ -169,12 +165,15 @@ static int zip_parse_file_entry(file_recovery_t *fr, const char **ext, const uns
   }
   fr->file_size += sizeof(file);
 #ifdef DEBUG_ZIP
-  log_info("%u Comp=%u %u CRC32=0x%08X ",
+  log_info("%u Comp=%u %u CRC32=0x%08X extra_length=%u ",
       le32(file.compressed_size),
       le16(file.compression),
       le32(file.uncompressed_size),
-      le32(file.crc32));
+      le32(file.crc32),
+      le16(file.extra_length));
 #endif
+  /* Avoid Jan  1  1980 files */
+  if(le16(file.last_mod_time)!=0 || le16(file.last_mod_date)!=33)
   {
     /* Use the more recent file to set the time/date of the recovered archive */
     const time_t tmp=date_dos2unix(le16(file.last_mod_time), le16(file.last_mod_date));
@@ -208,10 +207,17 @@ static int zip_parse_file_entry(file_recovery_t *fr, const char **ext, const uns
     {
       static int msoffice=0;
       static int sh3d=0;
+      static const char *ext_msoffice=NULL;
       if(file_nbr==0)
       {
 	msoffice=0;
 	sh3d=0;
+	ext_msoffice=NULL;
+      }
+      if(len==19 && memcmp(filename, "[Content_Types].xml", 19)==0)
+	msoffice=1;
+      else if(file_nbr==0)
+      {
 	if(len==8 && memcmp(filename, "mimetype", 8)==0 && le16(file.extra_length)==0)
 	{
 	  unsigned char buffer[128];
@@ -234,6 +240,8 @@ static int zip_parse_file_entry(file_recovery_t *fr, const char **ext, const uns
 	  }
 	  if(compressed_size==16      && memcmp(buffer,"image/openraster",16)==0)
 	    *ext="ora";
+	  else if(compressed_size==20 && memcmp(buffer,"application/epub+zip",20)==0)
+	    *ext="epub";
 	  else if(compressed_size==28 && memcmp(buffer,"application/vnd.sun.xml.calc",28)==0)
 	    *ext="sxc";
 	  else if(compressed_size==28 && memcmp(buffer,"application/vnd.sun.xml.draw",28)==0)
@@ -260,30 +268,32 @@ static int zip_parse_file_entry(file_recovery_t *fr, const char **ext, const uns
 	    *ext="sxw";
 	  }
 	}
-	else if(len==19 && memcmp(filename, "[Content_Types].xml", 19)==0)
-	  msoffice=1;
 	/* Zipped Keyhole Markup Language (KML) used by Google Earth */
 	else if(len==7 && memcmp(filename, "doc.kml", 7)==0)
 	  *ext="kmz";
 	else if(len==4 && memcmp(filename, "Home", 4)==0)
 	  sh3d=1;
+	/* Celtx, Screenwriting & Media Pre-production file */
+	else if(len==9 && memcmp(filename, "local.rdf", 9)==0)
+	  *ext="celtx";
+	else if(len==13 && memcmp(filename, "document.json", 13)==0)
+	  *ext="sketch";
       }
       else if(file_nbr==1 && sh3d==1)
       {
 	if(len==1 && filename[0]=='0')
 	  *ext="sh3d";
       }
-      else if(file_nbr==2 && msoffice!=0)
-      {
-	if(strncmp(filename, "word/", 5)==0)
-	  *ext="docx";
-	else if(strncmp(filename, "xl/", 3)==0)
-	  *ext="xlsx";
-	else if(strncmp(filename, "ppt/", 4)==0)
-	  *ext="pptx";
-	else
-	  *ext="docx";
-      }
+      if(strncmp(filename, "word/", 5)==0)
+	ext_msoffice="docx";
+      else if(strncmp(filename, "xl/", 3)==0)
+	ext_msoffice="xlsx";
+      else if(strncmp(filename, "ppt/", 4)==0)
+	ext_msoffice="pptx";
+      else if(strncmp(filename, "visio/", 6)==0)
+	ext_msoffice="vsdx";
+      if(msoffice && ext_msoffice!=NULL)
+	*ext=ext_msoffice;
     }
     if(*ext==NULL)
     {
@@ -297,6 +307,11 @@ static int zip_parse_file_entry(file_recovery_t *fr, const char **ext, const uns
       /* SMART Notebook */
       else if(len==15 && memcmp(filename, "imsmanifest.xml", 15)==0)
 	*ext="notebook";
+      /* Apple Numbers */
+      else if(len==18 && memcmp(filename, "Index/Document.iwa", 18)==0)
+	*ext="numbers";
+      else if(len==19 && memcmp(filename, "AndroidManifest.xml", 19)==0)
+	*ext="apk";
       else if(len==30 && memcmp(filename, "xsd/MindManagerApplication.xsd", 30)==0)
 	*ext="mmap";
     }
@@ -329,6 +344,9 @@ static int zip_parse_file_entry(file_recovery_t *fr, const char **ext, const uns
   if(len==0xffffffff && le16(extra.tag)==1)
   {
     len = le64(extra.compressed_size);
+    /* Avoid endless loop */
+    if( fr->file_size + len < fr->file_size)
+      return -1;
   }
   if(krita>0)
     len=krita;
@@ -346,8 +364,7 @@ static int zip_parse_file_entry(file_recovery_t *fr, const char **ext, const uns
 #endif
     fr->file_size += len;
   }
-
-  expected_compressed_size=0;
+  expected_compressed_size=len;
   if (file.has_descriptor && (le16(file.compression)==8 || le16(file.compression)==9))
   {
     /* The fields crc-32, compressed size and uncompressed size
@@ -456,6 +473,9 @@ static int zip64_parse_end_central_dir(file_recovery_t *fr)
   if (dir.end_size > 0)
   {
     const uint64_t len = le64(dir.end_size);
+    /* Avoid endless loop */
+    if( fr->file_size + len <= fr->file_size)
+      return -1;
     if (my_fseek(fr->handle, len, SEEK_CUR) == -1)
     {
 #ifdef DEBUG_ZIP
@@ -528,8 +548,9 @@ static int zip_parse_data_desc(file_recovery_t *fr)
   }
   fr->file_size += sizeof(desc);
 #ifdef DEBUG_ZIP
-  log_info("%u %u CRC32=0x%08X\n",
+  log_info("compressed_size=%u/%u uncompressed_size=%u CRC32=0x%08X\n",
       le32(desc.compressed_size),
+      expected_compressed_size,
       le32(desc.uncompressed_size),
       le32(desc.crc32));
 #endif
@@ -669,13 +690,13 @@ static void file_check_zip(file_recovery_t *fr)
   }
 }
 
-static void file_rename_zip(const char *old_filename)
+static void file_rename_zip(file_recovery_t *file_recovery)
 {
   const char *ext=NULL;
   unsigned int file_nbr=0;
   file_recovery_t fr;
   reset_file_recovery(&fr);
-  if((fr.handle=fopen(old_filename, "rb"))==NULL)
+  if((fr.handle=fopen(file_recovery->filename, "rb"))==NULL)
     return;
   fr.file_size = 0;
   fr.offset_error=0;
@@ -729,7 +750,7 @@ static void file_rename_zip(const char *old_filename)
 	if(ext!=NULL)
 	{
 	  fclose(fr.handle);
-	  file_rename(old_filename, NULL, 0, 0, ext, 1);
+	  file_rename(file_recovery, NULL, 0, 0, ext, 1);
 	  return;
 	}
         break;
@@ -764,111 +785,122 @@ static void file_rename_zip(const char *old_filename)
 	  first_filename[len]!='/' &&
 	  first_filename[len]!='\\';
 	  len++);
-      file_rename(old_filename, first_filename, len, 0, "zip", 0);
+      file_rename(file_recovery, first_filename, len, 0, "zip", 0);
       return;
     }
   }
 }
 
-static void register_header_check_zip(file_stat_t *file_stat)
-{
-  register_header_check(0, zip_header,sizeof(zip_header), &header_check_zip, file_stat);
-  register_header_check(0, zip_header2,sizeof(zip_header2), &header_check_zip, file_stat);
-}
-
 static int header_check_zip(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
 {
+  const zip_file_entry_t *file=(const zip_file_entry_t *)&buffer[4];
+  const unsigned int len=le16(file->filename_length);
 #ifdef DEBUG_ZIP
   log_trace("header_check_zip\n");
 #endif
-  if(memcmp(buffer,zip_header,sizeof(zip_header))==0)
+  if(file_recovery->file_stat!=NULL &&
+      file_recovery->file_stat->file_hint==&file_hint_doc)
   {
-    const zip_file_entry_t *file=(const zip_file_entry_t *)&buffer[4];
-    const unsigned int len=le16(file->filename_length);
-    if(file_recovery->file_stat!=NULL &&
-	file_recovery->file_stat->file_hint==&file_hint_doc &&
-	(strcmp(file_recovery->extension,"doc")==0 ||
-	 strcmp(file_recovery->extension,"psmodel")==0)
-	&& memcmp(&buffer[30], "macrolog_1.mac", 14)==0)
+    if(header_ignored_adv(file_recovery, file_recovery_new)==0)
       return 0;
-    /* A zip file begins by ZIP_FILE_ENTRY, this signature can also be
-     * found for each compressed file */
-    if(file_recovery->file_stat!=NULL &&
-	file_recovery->file_stat->file_hint==&file_hint_zip &&
-	safe_header_only==0)
+  }
+  /* A zip file begins by ZIP_FILE_ENTRY, this signature can also be
+   * found for each compressed file */
+  if(file_recovery->file_stat!=NULL &&
+      file_recovery->file_stat->file_hint==&file_hint_zip &&
+      safe_header_only==0)
+  {
+    if(header_ignored_adv(file_recovery, file_recovery_new)==0)
       return 0;
-    reset_file_recovery(file_recovery_new);
-    file_recovery_new->min_filesize=21;
-    file_recovery_new->file_check=&file_check_zip;
-    if(len==8 && memcmp(&buffer[30],"mimetype",8)==0)
-    {
-      const unsigned int compressed_size=le32(file->compressed_size);
-      /* Mypaint .ora */
-      if(compressed_size==16 && memcmp(&buffer[38],"image/openraster",16)==0)
-	file_recovery_new->extension="ora";
-      else if(compressed_size==28 && memcmp(&buffer[38],"application/vnd.sun.xml.calc",28)==0)
-	file_recovery_new->extension="sxc";
-      else if(compressed_size==28 && memcmp(&buffer[38],"application/vnd.sun.xml.draw",28)==0)
-	file_recovery_new->extension="sxd";
-      else if(compressed_size==31 && memcmp(&buffer[38],"application/vnd.sun.xml.impress",31)==0)
-	file_recovery_new->extension="sxi";
-      else if(compressed_size==30 && memcmp(&buffer[38],"application/vnd.sun.xml.writer",30)==0)
-	file_recovery_new->extension="sxw";
-      else if(compressed_size==39 && memcmp(&buffer[38],"application/vnd.oasis.opendocument.text",39)==0)
-	file_recovery_new->extension="odt";
-      else if(compressed_size==43 && memcmp(&buffer[38],"application/vnd.oasis.opendocument.graphics",43)==0)
-	file_recovery_new->extension="odg";
-      else if(compressed_size==46 && memcmp(&buffer[38],"application/vnd.oasis.opendocument.spreadsheet",46)==0)
-	file_recovery_new->extension="ods";
-      else if(compressed_size==47 && memcmp(&buffer[38],"application/vnd.oasis.opendocument.presentation",47)==0)
-	file_recovery_new->extension="odp";
-      else if(memcmp(&buffer[38],"application/x-krita",19)==0)
-	file_recovery_new->extension="kra";
-      else
-      { /* default to writer */
-	file_recovery_new->extension="sxw";
-      }
-    }
-    else if(len==19 && memcmp(&buffer[30],"[Content_Types].xml",19)==0)
-    {
-      if(pos_in_mem(&buffer[0], buffer_size, (const unsigned char*)"word/", 5)!=0)
-	file_recovery_new->extension="docx";
-      else if(pos_in_mem(&buffer[0], 2000, (const unsigned char*)"xl/", 3)!=0)
-	file_recovery_new->extension="xlsx";
-      else if(pos_in_mem(&buffer[0], buffer_size, (const unsigned char*)"ppt/", 4)!=0)
-	file_recovery_new->extension="pptx";
-      else
-	file_recovery_new->extension="docx";
-      file_recovery_new->file_rename=&file_rename_zip;
-    }
-    /* Extended Renoise song file */
-    else if(len==8 && memcmp(&buffer[30], "Song.xml", 8)==0)
-      file_recovery_new->extension="xrns";
-    else if(len==4 && memcmp(&buffer[30], "Home", 4)==0)
-      file_recovery_new->extension="sh3d";
+  }
+  reset_file_recovery(file_recovery_new);
+  file_recovery_new->min_filesize=21;
+  file_recovery_new->file_check=&file_check_zip;
+  if(len==8 && memcmp(&buffer[30],"mimetype",8)==0)
+  {
+    const unsigned int compressed_size=le32(file->compressed_size);
+    /* Mypaint .ora */
+    if(compressed_size==16 && memcmp(&buffer[38],"image/openraster",16)==0)
+      file_recovery_new->extension="ora";
+    else if(compressed_size==20 && memcmp(&buffer[38],"application/epub+zip",20)==0)
+      file_recovery_new->extension="epub";
+    else if(compressed_size==28 && memcmp(&buffer[38],"application/vnd.sun.xml.calc",28)==0)
+      file_recovery_new->extension="sxc";
+    else if(compressed_size==28 && memcmp(&buffer[38],"application/vnd.sun.xml.draw",28)==0)
+      file_recovery_new->extension="sxd";
+    else if(compressed_size==31 && memcmp(&buffer[38],"application/vnd.sun.xml.impress",31)==0)
+      file_recovery_new->extension="sxi";
+    else if(compressed_size==30 && memcmp(&buffer[38],"application/vnd.sun.xml.writer",30)==0)
+      file_recovery_new->extension="sxw";
+    else if(compressed_size==39 && memcmp(&buffer[38],"application/vnd.oasis.opendocument.text",39)==0)
+      file_recovery_new->extension="odt";
+    else if(compressed_size==43 && memcmp(&buffer[38],"application/vnd.oasis.opendocument.graphics",43)==0)
+      file_recovery_new->extension="odg";
+    else if(compressed_size==45 && memcmp(&buffer[38],"application/vnd.adobe.sparkler.project+dcxucf",45)==0)
+      file_recovery_new->extension="xd";
+    else if(compressed_size==46 && memcmp(&buffer[38],"application/vnd.oasis.opendocument.spreadsheet",46)==0)
+      file_recovery_new->extension="ods";
+    else if(compressed_size==47 && memcmp(&buffer[38],"application/vnd.oasis.opendocument.presentation",47)==0)
+      file_recovery_new->extension="odp";
+    else if(memcmp(&buffer[38],"application/x-krita",19)==0)
+      file_recovery_new->extension="kra";
     else
-    {
-      file_recovery_new->extension=file_hint_zip.extension;
-      file_recovery_new->file_rename=&file_rename_zip;
+    { /* default to writer */
+      file_recovery_new->extension="sxw";
     }
-    return 1;
   }
-  else if(memcmp(buffer,zip_header2,sizeof(zip_header2))==0)
+  else if(len==19 && memcmp(&buffer[30],"[Content_Types].xml",19)==0)
   {
-    reset_file_recovery(file_recovery_new);
-    file_recovery_new->file_check=&file_check_zip;
-    file_recovery_new->extension=file_hint_zip.extension;
-    return 1;
+    if(pos_in_mem(&buffer[0], buffer_size, (const unsigned char*)"word/", 5)!=0)
+      file_recovery_new->extension="docx";
+    else if(pos_in_mem(&buffer[0], 2000, (const unsigned char*)"xl/", 3)!=0)
+      file_recovery_new->extension="xlsx";
+    else if(pos_in_mem(&buffer[0], buffer_size, (const unsigned char*)"ppt/", 4)!=0)
+      file_recovery_new->extension="pptx";
+    else if(pos_in_mem(&buffer[0], buffer_size, (const unsigned char*)"visio/", 6)!=0)
+      file_recovery_new->extension="vsdx";
+    else
+      file_recovery_new->extension="docx";
+    file_recovery_new->file_rename=&file_rename_zip;
   }
-  return 0;
+  /* Extended Renoise song file */
+  else if(len==8 && memcmp(&buffer[30], "Song.xml", 8)==0)
+    file_recovery_new->extension="xrns";
+  else if(len==4 && memcmp(&buffer[30], "Home", 4)==0)
+    file_recovery_new->extension="sh3d";
+  /* Apple Numbers */
+  else if(len==18 && memcmp(&buffer[30], "Index/Document.iwa", 18)==0)
+    file_recovery_new->extension="numbers";
+  else
+  {
+    file_recovery_new->extension=file_hint_zip.extension;
+    file_recovery_new->file_rename=&file_rename_zip;
+  }
+  return 1;
+}
+
+static int header_check_winzip(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
+{
+  reset_file_recovery(file_recovery_new);
+  file_recovery_new->file_check=&file_check_zip;
+  file_recovery_new->extension=file_hint_zip.extension;
+  return 1;
 }
 
 static unsigned int pos_in_mem(const unsigned char *haystack, const unsigned int haystack_size, const unsigned char *needle, const unsigned int needle_size)
 {
   unsigned int i;
-  for(i=0;i<haystack_size;i++)
+  if(haystack_size < needle_size)
+    return 0;
+  for(i=0; i <= haystack_size - needle_size; i++)
     if(memcmp(&haystack[i],needle,needle_size)==0)
       return (i+needle_size);
   return 0;
 }
 
+static void register_header_check_zip(file_stat_t *file_stat)
+{
+  static const unsigned char zip_header2[8]  = { 'P', 'K', '0', '0', 'P', 'K', 0x03, 0x04}; /* WinZIPv8-compressed files. */
+  register_header_check(0, zip_header,sizeof(zip_header), &header_check_zip, file_stat);
+  register_header_check(0, zip_header2,sizeof(zip_header2), &header_check_winzip, file_stat);
+}
