@@ -31,17 +31,16 @@
 #endif
 #include <stdio.h>
 #include <ctype.h>
+#include <assert.h>
+#ifdef HAVE_TIME_H
+#include <time.h>
+#endif
 #include "types.h"
 #include "common.h"
 #include "filegen.h"
+#include "photorec.h"
 #include "log.h"
 
-#if defined(HAVE_FSEEKO) && !defined(__MINGW32__)
-#define my_fseek fseeko
-#else
-#define my_fseek fseek
-#endif
-
 static  file_check_t file_check_plist={
   .list = TD_LIST_HEAD_INIT(file_check_plist.list)
 };
@@ -199,6 +198,7 @@ void file_allow_nl(file_recovery_t *file_recovery, const unsigned int nl_mode)
 uint64_t file_rsearch(FILE *handle, uint64_t offset, const void*footer, const unsigned int footer_length)
 {
   unsigned char*buffer;
+  assert(footer_length < 4096);
   buffer=(unsigned char*)MALLOC(4096+footer_length-1);
   memset(buffer+4096,0,footer_length-1);
   do
@@ -221,7 +221,7 @@ uint64_t file_rsearch(FILE *handle, uint64_t offset, const void*footer, const un
         return offset + i;
       }
     }
-    memcpy(buffer+read_size,buffer,footer_length-1);
+    memcpy(buffer+4096,buffer,footer_length-1);
   } while(offset>0);
   free(buffer);
   return 0;
@@ -296,7 +296,13 @@ void file_check_size(file_recovery_t *file_recovery)
     file_recovery->file_size=file_recovery->calculated_file_size;
 }
 
-void file_check_size_lax(file_recovery_t *file_recovery)
+void file_check_size_min(file_recovery_t *file_recovery)
+{
+  if(file_recovery->file_size<file_recovery->calculated_file_size)
+    file_recovery->file_size=0;
+}
+
+void file_check_size_max(file_recovery_t *file_recovery)
 {
   if(file_recovery->file_size > file_recovery->calculated_file_size)
     file_recovery->file_size=file_recovery->calculated_file_size;
@@ -363,17 +369,18 @@ file_stat_t * init_file_stats(file_enable_t *files_enable)
 }
 
 /* The original filename begins at offset in buffer and is null terminated */
-void file_rename(const char *old_filename, const void *buffer, const int buffer_size, const int offset, const char *new_ext, const int force_ext)
+int file_rename(file_recovery_t *file_recovery, const void *buffer, const int buffer_size, const int offset, const char *new_ext, const int append_original_ext)
 {
   /* new_filename is large enough to avoid a buffer overflow */
   char *new_filename;
-  const char *src=old_filename;
-  const char *ext=src;
+  const char *src=file_recovery->filename;
+  const char *ext=NULL;
   char *dst;
   char *directory_sep;
-  int len=strlen(old_filename)+1;
+  int len;
   if(buffer_size<0)
-    return ;
+    return -1;
+  len=strlen(src)+1;
   if(offset < buffer_size && buffer!=NULL)
     len+=buffer_size-offset+1;
   if(new_ext!=NULL)
@@ -384,7 +391,10 @@ void file_rename(const char *old_filename, const void *buffer, const int buffer_
   while(*src!='\0')
   {
     if(*src=='/')
+    {
       directory_sep=dst;
+      ext=NULL;
+    }
     if(*src=='.')
       ext=src;
     *dst++ = *src++;
@@ -413,12 +423,13 @@ void file_rename(const char *old_filename, const void *buffer, const int buffer_
 	case '<':
 	case '>':
 	case '|':
+	case '\'':
 	  if(*(dst-1) != '_')
 	    *dst++ = '_';
 	  bad++;
 	  break;
 	default:
-	  if(isprint(*src) && !isspace(*src))
+	  if(isprint(*src) && !isspace(*src) && !ispunct(*src) && !iscntrl(*src))
 	  {
 	    *dst++ = *src;
 	    ok++;
@@ -448,33 +459,44 @@ void file_rename(const char *old_filename, const void *buffer, const int buffer_
     while(*src!='\0')
       *dst++ = *src++;
   }
-  else if(force_ext>0)
+  else if(append_original_ext>0)
   {
-    while(*ext!='\0')
-      *dst++ = *ext++;
+    if(ext!=NULL)
+    {
+      while(*ext!='\0')
+	*dst++ = *ext++;
+    }
   }
   *dst='\0';
-  if(rename(old_filename, new_filename)<0)
+  if(rename(file_recovery->filename, new_filename)<0)
   {
-    /* Rename has failed, try without the original filename */
-    if(buffer!=NULL)
-      file_rename(old_filename, NULL, 0, 0, new_ext, force_ext);
+    /* Rename has failed */
+    free(new_filename);
+    if(buffer==NULL)
+      return -1;
+    /* Try without the original filename */
+    return file_rename(file_recovery, NULL, 0, 0, new_ext, append_original_ext);
+  }
+  if(strlen(new_filename)<sizeof(file_recovery->filename))
+  {
+    strcpy(file_recovery->filename, new_filename);
   }
   free(new_filename);
+  return 0;
 }
 
 /* The original filename begins at offset in buffer and is null terminated */
-void file_rename_unicode(const char *old_filename, const void *buffer, const int buffer_size, const int offset, const char *new_ext, const int force_ext)
+int file_rename_unicode(file_recovery_t *file_recovery, const void *buffer, const int buffer_size, const int offset, const char *new_ext, const int append_original_ext)
 {
   /* new_filename is large enough to avoid a buffer overflow */
   char *new_filename;
-  const char *src=old_filename;
+  const char *src=file_recovery->filename;
   const char *ext=src;
   char *dst;
   char *directory_sep;
-  int len=strlen(old_filename)+1;
+  int len=strlen(src)+1;
   if(buffer_size<0)
-    return ;
+    return -1;
   if(offset < buffer_size && buffer!=NULL)
     len+=buffer_size-offset;
   if(new_ext!=NULL)
@@ -511,11 +533,16 @@ void file_rename_unicode(const char *old_filename, const void *buffer, const int
 	case '\\':
 	case ':':
 	case '*':
-	  *dst++ = '_';
+	case '<':
+	case '>':
+	case '|':
+	case '\'':
+	  if(*(dst-1) != '_')
+	    *dst++ = '_';
 	  bad++;
 	  break;
 	default:
-	  if(isprint(*src) && !isspace(*src))
+	  if(isprint(*src) && !isspace(*src) && !ispunct(*src) && !iscntrl(*src))
 	  {
 	    *dst++ = *src;
 	    ok++;
@@ -545,17 +572,214 @@ void file_rename_unicode(const char *old_filename, const void *buffer, const int
     while(*src!='\0')
       *dst++ = *src++;
   }
-  else if(force_ext>0)
+  else if(append_original_ext>0)
   {
     while(*ext!='\0')
       *dst++ = *ext++;
   }
   *dst='\0';
-  if(rename(old_filename, new_filename)<0)
+  if(rename(file_recovery->filename, new_filename)<0)
   {
-    /* Rename has failed, try without the original filename */
-    if(buffer!=NULL)
-      file_rename_unicode(old_filename, NULL, 0, 0, new_ext, force_ext);
+    /* Rename has failed */
+    free(new_filename);
+    if(buffer==NULL)
+      return -1;
+    /* Try without the original filename */
+    return file_rename_unicode(file_recovery, NULL, 0, 0, new_ext, append_original_ext);
+  }
+  if(strlen(new_filename)<sizeof(file_recovery->filename))
+  {
+    strcpy(file_recovery->filename, new_filename);
   }
   free(new_filename);
+  return 0;
+}
+
+static uint64_t offset_skipped_header=0;
+
+void header_ignored_cond_reset(uint64_t start, uint64_t end)
+{
+  if(start <= offset_skipped_header && offset_skipped_header <= end)
+    offset_skipped_header=0;
+}
+
+/* 0: file_recovery is bad *
+ * 1: file_recovery is ok  */
+int header_ignored_adv(const file_recovery_t *file_recovery, const file_recovery_t *file_recovery_new)
+{
+  file_recovery_t fr_test;
+  off_t offset;
+  assert(file_recovery!=NULL);
+  assert(file_recovery_new!=NULL);
+  if(file_recovery->file_check==NULL)
+  {
+    log_warning("header_ignored_adv: file_check==NULL\n");
+    return 1;
+  }
+  if(file_recovery->handle==NULL)
+  {
+    if(file_recovery_new->location.start==0 || offset_skipped_header==0)
+    {
+      offset_skipped_header=file_recovery_new->location.start;
+    }
+    return 0;
+  }
+
+  memcpy(&fr_test, file_recovery, sizeof(fr_test));
+#ifdef HAVE_FTELLO
+  if((offset=ftello(file_recovery->handle)) < 0)
+    offset=ftell(file_recovery->handle);
+#else
+  offset=ftell(file_recovery->handle);
+#endif
+  assert(offset >= 0);
+  file_recovery->file_check(&fr_test);
+  if(my_fseek(file_recovery->handle, offset, SEEK_SET) < 0)
+  {
+    log_error("BUG in header_ignored_adv: my_fseek() failed\n");
+    return 1;
+  }
+  if(fr_test.file_size>0)
+    return 1;
+  if(file_recovery_new->location.start==0 || offset_skipped_header==0)
+  {
+    offset_skipped_header=file_recovery_new->location.start;
+  }
+  return 0;
+}
+
+void header_ignored(const file_recovery_t *file_recovery_new)
+{
+  if(file_recovery_new==NULL)
+  {
+    offset_skipped_header=0;
+    return ;
+  }
+  if(file_recovery_new->location.start==0 || offset_skipped_header==0)
+    offset_skipped_header=file_recovery_new->location.start;
+}
+
+void get_prev_location_smart(alloc_data_t *list_search_space, alloc_data_t **current_search_space, uint64_t *offset, const uint64_t prev_location)
+{
+  alloc_data_t *file_space=*current_search_space;
+  if(offset_skipped_header==0)
+    return ;
+  while(1)
+  {
+    file_space=td_list_prev_entry(file_space, list);
+    if(file_space==list_search_space)
+      break;
+    if(file_space->start <= offset_skipped_header && offset_skipped_header < file_space->end)
+    {
+      *current_search_space=file_space;
+      *offset=offset_skipped_header;
+      offset_skipped_header=0;
+      return ;
+    }
+    if(file_space->start < prev_location)
+      break;
+  }
+#ifdef DEBUG_HEADER_CHECK
+  log_info("get_prev_location_smart: reset offset_skipped_header=%llu, offset=%llu\n",
+      (long long unsigned)(offset_skipped_header/512),
+      (long long unsigned)(*offset/512));
+#endif
+  while(1)
+  {
+    file_space=td_list_prev_entry(file_space, list);
+    if(file_space==list_search_space)
+    {
+      offset_skipped_header=0;
+      return;
+    }
+    if(file_space->start < prev_location || file_space->start < offset_skipped_header)
+    {
+#ifdef DEBUG_HEADER_CHECK
+      log_info("get_prev_location_smart: file_space->start < prev_location=%llu (in 512-bytes sectors), offset=%llu\n",
+	  (long long unsigned)(prev_location/512),
+	  (long long unsigned)(*offset/512));
+#endif
+      offset_skipped_header=0;
+      return ;
+    }
+    *current_search_space=file_space;
+    *offset=file_space->start;
+  }
+}
+
+int my_fseek(FILE *stream, off_t offset, int whence)
+{
+#if defined(HAVE_FSEEKO) && !defined(__MINGW32__) && !defined(__ARM_EABI__)
+  {
+    int res;
+    if((res=fseeko(stream, offset, whence))>=0)
+      return res;
+  }
+#endif
+ return fseek(stream, offset, whence);
+}
+
+time_t get_time_from_YYMMDDHHMMSS(const char *date_asc)
+{
+  struct tm tm_time;
+  memset(&tm_time, 0, sizeof(tm_time));
+  tm_time.tm_sec=(date_asc[10]-'0')*10+(date_asc[11]-'0');	/* seconds 0-59 */
+  tm_time.tm_min=(date_asc[8]-'0')*10+(date_asc[9]-'0');      /* minutes 0-59 */
+  tm_time.tm_hour=(date_asc[6]-'0')*10+(date_asc[7]-'0');     /* hours   0-23*/
+  tm_time.tm_mday=(date_asc[4]-'0')*10+(date_asc[5]-'0');	/* day of the month 1-31 */
+  tm_time.tm_mon=(date_asc[2]-'0')*10+(date_asc[3]-'0')-1;	/* month 1-12 */
+  tm_time.tm_year=(date_asc[0]-'0')*10+(date_asc[1]-'0');        	/* year */
+  if(tm_time.tm_year<80)
+    tm_time.tm_year+=100;	/* year 2000 - 2079 */
+  tm_time.tm_isdst = -1;	/* unknown daylight saving time */
+  return mktime(&tm_time);
+}
+
+time_t get_time_from_YYYY_MM_DD_HH_MM_SS(const char *date_asc)
+{
+  struct tm tm_time;
+  if(memcmp(date_asc, "0000", 4)==0)
+    return (time_t)0;
+  memset(&tm_time, 0, sizeof(tm_time));
+  tm_time.tm_sec=(date_asc[17]-'0')*10+(date_asc[18]-'0');      /* seconds 0-59 */
+  tm_time.tm_min=(date_asc[14]-'0')*10+(date_asc[15]-'0');      /* minutes 0-59 */
+  tm_time.tm_hour=(date_asc[11]-'0')*10+(date_asc[12]-'0');     /* hours   0-23*/
+  tm_time.tm_mday=(date_asc[8]-'0')*10+(date_asc[9]-'0');	/* day of the month 1-31 */
+  tm_time.tm_mon=(date_asc[5]-'0')*10+(date_asc[6]-'0')-1;	/* month 0-11 */
+  tm_time.tm_year=(date_asc[0]-'0')*1000+(date_asc[1]-'0')*100+
+    (date_asc[2]-'0')*10+(date_asc[3]-'0')-1900;        	/* year */
+  tm_time.tm_isdst = -1;		/* unknown daylight saving time */
+  return mktime(&tm_time);
+}
+
+time_t get_time_from_YYYY_MM_DD_HHMMSS(const char *date_asc)
+{
+  struct tm tm_time;
+  if(memcmp(date_asc, "0000", 4)==0)
+    return (time_t)0;
+  memset(&tm_time, 0, sizeof(tm_time));
+  tm_time.tm_sec=(date_asc[15]-'0')*10+(date_asc[16]-'0');      /* seconds 0-59 */
+  tm_time.tm_min=(date_asc[13]-'0')*10+(date_asc[14]-'0');      /* minutes 0-59 */
+  tm_time.tm_hour=(date_asc[11]-'0')*10+(date_asc[12]-'0');     /* hours   0-23*/
+  tm_time.tm_mday=(date_asc[8]-'0')*10+(date_asc[9]-'0');	/* day of the month 1-31 */
+  tm_time.tm_mon=(date_asc[5]-'0')*10+(date_asc[6]-'0')-1;	/* month 0-11 */
+  tm_time.tm_year=(date_asc[0]-'0')*1000+(date_asc[1]-'0')*100+
+    (date_asc[2]-'0')*10+(date_asc[3]-'0')-1900;        	/* year */
+  tm_time.tm_isdst = -1;		/* unknown daylight saving time */
+  return mktime(&tm_time);
+}
+
+time_t get_time_from_YYYYMMDD_HHMMSS(const char *date_asc)
+{
+  struct tm tm_time;
+  memset(&tm_time, 0, sizeof(tm_time));
+  tm_time.tm_sec=(date_asc[13]-'0')*10+(date_asc[14]-'0');      /* seconds 0-59 */
+  tm_time.tm_min=(date_asc[11]-'0')*10+(date_asc[12]-'0');      /* minutes 0-59 */
+  tm_time.tm_hour=(date_asc[9]-'0')*10+(date_asc[10]-'0');      /* hours   0-23*/
+  tm_time.tm_mday=(date_asc[6]-'0')*10+(date_asc[7]-'0');	/* day of the month 1-31 */
+  tm_time.tm_mon=(date_asc[4]-'0')*10+(date_asc[5]-'0')-1;	/* month 0-11 */
+  tm_time.tm_year=(date_asc[0]-'0')*1000+(date_asc[1]-'0')*100+
+    (date_asc[2]-'0')*10+(date_asc[3]-'0')-1900;		/* year */
+  tm_time.tm_isdst = -1;		/* unknown daylight saving time */
+  return mktime(&tm_time);
 }
