@@ -47,6 +47,7 @@
 #include "fat32.h"
 #include "tntfs.h"
 #include "thfs.h"
+#include "partgpt.h"
 #include "partmacn.h"
 
 #define RO 1
@@ -59,12 +60,10 @@ extern const arch_fnct_t arch_mac;
 extern const arch_fnct_t arch_none;
 extern const arch_fnct_t arch_sun;
 extern const arch_fnct_t arch_xbox;
-static list_part_t *reduce_structure(list_part_t *list_part);
 static int use_backup(disk_t *disk_car, const list_part_t *list_part, const int verbose,const int dump_ind, const unsigned int expert, char**current_cmd);
 static int interface_part_bad_log(disk_t *disk_car,list_part_t *list_part_bad);
 #ifdef HAVE_NCURSES
 static int interface_part_bad_ncurses(disk_t *disk_car, list_part_t *list_part_bad);
-static void warning_geometry_ncurses(disk_t *disk_car, const unsigned int recommanded_heads_per_cylinder);
 static void ask_mbr_order_i386(disk_t *disk_car,list_part_t *list_part);
 #define ANALYSE_X	0
 #define ANALYSE_Y	5
@@ -115,22 +114,39 @@ static unsigned int align_structure_aux(const uint64_t offset, const disk_t *dis
   return disk->sector_size;
 }
 
+static void align_structure_i386(list_part_t *list_part, const disk_t *disk, const unsigned int align)
+{
+  list_part_t *element;
+  for(element=list_part; element!=NULL; element=element->next)
+  {
+    uint64_t partition_end;
+    unsigned int location_boundary;
+    const partition_t *part=element->part;
+    if(align==0)
+      location_boundary=disk->sector_size;
+    else
+      location_boundary=align_structure_aux(part->part_offset, disk);
+    partition_end=(part->part_offset+part->part_size-1+location_boundary-1)/location_boundary*location_boundary-1;
+    if(align!=0 && element->next!=NULL)
+    {
+      const partition_t *next_partition=element->next->part;
+      if( next_partition->part_offset > part->part_offset + part->part_size -1 &&
+	  next_partition->part_offset <= partition_end)
+      {
+	/* Do not align the partition if it overlaps the next one because of that */
+	location_boundary=disk->sector_size;
+	partition_end=(part->part_offset + part->part_size-1+location_boundary-1)/location_boundary*location_boundary-1;
+      }
+    }
+    element->part->part_size=partition_end - part->part_offset+1;
+  }
+}
+
 static void align_structure(list_part_t *list_part, const disk_t *disk, const unsigned int align)
 {
   if(disk->arch==&arch_i386)
   {
-    list_part_t *element;
-    for(element=list_part; element!=NULL; element=element->next)
-    {
-      uint64_t partition_end;
-      unsigned int location_boundary;
-      if(align==0)
-	location_boundary=disk->sector_size;
-      else 
-	location_boundary=align_structure_aux(element->part->part_offset, disk);
-      partition_end=(element->part->part_offset+element->part->part_size-1+location_boundary-1)/location_boundary*location_boundary-1;
-      element->part->part_size=partition_end-element->part->part_offset+1;
-    }
+    align_structure_i386(list_part, disk, align);
     return ;
   }
   {
@@ -186,7 +202,7 @@ static int interface_part_bad_ncurses(disk_t *disk_car, list_part_t *list_part)
 	buffer_disk_size, buffer_disk_size_found);
   }
   wmove(stdscr,7,0);
-  wprintw(stdscr,"Check the harddisk size: HD jumpers settings, BIOS detection...");
+  wprintw(stdscr,"Check the harddisk size: HD jumper settings, BIOS detection...");
 #if defined(__CYGWIN__) || defined(__MINGW32__)
   if(disk_car->disk_size<=((uint64_t)1<<(28-1)) && disk_size>=((uint64_t)1<<(28-1)))
   {
@@ -305,7 +321,7 @@ static int interface_part_bad_log(disk_t *disk_car, list_part_t *list_part)
     }
   }
   log_warning("%s\n",disk_car->description(disk_car));
-  log_warning("Check the harddisk size: HD jumpers settings, BIOS detection...\n");
+  log_warning("Check the harddisk size: HD jumper settings, BIOS detection...\n");
 #if defined(__CYGWIN__) || defined(__MINGW32__)
   if(disk_car->disk_size<=((uint64_t)1<<(28-1)) && disk_size>=((uint64_t)1<<(28-1)))
   {
@@ -352,7 +368,7 @@ static void warning_geometry_ncurses(disk_t *disk_car, const unsigned int recomm
   wmove(stdscr,10,0);
   wprintw(stdscr,"- some partitions are not found by TestDisk");
   wmove(stdscr,11,0);
-  wprintw(stdscr,"- or the partition table can not be written because partitions overlaps.");
+  wprintw(stdscr,"- or the partition table can not be written because partitions overlap.");
   wmove(stdscr,22,0);
   wattrset(stdscr, A_REVERSE);
   wprintw(stdscr,"[ Continue ]");
@@ -418,7 +434,15 @@ static void search_add_hints(const disk_t *disk, uint64_t *try_offset, unsigned
 	hint_insert(try_offset, CHS2offset_inline(disk, &start), try_offset_nbr);
       }
     }
-    hint_insert(try_offset, (disk->disk_size-512)/(2048*512)*(2048*512)+(2048-1)*512, try_offset_nbr);
+    hint_insert(try_offset, (disk->disk_size-disk->sector_size)/(2048*512)*(2048*512)-disk->sector_size, try_offset_nbr);
+  }
+  else if(disk->arch==&arch_gpt)
+  {
+    /* Hint for NTFS backup */
+    const unsigned int gpt_entries_size=128*sizeof(struct gpt_ent);
+    const uint64_t hdr_lba_end=le64((disk->disk_size-1 - gpt_entries_size)/disk->sector_size - 1);
+    const uint64_t ntfs_backup_offset=(hdr_lba_end-1)*disk->sector_size/(2048*512)*(2048*512)-disk->sector_size;
+    hint_insert(try_offset, ntfs_backup_offset, try_offset_nbr);
   }
   else if(disk->arch==&arch_mac)
   {
@@ -597,9 +621,9 @@ static list_part_t *search_part(disk_t *disk_car, const list_part_t *list_part_o
     {
       wmove(stdscr,ANALYSE_Y,ANALYSE_X);
       wclrtoeol(stdscr);
-      wprintw(stdscr,"Analyse sector %11llu/%lu: %02u%%",
-	  search_location / disk_car->sector_size,
-	  (disk_car->disk_size-1)/disk_car->sector_size,
+      wprintw(stdscr,"Analyse sector %11llu/%llu: %02u%%",
+	  (long long unsigned)(search_location / disk_car->sector_size),
+	  (long long unsigned)((disk_car->disk_size-1)/disk_car->sector_size),
 	    (unsigned int)(search_location*100/disk_car->disk_size));
       wrefresh(stdscr);
       switch(check_enter_key_or_s(stdscr))
@@ -649,7 +673,7 @@ static list_part_t *search_part(disk_t *disk_car, const list_part_t *list_part_o
         int res=0;
         partition->part_size=(uint64_t)0;
         partition->part_offset=search_location;
-        if(res<=0 && test_nbr==0)
+        if(test_nbr==0)
         {
           if(search_now_raid>0 || fast_mode>1)
           { /* Search Linux software RAID */
@@ -698,7 +722,7 @@ static list_part_t *search_part(disk_t *disk_car, const list_part_t *list_part_o
 		 search_location%(2048*512)==(13-1)*disk_car->sector_size)) ||
 	      (disk_car->arch==&arch_gpt&& (search_location%(2048*512)==(13-1)*512)) ||
 	      (disk_car->arch!=&arch_i386 && (search_location%location_boundary==(13-1)*disk_car->sector_size)))
-	    res=search_EXFAT_backup(buffer_disk, disk_car, partition);
+	    res=search_exFAT_backup(buffer_disk, disk_car, partition);
 	  test_nbr++;
 	}
         if(res<=0 && test_nbr==3)
@@ -870,7 +894,7 @@ static list_part_t *search_part(disk_t *disk_car, const list_part_t *list_part_o
               }
               {
                 const uint64_t next_part_offset=partition->part_offset+partition->part_size-1+1;
-                const uint64_t head_size=disk_car->geom.sectors_per_head * disk_car->sector_size;
+                const uint64_t head_size=(uint64_t)disk_car->geom.sectors_per_head * disk_car->sector_size;
                 hint_insert(try_offset, next_part_offset, &try_offset_nbr);
                 hint_insert(try_offset, next_part_offset+head_size, &try_offset_nbr);
                 if(next_part_offset%head_size!=0)
@@ -986,7 +1010,7 @@ static void ask_mbr_order_i386(disk_t *disk_car,list_part_t *list_part)
       table[nbr_prim++]=element->part;
   }
   /* */
-  log_info("\nSelect primaries partition\n");
+  log_info("\nSelect primary partition\n");
   for(i=0;i<nbr_prim;i++)
       log_partition(disk_car,table[i]);
   /* */
@@ -1080,27 +1104,22 @@ static void ask_mbr_order_i386(disk_t *disk_car,list_part_t *list_part)
 }
 #endif
 
-static list_part_t *reduce_structure(list_part_t *list_part)
+static list_part_t *reduce_structure(const list_part_t *list_part_org)
 {
-  list_part_t *element=list_part;
-  list_part_t *prev=NULL;
-  while(element)
+
+  const list_part_t *element;
+  list_part_t *list_part=NULL;
+  for(element=list_part_org; element!=NULL; element=element->next)
   {
-    list_part_t *next=element->next;
-    if(element->part->status==STATUS_DELETED)
+    if(element->part->status!=STATUS_DELETED)
     {
-      if(prev==NULL)
-	list_part=next;
-      else
-	prev->next=next;
-      if(next!=NULL)
-	next->prev=prev;
-      free(element->part);
-      free(element);
+      int insert_error=0;
+      partition_t *new_partition=partition_new(NULL);
+      dup_partition_t(new_partition, element->part);
+      list_part=insert_new_partition(list_part, new_partition, 0, &insert_error);
+      if(insert_error>0)
+	free(new_partition);
     }
-    else
-      prev=element;
-    element=next;
   }
   return list_part;
 }
@@ -1153,6 +1172,7 @@ static list_part_t *add_ext_part_i386(disk_t *disk, list_part_t *list_part, cons
   }
   if(deb==NULL)
     return list_part;
+  assert(fin!=NULL);
   if(nbr_entries==4 || max_ext!=0)
   {
     if(verbose>0)
@@ -1310,7 +1330,142 @@ static int use_backup(disk_t *disk_car, const list_part_t *list_part, const int
   return 0;
 }
 
-int interface_recovery(disk_t *disk_car, const list_part_t * list_part_org, const int verbose, const int dump_ind, const int align, const int ask_part_order, const unsigned int expert, char **current_cmd)
+static void warning_geometry(const list_part_t *list_part, disk_t *disk, const int verbose, char **current_cmd)
+{
+  if(list_part!=NULL && (disk->arch==&arch_i386 || disk->arch==&arch_sun))
+  { /* Correct disk geometry is necessary for successfull Intel and Sun partition recovery */
+    const unsigned int heads_per_cylinder=get_geometry_from_list_part(disk, list_part, verbose);
+    if(disk->geom.heads_per_cylinder!=heads_per_cylinder)
+    {
+      log_warning("Warning: the current number of heads per cylinder is %u but the correct value may be %u.\n",
+	  disk->geom.heads_per_cylinder, heads_per_cylinder);
+#ifdef HAVE_NCURSES
+      if(*current_cmd==NULL)
+      {
+	warning_geometry_ncurses(disk, heads_per_cylinder);
+      }
+#endif
+    }
+  }
+}
+
+static int ask_write_partition_table(const list_part_t *list_part_org, disk_t *disk_car, const int verbose, const int dump_ind, const int ask_part_order, const unsigned int expert, char **current_cmd, unsigned int *menu, int *fast_mode)
+{
+  int res_interface_write;
+  int do_again=0;
+  int max_ext=0;
+  int can_ask_minmax_ext=0;
+  int no_confirm=0;
+  list_part_t *list_part;
+  list_part=reduce_structure(list_part_org);
+  /* sort list_part */
+  list_part=sort_partition_list(list_part);
+  /* Create PC/Intel Extended partition */
+  /* if(disk_car->arch==&arch_i386) */
+  {
+    list_part_t *parts;
+    uint64_t partext_offset=0;
+    uint64_t partext_size=0;
+    list_part=add_ext_part_i386(disk_car, list_part, !max_ext, verbose);
+    for(parts=list_part;parts!=NULL;parts=parts->next)
+      if(parts->part->status==STATUS_EXT)
+      {
+	partext_offset=parts->part->part_offset;
+	partext_size=parts->part->part_size;
+      }
+    if(partext_offset>0)
+    {
+      list_part=add_ext_part_i386(disk_car, list_part, max_ext, verbose);
+      for(parts=list_part;parts!=NULL;parts=parts->next)
+	if(parts->part->status==STATUS_EXT)
+	{
+	  if(partext_offset!=parts->part->part_offset || partext_size!=parts->part->part_size)
+	    can_ask_minmax_ext=1;
+	}
+    }
+  }
+  list_part=disk_car->arch->init_part_order(disk_car,list_part);
+  if(ask_part_order!=0)
+  {
+    /* Demande l'ordre des entrees dans le MBR */
+#ifdef HAVE_NCURSES
+    ask_mbr_order_i386(disk_car,list_part);
+#endif
+    /* Demande l'ordre des partitions etendues */
+  }
+  do
+  {
+    do_again=0;
+    res_interface_write=interface_write(disk_car,list_part,((*fast_mode)<1),can_ask_minmax_ext, &no_confirm, current_cmd, menu);
+    switch(res_interface_write)
+    {
+      case 'W':
+	if(disk_car->arch == &arch_mac)
+	{
+#ifdef HAVE_NCURSES
+	  write_part_mac_warning_ncurses();
+#endif
+	}
+	else if(disk_car->arch == &arch_sun)
+	{
+#ifdef HAVE_NCURSES
+	  not_implemented("write_part_sun");
+#endif
+	}
+	else if(disk_car->arch == &arch_xbox)
+	{
+#ifdef HAVE_NCURSES
+	  not_implemented("write_part_xbox");
+#endif
+	}
+	else if(disk_car->arch->write_part!=NULL)
+	{
+	  if(no_confirm!=0
+#ifdef HAVE_NCURSES
+	      || ask_confirmation("Write partition table, confirm ? (Y/N)")!=0
+#endif
+	    )
+	  {
+	    log_info("write!\n");
+	    if(disk_car->arch->write_part(disk_car, list_part, RW, verbose))
+	    {
+	      display_message(msg_PART_WR_ERR);
+	    }
+	    else
+	    {
+	      use_backup(disk_car,list_part,verbose,dump_ind,expert,current_cmd);
+	      if(no_confirm==0)
+		display_message("You will have to reboot for the change to take effect.\n");
+	    }
+	  }
+	  else
+	    log_info("Don't write, no confirmation\n");
+	}
+	break;
+      case 0:
+	if(disk_car->arch->write_part!=NULL)
+	{
+	  log_info("simulate write!\n");
+	  disk_car->arch->write_part(disk_car, list_part, RO, verbose);
+	}
+	break;
+      case 'S':
+	if((*fast_mode) < 2)
+	  (*fast_mode)++;
+	break;
+      case 'E':
+	max_ext=!max_ext;
+	list_part=add_ext_part_i386(disk_car, list_part, max_ext, verbose);
+	do_again=1;
+	break;
+    }
+  }
+  while(do_again==1);
+  part_free_list(list_part);
+  return res_interface_write;
+}
+
+int interface_recovery(disk_t *disk_car, const list_part_t *list_part_org, const int verbose, const int dump_ind, const int align, const int ask_part_order, const unsigned int expert, char **current_cmd)
 {
   int res_interface_write;
   int fast_mode=0;
@@ -1320,30 +1475,15 @@ int interface_recovery(disk_t *disk_car, const list_part_t * list_part_org, cons
     const list_part_t *element;
     unsigned int menu=0;
     if(fast_mode==0)
-      menu=3;	/* Search! */
+      menu=4;	/* Search! */
 #ifdef HAVE_NCURSES
     aff_copy(stdscr);
     wmove(stdscr,4,0);
     wprintw(stdscr,"%s",disk_car->description(disk_car));
     wmove(stdscr,5,0);
 #endif
-    res_interface_write=0;
     list_part=search_part(disk_car, list_part_org, verbose, dump_ind, fast_mode, current_cmd);
-    if(list_part!=NULL && (disk_car->arch==&arch_i386 || disk_car->arch==&arch_sun))
-    { /* Correct disk geometry is necessary for successfull Intel and Sun partition recovery */
-      const unsigned int heads_per_cylinder=get_geometry_from_list_part(disk_car, list_part, verbose);
-      if(disk_car->geom.heads_per_cylinder!=heads_per_cylinder)
-      {
-	log_warning("Warning: the current number of heads per cylinder is %u but the correct value may be %u.\n",
-	    disk_car->geom.heads_per_cylinder, heads_per_cylinder);
-#ifdef HAVE_NCURSES
-	if(*current_cmd==NULL)
-	{
-	  warning_geometry_ncurses(disk_car, heads_per_cylinder);
-	}
-#endif
-      }
-    }
+    warning_geometry(list_part, disk_car, verbose, current_cmd);
     align_structure(list_part, disk_car, align);
 
     disk_car->arch->init_structure(disk_car,list_part,verbose);
@@ -1358,7 +1498,7 @@ int interface_recovery(disk_t *disk_car, const list_part_t * list_part_org, cons
 	log_partition(disk_car,element->part);
 #ifdef TARGET_LINUX
       if(list_part!=NULL)
-	log_info("\nHint for advanced users. dmsetup may be used if you prefer to avoid to rewrite the partition table for the moment:\n");
+	log_info("\nHint for advanced users: dmsetup may be used if you prefer to avoid rewriting the partition table for the moment:\n");
       for(element=list_part;element!=NULL;element=element->next)
       {
 	const partition_t *partition=element->part;
@@ -1370,122 +1510,20 @@ int interface_recovery(disk_t *disk_car, const list_part_t * list_part_org, cons
       }
 #endif
     }
-    list_part=ask_structure(disk_car,list_part,verbose,current_cmd);
-    if(disk_car->arch->test_structure(list_part)==0)
+    log_flush();
+    do
     {
-      int do_again=0;
-      int max_ext=0;
-      int can_ask_minmax_ext=0;
-      int no_confirm=0;
-      list_part=reduce_structure(list_part);
-      /* sort list_part */
-      list_part=sort_partition_list(list_part);
-      /* Create PC/Intel Extended partition */
-      /* if(disk_car->arch==&arch_i386) */
+      list_part=ask_structure(disk_car,list_part,verbose,current_cmd);
+      if(disk_car->arch->test_structure(list_part)==0)
       {
-	list_part_t *parts;
-	uint64_t partext_offset=0;
-	uint64_t partext_size=0;
-	list_part=add_ext_part_i386(disk_car, list_part, !max_ext, verbose);
-	for(parts=list_part;parts!=NULL;parts=parts->next)
-	  if(parts->part->status==STATUS_EXT)
-	  {
-	    partext_offset=parts->part->part_offset;
-	    partext_size=parts->part->part_size;
-	  }
-	if(partext_offset>0)
-	{
-	  list_part=add_ext_part_i386(disk_car, list_part, max_ext, verbose);
-	  for(parts=list_part;parts!=NULL;parts=parts->next)
-	    if(parts->part->status==STATUS_EXT)
-	    {
-	      if(partext_offset!=parts->part->part_offset || partext_size!=parts->part->part_size)
-		can_ask_minmax_ext=1;
-	    }
-	}
+	res_interface_write=ask_write_partition_table(list_part, disk_car, verbose, dump_ind, ask_part_order, expert, current_cmd, &menu, &fast_mode);
       }
-      list_part=disk_car->arch->init_part_order(disk_car,list_part);
-      if(ask_part_order!=0)
+      else
       {
-	/* Demande l'ordre des entrees dans le MBR */
-#ifdef HAVE_NCURSES
-	ask_mbr_order_i386(disk_car,list_part);
-#endif
-	/* Demande l'ordre des partitions etendues */
+	display_message("Invalid partition structure.\n");
+	res_interface_write=0;
       }
-      do
-      {
-	do_again=0;
-	res_interface_write=interface_write(disk_car,list_part,(fast_mode<1),can_ask_minmax_ext, &no_confirm, current_cmd,&menu);
-	switch(res_interface_write)
-	{
-	  case 'W':
-	    if(disk_car->arch == &arch_mac)
-	    {
-#ifdef HAVE_NCURSES
-	      write_part_mac_warning_ncurses();
-#endif
-	    }
-	    else if(disk_car->arch == &arch_sun)
-	    {
-#ifdef HAVE_NCURSES
-	      not_implemented("write_part_sun");
-#endif
-	    }
-	    else if(disk_car->arch == &arch_xbox)
-	    {
-#ifdef HAVE_NCURSES
-	      not_implemented("write_part_xbox");
-#endif
-	    }
-	    else if(disk_car->arch->write_part!=NULL)
-	    {
-	      if(no_confirm!=0
-#ifdef HAVE_NCURSES
-		  || ask_confirmation("Write partition table, confirm ? (Y/N)")!=0
-#endif
-		)
-	      {
-		log_info("write!\n");
-		if(disk_car->arch->write_part(disk_car, list_part, RW, verbose))
-		{
-		  display_message(msg_PART_WR_ERR);
-		}
-		else
-		{
-		  use_backup(disk_car,list_part,verbose,dump_ind,expert,current_cmd);
-		  if(no_confirm==0)
-		    display_message("You will have to reboot for the change to take effect.\n");
-		}
-	      }
-	      else
-		log_info("Don't write, no confirmation\n");
-	    }
-	    break;
-	  case 0:
-	    if(disk_car->arch->write_part!=NULL)
-	    {
-	      log_info("simulate write!\n");
-	      disk_car->arch->write_part(disk_car, list_part, RO, verbose);
-	    }
-	    break;
-	  case 'S':
-	    if(fast_mode<2)
-	      fast_mode++;
-	    break;
-	  case 'E':
-	    max_ext=!max_ext;
-	    list_part=add_ext_part_i386(disk_car, list_part, max_ext, verbose);
-	    do_again=1;
-	    break;
-	}
-      }
-      while(do_again==1);
-    }
-    else
-    {
-      display_message("Invalid partition structure.\n");
-    }
+    } while(res_interface_write == 'R');
     part_free_list(list_part);
   } while(res_interface_write=='S');
   return 0;
