@@ -101,6 +101,9 @@
 #if defined(__CYGWIN__)
 #include <io.h>
 #endif
+#if defined(__HAIKU__)
+#include <Drivers.h>
+#endif
 #include "fnctdsk.h"
 #include "ewf.h"
 #include "log.h"
@@ -256,7 +259,7 @@ list_disk_t *hd_parse(list_disk_t *list_disk, const int verbose, const int testd
     if(list_disk==NULL)
       do_insert=1;
     {
-      for(i=0;i<16;i++)
+      for(i=0;i<64;i++)
       {
 	disk_t *disk_car;
 	sprintf(device_hd,"\\\\.\\PhysicalDrive%u", i);
@@ -299,7 +302,6 @@ list_disk_t *hd_parse(list_disk_t *list_disk, const int verbose, const int testd
     int j;
     char device[100];
     char device_ide[]="/dev/hda";
-    char device_scsi[]="/dev/sda";
     char device_ida[]="/dev/ida/c0d0";
     char device_cciss[]="/dev/cciss/c0d0";
     char device_p_ide[]="/dev/pda";
@@ -311,12 +313,6 @@ list_disk_t *hd_parse(list_disk_t *list_disk, const int verbose, const int testd
       device_ide[strlen(device_ide)-1]='a'+i;
       list_disk=insert_new_disk(list_disk, file_test_availability(device_ide, verbose, testdisk_mode));
     }
-    /* Disk SCSI */
-    for(i=0;i<26;i++)
-    {
-      device_scsi[strlen(device_scsi)-1]='a'+i;
-      list_disk=insert_new_disk(list_disk, file_test_availability(device_scsi, verbose, testdisk_mode));
-    }
     /* Device RAID Compaq */
     for(j=0;j<8;j++)
     {
@@ -363,14 +359,23 @@ list_disk_t *hd_parse(list_disk_t *list_disk, const int verbose, const int testd
       list_disk=insert_new_disk(list_disk, file_test_availability(device_mmc, verbose, testdisk_mode));
     }
 #ifdef HAVE_GLOB_H
+    /* Disk SCSI */
+    list_disk=hd_glob_parse("/dev/sd[a-z]", list_disk, verbose, testdisk_mode);
+    list_disk=hd_glob_parse("/dev/sd[a-z][a-z]", list_disk, verbose, testdisk_mode);
     list_disk=hd_glob_parse("/dev/mapper/*", list_disk, verbose, testdisk_mode);
     /* Software Raid (partition level) */
     list_disk=hd_glob_parse("/dev/md*", list_disk, verbose, testdisk_mode);
     list_disk=hd_glob_parse("/dev/sr?", list_disk, verbose, testdisk_mode);
     /* Software (ATA)Raid configured (disk level) via dmraid */
     list_disk=hd_glob_parse("/dev/dm-*", list_disk, verbose, testdisk_mode);
+    /* VirtIO block devices */
+    list_disk=hd_glob_parse("/dev/vd[a-z]", list_disk, verbose, testdisk_mode);
     /* Xen virtual disks */
     list_disk=hd_glob_parse("/dev/xvd?", list_disk, verbose, testdisk_mode);
+    /* Loop devices */
+    list_disk=hd_glob_parse("/dev/loop[0-9]*", list_disk, verbose, testdisk_mode);
+    /* NVME */
+    list_disk=hd_glob_parse("/dev/nvme[0-9]n[0-9]", list_disk, verbose, testdisk_mode);
 #endif
   }
 #elif defined(TARGET_SOLARIS)
@@ -388,7 +393,7 @@ list_disk_t *hd_parse(list_disk_t *list_disk, const int verbose, const int testd
 #elif defined(__HAIKU__)
   {
 #ifdef HAVE_GLOB_H
-    list_disk=hd_glob_parse("/dev/disk/*/*/master/raw", list_disk, verbose, testdisk_mode);
+    list_disk=hd_glob_parse("/dev/disk/*/*/*/raw", list_disk, verbose, testdisk_mode);
 #endif
   }
 #else
@@ -568,6 +573,21 @@ static unsigned int disk_get_sector_size(const int hd_h, const char *device, con
 #endif
     return disk_get_sector_size_win32(handle, device, verbose);
   }
+#endif
+#ifdef __HAIKU__
+  {
+    device_geometry g;
+    int error;
+    error = ioctl(hd_h, B_GET_GEOMETRY, &g, sizeof(g));
+    if(error==0)
+    {
+      if(verbose>1)
+      {
+	log_verbose("disk_get_sector_size B_GET_GEOMETRY %s sector_size=%u\n",device,g.bytes_per_sector);
+      }
+    return g.bytes_per_sector;
+    }
+  }
 #endif
   if(verbose>1)
   {
@@ -698,6 +718,23 @@ static void disk_get_geometry(CHSgeometry_t *geom, const int hd_h, const char *d
 #endif
     return disk_get_geometry_win32(geom, handle, device, verbose);
   }
+#endif
+#ifdef __HAIKU__
+  {
+    device_geometry g;
+    int error;
+    error = ioctl(hd_h, B_GET_GEOMETRY, &g, sizeof(g));
+    if(error==0)
+    {
+      if(verbose>1)
+      {
+	log_verbose("disk_get_geometry B_GET_GEOMETRY %s Ok\n", device);
+      }
+	geom->cylinders=g.cylinder_count;
+	geom->heads_per_cylinder=g.head_count;
+	geom->sectors_per_head=g.sectors_per_track;
+    }
+  }
 #endif
   if(geom->sectors_per_head>0 && geom->heads_per_cylinder>0)
     return ;
@@ -784,6 +821,23 @@ static uint64_t disk_get_size(const int hd_h, const char *device, const int verb
 #endif
     return disk_get_size_win32(handle, device, verbose);
   }
+#endif
+#ifdef __HAIKU__
+  {
+    device_geometry g;
+    int error;
+    error = ioctl(hd_h, B_GET_GEOMETRY, &g, sizeof(g));
+    if(error==0)
+    {
+      off_t o = (off_t)g.bytes_per_sector * g.sectors_per_track * g.cylinder_count * g.head_count;
+      if(verbose>1)
+      {
+	log_verbose("disk_get_size B_GET_GEOMETRY %s size %llu\n",
+	    device, (long long unsigned)o);
+      }
+      return o;
+    }
+  }
 #endif
   return compute_device_size(hd_h, device, verbose, sector_size);
 }
@@ -808,7 +862,7 @@ void update_disk_car_fields(disk_t *disk_car)
       (uint64_t)disk_car->sector_size;
     if(cylinder_num>0 && disk_car->geom.cylinders != cylinder_num)
     {
-      log_debug("Fix cylinder count for %s: number of cylinders %lu !=  %lu (calculated)\n",
+      log_debug("Fix cylinder count for %s: number of cylinders %lu != %lu (calculated)\n",
 	  disk_car->device, disk_car->geom.cylinders, cylinder_num);
       disk_car->geom.cylinders = cylinder_num;
     }
@@ -1020,7 +1074,8 @@ static uint64_t compute_device_size(const int hd_h, const char *device, const in
 {
 #ifdef HAVE_PREAD
   /* This function can failed if there are bad sectors */
-  uint64_t min_offset, max_offset;
+  uint64_t min_offset;
+  uint64_t max_offset;
   char *buffer=(char *)MALLOC(sector_size);
   min_offset=0;
   max_offset=sector_size;
@@ -1319,7 +1374,7 @@ static void autoset_geometry(disk_t *disk, const unsigned char *buffer, const in
     geometry.bytes_per_sector=0;
     disk->arch->get_geometry_from_mbr(buffer, verbose, &geometry);
     disk->autodetect=1;
-    if(geometry.sectors_per_head > 0)
+    if(geometry.sectors_per_head > 0 && geometry.heads_per_cylinder > 0)
     {
       disk->geom.heads_per_cylinder=geometry.heads_per_cylinder;
       disk->geom.sectors_per_head=geometry.sectors_per_head;
@@ -1394,14 +1449,22 @@ disk_t *file_test_availability(const char *device, const int verbose, int testdi
        * read-only loop devices may be openend read-write,
        * use BKROGET to detect the problem
        */
-      if (ioctl(hd_h, BLKROGET, &readonly) >= 0)
+      if (ioctl(hd_h, BLKROGET, &readonly) >= 0 &&
+	  readonly>0)
       {
-        if(readonly>0)
-        {
-          try_readonly=1;
-          close(hd_h);
-	  hd_h = -1;
-        }
+	try_readonly=1;
+	close(hd_h);
+	hd_h = -1;
+      }
+#endif
+#ifdef __HAIKU__
+      device_geometry g;
+      if (ioctl(hd_h, B_GET_GEOMETRY, &g, sizeof(g)) >= 0 &&
+        g.read_only>0)
+      {
+	try_readonly=1;
+	close(hd_h);
+	hd_h = -1;
       }
 #endif
     }
@@ -1465,23 +1528,22 @@ disk_t *file_test_availability(const char *device, const int verbose, int testdi
   data->handle=hd_h;
   data->mode=mode;
   disk_car->data=data;
-  disk_car->description=file_description;
-  disk_car->description_short=file_description_short;
-  disk_car->pread=file_pread;
-  disk_car->pwrite=((mode&O_RDWR)==O_RDWR?file_pwrite:file_nopwrite);
-  disk_car->sync=file_sync;
+  disk_car->description=&file_description;
+  disk_car->description_short=&file_description_short;
+  disk_car->pread=&file_pread;
+  disk_car->pwrite=((mode&O_RDWR)==O_RDWR?&file_pwrite:&file_nopwrite);
+  disk_car->sync=&file_sync;
   disk_car->access_mode=((mode&O_RDWR)==O_RDWR?TESTDISK_O_RDWR:TESTDISK_O_RDONLY);
 #ifdef O_DIRECT
   if((mode&O_DIRECT)==O_DIRECT)
     disk_car->access_mode|=TESTDISK_O_DIRECT;
 #endif
-  disk_car->clean=file_clean;
-  if(fstat(hd_h,&stat_rec)>=0)
+  disk_car->clean=&file_clean;
+  if(fstat(hd_h,&stat_rec)>=0 &&
+      S_ISREG(stat_rec.st_mode) &&
+      stat_rec.st_size > 0)
   {
-    if(S_ISREG(stat_rec.st_mode) && stat_rec.st_size > 0)
-    {
-      device_is_a_file=1;
-    }
+    device_is_a_file=1;
   }
 #ifndef DJGPP
   if(device_is_a_file==0)
