@@ -64,41 +64,12 @@ struct cache_struct
 
 static int cache_pread_aux(disk_t *disk_car, void *buffer, const unsigned int count, const uint64_t offset, const unsigned int read_ahead);
 static int cache_pread(disk_t *disk_car, void *buffer, const unsigned int count, const uint64_t offset);
-static void *cache_pread_fast(disk_t *disk, void *buffer, const unsigned int count, const uint64_t offset);
 static int cache_pwrite(disk_t *disk_car, const void *buffer, const unsigned int count, const uint64_t offset);
-static int cache_sync(disk_t *clean);
-static int cache_clean(disk_t *clean);
+static int cache_sync(disk_t *disk);
+static void cache_clean(disk_t *disk);
 static const char *cache_description(disk_t *disk_car);
 static const char *cache_description_short(disk_t *disk_car);
 
-static void *cache_get_data_p(disk_t *disk, const unsigned int count, const uint64_t offset)
-{
-  struct cache_struct *data=(struct cache_struct *)disk->data;
-  unsigned int i;
-  unsigned int cache_buffer_nbr;
-  for(i=0, cache_buffer_nbr=data->cache_buffer_nbr;
-      i<CACHE_BUFFER_NBR;
-      i++, cache_buffer_nbr=(cache_buffer_nbr+CACHE_BUFFER_NBR-1)%CACHE_BUFFER_NBR)
-  {
-    const struct cache_buffer_struct *cache=&data->cache[cache_buffer_nbr];
-    if(cache->buffer!=NULL && cache->cache_size>0 &&
-	cache->cache_offset <= offset &&
-	offset + count < cache->cache_offset + cache->cache_size)
-      return cache->buffer + offset - cache->cache_offset;
-  }
-  return NULL;
-}
-
-static void* cache_pread_fast(disk_t *disk, void *buffer, const unsigned int count, const uint64_t offset)
-{
-  void*data=cache_get_data_p(disk, count, offset);
-  if(data!=NULL)
-    return data;
-  if(cache_pread(disk, buffer, count, offset) == count)
-    return buffer;
-  return NULL;
-}
-
 static int cache_pread(disk_t *disk_car, void *buffer, const unsigned int count, const uint64_t offset)
 {
   const struct cache_struct *data=(const struct cache_struct *)disk_car->data;
@@ -121,9 +92,9 @@ static int cache_pread_aux(disk_t *disk_car, void *buffer, const unsigned int co
 	i++, cache_buffer_nbr=(cache_buffer_nbr+CACHE_BUFFER_NBR-1)%CACHE_BUFFER_NBR)
     {
       const struct cache_buffer_struct *cache=&data->cache[cache_buffer_nbr];
-      if(cache->buffer!=NULL && cache->cache_size>0 &&
-	  cache->cache_offset <= offset &&
-	  offset < cache->cache_offset +cache->cache_size)
+      if(cache->cache_offset <= offset &&
+	  offset < cache->cache_offset +cache->cache_size &&
+	  cache->buffer!=NULL && cache->cache_size>0)
       {
 	const unsigned int data_available= cache->cache_size + cache->cache_offset - offset;
 	const int res=cache->cache_status + cache->cache_offset - offset;
@@ -187,7 +158,7 @@ static int cache_pread_aux(disk_t *disk_car, void *buffer, const unsigned int co
     }
     /* Read failure */
     data->last_io_error_nbr++;
-    if(count<=disk_car->sector_size || disk_car->sector_size<=0 || data->last_io_error_nbr>1)
+    if(count_new<=disk_car->sector_size || disk_car->sector_size<=0 || data->last_io_error_nbr>1)
     {
       memcpy(buffer, cache->buffer, count);
       return cache->cache_status;
@@ -196,16 +167,16 @@ static int cache_pread_aux(disk_t *disk_car, void *buffer, const unsigned int co
     cache->cache_size=0;
     /* split the read sector by sector */
     {
-      unsigned int i;
+      unsigned int off;
       memset(buffer, 0, count);
-      for(i=0;i*disk_car->sector_size<count;i++)
+      for(off=0; off<count; off+=disk_car->sector_size)
       {
 	if(cache_pread_aux(disk_car, 
-	    (unsigned char*)buffer+i*disk_car->sector_size,
-	    (count>(i+1)*disk_car->sector_size?disk_car->sector_size:count - i*disk_car->sector_size),
-	    offset+i*disk_car->sector_size, 0) <= 0)
+	    (unsigned char*)buffer+off,
+	    (disk_car->sector_size < count - off ? disk_car->sector_size : count - off),
+	    offset+off, 0) <= 0)
 	{
-	  return i*disk_car->sector_size;
+	  return off;
 	}
       }
       return count;
@@ -230,7 +201,7 @@ static int cache_pwrite(disk_t *disk_car, const void *buffer, const unsigned int
   return data->disk_car->pwrite(data->disk_car, buffer, count, offset);
 }
 
-static int cache_clean(disk_t *disk_car)
+static void cache_clean(disk_t *disk_car)
 {
   if(disk_car->data)
   {
@@ -248,11 +219,10 @@ static int cache_clean(disk_t *disk_car)
       struct cache_buffer_struct *cache=&data->cache[i];
       free(cache->buffer);
     }
-    free(data->disk_car);
     free(disk_car->data);
     disk_car->data=NULL;
   }
-  return 0;
+  free(disk_car);
 }
 
 static int cache_sync(disk_t *disk_car)
@@ -294,13 +264,12 @@ disk_t *new_diskcache(disk_t *disk_car, const unsigned int testdisk_mode)
   new_disk_car->disk_real_size=disk_car->disk_real_size;
   new_disk_car->write_used=0;
   new_disk_car->data=data;
-  new_disk_car->pread_fast=cache_pread_fast;
-  new_disk_car->pread=cache_pread;
-  new_disk_car->pwrite=cache_pwrite;
-  new_disk_car->sync=cache_sync;
-  new_disk_car->clean=cache_clean;
-  new_disk_car->description=cache_description;
-  new_disk_car->description_short=cache_description_short;
+  new_disk_car->pread=&cache_pread;
+  new_disk_car->pwrite=&cache_pwrite;
+  new_disk_car->sync=&cache_sync;
+  new_disk_car->clean=&cache_clean;
+  new_disk_car->description=&cache_description;
+  new_disk_car->description_short=&cache_description_short;
   new_disk_car->rbuffer=NULL;
   new_disk_car->wbuffer=NULL;
   new_disk_car->rbuffer_size=0;
