@@ -45,9 +45,9 @@ struct search_location
 typedef struct search_location search_location_t;
 static inline uint64_t CHS_to_offset(const unsigned int C, const int H, const int S,const disk_t *disk_car);
 
-#define SEARCH_LOCATION_MAX 128
+#define SEARCH_LOCATION_MAX 256
 static unsigned int search_location_nbr=0;
-static search_location_t search_location_info[SEARCH_LOCATION_MAX];
+static search_location_t search_location_info[SEARCH_LOCATION_MAX+1];
 
 static inline uint64_t CHS_to_offset(const unsigned int C, const int H, const int S,const disk_t *disk_car)
 {
@@ -57,37 +57,56 @@ static inline uint64_t CHS_to_offset(const unsigned int C, const int H, const in
 static void update_location(void)
 {
   unsigned int i;
-  if(search_location_info[search_location_nbr].inc==0)
+  const search_location_t *src=&search_location_info[search_location_nbr];
+  if(src->inc==0)
   {
-    search_location_nbr++;
+    for(i=0; i<search_location_nbr; i++)
+    {
+      if(search_location_info[i].offset == src->offset)
+	return ;
+    }
+    if(search_location_nbr < SEARCH_LOCATION_MAX)
+      search_location_nbr++;
     return;
   }
   for(i=0; i<search_location_nbr; i++)
   {
-    if(search_location_info[i].offset==search_location_info[search_location_nbr].offset &&
-	search_location_info[i].inc >= search_location_info[search_location_nbr].inc &&
-	search_location_info[i].inc % search_location_info[search_location_nbr].inc==0)
+    search_location_t *cur=&search_location_info[i];
+    if(cur->offset == src->offset &&
+	cur->inc >= src->inc &&
+	cur->inc % src->inc==0)
     {
-      search_location_info[i].inc=search_location_info[search_location_nbr].inc;
+      cur->inc=src->inc;
       return ;
     }
-    if(search_location_info[i].offset==search_location_info[search_location_nbr].offset &&
-	search_location_info[search_location_nbr].inc >= search_location_info[i].inc &&
-	search_location_info[search_location_nbr].inc % search_location_info[i].inc==0)
-      return ;
-    if(search_location_info[i].inc==search_location_info[search_location_nbr].inc &&
-	search_location_info[i].offset >= search_location_info[search_location_nbr].offset &&
-	(search_location_info[i].offset - search_location_info[search_location_nbr].offset)%search_location_info[i].inc==0)
+    if(cur->inc == 0)
     {
-      search_location_info[i].offset=search_location_info[search_location_nbr].offset;
-      return ;
+      if(cur->offset == src->offset)
+      {
+	cur->inc = src->inc;
+	return ;
+      }
+    }
+    else
+    {
+      if(cur->offset == src->offset &&
+	  src->inc >= cur->inc && src->inc % cur->inc==0)
+	return ;
+      if(cur->inc==src->inc &&
+	  cur->offset >= src->offset &&
+	  (cur->offset - src->offset)%cur->inc==0)
+      {
+	cur->offset=src->offset;
+	return ;
+      }
+      if(cur->inc==src->inc &&
+	  src->offset >= cur->offset &&
+	  (src->offset - cur->offset)%cur->inc==0)
+	return ;
     }
-    if(search_location_info[i].inc==search_location_info[search_location_nbr].inc &&
-	search_location_info[search_location_nbr].offset >= search_location_info[i].offset &&
-	(search_location_info[search_location_nbr].offset - search_location_info[i].offset)%search_location_info[i].inc==0)
-      return ;
   }
-  search_location_nbr++;
+  if(search_location_nbr < SEARCH_LOCATION_MAX)
+    search_location_nbr++;
 }
 
 void search_location_init(const disk_t *disk_car, const unsigned int location_boundary, const int fast_mode)
