@@ -23,7 +23,7 @@
 extern "C" {
 #endif
 
-dir_partition_t dir_partition_ntfs_init(disk_t *disk_car, const partition_t *partition, dir_data_t *dir_data, const int verbose);
+dir_partition_t dir_partition_ntfs_init(disk_t *disk_car, const partition_t *partition, dir_data_t *dir_data, const int verbose, const int expert);
 const char*td_ntfs_version(void);
 
 #ifdef __cplusplus
