@@ -110,8 +110,6 @@
 #include "askloc.h"
 #include "setdate.h"
 
-extern const char *monstr[];
-
 struct options {
 	char		*dest;		/* Save file to this directory */
 };
@@ -120,8 +118,8 @@ struct filename {
 	struct td_list_head list;		/* Previous/Next links */
 	ntfschar	*uname;		/* Filename in unicode */
 	int		 uname_len;	/* and its length */
-	long long	 size_alloc;	/* Allocated size (multiple of cluster size) */
-	long long	 size_data;	/* Actual size of data */
+	uint64_t	 size_alloc;	/* Allocated size (multiple of cluster size) */
+	uint64_t	 size_data;	/* Actual size of data */
 	FILE_ATTR_FLAGS	 flags;
 	time_t		 date_c;	/* Time created */
 	time_t		 date_a;	/*	altered */
@@ -129,9 +127,8 @@ struct filename {
 	time_t		 date_r;	/*	read */
 	char		*name;		/* Filename in current locale */
 	FILE_NAME_TYPE_FLAGS name_space;
-	long long	 parent_mref;
+	uint64_t	 parent_mref;
 	char		*parent_name;
-	char		 padding[7];	/* Unused: padding to 64 bit. */
 };
 
 struct data {
@@ -142,28 +139,26 @@ struct data {
 	int		 resident;	/* Stream is resident */
 	int		 compressed;	/* Stream is compressed */
 	int		 encrypted;	/* Stream is encrypted */
-	long long	 size_alloc;	/* Allocated size (multiple of cluster size) */
-	long long	 size_data;	/* Actual size of data */
-	long long	 size_init;	/* Initialised size, may be less than data size */
-	long long	 size_vcn;	/* Highest VCN in the data runs */
+	uint64_t	 size_alloc;	/* Allocated size (multiple of cluster size) */
+	uint64_t	 size_data;	/* Actual size of data */
+	uint64_t	 size_init;	/* Initialised size, may be less than data size */
+	uint64_t	 size_vcn;	/* Highest VCN in the data runs */
 	runlist_element *runlist;	/* Decoded data runs */
-	int		 percent;	/* Amount potentially recoverable */
+	unsigned int	 percent;	/* Amount potentially recoverable */
 	void		*data;		/* If resident, a pointer to the data */
-	char		 padding[4];	/* Unused: padding to 64 bit. */
 };
 
 struct ufile {
-	long long	 inode;		/* MFT record number */
+	uint64_t	 inode;		/* MFT record number */
 	time_t		 date;		/* Last modification date/time */
 	struct td_list_head name;		/* A list of filenames */
 	struct td_list_head data;		/* A list of data streams */
 	char		*pref_name;	/* Preferred filename */
 	char		*pref_pname;	/*	     parent filename */
-	long long	 max_size;	/* Largest size we find */
+	uint64_t	 max_size;	/* Largest size we find */
 	int		 attr_list;	/* MFT record may be one of many */
 	int		 directory;	/* MFT record represents a directory */
 	MFT_RECORD	*mft;		/* Raw MFT record */
-	char		 padding[4];	/* Unused: padding to 64 bit. */
 };
 
 static const char *UNKNOWN   = "unknown";
@@ -292,7 +287,7 @@ static void get_parent_name(struct filename* name, ntfs_volume* vol)
   }
 
   {
-    long long inode_num;
+    uint64_t inode_num;
     int ok;
     inode_num = MREF(name->parent_mref);
     name->parent_name = NULL;
@@ -302,7 +297,7 @@ static void get_parent_name(struct filename* name, ntfs_volume* vol)
       ok=0;
       if (ntfs_attr_pread(mft_data, vol->mft_record_size * inode_num, vol->mft_record_size, rec) < 1)
       {
-	log_error("ERROR: Couldn't read MFT Record %lld.\n", inode_num);
+	log_error("ERROR: Couldn't read MFT Record %llu.\n", (long long unsigned)inode_num);
       }
       else if ((filename_attr = verify_parent(name, rec)))
       {
@@ -372,9 +367,7 @@ static void get_parent_name(struct filename* name, ntfs_volume* vol)
 static int get_filenames(struct ufile *file, ntfs_volume* vol)
 {
 	ATTR_RECORD *rec;
-	FILE_NAME_ATTR *attr;
 	ntfs_attr_search_ctx *ctx;
-	struct filename *name;
 	int count = 0;
 	int space = 4;
 
@@ -386,6 +379,8 @@ static int get_filenames(struct ufile *file, ntfs_volume* vol)
 		return -1;
 
 	while ((rec = find_attribute(AT_FILE_NAME, ctx))) {
+		struct filename *name;
+		FILE_NAME_ATTR *attr;
 		/* We know this will always be resident. */
 		attr = (FILE_NAME_ATTR *) ((char *) rec + le16_to_cpu(rec->value_offset));
 
@@ -455,7 +450,6 @@ static int get_data(struct ufile *file, ntfs_volume *vol)
 	ATTR_RECORD *rec;
 	ntfs_attr_search_ctx *ctx;
 	int count = 0;
-	struct data *data;
 
 	if (!file)
 		return -1;
@@ -465,6 +459,7 @@ static int get_data(struct ufile *file, ntfs_volume *vol)
 		return -1;
 
 	while ((rec = find_attribute(AT_DATA, ctx))) {
+		struct data *data;
 		data = (struct data *)calloc(1, sizeof(*data));
 		if (!data) {
 			log_error("ERROR: Couldn't allocate memory in get_data().\n");
@@ -524,7 +519,7 @@ static int get_data(struct ufile *file, ntfs_volume *vol)
  * Return:  Pointer  A ufile object containing the results
  *	    NULL     Error
  */
-static struct ufile * read_record(ntfs_volume *vol, long long record)
+static struct ufile * read_record(ntfs_volume *vol, uint64_t record)
 {
 	ATTR_RECORD *attr10, *attr20, *attr90;
 	struct ufile *file;
@@ -553,7 +548,7 @@ static struct ufile * read_record(ntfs_volume *vol, long long record)
 	}
 
 	if (ntfs_attr_mst_pread(mft, vol->mft_record_size * record, 1, vol->mft_record_size, file->mft) < 1) {
-		log_error("ERROR: Couldn't read MFT Record %lld.\n", record);
+		log_error("ERROR: Couldn't read MFT Record %llu.\n", (long long unsigned)record);
 		ntfs_attr_close(mft);
 		free_file(file);
 		return NULL;
@@ -612,103 +607,103 @@ static struct ufile * read_record(ntfs_volume *vol, long long record)
  * Return:  n  The percentage of the file that _could_ be recovered
  *	   -1  Error
  */
-static int calc_percentage(struct ufile *file, ntfs_volume *vol)
+static unsigned int calc_percentage(struct ufile *file, ntfs_volume *vol)
 {
-	struct td_list_head *pos;
-	int percent = 0;
+  struct td_list_head *pos;
+  unsigned int percent = 0;
 
-	if (!file || !vol)
-		return -1;
+  if (!file || !vol)
+    return -1;
 
-	if (file->directory) {
-		return 0;
-	}
+  if (file->directory) {
+    return 0;
+  }
 
-	if (td_list_empty(&file->data)) {
-		return 0;
-	}
+  if (td_list_empty(&file->data)) {
+    return 0;
+  }
 
-	td_list_for_each(pos, &file->data) {
-		runlist_element *rl = NULL;
-		long long i;
-		long long start, end;
-	  	int clusters_inuse, clusters_free;
-		struct data *data;
-		data  = td_list_entry(pos, struct data, list);
-		clusters_inuse = 0;
-		clusters_free  = 0;
+  td_list_for_each(pos, &file->data) {
+    runlist_element *rl = NULL;
+    uint64_t i;
+    unsigned int clusters_inuse, clusters_free;
+    struct data *data;
+    data  = td_list_entry(pos, struct data, list);
+    clusters_inuse = 0;
+    clusters_free  = 0;
 
-		if (data->encrypted) {
-			log_debug("File is encrypted, recovery is impossible.\n");
-			continue;
-		}
+    if (data->encrypted) {
+      log_debug("File is encrypted, recovery is impossible.\n");
+      continue;
+    }
 
-		if (data->compressed) {
-			log_debug("File is compressed, recovery not yet implemented.\n");
-			continue;
-		}
+    if (data->compressed) {
+      log_debug("File is compressed, recovery not yet implemented.\n");
+      continue;
+    }
 
-		if (data->resident) {
-			percent = 100;
-			data->percent = 100;
-			continue;
-		}
+    if (data->resident) {
+      percent = 100;
+      data->percent = 100;
+      continue;
+    }
 
-		rl = data->runlist;
-		if (!rl) {
-			log_debug("File has no runlist, hence no data.\n");
-			continue;
-		}
+    rl = data->runlist;
+    if (!rl) {
+      log_debug("File has no runlist, hence no data.\n");
+      continue;
+    }
 
-		if (rl[0].length <= 0) {
-			log_debug("File has an empty runlist, hence no data.\n");
-			continue;
-		}
+    if (rl[0].length <= 0) {
+      log_debug("File has an empty runlist, hence no data.\n");
+      continue;
+    }
 
-		if (rl[0].lcn == LCN_RL_NOT_MAPPED) {	/* extended mft record */
-			log_debug("Missing segment at beginning, %lld clusters\n", (long long)rl[0].length);
-			clusters_inuse += rl[0].length;
-			rl++;
-		}
+    if (rl[0].lcn == LCN_RL_NOT_MAPPED) {	/* extended mft record */
+      log_debug("Missing segment at beginning, %lld clusters\n", (long long)rl[0].length);
+      clusters_inuse += rl[0].length;
+      rl++;
+    }
 
-		for (i = 0; rl[i].length > 0; i++) {
-			long long j;
-			if (rl[i].lcn == LCN_RL_NOT_MAPPED) {
-				log_debug("Missing segment at end, %lld clusters\n",
-						(long long)rl[i].length);
-				clusters_inuse += rl[i].length;
-				continue;
-			}
+    for (i = 0; rl[i].length > 0; i++) {
+      uint64_t start, end;
+      uint64_t j;
+      if (rl[i].lcn == LCN_RL_NOT_MAPPED) {
+	log_debug("Missing segment at end, %lld clusters\n",
+	    (long long)rl[i].length);
+	clusters_inuse += rl[i].length;
+	continue;
+      }
 
-			if (rl[i].lcn == LCN_HOLE) {
-				clusters_free += rl[i].length;
-				continue;
-			}
+      if (rl[i].lcn == LCN_HOLE) {
+	clusters_free += rl[i].length;
+	continue;
+      }
 
-			start = rl[i].lcn;
-			end   = rl[i].lcn + rl[i].length;
+      start = rl[i].lcn;
+      end   = rl[i].lcn + rl[i].length;
 
-			for (j = start; j < end; j++) {
-				if (utils_cluster_in_use(vol, j))
-					clusters_inuse++;
-				else
-					clusters_free++;
-			}
-		}
+      for (j = start; j < end; j++) {
+	if (utils_cluster_in_use(vol, j))
+	  clusters_inuse++;
+	else
+	  clusters_free++;
+      }
+    }
 
-		if ((clusters_inuse + clusters_free) == 0) {
-			log_error("ERROR: Unexpected error whilst "
-				"calculating percentage for inode %lld\n",
-				file->inode);
-			continue;
-		}
+    if ((clusters_inuse + clusters_free) == 0) {
+      log_error("ERROR: Unexpected error whilst "
+	  "calculating percentage for inode %llu\n",
+	  (long long unsigned)file->inode);
+      continue;
+    }
 
-		data->percent = (clusters_free * 100) /
-				(clusters_inuse + clusters_free);
+    data->percent = (clusters_free * 100) /
+      (clusters_inuse + clusters_free);
 
-		percent = max(percent, data->percent);
-	}
-	return percent;
+    percent = max(percent, data->percent);
+  }
+  return percent;
 }
 
 /**
@@ -821,7 +816,7 @@ static int open_file(const char *pathname)
 {
   int fh;
   fh=open(pathname, O_RDWR | O_CREAT | O_TRUNC, S_IRUSR | S_IWUSR);
-  if(fh>=0 || errno!=ENOENT)
+  if(fh != -1 || errno!=ENOENT)
     return fh;
   mkdir_local_for_file(pathname);
   return open(pathname, O_RDWR | O_CREAT | O_TRUNC, S_IRUSR | S_IWUSR);
@@ -849,224 +844,228 @@ static int open_file(const char *pathname)
  * Return:  -2  Error, something went wrong
  *	    0  Success, the data was recovered
  */
-static int undelete_file(ntfs_volume *vol, long long inode)
+static int undelete_file(ntfs_volume *vol, uint64_t inode)
 {
-	char *buffer = NULL;
-	unsigned int bufsize;
-	struct ufile *file;
-	int i, j;
-	long long start, end;
-	runlist_element *rl;
-	struct td_list_head *item;
-	long long k;
-	int result = -2;
-	char *name;
-	long long cluster_count;	/* I'll need this variable (see below). +mabs */
+  char *buffer = NULL;
+  unsigned int bufsize;
+  struct ufile *file;
+  struct td_list_head *item;
 
-	if (!vol)
-		return -2;
+  if (!vol)
+    return -2;
 
-	/* try to get record */
-	file = read_record(vol, inode);
-	if (!file || !file->mft) {
-		log_error("Can't read info from mft record %lld.\n", inode);
-		return -2;
+  /* try to get record */
+  file = read_record(vol, inode);
+  if (!file || !file->mft) {
+    log_error("Can't read info from mft record %llu.\n", (long long unsigned)inode);
+    return -2;
+  }
+
+
+  bufsize = vol->cluster_size;
+  buffer = (char *)MALLOC(bufsize);
+
+  /* calc_percentage() must be called before 
+   * list_record(). Otherwise, when undeleting, a file will always be
+   * listed as 0% recoverable even if successfully undeleted. +mabs
+   */
+  if (file->mft->flags & MFT_RECORD_IN_USE) {
+    log_error("Record is in use by the mft\n");
+    free(buffer);
+    free_file(file);
+    return -2;
+  }
+
+  if (calc_percentage(file, vol) == 0) {
+    log_error("File has no recoverable data.\n");
+    goto free;
+  }
+
+  if (td_list_empty(&file->data)) {
+    log_warning("File has no data.  There is nothing to recover.\n");
+    goto free;
+  }
+
+  td_list_for_each(item, &file->data) {
+    char pathname[256];
+    char defname[64];
+    char *name;
+    struct data *d = td_list_entry(item, struct data, list);
+    if(file->pref_name)
+    {
+      name = file->pref_name;
+    }
+    else
+    {
+      sprintf(defname, "inode_%llu", (long long unsigned)file->inode);
+      name = defname;
+    }
+
+    //dir_data->local_dir;
+    create_pathname(opts.dest, file->pref_pname, name, d->name, pathname, sizeof(pathname));
+    if (d->resident) {
+      int fd;
+      fd = open_file(pathname);
+      if (fd < 0) {
+	log_error("Couldn't create file %s\n", pathname);
+	goto free;
+      }
+
+      log_verbose("File has resident data.\n");
+      if (write_data(fd, (const char *)d->data, d->size_data) < d->size_data) {
+	log_error("Write failed\n");
+	close(fd);
+	goto free;
+      }
+
+      if (close(fd) < 0) {
+	log_error("Close failed\n");
+      }
+    } else {
+      int i;
+      int fd;
+      uint64_t cluster_count;	/* I'll need this variable (see below). +mabs */
+      runlist_element *rl;
+      rl = d->runlist;
+      if (!rl) {
+	log_verbose("File has no runlist, hence no data.\n");
+	continue;
+      }
+
+      if (rl[0].length <= 0) {
+	log_verbose("File has an empty runlist, hence no data.\n");
+	continue;
+      }
+
+      fd = open_file(pathname);
+      if (fd < 0) {
+	log_error("Couldn't create output file %s\n", pathname);
+	goto free;
+      }
+
+      if (rl[0].lcn == LCN_RL_NOT_MAPPED) {	/* extended mft record */
+	uint64_t k;
+	log_verbose("Missing segment at beginning, %lld "
+	    "clusters.\n",
+	    (long long)rl[0].length);
+	memset(buffer, 0, bufsize);
+	for (k = 0; k < (uint64_t)rl[0].length * vol->cluster_size; k += bufsize) {
+	  if (write_data(fd, buffer, bufsize) < bufsize) {
+	    log_error("Write failed\n");
+	    close(fd);
+	    goto free;
+	  }
+	}
+      }
+
+      cluster_count = 0;
+      for (i = 0; rl[i].length > 0; i++) {
+	uint64_t start, end;
+	uint64_t j;
+
+	if (rl[i].lcn == LCN_RL_NOT_MAPPED) {
+	  uint64_t k;
+	  log_verbose("Missing segment at end, "
+	      "%lld clusters.\n",
+	      (long long)rl[i].length);
+	  memset(buffer, 0, bufsize);
+	  for (k = 0; k < (uint64_t)rl[i].length * vol->cluster_size; k += bufsize) {
+	    if (write_data(fd, buffer, bufsize) < bufsize) {
+	      log_error("Write failed\n");
+	      close(fd);
+	      goto free;
+	    }
+	    cluster_count++;
+	  }
+	  continue;
 	}
 
-
-	bufsize = vol->cluster_size;
-	buffer = (char *)MALLOC(bufsize);
-
-	/* calc_percentage() must be called before 
-	 * list_record(). Otherwise, when undeleting, a file will always be
-	 * listed as 0% recoverable even if successfully undeleted. +mabs
-	 */
-	if (file->mft->flags & MFT_RECORD_IN_USE) {
-          log_error("Record is in use by the mft\n");
-          free(buffer);
-          free_file(file);
-          return -2;
+	if (rl[i].lcn == LCN_HOLE) {
+	  uint64_t k;
+	  log_verbose("File has a sparse section.\n");
+	  memset(buffer, 0, bufsize);
+	  for (k = 0; k < (uint64_t)rl[i].length * vol->cluster_size; k += bufsize) {
+	    if (write_data(fd, buffer, bufsize) < bufsize) {
+	      log_error("Write failed\n");
+	      close(fd);
+	      goto free;
+	    }
+	  }
+	  continue;
 	}
 
-	if (calc_percentage(file, vol) == 0) {
-		log_error("File has no recoverable data.\n");
-		goto free;
-	}
+	start = rl[i].lcn;
+	end   = rl[i].lcn + rl[i].length;
 
-	if (td_list_empty(&file->data)) {
-		log_warning("File has no data.  There is nothing to recover.\n");
-		goto free;
-	}
-
-	td_list_for_each(item, &file->data) {
-	  char pathname[256];
-	  char defname[64];
-		struct data *d = td_list_entry(item, struct data, list);
-		if(file->pref_name)
-		{
-		  name = file->pref_name;
-		}
-		else
-		{
-		  sprintf(defname, "inode_%llu", (long long unsigned)file->inode);
-		  name = defname;
-		}
-
-		//dir_data->local_dir;
-		create_pathname(opts.dest, file->pref_pname, name, d->name, pathname, sizeof(pathname));
-		if (d->resident) {
-		        int fd;
-			fd = open_file(pathname);
-			if (fd < 0) {
-				log_error("Couldn't create file %s\n", pathname);
-				goto free;
-			}
-
-			log_verbose("File has resident data.\n");
-			if (write_data(fd, (const char *)d->data, d->size_data) < d->size_data) {
-				log_error("Write failed\n");
-				close(fd);
-				goto free;
-			}
-
-			if (close(fd) < 0) {
-				log_error("Close failed\n");
-			}
-		} else {
-		        int fd;
-			rl = d->runlist;
-			if (!rl) {
-				log_verbose("File has no runlist, hence no data.\n");
-				continue;
-			}
-
-			if (rl[0].length <= 0) {
-				log_verbose("File has an empty runlist, hence no data.\n");
-				continue;
-			}
-
-			fd = open_file(pathname);
-			if (fd < 0) {
-				log_error("Couldn't create output file %s\n", pathname);
-				goto free;
-			}
-
-			if (rl[0].lcn == LCN_RL_NOT_MAPPED) {	/* extended mft record */
-				log_verbose("Missing segment at beginning, %lld "
-						"clusters.\n",
-						(long long)rl[0].length);
-				memset(buffer, 0, bufsize);
-				for (k = 0; k < rl[0].length * vol->cluster_size; k += bufsize) {
-					if (write_data(fd, buffer, bufsize) < bufsize) {
-						log_error("Write failed\n");
-						close(fd);
-						goto free;
-					}
-				}
-			}
-
-			cluster_count = (long long)0;
-			for (i = 0; rl[i].length > 0; i++) {
-
-				if (rl[i].lcn == LCN_RL_NOT_MAPPED) {
-					log_verbose("Missing segment at end, "
-							"%lld clusters.\n",
-							(long long)rl[i].length);
-					memset(buffer, 0, bufsize);
-					for (k = 0; k < rl[i].length * vol->cluster_size; k += bufsize) {
-						if (write_data(fd, buffer, bufsize) < bufsize) {
-							log_error("Write failed\n");
-							close(fd);
-							goto free;
-						}
-						cluster_count++;
-					}
-					continue;
-				}
-
-				if (rl[i].lcn == LCN_HOLE) {
-					log_verbose("File has a sparse section.\n");
-					memset(buffer, 0, bufsize);
-					for (k = 0; k < rl[i].length * vol->cluster_size; k += bufsize) {
-						if (write_data(fd, buffer, bufsize) < bufsize) {
-							log_error("Write failed\n");
-							close(fd);
-							goto free;
-						}
-					}
-					continue;
-				}
-
-				start = rl[i].lcn;
-				end   = rl[i].lcn + rl[i].length;
-
-				for (j = start; j < end; j++)
-				{
-				  /* Don't check if clusters are in used or not */
+	for (j = start; j < end; j++)
+	{
+	  /* Don't check if clusters are in used or not */
 #if 0
-					if (utils_cluster_in_use(vol, j) && !opts.optimistic)
-					{
-						memset(buffer, 0, bufsize);
-						if (write_data(fd, buffer, bufsize) < bufsize)
-						{
-							log_error("Write failed\n");
-							close(fd);
-							goto free;
-						}
-					}
-					else
+	  if (utils_cluster_in_use(vol, j) && !opts.optimistic)
+	  {
+	    memset(buffer, 0, bufsize);
+	    if (write_data(fd, buffer, bufsize) < bufsize)
+	    {
+	      log_error("Write failed\n");
+	      close(fd);
+	      goto free;
+	    }
+	  }
+	  else
 #endif
-					{
-						if (ntfs_cluster_read(vol, j, 1, buffer) < 1) {
-							log_error("Read failed\n");
-							close(fd);
-							goto free;
-						}
-						if (write_data(fd, buffer, bufsize) < bufsize) {
-							log_error("Write failed\n");
-							close(fd);
-							goto free;
-						}
-						cluster_count++;
-					}
-				}
-			}
-
-			/*
-			 * IF data stream currently being recovered is
-			 * non-resident AND data stream has no holes (100% recoverability) AND
-			 * 0 <= (data->size_alloc - data->size_data) <= vol->cluster_size AND
-			 * cluster_count * vol->cluster_size == data->size_alloc THEN file
-			 * currently being written is truncated to data->size_data bytes before
-			 * it's closed.
-			 * This multiple checks try to ensure that only files with consistent
-			 * values of size/occupied clusters are eligible for truncation. Note
-			 * that resident streams need not be truncated, since the original code
-			 * already recovers their exact length.                           +mabs
-			 */
-			  if (d->percent == 100 && d->size_alloc >= d->size_data &&
-			      (d->size_alloc - d->size_data) <= (long long)vol->cluster_size &&
-			      cluster_count * (long long)vol->cluster_size == d->size_alloc)
-			  {
-			    if (ftruncate(fd, (off_t)d->size_data))
-			      log_error("Truncation failed\n");
-			  }
-			  else
-			    log_warning("Truncation not performed because file has an "
-				"inconsistent $MFT record.\n");
-
-			if (close(fd) < 0) {
-				log_error("Close failed\n");
-			}
-
-		}
-		set_date(pathname, file->date, file->date);
+	  {
+	    if (ntfs_cluster_read(vol, j, 1, buffer) < 1) {
+	      log_error("Read failed\n");
+	      close(fd);
+	      goto free;
+	    }
+	    if (write_data(fd, buffer, bufsize) < bufsize) {
+	      log_error("Write failed\n");
+	      close(fd);
+	      goto free;
+	    }
+	    cluster_count++;
+	  }
 	}
-	result = 0;
+      }
+
+      /*
+       * IF data stream currently being recovered is
+       * non-resident AND data stream has no holes (100% recoverability) AND
+       * 0 <= (data->size_alloc - data->size_data) <= vol->cluster_size AND
+       * cluster_count * vol->cluster_size == data->size_alloc THEN file
+       * currently being written is truncated to data->size_data bytes before
+       * it's closed.
+       * This multiple checks try to ensure that only files with consistent
+       * values of size/occupied clusters are eligible for truncation. Note
+       * that resident streams need not be truncated, since the original code
+       * already recovers their exact length.                           +mabs
+       */
+      if (d->percent == 100 && d->size_alloc >= d->size_data &&
+	  (d->size_alloc - d->size_data) <= (uint64_t)vol->cluster_size &&
+	  cluster_count * (uint64_t)vol->cluster_size == d->size_alloc)
+      {
+	if (ftruncate(fd, (off_t)d->size_data))
+	  log_error("Truncation failed\n");
+      }
+      else
+	log_warning("Truncation not performed because file has an "
+	    "inconsistent $MFT record.\n");
+
+      if (close(fd) < 0) {
+	log_error("Close failed\n");
+      }
+
+    }
+    set_date(pathname, file->date, file->date);
+  }
+  free(buffer);
+  free_file(file);
+  return 0;
 free:
-	free(buffer);
-	free_file(file);
-	return result;
+  free(buffer);
+  free_file(file);
+  return -2;
 }
 
 static file_info_t *ufile_to_file_data(const struct ufile *file, const struct data *d)
@@ -1107,82 +1106,84 @@ static file_info_t *ufile_to_file_data(const struct ufile *file, const struct da
  */
 static void scan_disk(ntfs_volume *vol, file_info_t *dir_list)
 {
-	s64 nr_mft_records;
-	const int BUFSIZE = 8192;
-	char *buffer = NULL;
-	int results = 0;
-	ntfs_attr *attr;
-	long long size;
-	long long bmpsize;
-	int i, j, k, b;
-	struct ufile *file;
-	if (!vol)
-	  return;
+  uint64_t nr_mft_records;
+  const unsigned int BUFSIZE = 8192;
+  char *buffer = NULL;
+  unsigned int results = 0;
+  ntfs_attr *attr;
+  uint64_t bmpsize;
+  uint64_t i;
+  struct ufile *file;
+  if (!vol)
+    return;
 #ifdef NTFS_LOG_LEVEL_VERBOSE
-	ntfs_log_set_levels(NTFS_LOG_LEVEL_QUIET);
-	ntfs_log_set_handler(ntfs_log_handler_stderr);
+  ntfs_log_set_levels(NTFS_LOG_LEVEL_QUIET);
+  ntfs_log_set_handler(ntfs_log_handler_stderr);
 #endif
 
-	attr = ntfs_attr_open(vol->mft_ni, AT_BITMAP, AT_UNNAMED, 0);
-	if (!attr)
+  attr = ntfs_attr_open(vol->mft_ni, AT_BITMAP, AT_UNNAMED, 0);
+  if (!attr)
+  {
+    log_error("ERROR: Couldn't open $MFT/$BITMAP\n");
+    return;
+  }
+  bmpsize = attr->initialized_size;
+
+  buffer = (char *) MALLOC(BUFSIZE);
+
+  nr_mft_records = vol->mft_na->initialized_size >>
+    vol->mft_record_size_bits;
+
+  for (i = 0; i < bmpsize; i += BUFSIZE) {
+    int64_t size;
+    unsigned int j;
+    uint64_t read_count = min((bmpsize - i), BUFSIZE);
+    size = ntfs_attr_pread(attr, i, read_count, buffer);
+    if (size < 0)
+      break;
+
+    for (j = 0; j < size; j++) {
+      unsigned int k;
+      unsigned int b;
+      b = buffer[j];
+      for (k = 0; k < 8; k++, b>>=1)
+      {
+	unsigned int percent;
+	if (((i+j)*8+k) >= nr_mft_records)
+	  goto done;
+	if (b & 1)
+	  continue;
+	file = read_record(vol, (i+j)*8+k);
+	if (!file) {
+	  log_error("Couldn't read MFT Record %llu.\n", (long long unsigned)(i+j)*8+k);
+	  continue;
+	}
+
+	percent = calc_percentage(file, vol);
+	if (percent >0)
 	{
-	  log_error("ERROR: Couldn't open $MFT/$BITMAP\n");
-	  return;
-	}
-	bmpsize = attr->initialized_size;
-
-	buffer = (char *) MALLOC(BUFSIZE);
-
-	nr_mft_records = vol->mft_na->initialized_size >>
-			vol->mft_record_size_bits;
-
-	for (i = 0; i < bmpsize; i += BUFSIZE) {
-		long long read_count = min((bmpsize - i), BUFSIZE);
-		size = ntfs_attr_pread(attr, i, read_count, buffer);
-		if (size < 0)
-			break;
-
-		for (j = 0; j < size; j++) {
-			b = buffer[j];
-			for (k = 0; k < 8; k++, b>>=1)
-			{
-			  int percent;
-			  if (((i+j)*8+k) >= nr_mft_records)
-			    goto done;
-			  if (b & 1)
-			    continue;
-			  file = read_record(vol, (i+j)*8+k);
-			  if (!file) {
-			    log_error("Couldn't read MFT Record %d.\n", (i+j)*8+k);
-			    continue;
-			  }
-
-			  percent = calc_percentage(file, vol);
-			  if (percent >0)
-			  {
-			    struct td_list_head *item;
-			    td_list_for_each(item, &file->data)
-			    {
-			      const struct data *d = td_list_entry(item, struct data, list);
-			      file_info_t *new_file;
-			      new_file=ufile_to_file_data(file, d);
-			      if(new_file!=NULL)
-			      {
-				td_list_add_tail(&new_file->list, &dir_list->list);
-				results++;
-			      }
-			    }
-			  }
-			  free_file(file);
-			}
-		}
+	  struct td_list_head *item;
+	  td_list_for_each(item, &file->data)
+	  {
+	    const struct data *d = td_list_entry_const(item, const struct data, list);
+	    file_info_t *new_file;
+	    new_file=ufile_to_file_data(file, d);
+	    if(new_file!=NULL)
+	    {
+	      td_list_add_tail(&new_file->list, &dir_list->list);
+	      results++;
+	    }
+	  }
 	}
+	free_file(file);
+      }
+    }
+  }
 done:
-	log_info("\nFiles with potentially recoverable content: %d\n", results);
-	free(buffer);
-	if (attr)
-		ntfs_attr_close(attr);
-	td_list_sort(&dir_list->list, filesort);
+  log_info("\nFiles with potentially recoverable content: %u\n", results);
+  free(buffer);
+  ntfs_attr_close(attr);
+  td_list_sort(&dir_list->list, filesort);
 }
 
 #ifdef HAVE_NCURSES
@@ -1195,7 +1196,7 @@ static struct td_list_head *ntfs_next_non_deleted(struct td_list_head *current_f
   {
     const file_info_t *file_info;
     walker=walker->next;
-    file_info=td_list_entry(walker, file_info_t, list);
+    file_info=td_list_entry_const(walker, const file_info_t, list);
     if((file_info->status&FILE_STATUS_DELETED)==0)
       return walker;
   }
@@ -1209,7 +1210,7 @@ static struct td_list_head *ntfs_prev_non_deleted(struct td_list_head *current_f
   {
     const file_info_t *file_info;
     walker=walker->prev;
-    file_info=td_list_entry(walker, file_info_t, list);
+    file_info=td_list_entry_const(walker, const file_info_t, list);
     if((file_info->status&FILE_STATUS_DELETED)==0)
       return walker;
   }
@@ -1262,29 +1263,19 @@ static void ntfs_undelete_menu_ncurses(disk_t *disk_car, const partition_t *part
 	  waddstr(window, " ");
 	if((file_info->status&FILE_STATUS_MARKED)!=0 && has_colors())
 	  wbkgdset(window,' ' | COLOR_PAIR(2));
-	if(file_info->td_mtime!=0)
-	{
-	  const struct tm *tm_p= localtime(&file_info->td_mtime);
-	  snprintf(datestr, sizeof(datestr),"%2d-%s-%4d %02d:%02d",
-	      tm_p->tm_mday, monstr[tm_p->tm_mon],
-	      1900 + tm_p->tm_year, tm_p->tm_hour,
-	      tm_p->tm_min);
-	  /* May have to use %d instead of %e */
-	} else {
-	  strncpy(datestr, "                 ",sizeof(datestr));
-	}
+	set_date((char *)&datestr, sizeof(datestr), file_info->td_mtime);
 	if(COLS <= 1+17+1+9+1)
 	  wprintw(window, "%s", file_info->name);
 	else
 	{
-	  const unsigned int nbr=COLS - (1+17+1+9+1);
+	  const unsigned int nbr=COLS - (1+17+1+11+1);
 	  if(strlen(file_info->name) < nbr)
 	    wprintw(window, "%-*s", nbr, file_info->name);
 	  else
 	    wprintw(window, "%-*s", nbr, &file_info->name[strlen(file_info->name) - nbr]);
 	}
 	wprintw(window, " %s ", datestr);
-	wprintw(window, "%9llu", (long long unsigned int)file_info->st_size);
+	wprintw(window, "%11llu", (long long unsigned int)file_info->st_size);
 	if((file_info->status&FILE_STATUS_MARKED)!=0 && has_colors())
 	  wbkgdset(window,' ' | COLOR_PAIR(0));
 	if(file_walker==current_file)
@@ -1597,11 +1588,40 @@ static void ntfs_undelete_menu_ncurses(disk_t *disk_car, const partition_t *part
 }
 #endif
 
+static void ntfs_undelete_cli(dir_data_t *dir_data, file_info_t *dir_list)
+{
+  unsigned int file_ok=0;
+  unsigned int file_bad=0;
+  struct td_list_head *file_walker = NULL;
+  struct ntfs_dir_struct *ls=(struct ntfs_dir_struct *)dir_data->private_dir_data;
+  char *dst_path;
+  dst_path=get_default_location();
+  dir_data->local_dir=dst_path;
+  opts.dest=dst_path;
+  td_list_for_each(file_walker,&dir_list->list)
+  {
+    const file_info_t *file_info=td_list_entry_const(file_walker, const file_info_t, list);
+    if(undelete_file(ls->vol, file_info->st_ino) < 0)
+      file_bad++;
+    else
+      file_ok++;
+  }
+  log_info("NTFS undelete done (%u/%u)\n", file_ok, (file_ok+file_bad));
+  free(dst_path);
+  dir_data->local_dir=NULL;
+  opts.dest=NULL;
+}
+
 static void ntfs_undelete_menu(disk_t *disk_car, const partition_t *partition, dir_data_t *dir_data, file_info_t *dir_list, char**current_cmd)
 {
   log_list_file(disk_car, partition, dir_data, dir_list);
   if(*current_cmd!=NULL)
   {
+    skip_comma_in_command(current_cmd);
+    if(check_command(current_cmd,"allundelete",11)==0)
+    {
+      ntfs_undelete_cli(dir_data, dir_list);
+    }
     return;	/* Quit */
   }
 #ifdef HAVE_NCURSES
@@ -1615,7 +1635,7 @@ int ntfs_undelete_part(disk_t *disk_car, const partition_t *partition, const int
 #ifdef HAVE_NCURSES
   WINDOW *window;
 #endif
-  int res=dir_partition_ntfs_init(disk_car,partition,&dir_data,verbose);
+  dir_partition_t res=dir_partition_ntfs_init(disk_car, partition, &dir_data, verbose, 0);
 #ifdef HAVE_NCURSES
   window=newwin(LINES, COLS, 0, 0);	/* full screen */
   dir_data.display=window;
@@ -1626,7 +1646,7 @@ int ntfs_undelete_part(disk_t *disk_car, const partition_t *partition, const int
   log_info("\n");
   switch(res)
   {
-    case -2:
+    case DIR_PART_ENOSYS:
       screen_buffer_reset();
 #ifdef HAVE_NCURSES
       aff_copy(window);
@@ -1634,7 +1654,7 @@ int ntfs_undelete_part(disk_t *disk_car, const partition_t *partition, const int
       aff_part(window,AFF_PART_ORDER|AFF_PART_STATUS,disk_car,partition);
 #endif
       log_partition(disk_car,partition);
-      screen_buffer_add("Support for this filesystem hasn't been enable during compilation.\n");
+      screen_buffer_add("Support for this filesystem wasn't enabled during compilation.\n");
       screen_buffer_to_log();
       if(*current_cmd==NULL)
       {
@@ -1643,7 +1663,7 @@ int ntfs_undelete_part(disk_t *disk_car, const partition_t *partition, const int
 #endif
       }
       break;
-    case -1:
+    case DIR_PART_EIO:
       screen_buffer_reset();
 #ifdef HAVE_NCURSES
       aff_copy(window);
@@ -1662,11 +1682,9 @@ int ntfs_undelete_part(disk_t *disk_car, const partition_t *partition, const int
       break;
     default:
       {
-	file_info_t dir_list = {
-	  .list = TD_LIST_HEAD_INIT(dir_list.list),
-	  .name = NULL
-	};
 	struct ntfs_dir_struct *ls=(struct ntfs_dir_struct *)dir_data.private_dir_data;
+	file_info_t dir_list;
+	TD_INIT_LIST_HEAD(&dir_list.list);
 	scan_disk(ls->vol, &dir_list);
 	ntfs_undelete_menu(disk_car, partition, &dir_data, &dir_list, current_cmd);
 	delete_list_file(&dir_list);
@@ -1699,15 +1717,13 @@ int ntfs_undelete_part(disk_t *disk_car, const partition_t *partition, const int
   aff_part(window,AFF_PART_ORDER|AFF_PART_STATUS,disk_car,partition);
 #endif
   log_partition(disk_car,partition);
-  screen_buffer_add("Support for this filesystem hasn't been enable during compilation.\n");
+  screen_buffer_add("Support for this filesystem wasn't enabled during compilation.\n");
   screen_buffer_to_log();
+#ifdef HAVE_NCURSES
   if(*current_cmd==NULL)
   {
-#ifdef HAVE_NCURSES
     screen_buffer_display(window,"",NULL);
-#endif
   }
-#ifdef HAVE_NCURSES
   delwin(window);
   (void) clearok(stdscr, TRUE);
 #ifdef HAVE_TOUCHWIN
