@@ -42,6 +42,7 @@
 #if defined(HAVE_SYS_UUID_H)
 #include <sys/uuid.h>
 #endif
+#include <assert.h>
 #include "common.h"
 #include "fnctdsk.h"
 #include "lang.h"
@@ -56,6 +57,7 @@
 #include "hfsp.h"
 #include "lvm.h"
 #include "ntfs.h"
+#include "refs.h"
 #include "log.h"
 #include "log_part.h"
 #include "md.h"
@@ -76,6 +78,7 @@ static const char *get_gpt_typename(const efi_guid_t part_type_gpt);
 
 const struct systypes_gtp gpt_sys_types[] = {
   { GPT_ENT_TYPE_EFI, 			"EFI System"		},
+  { GPT_ENT_TYPE_EBP, 			"Extended Boot"		},
   { GPT_ENT_TYPE_MBR,			"MBR"			},
   { GPT_ENT_TYPE_FREEBSD,		"FreeBSD"		},
   { GPT_ENT_TYPE_FREEBSD_SWAP,		"FreeBSD Swap"		},
@@ -87,13 +90,18 @@ const struct systypes_gtp gpt_sys_types[] = {
   { GPT_ENT_TYPE_MS_BASIC_DATA,		"MS Data"		},
   { GPT_ENT_TYPE_MS_LDM_METADATA,	"MS LDM MetaData"	},
   { GPT_ENT_TYPE_MS_LDM_DATA,		"MS LDM Data"		},
+  { GPT_ENT_TYPE_MS_RECOVERY,		"Windows Recovery Env"  },
 //  { GPT_ENT_TYPE_LINUX_DATA
   { GPT_ENT_TYPE_LINUX_RAID,		"Linux Raid"		},
   { GPT_ENT_TYPE_LINUX_SWAP,		"Linux Swap"		},
   { GPT_ENT_TYPE_LINUX_LVM,		"Linux LVM"		},
   { GPT_ENT_TYPE_LINUX_RESERVED,	"Linux Reserved"	},
+  { GPT_ENT_TYPE_LINUX_HOME,		"Linux /home"		},
+  { GPT_ENT_TYPE_LINUX_SRV,		"Linux /src"		},
+  { GPT_ENT_TYPE_LINUX_DATA,		"Linux filesys. data"	},
   { GPT_ENT_TYPE_HPUX_DATA,		"HPUX Data"		},
   { GPT_ENT_TYPE_HPUX_SERVICE,		"HPUX Service"		},
+  { GPT_ENT_TYPE_MAC_AFS,		"Apple APFS"		},
   { GPT_ENT_TYPE_MAC_HFS,		"Mac HFS"		},
   { GPT_ENT_TYPE_MAC_UFS,		"Mac UFS"		},
   { GPT_ENT_TYPE_MAC_RAID,		"Mac Raid"		},
@@ -122,20 +130,20 @@ arch_fnct_t arch_gpt=
   .part_name="EFI GPT",
   .part_name_option="partition_gpt",
   .msg_part_type="                P=Primary  D=Deleted",
-  .read_part=read_part_gpt,
-  .write_part=write_part_gpt,
-  .init_part_order=init_part_order_gpt,
+  .read_part=&read_part_gpt,
+  .write_part=&write_part_gpt,
+  .init_part_order=&init_part_order_gpt,
   .get_geometry_from_mbr=NULL,
-  .check_part=check_part_gpt,
+  .check_part=&check_part_gpt,
   .write_MBR_code=NULL,
-  .set_prev_status=set_next_status_gpt,
-  .set_next_status=set_next_status_gpt,
-  .test_structure=test_structure_gpt,
+  .set_prev_status=&set_next_status_gpt,
+  .set_next_status=&set_next_status_gpt,
+  .test_structure=&test_structure_gpt,
   .get_part_type=NULL,
   .set_part_type=NULL,
-  .init_structure=init_structure_gpt,
+  .init_structure=&init_structure_gpt,
   .erase_list_part=NULL,
-  .get_partition_typename=get_partition_typename_gpt,
+  .get_partition_typename=&get_partition_typename_gpt,
   .is_part_known=&is_part_known_gpt
 };
 
@@ -331,17 +339,17 @@ static list_part_t *init_part_order_gpt(const disk_t *disk_car, list_part_t *lis
 
 list_part_t *add_partition_gpt_cli(disk_t *disk_car,list_part_t *list_part, char **current_cmd)
 {
-  partition_t *new_partition=partition_new(&arch_gpt);
+  partition_t *new_partition;
+  assert(current_cmd!=NULL);
+  new_partition=partition_new(&arch_gpt);
   new_partition->part_offset=disk_car->sector_size;
   new_partition->part_size=disk_car->disk_size-new_partition->part_offset;
-  while(*current_cmd[0]==',')
-    (*current_cmd)++;
   while(1)
   {
-    if(strncmp(*current_cmd,"s,",2)==0)
+    skip_comma_in_command(current_cmd);
+    if(check_command(current_cmd,"s,",2)==0)
     {
       uint64_t part_offset;
-      (*current_cmd)+=2;
       part_offset=new_partition->part_offset;
       new_partition->part_offset=(uint64_t)ask_number_cli(
           current_cmd,
@@ -352,9 +360,8 @@ list_part_t *add_partition_gpt_cli(disk_t *disk_car,list_part_t *list_part, char
         (uint64_t)disk_car->sector_size;
       new_partition->part_size=new_partition->part_size + part_offset - new_partition->part_offset;
     }
-    else if(strncmp(*current_cmd,"S,",2)==0)
+    else if(check_command(current_cmd,"S,",2)==0)
     {
-      (*current_cmd)+=2;
       new_partition->part_size=(uint64_t)ask_number_cli(
           current_cmd,
           (new_partition->part_offset+new_partition->part_size-1)/disk_car->sector_size,
@@ -364,9 +371,8 @@ list_part_t *add_partition_gpt_cli(disk_t *disk_car,list_part_t *list_part, char
         (uint64_t)disk_car->sector_size +
         disk_car->sector_size - new_partition->part_offset;
     }
-    else if(strncmp(*current_cmd,"T,",2)==0)
+    else if(check_command(current_cmd,"T,",2)==0)
     {
-      (*current_cmd)+=2;
       change_part_type_cli(disk_car,new_partition,current_cmd);
     }
     else if(new_partition->part_size>0 && guid_cmp(new_partition->part_type_gpt, GPT_ENT_TYPE_UNUSED)!=0)
@@ -456,9 +462,11 @@ static int check_part_gpt(disk_t *disk, const int verbose,partition_t *partition
   {
     ret=check_FAT(disk,partition,verbose);
     if(ret!=0)
-      ret=check_EXFAT(disk, partition);
+      ret=check_exFAT(disk, partition);
     if(ret!=0)
       ret=check_NTFS(disk,partition,verbose,0);
+    if(ret!=0)
+      ret=check_ReFS(disk, partition);
     if(ret!=0)
       ret=check_linux(disk, partition, verbose);
     if(ret!=0)
@@ -506,6 +514,18 @@ static const char *get_gpt_typename(const efi_guid_t part_type_gpt)
   for(i=0; gpt_sys_types[i].name!=NULL; i++)
     if(guid_cmp(gpt_sys_types[i].part_type, part_type_gpt)==0)
       return gpt_sys_types[i].name;
+  log_info("%8x %04x %04x %02x %02x %02x %02x %02x %02x %02x %02x\n",
+      part_type_gpt.time_low,
+      part_type_gpt.time_mid,
+      part_type_gpt.time_hi_and_version,
+      part_type_gpt.clock_seq_hi_and_reserved,
+      part_type_gpt.clock_seq_low,
+      part_type_gpt.node[0],
+      part_type_gpt.node[1],
+      part_type_gpt.node[2],
+      part_type_gpt.node[3],
+      part_type_gpt.node[4],
+      part_type_gpt.node[5]);
   return NULL;
 }
 
