@@ -44,7 +44,7 @@ struct gpt_hdr
   uint32_t	hdr_entsz;                      /* 0x54 */
   uint32_t	hdr_crc_table;                  /* 0x58 */
   uint8_t 	padding[420];                   /* 0x5c */
-} __attribute__ ((__packed__));
+} __attribute__ ((gcc_struct, __packed__));
 
 struct gpt_ent
 {
@@ -63,7 +63,7 @@ struct systypes_gtp {
 };
 
 list_part_t *add_partition_gpt_cli(disk_t *disk_car,list_part_t *list_part, char **current_cmd);
-int write_part_gpt(disk_t *disk_car, const list_part_t *list_part, const int ro, const int verbose, const int align);
+int write_part_gpt(disk_t *disk_car, const list_part_t *list_part, const int ro, const int verbose);
 
 #ifdef __cplusplus
 } /* closing brace for extern "C" */
