@@ -33,6 +33,7 @@
 #include <string.h>
 #endif
 #include <ctype.h>      /* tolower */
+#include <assert.h>
 #include "types.h"
 #include "common.h"
 #include "fnctdsk.h"
@@ -56,6 +57,7 @@
 #include "log.h"
 #include "parti386.h"
 #include "partgpt.h"
+#include "partxbox.h"
 
 #define TAB_PART 0x1BE
 static int is_extended(const unsigned int part_type);
@@ -95,7 +97,7 @@ static int write_all_log_i386(disk_t *disk_car, const list_part_t *list_part, co
 static int diff(const unsigned char buffer[DEFAULT_SECTOR_SIZE], const unsigned char buffer_org[DEFAULT_SECTOR_SIZE]);
 
 static list_part_t *read_part_i386(disk_t *disk_car, const int verbose, const int saveheader);
-static int write_part_i386(disk_t *disk_car, const list_part_t *list_part, const int ro , const int verbose, const int align);
+static int write_part_i386(disk_t *disk_car, const list_part_t *list_part, const int ro , const int verbose);
 static list_part_t *init_part_order_i386(const disk_t *disk_car, list_part_t *list_part);
 static int write_MBR_code_i386(disk_t *disk_car);
 static int write_MBR_code_i386_aux(unsigned char *buffer);
@@ -197,6 +199,7 @@ static const struct systypes i386_sys_types[] = {
   {0xe1,		"SpeedStor FAT12 ext"},
   {0xe3,		"DOS RO"},
   {0xe4,		"SpeedStor FAT16 ext"},
+  {0xea,		"Boot (BLS)"},
   {P_BEOS,		"BeFS"},
   {0xee,		"EFI GPT"},          /* Intel EFI GUID Partition Table */
   {0xef,		"EFI (FAT-12/16/32)"},/* Intel EFI System Partition */
@@ -215,21 +218,21 @@ arch_fnct_t arch_i386= {
   .part_name="Intel",
   .part_name_option="partition_i386",
   .msg_part_type="*=Primary bootable  P=Primary  L=Logical  E=Extended  D=Deleted",
-  .read_part=read_part_i386,
-  .write_part=write_part_i386,
-  .init_part_order=init_part_order_i386,
-  .get_geometry_from_mbr=get_geometry_from_i386mbr,
-  .check_part=check_part_i386,
-  .write_MBR_code=write_MBR_code_i386,
-  .set_prev_status=set_prev_status_i386,
-  .set_next_status=set_next_status_i386,
-  .test_structure=test_structure_i386,
-  .get_part_type=get_part_type_i386,
-  .set_part_type=set_part_type_i386,
-  .init_structure=init_structure_i386,
-  .erase_list_part=erase_list_part_i386,
-  .get_partition_typename=get_partition_typename_i386,
-  .is_part_known=is_part_known_i386
+  .read_part=&read_part_i386,
+  .write_part=&write_part_i386,
+  .init_part_order=&init_part_order_i386,
+  .get_geometry_from_mbr=&get_geometry_from_i386mbr,
+  .check_part=&check_part_i386,
+  .write_MBR_code=&write_MBR_code_i386,
+  .set_prev_status=&set_prev_status_i386,
+  .set_next_status=&set_next_status_i386,
+  .test_structure=&test_structure_i386,
+  .get_part_type=&get_part_type_i386,
+  .set_part_type=&set_part_type_i386,
+  .init_structure=&init_structure_i386,
+  .erase_list_part=&erase_list_part_i386,
+  .get_partition_typename=&get_partition_typename_i386,
+  .is_part_known=&is_part_known_i386
 };
 
 static uint64_t C_H_S2offset(const disk_t *disk_car,const unsigned int C, const unsigned int H, const unsigned int S)
@@ -374,33 +377,33 @@ static list_part_t *read_part_i386(disk_t *disk_car, const int verbose, const in
   for(i=0;i<4;i++)
   {
     const struct partition_dos *p=pt_offset(buffer,i);
-    status_type_t status=STATUS_PRIM;
+    status_type_t status;
     switch(p->sys_ind)
     {
-      case P_NO_OS:
-	break;
       case P_EXTENDX:
       case P_EXTENDED:
       case P_LINUXEXTENDX:
 	status=STATUS_EXT;
-	/* don't put a break */
-      default:
-	{
-	  int insert_error=0;
-	  partition_t *new_partition=partition_new(&arch_i386);
-	  i386_entry2partition(disk_car, (uint64_t)0, new_partition, p, status,i+1,verbose,saveheader);
-	  if(verbose>1)
-	    log_dos_entry(p);
-	  aff_part_buffer(AFF_PART_ORDER|AFF_PART_STATUS,disk_car,new_partition);
-	  if(new_partition->errcode!=BAD_NOERR)
-	  {
-	    screen_buffer_add("%s\n",errmsg_i386_entry2partition(new_partition->errcode));
-	  }
-	  new_list_part=insert_new_partition(new_list_part,new_partition, 0, &insert_error);
-	  if(insert_error>0)
-	    free(new_partition);
-	}
 	break;
+      default:
+	status=STATUS_PRIM;
+	break;
+    }
+    if(p->sys_ind != P_NO_OS)
+    {
+      int insert_error=0;
+      partition_t *new_partition=partition_new(&arch_i386);
+      i386_entry2partition(disk_car, (uint64_t)0, new_partition, p, status,i+1,verbose,saveheader);
+      if(verbose>1)
+	log_dos_entry(p);
+      aff_part_buffer(AFF_PART_ORDER|AFF_PART_STATUS,disk_car,new_partition);
+      if(new_partition->errcode!=BAD_NOERR)
+      {
+	screen_buffer_add("%s\n",errmsg_i386_entry2partition(new_partition->errcode));
+      }
+      new_list_part=insert_new_partition(new_list_part,new_partition, 0, &insert_error);
+      if(insert_error>0)
+	free(new_partition);
     }
   }
   test_MBR_data(new_list_part);
@@ -420,9 +423,10 @@ static void test_MBR_data(list_part_t *list_part)
     const partition_t *partition=element->part;
     switch(partition->status)
     {
-      case STATUS_PRIM_BOOT:
-	nb_boot++;
       case STATUS_PRIM:
+      case STATUS_PRIM_BOOT:
+	if(partition->status == STATUS_PRIM_BOOT)
+	  nb_boot++;
 	switch(partition->part_type_i386)
 	{
 	  case P_12FAT:
@@ -465,143 +469,157 @@ static void test_MBR_data(list_part_t *list_part)
       screen_buffer_add(msg_ONLY1MUSTBOOT);
 }
 
-static list_part_t *get_ext_data_i386(disk_t *disk_car, list_part_t *list_part, const int verbose, const int saveheader)
+static partition_t *get_ext_partition_i386(const list_part_t *list_part)
 {
-  list_part_t *element;
-  partition_t *partition_main_ext=NULL;
-  for(element=list_part;element!=NULL;element=element->next)
+  const list_part_t *element;
+  for(element=list_part; element!=NULL; element=element->next)
   {
     if(element->part->status==STATUS_EXT)
-      partition_main_ext=element->part;
+      return element->part;
   }
-  if(partition_main_ext!=NULL)
+  return NULL;
+}
+
+static list_part_t *get_ext_data_i386(disk_t *disk_car, list_part_t *list_part, const int verbose, const int saveheader)
+{
+  partition_t *partition_main_ext;
+  partition_t *partition_ext;
+  partition_t *partition_next_ext;
+  unsigned int order=5;
+  unsigned int nbr_part=0;
+  if((partition_main_ext=get_ext_partition_i386(list_part))==NULL)
+    return list_part;
+  for(partition_ext=partition_main_ext;
+      partition_ext!=NULL && nbr_part<32;
+      partition_ext=partition_next_ext)
   {
-    partition_t *partition_ext=partition_main_ext;
-    partition_t *partition_next_ext=NULL;
-    unsigned int order=5;
-    do
+    unsigned char buffer[DEFAULT_SECTOR_SIZE];
+    int nb_hidden=0, nb_mb=0, nb_part=0, nb_ext=0, nb_boot=0;
+    unsigned int i;
+    partition_next_ext=NULL;
+    if(partition_ext->part_offset==0)
+      return list_part;
+    if(disk_car->pread(disk_car, &buffer, sizeof(buffer), partition_ext->part_offset) != sizeof(buffer))
+      return list_part;
+    if((buffer[0x1FE]!=(unsigned char)0x55)||(buffer[0x1FF]!=(unsigned char)0xAA))
     {
-      unsigned char buffer[DEFAULT_SECTOR_SIZE];
-      int nb_hidden=0, nb_mb=0, nb_part=0, nb_ext=0, nb_boot=0;
-      unsigned int i;
-      if(disk_car->pread(disk_car, &buffer, sizeof(buffer), partition_ext->part_offset) != sizeof(buffer))
-	return list_part;
-      if((buffer[0x1FE]!=(unsigned char)0x55)||(buffer[0x1FF]!=(unsigned char)0xAA))
+      screen_buffer_add("\ntest_logical: " msg_TBL_NMARK);
+      return list_part;
+    }
+    for(i=0;i<4;i++)
+    {
+      const struct partition_dos *p=pt_offset(buffer,i);
+      if(p->boot_ind==(unsigned char)0x80)
+	nb_boot++;
+      switch(p->sys_ind)
       {
-	screen_buffer_add("\ntest_logical: " msg_TBL_NMARK);
-	return list_part;
+	case P_16FATBDH:
+	case P_16FATH:
+	case P_NTFSH:
+	  nb_hidden++;
+	  break;
+	case P_OS2MB:
+	  nb_mb++;
+	  break;
+	case P_EXTENDX:
+	case P_EXTENDED:
+	case P_LINUXEXTENDX:
+	  nb_ext++;
+	  break;
+	case P_NO_OS:
+	  break;
+	default:
+	  nb_part++;
       }
-      for(i=0;i<4;i++)
+    }
+    if(nb_hidden>0)
+      screen_buffer_add("Partition must not be hidden\n");
+    if(nb_mb>0)
+      screen_buffer_add("Multiboot must be a primary partition, not a logical\n");
+    if(nb_ext>1)
+      screen_buffer_add("A logical partition must not have more than one link to another logical partition\n");
+    if(nb_part>1)
+      screen_buffer_add("A logical partition must contain only one partition\n");
+    if(nb_boot>0)
+      screen_buffer_add("Logical partition must not be bootable\n");
+    for(i=0;i<4;i++)
+    {
+      const struct partition_dos *p=pt_offset(buffer,i);
+      if(p->sys_ind!=0)
       {
-	const struct partition_dos *p=pt_offset(buffer,i);
-	if(p->boot_ind==(unsigned char)0x80)
-	  nb_boot++;
-	switch(p->sys_ind)
+	int insert_error=0;
+	partition_t *new_partition=partition_new(&arch_i386);
+	new_partition->order=order;
+	if(verbose>1)
+	  log_dos_entry(p);
+	if(is_extended(p->sys_ind))
 	{
-	  case P_16FATBDH:
-	  case P_16FATH:
-	  case P_NTFSH:
-	    nb_hidden++;
-	    break;
-	  case P_OS2MB:
-	    nb_mb++;
-	    break;
-	  case P_EXTENDX:
-	  case P_EXTENDED:
-	  case P_LINUXEXTENDX:
-	    nb_ext++;
-	    break;
-	  case P_NO_OS:
-	    break;
-	  default:
-	    nb_part++;
-	}
-      }
-      if(nb_hidden>0)
-	screen_buffer_add("Partition must not be hidden\n");
-      if(nb_mb>0)
-	screen_buffer_add("Multiboot must be a primary partition, not a logical\n");
-      if(nb_ext>1)
-	screen_buffer_add("A logical partition must not have more than one link to another logical partition\n");
-      if(nb_part>1)
-	screen_buffer_add("A logical partition must contain only one partition\n");
-      if(nb_boot>0)
-	screen_buffer_add("Logical partition must not be bootable\n");
-      partition_next_ext=NULL;
-      for(i=0;i<4;i++)
-      {
-	const struct partition_dos *p=pt_offset(buffer,i);
-	if(p->sys_ind!=0)
-	{
-	  int insert_error=0;
-	  partition_t *new_partition=partition_new(&arch_i386);
-	  new_partition->order=order;
-	  if(verbose>1)
-	    log_dos_entry(p);
-	  if(is_extended(p->sys_ind))
+	  i386_entry2partition(disk_car, partition_main_ext->part_offset, new_partition, p, STATUS_EXT_IN_EXT,order,verbose,saveheader);
+	  aff_part_buffer(AFF_PART_ORDER|AFF_PART_STATUS,disk_car,new_partition);
+	  if(new_partition->errcode!=BAD_NOERR)
 	  {
-	    partition_next_ext=new_partition;
-	    i386_entry2partition(disk_car, partition_main_ext->part_offset, new_partition, p, STATUS_EXT_IN_EXT,order,verbose,saveheader);
-	    aff_part_buffer(AFF_PART_ORDER|AFF_PART_STATUS,disk_car,new_partition);
-	    if(new_partition->errcode!=BAD_NOERR)
-	    {
-	      screen_buffer_add("%s\n",errmsg_i386_entry2partition(new_partition->errcode));
+	    screen_buffer_add("%s\n",errmsg_i386_entry2partition(new_partition->errcode));
+	  }
+	  {
+	    if((new_partition->part_offset<=partition_main_ext->part_offset) ||
+		(new_partition->part_offset+new_partition->part_size-1 > partition_main_ext->part_offset+partition_main_ext->part_size-1))
+	    {	/* Must be IN partition_main_ext */
+	      screen_buffer_add("Must be in extended partition\n");
+	      aff_part_buffer(AFF_PART_ORDER|AFF_PART_STATUS,disk_car,partition_main_ext);
+	      aff_part_buffer(AFF_PART_ORDER|AFF_PART_STATUS,disk_car,new_partition);
 	    }
+	    else
 	    {
-	      if((new_partition->part_offset<=partition_main_ext->part_offset) ||
-		  (new_partition->part_offset+new_partition->part_size-1 > partition_main_ext->part_offset+partition_main_ext->part_size-1))
-	      {	/* Must be IN partition_main_ext */
-		screen_buffer_add("Must be in extended partition\n");
-		aff_part_buffer(AFF_PART_ORDER|AFF_PART_STATUS,disk_car,partition_main_ext);
-		aff_part_buffer(AFF_PART_ORDER|AFF_PART_STATUS,disk_car,new_partition);
-	      }
-	      else
+	      list_part_t *element;
+	      for(element=list_part;element!=NULL;element=element->next)
 	      {
-		for(element=list_part;element!=NULL;element=element->next)
+		partition_t *partition=element->part;
+		if(partition->status==STATUS_EXT_IN_EXT)
 		{
-		  partition_t *partition=element->part;
-		  if(partition->status==STATUS_EXT_IN_EXT)
-		  {
-		    if(((partition->part_offset>=new_partition->part_offset) && (partition->part_offset<=new_partition->part_offset+new_partition->part_size-1)) ||
-			((partition->part_offset+partition->part_size-1>=new_partition->part_offset) && (partition->part_offset+partition->part_size-1<=new_partition->part_offset+partition->part_size-1)))
-		    { /* New Partition start or end mustn't been in partition */
-		      screen_buffer_add( "Logical partition must be in its own extended partition\n");
-		      aff_part_buffer(AFF_PART_ORDER|AFF_PART_STATUS,disk_car,partition);
-		      aff_part_buffer(AFF_PART_ORDER|AFF_PART_STATUS,disk_car,new_partition);
-		    }
+		  if(((partition->part_offset>=new_partition->part_offset) && (partition->part_offset<=new_partition->part_offset+new_partition->part_size-1)) ||
+		      ((partition->part_offset+partition->part_size-1>=new_partition->part_offset) && (partition->part_offset+partition->part_size-1<=new_partition->part_offset+partition->part_size-1)))
+		  { /* New Partition start or end mustn't been in partition */
+		    screen_buffer_add( "Logical partition must be in its own extended partition\n");
+		    aff_part_buffer(AFF_PART_ORDER|AFF_PART_STATUS,disk_car,partition);
+		    aff_part_buffer(AFF_PART_ORDER|AFF_PART_STATUS,disk_car,new_partition);
 		  }
 		}
 	      }
 	    }
 	  }
-	  else
+	}
+	else
+	{
+	  i386_entry2partition(disk_car,partition_ext->part_offset, new_partition, p, STATUS_LOG,order,verbose,saveheader);
+	  order++;
+	  if(verbose>1)
+	    log_dos_entry(p);
+	  aff_part_buffer(AFF_PART_ORDER|AFF_PART_STATUS,disk_car,new_partition);
+	  if(new_partition->errcode!=BAD_NOERR)
 	  {
-	    i386_entry2partition(disk_car,partition_ext->part_offset, new_partition, p, STATUS_LOG,order,verbose,saveheader);
-	    order++;
-	    if(verbose>1)
-	      log_dos_entry(p);
-	    aff_part_buffer(AFF_PART_ORDER|AFF_PART_STATUS,disk_car,new_partition);
-	    if(new_partition->errcode!=BAD_NOERR)
-	    {
-	      screen_buffer_add("%s\n",errmsg_i386_entry2partition(new_partition->errcode));
-	    }
-	    {
-	      if((new_partition->part_offset<=partition_main_ext->part_offset) ||
-		  (new_partition->part_offset+new_partition->part_size-1 > partition_main_ext->part_offset+partition_main_ext->part_size-1))
-	      {	/* Must be IN partition_main_ext */
-		screen_buffer_add( msg_SAME_SPACE);
-		aff_part_buffer(AFF_PART_ORDER|AFF_PART_STATUS,disk_car,partition_main_ext);
-		aff_part_buffer(AFF_PART_ORDER|AFF_PART_STATUS,disk_car,new_partition);
-	      }
+	    screen_buffer_add("%s\n",errmsg_i386_entry2partition(new_partition->errcode));
+	  }
+	  {
+	    if((new_partition->part_offset<=partition_main_ext->part_offset) ||
+		(new_partition->part_offset+new_partition->part_size-1 > partition_main_ext->part_offset+partition_main_ext->part_size-1))
+	    {	/* Must be IN partition_main_ext */
+	      screen_buffer_add( msg_SAME_SPACE);
+	      aff_part_buffer(AFF_PART_ORDER|AFF_PART_STATUS,disk_car,partition_main_ext);
+	      aff_part_buffer(AFF_PART_ORDER|AFF_PART_STATUS,disk_car,new_partition);
 	    }
 	  }
-	  list_part=insert_new_partition(list_part, new_partition, 0, &insert_error);
-	  if(insert_error>0)
-	    free(new_partition);
+	}
+	list_part=insert_new_partition(list_part, new_partition, 0, &insert_error);
+	if(insert_error>0)
+	  free(new_partition);
+	else
+	{
+	  nbr_part++;
+	  if(is_extended(p->sys_ind))
+	    partition_next_ext=new_partition;
 	}
       }
-      partition_ext=partition_next_ext;
-    } while ((partition_ext!=NULL) && (order<30));
+    }
   }
   return list_part;
 }
@@ -649,7 +667,7 @@ static int test_MBR_over(disk_t *disk_car,list_part_t *list_part)
   return res;
 }
 
-static int write_part_i386(disk_t *disk_car, const list_part_t *list_part, const int ro, const int verbose, const int align)
+static int write_part_i386(disk_t *disk_car, const list_part_t *list_part, const int ro, const int verbose)
 {
   int res=0;
   res+=write_mbr_i386(disk_car,list_part,ro,verbose);
@@ -661,8 +679,12 @@ static int write_part_i386(disk_t *disk_car, const list_part_t *list_part, const
 static int write_mbr_i386(disk_t *disk_car, const list_part_t *list_part, const int ro, const int verbose)
 {
   const list_part_t *element;
-  unsigned char *buffer=(unsigned char *)MALLOC(disk_car->sector_size);
-  unsigned char *buffer_org=(unsigned char *)MALLOC(disk_car->sector_size);
+  unsigned char *buffer;
+  unsigned char *buffer_org;
+  if(disk_car->sector_size < DEFAULT_SECTOR_SIZE)
+    return 0;
+  buffer=(unsigned char *)MALLOC(disk_car->sector_size);
+  buffer_org=(unsigned char *)MALLOC(disk_car->sector_size);
   if(verbose>0)
   {
     log_trace("\nwrite_mbr_i386: starting...\n");
@@ -888,7 +910,7 @@ static int diff(const unsigned char buffer[DEFAULT_SECTOR_SIZE], const unsigned
 {
   if(memcmp(buffer,buffer_org,DEFAULT_SECTOR_SIZE))
   {
-    int j;
+    unsigned int j;
     log_info("\nSectors are different.\n");
     log_info("buffer_org\n");
     for(j=0;j<4;j++)
@@ -1175,6 +1197,9 @@ static int test_structure_i386(list_part_t *list_part)
       case STATUS_PRIM_BOOT:
 	if(nbr_prim_boot++)
 	  return 1;
+	nbr_prim++;
+	first_log=NULL;
+	break;
       case STATUS_PRIM:
 	nbr_prim++;
 	first_log=NULL;
@@ -1203,55 +1228,48 @@ list_part_t *add_partition_i386_cli(disk_t *disk_car, list_part_t *list_part, ch
 {
   CHS_t start,end;
   partition_t *new_partition=partition_new(&arch_i386);
+  assert(current_cmd!=NULL);
   start.cylinder=0;
   start.head=0;
   start.sector=1;
   end.cylinder=disk_car->geom.cylinders-1;
   end.head=disk_car->geom.heads_per_cylinder-1;
   end.sector=disk_car->geom.sectors_per_head;
-  while(*current_cmd[0]==',')
-    (*current_cmd)++;
   while(1)
   {
-    if(strncmp(*current_cmd,"c,",2)==0)
+    skip_comma_in_command(current_cmd);
+    if(check_command(current_cmd,"c,",2)==0)
     {
-      (*current_cmd)+=2;
       start.cylinder=ask_number_cli(current_cmd, start.cylinder,
 	  0, disk_car->geom.cylinders-1, "Enter the starting cylinder ");
     }
-    else if(strncmp(*current_cmd,"h,",2)==0)
+    else if(check_command(current_cmd,"h,",2)==0)
     {
-      (*current_cmd)+=2;
       start.head=ask_number_cli(current_cmd, start.head,
 	  0, disk_car->geom.heads_per_cylinder-1, "Enter the starting head ");
     }
-    else if(strncmp(*current_cmd,"s,",2)==0)
+    else if(check_command(current_cmd,"s,",2)==0)
     {
-      (*current_cmd)+=2;
       start.sector=ask_number_cli(current_cmd, start.sector,
 	  1, disk_car->geom.sectors_per_head, "Enter the starting sector ");
     }
-    else if(strncmp(*current_cmd,"C,",2)==0)
+    else if(check_command(current_cmd,"C,",2)==0)
     {
-      (*current_cmd)+=2;
       end.cylinder=ask_number_cli(current_cmd, end.cylinder,
 	  start.cylinder, disk_car->geom.cylinders-1, "Enter the ending cylinder ");
     }
-    else if(strncmp(*current_cmd,"H,",2)==0)
+    else if(check_command(current_cmd,"H,",2)==0)
     {
-      (*current_cmd)+=2;
       end.head=ask_number_cli(current_cmd, end.head,
 	  0, disk_car->geom.heads_per_cylinder-1, "Enter the ending head ");
     }
-    else if(strncmp(*current_cmd,"S,",2)==0)
+    else if(check_command(current_cmd,"S,",2)==0)
     {
-      (*current_cmd)+=2;
       end.sector=ask_number_cli(current_cmd, end.sector,
 	  1, disk_car->geom.sectors_per_head-1, "Enter the ending sector ");
     }
-    else if(strncmp(*current_cmd,"T,",2)==0)
+    else if(check_command(current_cmd,"T,",2)==0)
     {
-      (*current_cmd)+=2;
       change_part_type_cli(disk_car,new_partition,current_cmd);
     }
     else if((CHS2offset(disk_car,&end)>new_partition->part_offset) &&
@@ -1479,10 +1497,29 @@ static int erase_list_part_i386(disk_t *disk)
     memset(buffer,0,sizeof(buffer));
   }
   memset(buffer+TAB_PART,0,0x40);
+  /* Remove Mac signature */
+  if(buffer[0]==0x45 && buffer[1]==0x52)
+    buffer[0]=0;
+  /* Remove Sun signature */
+  if(buffer[0x1fc]==0xda && buffer[0x1fd]==0xbe)
+    buffer[0x1fc]=0;
   if(disk->pwrite(disk, buffer, DEFAULT_SECTOR_SIZE, (uint64_t)0) != DEFAULT_SECTOR_SIZE)
   {
     return 1;
   }
+  {
+    /* Erase XBOX signature if present */
+    struct xbox_partition *xboxlabel=(struct xbox_partition*)MALLOC(0x800);
+    if((unsigned)disk->pread(disk, xboxlabel, 0x800, 0) == 0x800)
+    {
+      if (memcmp(xboxlabel->magic,"BRFR",4)==0)
+      {
+	memset(xboxlabel->magic, 0, 4);
+	disk->pwrite(disk, xboxlabel, 0x800, 0);
+      }
+    }
+    free(xboxlabel);
+  }
   {
     /* Erase EFI GPT signature if present */
     struct gpt_hdr *gpt=(struct gpt_hdr*)MALLOC(disk->sector_size);
@@ -1559,10 +1596,10 @@ static int check_part_i386(disk_t *disk_car,const int verbose,partition_t *parti
       ret=check_NTFS(disk_car,partition,verbose,0);
       if(ret!=0)
       {
-	ret=check_EXFAT(disk_car, partition);
+	ret=check_exFAT(disk_car, partition);
       }
       if(ret!=0)
-      { screen_buffer_add("Invalid NTFS or EXFAT boot\n"); }
+      { screen_buffer_add("Invalid NTFS or exFAT boot\n"); }
       break;
     case P_OPENBSD:
       ret=check_BSD(disk_car,partition,verbose,OPENBSD_MAXPARTITIONS);
