@@ -33,6 +33,7 @@
 #include <string.h>
 #endif
 #include <ctype.h>      /* tolower */
+#include <assert.h>
 #include "types.h"
 #include "common.h"
 #include "fnctdsk.h"
@@ -84,21 +85,21 @@ arch_fnct_t arch_mac=
   .part_name="Mac",
   .part_name_option="partition_mac",
   .msg_part_type="                P=Primary  D=Deleted",
-  .read_part=read_part_mac,
-  .write_part=write_part_mac,
-  .init_part_order=init_part_order_mac,
+  .read_part=&read_part_mac,
+  .write_part=&write_part_mac,
+  .init_part_order=&init_part_order_mac,
   .get_geometry_from_mbr=NULL,
-  .check_part=check_part_mac,
+  .check_part=&check_part_mac,
   .write_MBR_code=NULL,
-  .set_prev_status=set_next_status_mac,
-  .set_next_status=set_next_status_mac,
-  .test_structure=test_structure_mac,
-  .get_part_type=get_part_type_mac,
-  .set_part_type=set_part_type_mac,
-  .init_structure=init_structure_mac,
+  .set_prev_status=&set_next_status_mac,
+  .set_next_status=&set_next_status_mac,
+  .test_structure=&test_structure_mac,
+  .get_part_type=&get_part_type_mac,
+  .set_part_type=&set_part_type_mac,
+  .init_structure=&init_structure_mac,
   .erase_list_part=NULL,
-  .get_partition_typename=get_partition_typename_mac,
-  .is_part_known=is_part_known_mac
+  .get_partition_typename=&get_partition_typename_mac,
+  .is_part_known=&is_part_known_mac
 };
 
 static unsigned int get_part_type_mac(const partition_t *partition)
@@ -212,16 +213,15 @@ static list_part_t *init_part_order_mac(const disk_t *disk_car, list_part_t *lis
 list_part_t *add_partition_mac_cli(disk_t *disk_car,list_part_t *list_part, char **current_cmd)
 {
   partition_t *new_partition=partition_new(&arch_mac);
+  assert(current_cmd!=NULL);
   new_partition->part_offset=disk_car->sector_size;
   new_partition->part_size=disk_car->disk_size-disk_car->sector_size;
-  while(*current_cmd[0]==',')
-    (*current_cmd)++;
   while(1)
   {
-    if(strncmp(*current_cmd,"s,",2)==0)
+    skip_comma_in_command(current_cmd);
+    if(check_command(current_cmd,"s,",2)==0)
     {
       uint64_t part_offset;
-      (*current_cmd)+=2;
       part_offset=new_partition->part_offset;
       new_partition->part_offset=(uint64_t)ask_number_cli(
 	  current_cmd,
@@ -232,9 +232,8 @@ list_part_t *add_partition_mac_cli(disk_t *disk_car,list_part_t *list_part, char
 	(uint64_t)disk_car->sector_size;
       new_partition->part_size=new_partition->part_size + part_offset - new_partition->part_offset;
     }
-    else if(strncmp(*current_cmd,"S,",2)==0)
+    else if(check_command(current_cmd,"S,",2)==0)
     {
-      (*current_cmd)+=2;
       new_partition->part_size=(uint64_t)ask_number_cli(
 	  current_cmd,
 	  (new_partition->part_offset+new_partition->part_size-1)/disk_car->sector_size,
@@ -244,9 +243,8 @@ list_part_t *add_partition_mac_cli(disk_t *disk_car,list_part_t *list_part, char
 	(uint64_t)disk_car->sector_size +
 	disk_car->sector_size - new_partition->part_offset;
     }
-    else if(strncmp(*current_cmd,"T,",2)==0)
+    else if(check_command(current_cmd,"T,",2)==0)
     {
-      (*current_cmd)+=2;
       change_part_type_cli(disk_car,new_partition,current_cmd);
     }
     else if(new_partition->part_size>0 && new_partition->part_type_mac>0)
