@@ -25,7 +25,7 @@ extern "C" {
 #endif
 
 void write_part_mac_warning_ncurses(void);
-list_part_t *add_partition_mac_ncurses(disk_t *disk_car,list_part_t *list_part, char **current_cmd);
+list_part_t *add_partition_mac_ncurses(disk_t *disk, list_part_t *list_part);
 
 #ifdef __cplusplus
 } /* closing brace for extern "C" */
