@@ -33,6 +33,7 @@
 #include <string.h>
 #endif
 #include <ctype.h>      /* tolower */
+#include <assert.h>
 #include "types.h"
 #include "common.h"
 #include "fnctdsk.h"
@@ -68,21 +69,21 @@ arch_fnct_t arch_xbox=
   .part_name="XBox",
   .part_name_option="partition_xbox",
   .msg_part_type="                P=Primary  D=Deleted",
-  .read_part=read_part_xbox,
-  .write_part=write_part_xbox,
-  .init_part_order=init_part_order_xbox,
+  .read_part=&read_part_xbox,
+  .write_part=&write_part_xbox,
+  .init_part_order=&init_part_order_xbox,
   .get_geometry_from_mbr=NULL,
-  .check_part=check_part_xbox,
+  .check_part=&check_part_xbox,
   .write_MBR_code=NULL,
-  .set_prev_status=set_next_status_xbox,
-  .set_next_status=set_next_status_xbox,
-  .test_structure=test_structure_xbox,
-  .get_part_type=get_part_type_xbox,
-  .set_part_type=set_part_type_xbox,
-  .init_structure=init_structure_xbox,
+  .set_prev_status=&set_next_status_xbox,
+  .set_next_status=&set_next_status_xbox,
+  .test_structure=&test_structure_xbox,
+  .get_part_type=&get_part_type_xbox,
+  .set_part_type=&set_part_type_xbox,
+  .init_structure=&init_structure_xbox,
   .erase_list_part=NULL,
-  .get_partition_typename=get_partition_typename_xbox,
-  .is_part_known=is_part_known_xbox
+  .get_partition_typename=&get_partition_typename_xbox,
+  .is_part_known=&is_part_known_xbox
 };
 
 static unsigned int get_part_type_xbox(const partition_t *partition)
@@ -147,16 +148,15 @@ static list_part_t *init_part_order_xbox(const disk_t *disk_car, list_part_t *li
 list_part_t *add_partition_xbox_cli(disk_t *disk_car,list_part_t *list_part, char **current_cmd)
 {
   partition_t *new_partition=partition_new(&arch_xbox);
+  assert(current_cmd!=NULL);
   new_partition->part_offset=disk_car->sector_size;
   new_partition->part_size=disk_car->disk_size-disk_car->sector_size;
-  while(*current_cmd[0]==',')
-    (*current_cmd)++;
   while(1)
   {
-    if(strncmp(*current_cmd,"s,",2)==0)
+    skip_comma_in_command(current_cmd);
+    if(check_command(current_cmd,"s,",2)==0)
     {
       uint64_t part_offset;
-      (*current_cmd)+=2;
       part_offset=new_partition->part_offset;
       new_partition->part_offset=(uint64_t)ask_number_cli(
 	  current_cmd,
@@ -167,9 +167,8 @@ list_part_t *add_partition_xbox_cli(disk_t *disk_car,list_part_t *list_part, cha
 	(uint64_t)disk_car->sector_size;
       new_partition->part_size=new_partition->part_size + part_offset - new_partition->part_offset;
     }
-    else if(strncmp(*current_cmd,"S,",2)==0)
+    else if(check_command(current_cmd,"S,",2)==0)
     {
-      (*current_cmd)+=2;
       new_partition->part_size=(uint64_t)ask_number_cli(
 	  current_cmd,
 	  (new_partition->part_offset+new_partition->part_size-1)/disk_car->sector_size,
@@ -179,9 +178,8 @@ list_part_t *add_partition_xbox_cli(disk_t *disk_car,list_part_t *list_part, cha
 	(uint64_t)disk_car->sector_size +
 	disk_car->sector_size - new_partition->part_offset;
     }
-    else if(strncmp(*current_cmd,"T,",2)==0)
+    else if(check_command(current_cmd,"T,",2)==0)
     {
-      (*current_cmd)+=2;
       change_part_type_cli(disk_car,new_partition,current_cmd);
     }
     else if(new_partition->part_size>0 && new_partition->part_type_xbox>0)
