@@ -71,12 +71,6 @@
 #include "phbf.h"
 #include "phnc.h"
 
-#if defined(HAVE_FSEEKO) && !defined(__MINGW32__)
-#define my_fseek fseeko
-#else
-#define my_fseek fseek
-#endif
-
 //#define DEBUG_BF
 //#define DEBUG_BF2
 #define READ_SIZE 1024*512
@@ -455,7 +449,7 @@ static bf_status_t photorec_bf_frag_fast(struct ph_param *params, file_recovery_
   const uint64_t original_offset_ok=file_recovery->offset_ok;
   const unsigned int blocs_to_skip=file_recovery->extra / blocksize;
   unsigned int i;
-  log_info("photorec_bf_frag_fast %s, original_offset_ok=%llu, original_offset_error=%llu, blocs_to_skip=%d, extra=%llu\n",
+  log_info("photorec_bf_frag_fast %s, original_offset_ok=%llu, original_offset_error=%llu, blocs_to_skip=%u, extra=%llu\n",
       file_recovery->filename,
       (long long unsigned)original_offset_ok,
       (long long unsigned)original_offset_error,
@@ -596,15 +590,17 @@ static bf_status_t photorec_bf_frag(struct ph_param *params, file_recovery_t *fi
     extrablock_offset=0;
     if(!td_list_empty(&file_recovery->location.list))
     {
-      const alloc_list_t *element=td_list_entry(file_recovery->location.list.prev, alloc_list_t, list);
+      const alloc_list_t *element=td_list_last_entry(&file_recovery->location.list, alloc_list_t, list);
       extrablock_offset=element->end/blocksize*blocksize;
     }
     /* Get the corresponding search_place */
-    extractblock_search_space=td_list_entry(list_search_space->list.next, alloc_data_t, list);
+    extractblock_search_space=td_list_first_entry(&list_search_space->list, alloc_data_t, list);
     while(extractblock_search_space != list_search_space &&
 	!(extractblock_search_space->start <= extrablock_offset &&
 	  extrablock_offset <= extractblock_search_space->end))
-      extractblock_search_space=td_list_entry(extractblock_search_space->list.next, alloc_data_t, list);
+    {
+      extractblock_search_space=td_list_next_entry(extractblock_search_space, list);
+    }
     /* Update extractblock_search_space & extrablock_offset */
     get_next_sector(list_search_space, &extractblock_search_space, &extrablock_offset, blocksize);
     /* */
