@@ -22,7 +22,7 @@
 #ifdef __cplusplus
 extern "C" {
 #endif
-int photorec_bf(struct ph_param *params, const struct ph_options *options, alloc_data_t *list_search_space);
+pstatus_t photorec_bf(struct ph_param *params, const struct ph_options *options, alloc_data_t *list_search_space);
 #ifdef __cplusplus
 } /* closing brace for extern "C" */
 #endif
