@@ -55,12 +55,6 @@
 #include "setdate.h"
 #include "dfxml.h"
 
-#if defined(HAVE_FSEEKO) && !defined(__MINGW32__)
-#define my_fseek fseeko
-#else
-#define my_fseek fseek
-#endif
-
 /* #define DEBUG_FILE_FINISH */
 /* #define DEBUG_UPDATE_SEARCH_SPACE */
 /* #define DEBUG_FREE */
@@ -77,7 +71,7 @@ void file_block_log(const file_recovery_t *file_recovery, const unsigned int sec
   log_info("%s\t",file_recovery->filename);
   td_list_for_each(tmp, &file_recovery->location.list)
   {
-    const alloc_list_t *element=td_list_entry(tmp, alloc_list_t, list);
+    const alloc_list_t *element=td_list_entry_const(tmp, const alloc_list_t, list);
     if(element->data>0)
       log_info(" %lu-%lu", (unsigned long)(element->start/sector_size), (unsigned long)(element->end/sector_size));
     else
@@ -133,7 +127,7 @@ static void update_search_space_aux(alloc_data_t *list_search_space, const uint6
       if(offset!=NULL && new_current_search_space!=NULL &&
           current_search_space->start<=*offset && *offset<=current_search_space->end)
       {
-        *new_current_search_space=td_list_entry(current_search_space->list.next, alloc_data_t, list);
+        *new_current_search_space=td_list_next_entry(current_search_space, list);
         *offset=(*new_current_search_space)->start;
       }
       td_list_del(search_walker);
@@ -152,7 +146,7 @@ static void update_search_space_aux(alloc_data_t *list_search_space, const uint6
         if(offset!=NULL && new_current_search_space!=NULL &&
             start<=*offset && *offset<=current_search_space->end)
         {
-          *new_current_search_space=td_list_entry(current_search_space->list.next, alloc_data_t, list);
+	  *new_current_search_space=td_list_next_entry(current_search_space, list);
           *offset=(*new_current_search_space)->start;
         }
         current_search_space->end=start-1;
@@ -162,7 +156,7 @@ static void update_search_space_aux(alloc_data_t *list_search_space, const uint6
       if(offset!=NULL && new_current_search_space!=NULL &&
           current_search_space->start<=*offset && *offset<=current_search_space->end)
       {
-        *new_current_search_space=td_list_entry(current_search_space->list.next, alloc_data_t, list);
+	*new_current_search_space=td_list_next_entry(current_search_space, list);
         *offset=(*new_current_search_space)->start;
       }
       td_list_del(search_walker);
@@ -259,7 +253,7 @@ unsigned int photorec_mkdir(const char *recup_dir, const unsigned int initial_di
       if(mkdir(working_recup_dir, 0775)!=0 && errno==EEXIST)
 #endif
 #else
-#warning You need a mkdir function!
+#warning "You need a mkdir function!"
 #endif
       {
 	dir_num++;
@@ -281,26 +275,6 @@ unsigned int photorec_mkdir(const char *recup_dir, const unsigned int initial_di
   return dir_num;
 }
 
-void get_prev_location(alloc_data_t *list_search_space, alloc_data_t **current_search_space, uint64_t *offset, const uint64_t prev_location)
-{
-  int nbr;
-  alloc_data_t *file_space=*current_search_space;
-  uint64_t size=0;
-  /* Search backward the first fragment of a file not successfully recovered
-   * Limit the search to 10 fragments or 1GB */
-  for(nbr=0; nbr<3 && size < (uint64_t)200*1024*1024; nbr++)
-  {
-    file_space=td_list_entry(file_space->list.prev, alloc_data_t, list);
-    if(file_space==list_search_space)
-      return;
-    size+=file_space->end - file_space->start + 1;
-    if(file_space->start < prev_location)
-      return ;
-    *current_search_space=file_space;
-    *offset=file_space->start;
-  }
-}
-
 int get_prev_file_header(alloc_data_t *list_search_space, alloc_data_t **current_search_space, uint64_t *offset)
 {
   int nbr;
@@ -310,7 +284,7 @@ int get_prev_file_header(alloc_data_t *list_search_space, alloc_data_t **current
    * Limit the search to 10 fragments or 1GB */
   for(nbr=0; nbr<3 && size < (uint64_t)200*1024*1024; nbr++)
   {
-    file_space=td_list_entry(file_space->list.prev, alloc_data_t, list);
+    file_space=td_list_prev_entry(file_space, list);
     if(file_space==list_search_space)
       return -1;
     size+=file_space->end - file_space->start + 1;
@@ -411,7 +385,7 @@ void write_stats_log(const file_stat_t *file_stats)
     }
   }
   free(new_file_stats);
-  if(file_nbr>1)
+  if(file_nbr!=1)
   {
     log_info("Total: %u files found\n\n",file_nbr);
   }
@@ -451,13 +425,13 @@ unsigned int find_blocksize(alloc_data_t *list_search_space, const unsigned int
   *offset=0;
   if(td_list_empty(&list_search_space->list))
     return default_blocksize;
-  *offset=(td_list_entry(list_search_space->list.next, alloc_data_t, list))->start % blocksize;
+  *offset=(td_list_first_entry(&list_search_space->list, alloc_data_t, list))->start % blocksize;
   do
   {
     run_again=0;
     td_list_for_each(search_walker, &list_search_space->list)
     {
-      const alloc_data_t *tmp=td_list_entry(search_walker, alloc_data_t, list);
+      const alloc_data_t *tmp=td_list_entry_const(search_walker, const alloc_data_t, list);
       if(tmp->file_stat!=NULL)
       {
 	if(tmp->start%blocksize!=*offset && blocksize>default_blocksize)
@@ -474,7 +448,7 @@ unsigned int find_blocksize(alloc_data_t *list_search_space, const unsigned int
 
 void update_blocksize(const unsigned int blocksize, alloc_data_t *list_search_space, const uint64_t offset)
 {
-  struct td_list_head *search_walker = NULL;
+  struct td_list_head *search_walker;
   struct td_list_head *search_walker_prev = NULL;
   log_info("blocksize=%u, offset=%u\n", blocksize, (unsigned int)(offset%blocksize));
   /* Align end of last range (round up) */
@@ -536,6 +510,7 @@ void file_block_free(alloc_list_t *list_allocation)
   {
     alloc_list_t *allocated_space;
     allocated_space=td_list_entry(tmp, alloc_list_t, list);
+    header_ignored_cond_reset(allocated_space->start, allocated_space->end);
     free_list_allocation_end=allocated_space->end;
     td_list_del(tmp);
     free(allocated_space);
@@ -590,7 +565,7 @@ static void file_finish_aux(file_recovery_t *file_recovery, struct ph_param *par
   if(file_recovery->time!=0 && file_recovery->time!=(time_t)-1)
     set_date(file_recovery->filename, file_recovery->time, file_recovery->time);
   if(file_recovery->file_rename!=NULL)
-    file_recovery->file_rename(file_recovery->filename);
+    file_recovery->file_rename(file_recovery);
   if((++params->file_nbr)%MAX_FILES_PER_DIR==0)
   {
     params->dir_num=photorec_mkdir(params->recup_dir, params->dir_num+1);
@@ -641,16 +616,22 @@ int file_finish_bf(file_recovery_t *file_recovery, struct ph_param *params,
   return 1;
 }
 
-/*  file_finish2()
-    @param file_recovery - 
-    @param struct ph_param *params
-    const struct ph_options *options
-    @param alloc_data_t *list_search_space
+void file_recovery_aborted(file_recovery_t *file_recovery, struct ph_param *params, alloc_data_t *list_search_space)
+{
+  if(file_recovery->file_stat==NULL)
+    return ;
+  params->offset=file_recovery->location.start;
+  if(file_recovery->handle)
+  {
+    fclose(file_recovery->handle);
+    file_recovery->handle=NULL;
+    /* File is zero-length; erase it */
+    unlink(file_recovery->filename);
+  }
+  file_block_truncate_zero(file_recovery, list_search_space);
+  reset_file_recovery(file_recovery);
+}
 
-    @returns:
-    0: file not recovered
-    1: file recovered
- */
 pfstatus_t file_finish2(file_recovery_t *file_recovery, struct ph_param *params, const int paranoid, alloc_data_t *list_search_space)
 {
   int file_truncated;
@@ -701,7 +682,7 @@ void info_list_search_space(const alloc_data_t *list_search_space, const alloc_d
            "(null)"));
     }
   }
-  log_info("%llu sectors contains unknown data, %lu invalid files found %s.\n",
+  log_info("%llu sectors contain unknown data, %lu invalid files found %s.\n",
       (long long unsigned)sectors_with_unknown_data, (long unsigned)nbr_headers,
       (keep_corrupted_file>0?"but saved":"and rejected"));
 }
@@ -726,16 +707,16 @@ void set_filename(file_recovery_t *file_recovery, struct ph_param *params)
   if(file_recovery->extension==NULL || file_recovery->extension[0]=='\0')
   {
     snprintf(file_recovery->filename,sizeof(file_recovery->filename)-1,
-	"%s.%u/%c%07u", params->recup_dir,
+	"%s.%u/%c%07lu", params->recup_dir,
 	params->dir_num, (broken?'b':'f'),
-	(unsigned int)((file_recovery->location.start - params->partition->part_offset)/ params->disk->sector_size));
+	(unsigned long int)((file_recovery->location.start - params->partition->part_offset)/ params->disk->sector_size));
   }
   else
   {
     snprintf(file_recovery->filename,sizeof(file_recovery->filename)-1,
-	"%s.%u/%c%07u.%s", params->recup_dir,
+	"%s.%u/%c%07lu.%s", params->recup_dir,
 	params->dir_num, (broken?'b':'f'),
-	(unsigned int)((file_recovery->location.start - params->partition->part_offset) / params->disk->sector_size), file_recovery->extension);
+	(unsigned long int)((file_recovery->location.start - params->partition->part_offset) / params->disk->sector_size), file_recovery->extension);
   }
 }
 
@@ -768,8 +749,7 @@ uint64_t set_search_start(struct ph_param *params, alloc_data_t **new_current_se
   else if(params->cmd_run!=NULL && params->cmd_run[0]!='\0')
   {
     offset=0;
-    while(*params->cmd_run==',')
-      params->cmd_run++;
+    skip_comma_in_command(&params->cmd_run);
     while(*params->cmd_run >= '0' && *params->cmd_run <= '9')
     {
       offset=offset * 10 + (*params->cmd_run - '0');
@@ -890,7 +870,7 @@ static inline void file_block_remove_from_sp_aux(alloc_data_t *tmp, alloc_data_t
     tmp->file_stat=NULL;
     if(tmp->start <= tmp->end)
       return ;
-    *new_current_search_space=td_list_entry(tmp->list.next, alloc_data_t, list);
+    *new_current_search_space=td_list_next_entry(tmp, list);
     *offset=(*new_current_search_space)->start;
     td_list_del(&tmp->list);
     free(tmp);
@@ -899,7 +879,7 @@ static inline void file_block_remove_from_sp_aux(alloc_data_t *tmp, alloc_data_t
   if(*offset + blocksize == tmp->end + 1)
   {
     tmp->end-=blocksize;
-    *new_current_search_space=td_list_entry(tmp->list.next, alloc_data_t, list);
+    *new_current_search_space=td_list_next_entry(tmp, list);
     *offset=(*new_current_search_space)->start;
     return ;
   }
@@ -927,14 +907,20 @@ static inline void file_block_remove_from_sp(alloc_data_t *list_search_space, al
     alloc_data_t *tmp;
     tmp=td_list_entry(search_walker, alloc_data_t, list);
     if(tmp->start <= *offset && *offset + blocksize <= tmp->end + 1)
-      return file_block_remove_from_sp_aux(tmp, new_current_search_space, offset, blocksize);
+    {
+      file_block_remove_from_sp_aux(tmp, new_current_search_space, offset, blocksize);
+      return;
+    }
   }
   td_list_for_each(search_walker, &list_search_space->list)
   {
     alloc_data_t *tmp;
     tmp=td_list_entry(search_walker, alloc_data_t, list);
     if(tmp->start <= *offset && *offset + blocksize <= tmp->end + 1)
-      return file_block_remove_from_sp_aux(tmp, new_current_search_space, offset, blocksize);
+    {
+      file_block_remove_from_sp_aux(tmp, new_current_search_space, offset, blocksize);
+      return ;
+    }
   }
   log_critical("file_block_remove_from_sp(list_search_space, alloc_data_t **new_current_search_space, uint64_t *offset, const unsigned int blocksize) failed\n");
 }
@@ -943,7 +929,7 @@ static inline void file_block_add_to_file(alloc_list_t *list, const uint64_t off
 {
   if(!td_list_empty(&list->list))
   {
-    alloc_list_t *prev=td_list_entry(list->list.prev, alloc_list_t, list);
+    alloc_list_t *prev=td_list_last_entry(&list->list, alloc_list_t, list);
     if(prev->end+1==offset && prev->data==data)
     {
       prev->end=offset+blocksize-1;
