@@ -109,9 +109,9 @@ static void recovery_finished(disk_t *disk, const partition_t *partition, const
       if(file_nbr > 0)
       {
 	wmove(stdscr, 12, 0);
-	wprintw(stdscr, "You are welcome to donate to support further development and encouragement");
+	wprintw(stdscr, "You are welcome to donate to support and encourage further development");
 	wmove(stdscr, 13, 0);
-	wprintw(stdscr, "http://www.cgsecurity.org/wiki/Donation");
+	wprintw(stdscr, "https://www.cgsecurity.org/wiki/Donation");
       }
       break;
     case PSTATUS_STOP:
@@ -181,7 +181,7 @@ static int interface_cannot_create_file(void)
   wmove(stdscr,5,0);
   wprintw(stdscr,"This problem may be due to antivirus blocking write access while scanning files created by PhotoRec.");
   wmove(stdscr,6,0);
-  wprintw(stdscr,"If possible, temporary disable your antivirus live protection.");
+  wprintw(stdscr,"If possible, temporarily disable your live antivirus protection.");
   car= wmenuSelect_ext(stdscr, 23, INTER_MAIN_Y, INTER_MAIN_X, menuMain, 10,
       "CQ", MENU_VERT | MENU_VERT_WARN | MENU_BUTTON, &menu,NULL);
   if(car=='c' || car=='C')
@@ -236,47 +236,38 @@ int photorec(struct ph_param *params, const struct ph_options *options, alloc_da
   params_reset(params, options);
   if(params->cmd_run!=NULL && params->cmd_run[0]!='\0')
   {
-    while(params->cmd_run[0]==',')
-      params->cmd_run++;
-    if(strncmp(params->cmd_run,"status=unformat",15)==0)
+    skip_comma_in_command(&params->cmd_run);
+    if(check_command(&params->cmd_run,"status=unformat",15)==0)
     {
       params->status=STATUS_UNFORMAT;
-      params->cmd_run+=15;
     }
-    else if(strncmp(params->cmd_run,"status=find_offset",18)==0)
+    else if(check_command(&params->cmd_run,"status=find_offset",18)==0)
     {
       params->status=STATUS_FIND_OFFSET;
-      params->cmd_run+=18;
     }
-    else if(strncmp(params->cmd_run,"status=ext2_on_bf",17)==0)
+    else if(check_command(&params->cmd_run,"status=ext2_on_bf",17)==0)
     {
       params->status=STATUS_EXT2_ON_BF;
-      params->cmd_run+=17;
     }
-    else if(strncmp(params->cmd_run,"status=ext2_on_save_everything",33)==0)
+    else if(check_command(&params->cmd_run,"status=ext2_on_save_everything",30)==0)
     {
       params->status=STATUS_EXT2_ON_SAVE_EVERYTHING;
-      params->cmd_run+=33;
     }
-    else if(strncmp(params->cmd_run,"status=ext2_on",14)==0)
+    else if(check_command(&params->cmd_run,"status=ext2_on",14)==0)
     {
       params->status=STATUS_EXT2_ON;
-      params->cmd_run+=14;
     }
-    else if(strncmp(params->cmd_run,"status=ext2_off_bf",18)==0)
+    else if(check_command(&params->cmd_run,"status=ext2_off_bf",18)==0)
     {
       params->status=STATUS_EXT2_OFF_BF;
-      params->cmd_run+=18;
     }
-    else if(strncmp(params->cmd_run,"status=ext2_off_save_everything",34)==0)
+    else if(check_command(&params->cmd_run,"status=ext2_off_save_everything",31)==0)
     {
       params->status=STATUS_EXT2_OFF_SAVE_EVERYTHING;
-      params->cmd_run+=34;
     }
-    else if(strncmp(params->cmd_run,"status=ext2_off",15)==0)
+    else if(check_command(&params->cmd_run,"status=ext2_off",15)==0)
     {
       params->status=STATUS_EXT2_OFF;
-      params->cmd_run+=15;
     }
   }
   else
@@ -333,7 +324,7 @@ int photorec(struct ph_param *params, const struct ph_options *options, alloc_da
 	  {
 	    ind_stop=PSTATUS_OK;
 	    if(!td_list_empty(&list_search_space->list))
-	      start_offset=(td_list_entry(list_search_space->list.next, alloc_data_t, list))->start % params->blocksize;
+	      start_offset=(td_list_first_entry(&list_search_space->list, alloc_data_t, list))->start % params->blocksize;
 	  }
 	  else
 	  {
@@ -371,7 +362,7 @@ int photorec(struct ph_param *params, const struct ph_options *options, alloc_da
 	    if(res!=NULL)
 	      *res='\0';
 	  }
-	  res=ask_location("Warning: not enough free space available. Please select a destination to save the recovered files.\nDo not choose to write the files to the same partition they were stored on.", "", dst);
+	  res=ask_location("Warning: not enough free space available. Please select a destination to save the recovered files to.\nDo not choose to write the files to the same partition they were stored on.", "", dst);
 	  free(dst);
 	  if(res==NULL)
 	    params->status=STATUS_QUIT;
@@ -380,7 +371,8 @@ int photorec(struct ph_param *params, const struct ph_options *options, alloc_da
 	    free(params->recup_dir);
 	    params->recup_dir=(char *)MALLOC(strlen(res)+1+strlen(DEFAULT_RECUP_DIR)+1);
 	    strcpy(params->recup_dir,res);
-	    strcat(params->recup_dir,"/");
+	    if(strcmp(params->recup_dir,"/")!=0)
+	      strcat(params->recup_dir,"/");
 	    strcat(params->recup_dir,DEFAULT_RECUP_DIR);
 	    free(res);
 	    /* Create the directory */
@@ -406,6 +398,7 @@ int photorec(struct ph_param *params, const struct ph_options *options, alloc_da
 	}
 	else
 	{
+	  log_flush();
 #ifdef HAVE_NCURSES
 	  if(ask_confirmation("Answer Y to really Quit, N to resume the recovery")!=0)
 #endif
@@ -463,7 +456,8 @@ int photorec(struct ph_param *params, const struct ph_options *options, alloc_da
       }
       filename=(char *)MALLOC(strlen(dst_path) + 1 + strlen(DEFAULT_IMAGE_NAME) + 1);
       strcpy(filename, dst_path);
-      strcat(filename, "/");
+      if(strcmp(params->recup_dir,"/")!=0)
+	strcat(filename, "/");
       strcat(filename, DEFAULT_IMAGE_NAME);
       gen_image(filename, params->disk, list_search_space);
       free(filename);
@@ -596,7 +590,7 @@ void interface_file_select_ncurses(file_enable_t *files_enable)
     wmove(stdscr,5,4);
     if(offset>0)
       wprintw(stdscr,"Previous");
-    for(i=offset;files_enable[i].file_hint!=NULL && i<offset+INTER_FSELECT;i++)
+    for(i=offset; i<offset+INTER_FSELECT && files_enable[i].file_hint!=NULL; i++)
     {
       wmove(stdscr,6+i-offset,0);
       wclrtoeol(stdscr);	/* before addstr for BSD compatibility */
