@@ -24,8 +24,8 @@ extern "C" {
 #endif
 
 int photorec(struct ph_param *params, const struct ph_options *options, alloc_data_t *list_search_space);
-void interface_file_select(file_enable_t *files_enable, char**current_cmd);
 #ifdef HAVE_NCURSES
+void interface_file_select_ncurses(file_enable_t *files_enable);
 void interface_options_photorec_ncurses(struct ph_options *options);
 #endif
 
