@@ -68,153 +68,8 @@
 #include "phnc.h"
 #endif
 #include "psearchn.h"
-
+#include "photorec_check_header.h"
 #define READ_SIZE 1024*512
-extern const file_hint_t file_hint_tar;
-extern const file_hint_t file_hint_dir;
-extern file_check_list_t file_check_list;
-
-#if defined(__CYGWIN__) || defined(__MINGW32__)
-/* Live antivirus protection may open file as soon as they are created by *
- * PhotoRec. PhotoRec will not be able to overwrite a file as long as the *
- * antivirus is scanning it, so let's wait a little bit if the creation   *
- * failed. */
-
-#ifndef HAVE_SLEEP
-#define sleep(x) Sleep((x)*1000)
-#endif
-
-static FILE *fopen_with_retry(const char *path, const char *mode)
-{
-  FILE *handle;
-  if((handle=fopen(path, mode))!=NULL)
-    return handle;
-#ifdef __MINGW32__
-  Sleep(1000);
-#else
-  sleep(1);
-#endif
-  if((handle=fopen(path, mode))!=NULL)
-    return handle;
-#ifdef __MINGW32__
-  Sleep(2000);
-#else
-  sleep(2);
-#endif
-  if((handle=fopen(path, mode))!=NULL)
-    return handle;
-  return NULL;
-}
-#endif
-
-static void photorec_dir_fat(const unsigned char *buffer, const unsigned int read_size, const unsigned long long sector)
-{
-  file_info_t dir_list = {
-    .list = TD_LIST_HEAD_INIT(dir_list.list),
-    .name = NULL
-  };
-  dir_fat_aux(buffer, read_size, 0, &dir_list);
-  if(!td_list_empty(&dir_list.list))
-  {
-    log_info("Sector %llu\n", sector);
-    dir_aff_log(NULL, &dir_list);
-    delete_list_file(&dir_list);
-  }
-}
-
-static pstatus_t photorec_new_file(file_recovery_t *file_recovery, struct ph_param *params, const uint64_t offset)
-{
-  set_filename(file_recovery, params);
-  if(file_recovery->file_stat->file_hint->recover==1)
-  {
-#if defined(__CYGWIN__) || defined(__MINGW32__)
-    file_recovery->handle=fopen_with_retry(file_recovery->filename,"w+b");
-#else
-    file_recovery->handle=fopen(file_recovery->filename,"w+b");
-#endif
-    if(!file_recovery->handle)
-    { 
-      log_critical("Cannot create file %s: %s\n", file_recovery->filename, strerror(errno));
-      params->offset=offset;
-      return PSTATUS_EACCES;
-    }
-  }
-  return PSTATUS_OK;
-}
-
-static pstatus_t photorec_header_found(file_recovery_t *file_recovery_new, file_recovery_t *file_recovery, struct ph_param *params, const struct ph_options *options, alloc_data_t *list_search_space, const unsigned char *buffer, pfstatus_t *file_recovered, const uint64_t offset)
-{
-  *file_recovered=PFSTATUS_BAD;
-  if(file_recovery_new->file_stat==NULL || file_recovery_new->file_stat->file_hint==NULL)
-    return PSTATUS_OK;
-  file_recovery_new->location.start=offset;
-  if(file_recovery->file_stat!=NULL)
-  {
-    if(options->verbose > 1)
-      log_trace("A known header has been found, recovery of the previous file is finished\n");
-    *file_recovered=file_finish2(file_recovery, params, options->paranoid, list_search_space);
-    if(*file_recovered==PFSTATUS_OK_TRUNCATED)
-      return PSTATUS_OK;
-  }
-  file_recovery_cpy(file_recovery, file_recovery_new);
-  if(options->verbose > 1)
-  {
-    log_info("%s header found at sector %lu\n",
-	((file_recovery->extension!=NULL && file_recovery->extension[0]!='\0')?
-	 file_recovery->extension:file_recovery->file_stat->file_hint->description),
-	(unsigned long)((file_recovery->location.start-params->partition->part_offset)/params->disk->sector_size));
-    log_info("file_recovery->location.start=%lu\n",
-	(unsigned long)(file_recovery->location.start/params->disk->sector_size));
-  }
-  if(file_recovery->file_stat->file_hint==&file_hint_dir && options->verbose > 0)
-  { /* FAT directory found, list the file */
-    const unsigned int blocksize=params->blocksize;
-    const unsigned int read_size=(blocksize>65536?blocksize:65536);
-    photorec_dir_fat(buffer, read_size, file_recovery->location.start/params->disk->sector_size);
-  }
-  return photorec_new_file(file_recovery, params, offset);
-}
-
-inline static pstatus_t photorec_check_header(file_recovery_t *file_recovery, struct ph_param *params, const struct ph_options *options, alloc_data_t *list_search_space, const unsigned char *buffer, pfstatus_t *file_recovered, const uint64_t offset)
-{
-  const struct td_list_head *tmpl;
-  const unsigned int blocksize=params->blocksize;
-  const unsigned int read_size=(blocksize>65536?blocksize:65536);
-  file_recovery_t file_recovery_new;
-  file_recovery_new.blocksize=blocksize;
-  if(file_recovery->file_stat!=NULL &&
-      file_recovery->file_stat->file_hint->min_header_distance > 0 &&
-      file_recovery->file_size<=file_recovery->file_stat->file_hint->min_header_distance)
-    return PSTATUS_OK;
-  if(file_recovery->file_stat!=NULL && file_recovery->file_stat->file_hint==&file_hint_tar &&
-      header_check_tar(buffer-0x200,0x200, 0, file_recovery, &file_recovery_new))
-  { /* Currently saving a tar, do not check the data for know header */
-    if(options->verbose > 1)
-    {
-      log_verbose("Currently saving a tar file, sector %lu.\n",
-	  (unsigned long)((offset-params->partition->part_offset)/params->disk->sector_size));
-    }
-    return PSTATUS_OK;
-  }
-  file_recovery_new.file_stat=NULL;
-  file_recovery_new.location.start=offset;
-  td_list_for_each(tmpl, &file_check_list.list)
-  {
-    const struct td_list_head *tmp;
-    const file_check_list_t *pos=td_list_entry_const(tmpl, const file_check_list_t, list);
-    td_list_for_each(tmp, &pos->file_checks[buffer[pos->offset]].list)
-    {
-      const file_check_t *file_check=td_list_entry_const(tmp, const file_check_t, list);
-      if((file_check->length==0 || memcmp(buffer + file_check->offset, file_check->value, file_check->length)==0) &&
-	  file_check->header_check(buffer, read_size, 0, file_recovery, &file_recovery_new)!=0)
-      {
-	file_recovery_new.file_stat=file_check->file_stat;
-	return photorec_header_found(&file_recovery_new, file_recovery, params, options, list_search_space, buffer, file_recovered, offset);
-      }
-    }
-  }
-  return PSTATUS_OK;
-}
 
 pstatus_t photorec_aux(struct ph_param *params, const struct ph_options *options, alloc_data_t *list_search_space)
 {
@@ -244,7 +99,7 @@ pstatus_t photorec_aux(struct ph_param *params, const struct ph_options *options
   previous_time=start_time;
   next_checkpoint=start_time+5*60;
   memset(buffer_olddata,0,blocksize);
-  current_search_space=td_list_entry(list_search_space->list.next, alloc_data_t, list);
+  current_search_space=td_list_first_entry(&list_search_space->list, alloc_data_t, list);
   offset=set_search_start(params, &current_search_space, list_search_space);
   if(options->verbose > 0)
     info_list_search_space(list_search_space, current_search_space, params->disk->sector_size, 0, options->verbose);
@@ -255,11 +110,12 @@ pstatus_t photorec_aux(struct ph_param *params, const struct ph_options *options
 	(unsigned long long)((params->partition->part_size-1)/params->disk->sector_size));
   }
   params->disk->pread(params->disk, buffer, READ_SIZE, offset);
+  header_ignored(NULL);
   while(current_search_space!=list_search_space)
   {
     pfstatus_t file_recovered=PFSTATUS_BAD;
     uint64_t old_offset=offset;
-    int res=DC_SCAN;
+    data_check_t res=DC_SCAN;
 #ifdef DEBUG
     log_debug("sector %llu\n",
         (unsigned long long)((offset-params->partition->part_offset)/params->disk->sector_size));
@@ -329,13 +185,13 @@ pstatus_t photorec_aux(struct ph_param *params, const struct ph_options *options
       if(res!=DC_STOP && res!=DC_ERROR && file_recovery.file_stat->file_hint->max_filesize>0 && file_recovery.file_size>=file_recovery.file_stat->file_hint->max_filesize)
       {
 	res=DC_STOP;
-	log_verbose("File should not be bigger than %llu, stop adding data\n",
+	log_verbose("File should not be bigger than %llu, stopped adding data\n",
 	    (long long unsigned)file_recovery.file_stat->file_hint->max_filesize);
       }
       if(res!=DC_STOP && res!=DC_ERROR && file_recovery.file_size + blocksize >= PHOTOREC_MAX_SIZE_32 && is_fat(params->partition))
       {
       	res=DC_STOP;
-	log_verbose("File should not be bigger than %llu, stop adding data\n",
+	log_verbose("File should not be bigger than %llu, stopped adding data\n",
 	    (long long unsigned)file_recovery.file_stat->file_hint->max_filesize);
       }
       if(res==DC_STOP || res==DC_ERROR)
@@ -350,9 +206,11 @@ pstatus_t photorec_aux(struct ph_param *params, const struct ph_options *options
     if(ind_stop!=PSTATUS_OK)
     {
       log_info("PhotoRec has been stopped\n");
-      current_search_space=list_search_space;
+      file_recovery_aborted(&file_recovery, params, list_search_space);
+      free(buffer_start);
+      return ind_stop;
     }
-    else if(file_recovered==PFSTATUS_BAD)
+    if(file_recovered==PFSTATUS_BAD)
     {
       if(res==DC_SCAN)
       {
@@ -372,7 +230,7 @@ pstatus_t photorec_aux(struct ph_param *params, const struct ph_options *options
       else
       {
 	back=0;
-	get_prev_location(list_search_space, &current_search_space, &offset, file_recovery.location.start);
+	get_prev_location_smart(list_search_space, &current_search_space, &offset, file_recovery.location.start);
       }
     }
     if(current_search_space==list_search_space)
@@ -384,7 +242,7 @@ pstatus_t photorec_aux(struct ph_param *params, const struct ph_options *options
 #endif
       file_recovered=file_finish2(&file_recovery, params, options->paranoid, list_search_space);
       if(file_recovered!=PFSTATUS_BAD)
-	get_prev_location(list_search_space, &current_search_space, &offset, file_recovery.location.start);
+	get_prev_location_smart(list_search_space, &current_search_space, &offset, file_recovery.location.start);
       if(options->lowmem > 0)
 	forget(list_search_space,current_search_space);
     }
@@ -425,17 +283,15 @@ pstatus_t photorec_aux(struct ph_param *params, const struct ph_options *options
           ind_stop=photorec_progressbar(stdscr, params->pass, params, offset, current_time);
 #endif
 	  params->offset=offset;
-	  if(current_time >= next_checkpoint)
-	  {
-	    /* Save current progress */
-	    session_save(list_search_space, params, options);
-	    next_checkpoint=current_time+5*60;
-	  }
 	  if(ind_stop!=PSTATUS_OK)
 	  {
 	    log_info("PhotoRec has been stopped\n");
-	    current_search_space=list_search_space;
+	    file_recovery_aborted(&file_recovery, params, list_search_space);
+	    free(buffer_start);
+	    return ind_stop;
 	  }
+	  if(current_time >= next_checkpoint)
+	    next_checkpoint=regular_session_save(list_search_space, params, options, current_time);
         }
       }
     }
