@@ -25,6 +25,7 @@
 #endif
 
 #include <QApplication>
+#include <QTranslator>
 #include <stdio.h>
 #include <stdlib.h>
 #ifdef HAVE_UNISTD_H
@@ -59,11 +60,13 @@ static void display_help(void)
 
 static void display_version(void)
 {
-  printf("QPhotoRec %s, Data Recovery Utility, %s\nChristophe GRENIER <grenier@cgsecurity.org>\nhttp://www.cgsecurity.org\n",VERSION,TESTDISKDATE);
+  printf("QPhotoRec %s, Data Recovery Utility, %s\nChristophe GRENIER <grenier@cgsecurity.org>\nhttps://www.cgsecurity.org\n",VERSION,TESTDISKDATE);
   printf("\n");
   printf("Version: %s\n", VERSION);
   printf("Compiler: %s\n", get_compiler());
+#ifdef RECORD_COMPILATION_DATE
   printf("Compilation date: %s\n", get_compilation_date());
+#endif
   printf("ext2fs lib: %s, ntfs lib: %s, ewf lib: %s, libjpeg: %s\n",
       td_ext2fs_version(), td_ntfs_version(), td_ewf_version(), td_jpeg_version());
   printf("OS: %s\n" , get_os());
@@ -101,8 +104,20 @@ int main(int argc, char *argv[])
     }
   }
 #endif
-  QApplication a(argc, argv);
   log_handle=log_open("qphotorec.log", TD_LOG_CREATE, &log_errno);
+  QApplication a(argc, argv);
+  QTranslator translator;
+  QString locale = QLocale::system().name().section('_', 0, 0);
+  log_info("%s\n", locale.toStdString().c_str());
+  if(translator.load(QLocale(), "qphotorec", ".", ":lang/", ".qm"))
+  {
+    a.installTranslator(&translator);
+    log_info("translator installed\n");
+  }
+  else
+  {
+    log_info("translator not installed\n");
+  }
 #ifdef HAVE_DUP2
   if(log_handle)
   {
@@ -111,10 +126,12 @@ int main(int argc, char *argv[])
 #endif
   my_time=time(NULL);
   log_info("\n\n%s",ctime(&my_time));
-  log_info("PhotoRec %s, Data Recovery Utility, %s\nChristophe GRENIER <grenier@cgsecurity.org>\nhttp://www.cgsecurity.org\n", VERSION, TESTDISKDATE);
+  log_info("PhotoRec %s, Data Recovery Utility, %s\nChristophe GRENIER <grenier@cgsecurity.org>\nhttps://www.cgsecurity.org\n", VERSION, TESTDISKDATE);
   log_info("OS: %s\n" , get_os());
   log_info("Compiler: %s\n", get_compiler());
+#ifdef RECORD_COMPILATION_DATE
   log_info("Compilation date: %s\n", get_compilation_date());
+#endif
   log_info("ext2fs lib: %s, ntfs lib: %s, ewf lib: %s, libjpeg: %s\n",
       td_ext2fs_version(), td_ntfs_version(), td_ewf_version(), td_jpeg_version());
 
