@@ -62,7 +62,9 @@ static int testdisk_disk_selection_ncurses(int verbose,int dump_ind, const list_
   unsigned int menu=0;
   int offset=0;
   int pos_num=0;
+#ifdef SUDO_BIN
   int use_sudo=0;
+#endif
   const list_disk_t *element_disk;
   const list_disk_t *current_disk;
   static const struct MenuItem menuMain[]=
@@ -130,10 +132,11 @@ static int testdisk_disk_selection_ncurses(int verbose,int dump_ind, const list_
 #endif
       waddstr(stdscr,"Disk capacity must be correctly detected for a successful recovery.");
       wmove(stdscr,line++,0);
-      wprintw(stdscr,"If a disk listed above has incorrect size, check HD jumper settings, BIOS");
+      wprintw(stdscr,"If a disk listed above has an incorrect size, check HD jumper settings and BIOS");
       wmove(stdscr,line,0);
       wprintw(stdscr,"detection, and install the latest OS patches and disk drivers."); 
     }
+#ifdef SUDO_BIN
     if(use_sudo > 0)
     {
       if(i<=NBR_DISK_MAX && element_disk==NULL)
@@ -142,6 +145,7 @@ static int testdisk_disk_selection_ncurses(int verbose,int dump_ind, const list_
 	options="PNOSQ";
     }
     else
+#endif
     {
       if(i<=NBR_DISK_MAX && element_disk==NULL)
 	options="OQ";
@@ -237,8 +241,7 @@ static int testdisk_disk_selection_cli(int verbose,int dump_ind, const list_disk
   }
   if(*current_cmd!=NULL)
   {
-    while(*current_cmd[0]==',')
-      (*current_cmd)++;
+    skip_comma_in_command(current_cmd);
     {
       disk_t *disk=current_disk->disk;
       autodetect_arch(disk, NULL);
