@@ -43,9 +43,7 @@
 #include "types.h"
 #include "common.h"
 #include "intrf.h"
-#ifdef HAVE_NCURSES
 #include "intrfn.h"
-#endif
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
@@ -117,7 +115,9 @@ static void display_version(void)
   printf("\n");
   printf("Version: %s\n", VERSION);
   printf("Compiler: %s\n", get_compiler());
+#ifdef RECORD_COMPILATION_DATE
   printf("Compilation date: %s\n", get_compilation_date());
+#endif
   printf("ext2fs lib: %s, ntfs lib: %s, reiserfs lib: %s, ewf lib: %s, curses lib: %s\n",
       td_ext2fs_version(), td_ntfs_version(), td_reiserfs_version(), td_ewf_version(), td_curses_version());
   printf("OS: %s\n" , get_os());
@@ -201,7 +201,9 @@ static int display_disk_list(list_disk_t *list_disk, const int testdisk_mode,
 int main( int argc, char **argv )
 {
   int i;
+#ifdef SUDO_BIN
   int use_sudo=0;
+#endif
   int verbose=0, dump_ind=0;
   int create_log=TD_LOG_NONE;
   int do_list=0;
@@ -227,7 +229,7 @@ int main( int argc, char **argv )
   sigemptyset(&action.sa_mask);
   sigaddset(&action.sa_mask, SIGINT);
   sigaddset(&action.sa_mask, SIGHUP);
-  action.sa_handler  = sighup_hdlr;
+  action.sa_handler  = &sighup_hdlr;
   action.sa_flags = 0;
   if(sigaction(SIGINT, &action, NULL)==-1)
   {
@@ -240,7 +242,7 @@ int main( int argc, char **argv )
     return -1;
   }
 #endif
-  printf("TestDisk %s, Data Recovery Utility, %s\nChristophe GRENIER <grenier@cgsecurity.org>\nhttp://www.cgsecurity.org\n",VERSION,TESTDISKDATE);
+  printf("TestDisk %s, Data Recovery Utility, %s\nChristophe GRENIER <grenier@cgsecurity.org>\nhttps://www.cgsecurity.org\n",VERSION,TESTDISKDATE);
   for(i=1;i<argc;i++)
   {
     if((strcmp(argv[i],"/dump")==0) || (strcmp(argv[i],"-dump")==0))
@@ -255,6 +257,10 @@ int main( int argc, char **argv )
       }
       logfile=argv[++i];
     }
+    else if((strcmp(argv[i],"/nolog")==0) ||(strcmp(argv[i],"-nolog")==0))
+    {
+      create_log=TD_LOG_NONE;
+    }
     else if((strcmp(argv[i],"/log")==0) ||(strcmp(argv[i],"-log")==0))
     {
       if(create_log==TD_LOG_NONE)
@@ -401,10 +407,12 @@ int main( int argc, char **argv )
       log_info(" %s", argv[i]);
     log_info("\n\n");
   }
-  log_info("TestDisk %s, Data Recovery Utility, %s\nChristophe GRENIER <grenier@cgsecurity.org>\nhttp://www.cgsecurity.org\n", VERSION, TESTDISKDATE);
+  log_info("TestDisk %s, Data Recovery Utility, %s\nChristophe GRENIER <grenier@cgsecurity.org>\nhttps://www.cgsecurity.org\n", VERSION, TESTDISKDATE);
   log_info("OS: %s\n" , get_os());
   log_info("Compiler: %s\n", get_compiler());
+#ifdef RECORD_COMPILATION_DATE
   log_info("Compilation date: %s\n", get_compilation_date());
+#endif
   log_info("ext2fs lib: %s, ntfs lib: %s, reiserfs lib: %s, ewf lib: %s, curses lib: %s\n",
       td_ext2fs_version(), td_ntfs_version(), td_reiserfs_version(), td_ewf_version(), td_curses_version());
 #if defined(__CYGWIN__) || defined(__MINGW32__) || defined(DJGPP)
@@ -451,9 +459,11 @@ int main( int argc, char **argv )
 #endif
 #endif
   }
-#endif
   if(use_sudo==0)
     use_sudo=do_curses_testdisk(verbose,dump_ind,list_disk,saveheader,cmd_device,&cmd_run);
+#else
+  do_curses_testdisk(verbose,dump_ind,list_disk,saveheader,cmd_device,&cmd_run);
+#endif
 #ifdef HAVE_NCURSES
   end_ncurses();
 #endif
@@ -463,11 +473,9 @@ int main( int argc, char **argv )
     int command='Q';
     if(cmd_run!=NULL)
     {
-      while(cmd_run[0]==',')
-	(cmd_run)++;
-      if(strncmp(cmd_run,"list",4)==0)
+      skip_comma_in_command(&cmd_run);
+      if(check_command(&cmd_run,"list",4)==0)
       {
-	(cmd_run)+=4;
 	command='L';
       }
       else if(cmd_run[0]!='\0')
@@ -509,7 +517,7 @@ int main( int argc, char **argv )
     printf("TestDisk will try to restart itself using the sudo command to get\n");
     printf("root (superuser) privileges.\n");
     printf("\n");
-    run_sudo(argc, argv);
+    run_sudo(argc, argv, create_log);
   }
 #endif
   return 0;
