@@ -35,8 +35,8 @@
 #include "log.h"
 #include "vmfs.h"
 
-static int test_VMFS(disk_t *disk, const struct vmfs_volume *sb, partition_t *partition, const int dump_ind);
-static int set_VMFS_info(const struct vmfs_volume *sb, partition_t *partition);
+static int test_VMFS(disk_t *disk, const struct vmfs_volume *sb, const partition_t *partition, const int dump_ind);
+static void set_VMFS_info(const struct vmfs_volume *sb, partition_t *partition);
 
 int check_VMFS(disk_t *disk,partition_t *partition)
 {
@@ -56,10 +56,10 @@ int check_VMFS(disk_t *disk,partition_t *partition)
   return 0;
 }
 
-static int set_VMFS_info(const struct vmfs_volume *sb, partition_t *partition)
+static void set_VMFS_info(const struct vmfs_volume *sb, partition_t *partition)
 {
+  partition->upart_type=UP_VMFS;
   sprintf(partition->info,"VMFS %lu", (long unsigned)le32(sb->version));
-  return 0;
 }
 
 int recover_VMFS(disk_t *disk, const struct vmfs_volume *sb, partition_t *partition, const int verbose, const int dump_ind)
@@ -82,7 +82,7 @@ int recover_VMFS(disk_t *disk, const struct vmfs_volume *sb, partition_t *partit
   return 0;
 }
 
-static int test_VMFS(disk_t *disk, const struct vmfs_volume *sb, partition_t *partition, const int dump_ind)
+static int test_VMFS(disk_t *disk, const struct vmfs_volume *sb, const partition_t *partition, const int dump_ind)
 {
   if(le32(sb->magic)!=0xc001d00d || le32(sb->version)>20)
     return 1;
@@ -95,8 +95,5 @@ static int test_VMFS(disk_t *disk, const struct vmfs_volume *sb, partition_t *pa
           offset2sector(disk,partition->part_offset));
     dump_log(sb,DEFAULT_SECTOR_SIZE);
   }
-  if(partition==NULL)
-    return 0;
-  partition->upart_type=UP_VMFS;
   return 0;
 }
